// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_

#include <stddef.h>

#include "base/check_op.h"
#include "third_party/blink/public/mojom/use_counter/metrics/css_property_id.mojom-blink-forward.h"
#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/platform/wtf/allocator/allocator.h"
#include "third_party/blink/renderer/platform/wtf/hash_functions.h"
#include "third_party/blink/renderer/platform/wtf/hash_traits.h"

namespace WTF {
class AtomicString;
class String;
}  // namespace WTF

namespace blink {

class ExecutionContext;

enum class CSSPropertyID {
    kInvalid = 0,
    kVariable = 1,
    kColorScheme = 2,
    kForcedColorAdjust = 3,
    kMaskImage = 4,
    kMathDepth = 5,
    kPosition = 6,
    kPositionAnchor = 7,
    kTextSizeAdjust = 8,
    kAppearance = 9,
    kColor = 10,
    kDirection = 11,
    kFontFamily = 12,
    kFontFeatureSettings = 13,
    kFontKerning = 14,
    kFontOpticalSizing = 15,
    kFontPalette = 16,
    kFontSize = 17,
    kFontSizeAdjust = 18,
    kFontStretch = 19,
    kFontStyle = 20,
    kFontSynthesisSmallCaps = 21,
    kFontSynthesisStyle = 22,
    kFontSynthesisWeight = 23,
    kFontVariantAlternates = 24,
    kFontVariantCaps = 25,
    kFontVariantEastAsian = 26,
    kFontVariantEmoji = 27,
    kFontVariantLigatures = 28,
    kFontVariantNumeric = 29,
    kFontVariantPosition = 30,
    kFontVariationSettings = 31,
    kFontWeight = 32,
    kInsetArea = 33,
    kInternalVisitedColor = 34,
    kPositionArea = 35,
    kTextOrientation = 36,
    kTextRendering = 37,
    kTextSpacingTrim = 38,
    kWebkitFontSmoothing = 39,
    kWebkitLocale = 40,
    kWebkitTextOrientation = 41,
    kWebkitWritingMode = 42,
    kWritingMode = 43,
    kZoom = 44,
    kAccentColor = 45,
    kAdditiveSymbols = 46,
    kAlignContent = 47,
    kAlignItems = 48,
    kAlignSelf = 49,
    kAlignmentBaseline = 50,
    kAll = 51,
    kAnchorName = 52,
    kAnchorScope = 53,
    kAnimationComposition = 54,
    kAnimationDelay = 55,
    kAnimationDirection = 56,
    kAnimationDuration = 57,
    kAnimationFillMode = 58,
    kAnimationIterationCount = 59,
    kAnimationName = 60,
    kAnimationPlayState = 61,
    kAnimationRangeEnd = 62,
    kAnimationRangeStart = 63,
    kAnimationTimeline = 64,
    kAnimationTimingFunction = 65,
    kAppRegion = 66,
    kAscentOverride = 67,
    kAspectRatio = 68,
    kBackdropFilter = 69,
    kBackfaceVisibility = 70,
    kBackgroundAttachment = 71,
    kBackgroundBlendMode = 72,
    kBackgroundClip = 73,
    kBackgroundColor = 74,
    kBackgroundImage = 75,
    kBackgroundOrigin = 76,
    kBackgroundPositionX = 77,
    kBackgroundPositionY = 78,
    kBackgroundRepeat = 79,
    kBackgroundSize = 80,
    kBasePalette = 81,
    kBaselineShift = 82,
    kBaselineSource = 83,
    kBlockSize = 84,
    kBorderBlockEndColor = 85,
    kBorderBlockEndStyle = 86,
    kBorderBlockEndWidth = 87,
    kBorderBlockStartColor = 88,
    kBorderBlockStartStyle = 89,
    kBorderBlockStartWidth = 90,
    kBorderBottomColor = 91,
    kBorderBottomLeftRadius = 92,
    kBorderBottomRightRadius = 93,
    kBorderBottomStyle = 94,
    kBorderBottomWidth = 95,
    kBorderCollapse = 96,
    kBorderEndEndRadius = 97,
    kBorderEndStartRadius = 98,
    kBorderImageOutset = 99,
    kBorderImageRepeat = 100,
    kBorderImageSlice = 101,
    kBorderImageSource = 102,
    kBorderImageWidth = 103,
    kBorderInlineEndColor = 104,
    kBorderInlineEndStyle = 105,
    kBorderInlineEndWidth = 106,
    kBorderInlineStartColor = 107,
    kBorderInlineStartStyle = 108,
    kBorderInlineStartWidth = 109,
    kBorderLeftColor = 110,
    kBorderLeftStyle = 111,
    kBorderLeftWidth = 112,
    kBorderRightColor = 113,
    kBorderRightStyle = 114,
    kBorderRightWidth = 115,
    kBorderStartEndRadius = 116,
    kBorderStartStartRadius = 117,
    kBorderTopColor = 118,
    kBorderTopLeftRadius = 119,
    kBorderTopRightRadius = 120,
    kBorderTopStyle = 121,
    kBorderTopWidth = 122,
    kBottom = 123,
    kBoxDecorationBreak = 124,
    kBoxShadow = 125,
    kBoxSizing = 126,
    kBreakAfter = 127,
    kBreakBefore = 128,
    kBreakInside = 129,
    kBufferedRendering = 130,
    kCaptionSide = 131,
    kCaretColor = 132,
    kClear = 133,
    kClip = 134,
    kClipPath = 135,
    kClipRule = 136,
    kColorInterpolation = 137,
    kColorInterpolationFilters = 138,
    kColorRendering = 139,
    kColumnCount = 140,
    kColumnFill = 141,
    kColumnGap = 142,
    kColumnRuleColor = 143,
    kColumnRuleStyle = 144,
    kColumnRuleWidth = 145,
    kColumnSpan = 146,
    kColumnWidth = 147,
    kContain = 148,
    kContainIntrinsicBlockSize = 149,
    kContainIntrinsicHeight = 150,
    kContainIntrinsicInlineSize = 151,
    kContainIntrinsicWidth = 152,
    kContainerName = 153,
    kContainerType = 154,
    kContent = 155,
    kContentVisibility = 156,
    kCounterIncrement = 157,
    kCounterReset = 158,
    kCounterSet = 159,
    kCursor = 160,
    kCx = 161,
    kCy = 162,
    kD = 163,
    kDescentOverride = 164,
    kDisplay = 165,
    kDominantBaseline = 166,
    kDynamicRangeLimit = 167,
    kEmptyCells = 168,
    kFallback = 169,
    kFieldSizing = 170,
    kFill = 171,
    kFillOpacity = 172,
    kFillRule = 173,
    kFilter = 174,
    kFlexBasis = 175,
    kFlexDirection = 176,
    kFlexGrow = 177,
    kFlexShrink = 178,
    kFlexWrap = 179,
    kFloat = 180,
    kFloodColor = 181,
    kFloodOpacity = 182,
    kFontDisplay = 183,
    kGridAutoColumns = 184,
    kGridAutoFlow = 185,
    kGridAutoRows = 186,
    kGridColumnEnd = 187,
    kGridColumnStart = 188,
    kGridRowEnd = 189,
    kGridRowStart = 190,
    kGridTemplateAreas = 191,
    kGridTemplateColumns = 192,
    kGridTemplateRows = 193,
    kHeight = 194,
    kHyphenateCharacter = 195,
    kHyphenateLimitChars = 196,
    kHyphens = 197,
    kImageOrientation = 198,
    kImageRendering = 199,
    kInherits = 200,
    kInitialLetter = 201,
    kInitialValue = 202,
    kInlineSize = 203,
    kInsetBlockEnd = 204,
    kInsetBlockStart = 205,
    kInsetInlineEnd = 206,
    kInsetInlineStart = 207,
    kInternalAlignContentBlock = 208,
    kInternalEmptyLineHeight = 209,
    kInternalFontSizeDelta = 210,
    kInternalForcedBackgroundColor = 211,
    kInternalForcedBorderColor = 212,
    kInternalForcedColor = 213,
    kInternalForcedOutlineColor = 214,
    kInternalForcedVisitedColor = 215,
    kInternalOverflowBlock = 216,
    kInternalOverflowInline = 217,
    kInternalVisitedBackgroundColor = 218,
    kInternalVisitedBorderBlockEndColor = 219,
    kInternalVisitedBorderBlockStartColor = 220,
    kInternalVisitedBorderBottomColor = 221,
    kInternalVisitedBorderInlineEndColor = 222,
    kInternalVisitedBorderInlineStartColor = 223,
    kInternalVisitedBorderLeftColor = 224,
    kInternalVisitedBorderRightColor = 225,
    kInternalVisitedBorderTopColor = 226,
    kInternalVisitedCaretColor = 227,
    kInternalVisitedColumnRuleColor = 228,
    kInternalVisitedFill = 229,
    kInternalVisitedOutlineColor = 230,
    kInternalVisitedStroke = 231,
    kInternalVisitedTextDecorationColor = 232,
    kInternalVisitedTextEmphasisColor = 233,
    kInternalVisitedTextFillColor = 234,
    kInternalVisitedTextStrokeColor = 235,
    kInterpolateSize = 236,
    kIsolation = 237,
    kJustifyContent = 238,
    kJustifyItems = 239,
    kJustifySelf = 240,
    kLeft = 241,
    kLetterSpacing = 242,
    kLightingColor = 243,
    kLineBreak = 244,
    kLineClamp = 245,
    kLineGapOverride = 246,
    kLineHeight = 247,
    kListStyleImage = 248,
    kListStylePosition = 249,
    kListStyleType = 250,
    kMarginBlockEnd = 251,
    kMarginBlockStart = 252,
    kMarginBottom = 253,
    kMarginInlineEnd = 254,
    kMarginInlineStart = 255,
    kMarginLeft = 256,
    kMarginRight = 257,
    kMarginTop = 258,
    kMarkerEnd = 259,
    kMarkerMid = 260,
    kMarkerStart = 261,
    kMaskClip = 262,
    kMaskComposite = 263,
    kMaskMode = 264,
    kMaskOrigin = 265,
    kMaskRepeat = 266,
    kMaskSize = 267,
    kMaskType = 268,
    kMasonrySlack = 269,
    kMasonryTemplateTracks = 270,
    kMasonryTrackEnd = 271,
    kMasonryTrackStart = 272,
    kMathShift = 273,
    kMathStyle = 274,
    kMaxBlockSize = 275,
    kMaxHeight = 276,
    kMaxInlineSize = 277,
    kMaxWidth = 278,
    kMinBlockSize = 279,
    kMinHeight = 280,
    kMinInlineSize = 281,
    kMinWidth = 282,
    kMixBlendMode = 283,
    kNavigation = 284,
    kNegative = 285,
    kObjectFit = 286,
    kObjectPosition = 287,
    kObjectViewBox = 288,
    kOffsetAnchor = 289,
    kOffsetDistance = 290,
    kOffsetPath = 291,
    kOffsetPosition = 292,
    kOffsetRotate = 293,
    kOpacity = 294,
    kOrder = 295,
    kOriginTrialTestProperty = 296,
    kOrphans = 297,
    kOutlineColor = 298,
    kOutlineOffset = 299,
    kOutlineStyle = 300,
    kOutlineWidth = 301,
    kOverflowAnchor = 302,
    kOverflowBlock = 303,
    kOverflowClipMargin = 304,
    kOverflowInline = 305,
    kOverflowWrap = 306,
    kOverflowX = 307,
    kOverflowY = 308,
    kOverlay = 309,
    kOverrideColors = 310,
    kOverscrollBehaviorBlock = 311,
    kOverscrollBehaviorInline = 312,
    kOverscrollBehaviorX = 313,
    kOverscrollBehaviorY = 314,
    kPad = 315,
    kPaddingBlockEnd = 316,
    kPaddingBlockStart = 317,
    kPaddingBottom = 318,
    kPaddingInlineEnd = 319,
    kPaddingInlineStart = 320,
    kPaddingLeft = 321,
    kPaddingRight = 322,
    kPaddingTop = 323,
    kPage = 324,
    kPageOrientation = 325,
    kPaintOrder = 326,
    kPerspective = 327,
    kPerspectiveOrigin = 328,
    kPointerEvents = 329,
    kPopoverHideDelay = 330,
    kPopoverShowDelay = 331,
    kPositionTryFallbacks = 332,
    kPositionTryOptions = 333,
    kPositionTryOrder = 334,
    kPositionVisibility = 335,
    kPrefix = 336,
    kQuotes = 337,
    kR = 338,
    kRange = 339,
    kReadingFlow = 340,
    kResize = 341,
    kRight = 342,
    kRotate = 343,
    kRowGap = 344,
    kRubyAlign = 345,
    kRubyPosition = 346,
    kRx = 347,
    kRy = 348,
    kScale = 349,
    kScrollBehavior = 350,
    kScrollMarginBlockEnd = 351,
    kScrollMarginBlockStart = 352,
    kScrollMarginBottom = 353,
    kScrollMarginInlineEnd = 354,
    kScrollMarginInlineStart = 355,
    kScrollMarginLeft = 356,
    kScrollMarginRight = 357,
    kScrollMarginTop = 358,
    kScrollMarkerGroup = 359,
    kScrollPaddingBlockEnd = 360,
    kScrollPaddingBlockStart = 361,
    kScrollPaddingBottom = 362,
    kScrollPaddingInlineEnd = 363,
    kScrollPaddingInlineStart = 364,
    kScrollPaddingLeft = 365,
    kScrollPaddingRight = 366,
    kScrollPaddingTop = 367,
    kScrollSnapAlign = 368,
    kScrollSnapStop = 369,
    kScrollSnapType = 370,
    kScrollStartBlock = 371,
    kScrollStartInline = 372,
    kScrollStartTargetBlock = 373,
    kScrollStartTargetInline = 374,
    kScrollStartTargetX = 375,
    kScrollStartTargetY = 376,
    kScrollStartX = 377,
    kScrollStartY = 378,
    kScrollTimelineAxis = 379,
    kScrollTimelineName = 380,
    kScrollbarColor = 381,
    kScrollbarGutter = 382,
    kScrollbarWidth = 383,
    kShapeImageThreshold = 384,
    kShapeMargin = 385,
    kShapeOutside = 386,
    kShapeRendering = 387,
    kSize = 388,
    kSizeAdjust = 389,
    kSpeak = 390,
    kSpeakAs = 391,
    kSrc = 392,
    kStopColor = 393,
    kStopOpacity = 394,
    kStroke = 395,
    kStrokeDasharray = 396,
    kStrokeDashoffset = 397,
    kStrokeLinecap = 398,
    kStrokeLinejoin = 399,
    kStrokeMiterlimit = 400,
    kStrokeOpacity = 401,
    kStrokeWidth = 402,
    kSuffix = 403,
    kSymbols = 404,
    kSyntax = 405,
    kSystem = 406,
    kTabSize = 407,
    kTableLayout = 408,
    kTextAlign = 409,
    kTextAlignLast = 410,
    kTextAnchor = 411,
    kTextAutospace = 412,
    kTextBoxEdge = 413,
    kTextBoxTrim = 414,
    kTextCombineUpright = 415,
    kTextDecorationColor = 416,
    kTextDecorationLine = 417,
    kTextDecorationSkipInk = 418,
    kTextDecorationStyle = 419,
    kTextDecorationThickness = 420,
    kTextEmphasisColor = 421,
    kTextEmphasisPosition = 422,
    kTextEmphasisStyle = 423,
    kTextIndent = 424,
    kTextOverflow = 425,
    kTextShadow = 426,
    kTextTransform = 427,
    kTextUnderlineOffset = 428,
    kTextUnderlinePosition = 429,
    kTextWrapMode = 430,
    kTextWrapStyle = 431,
    kTimelineScope = 432,
    kTop = 433,
    kTouchAction = 434,
    kTransform = 435,
    kTransformBox = 436,
    kTransformOrigin = 437,
    kTransformStyle = 438,
    kTransitionBehavior = 439,
    kTransitionDelay = 440,
    kTransitionDuration = 441,
    kTransitionProperty = 442,
    kTransitionTimingFunction = 443,
    kTranslate = 444,
    kTypes = 445,
    kUnicodeBidi = 446,
    kUnicodeRange = 447,
    kUserSelect = 448,
    kVectorEffect = 449,
    kVerticalAlign = 450,
    kViewTimelineAxis = 451,
    kViewTimelineInset = 452,
    kViewTimelineName = 453,
    kViewTransitionClass = 454,
    kViewTransitionGroup = 455,
    kViewTransitionName = 456,
    kVisibility = 457,
    kWebkitBorderHorizontalSpacing = 458,
    kWebkitBorderImage = 459,
    kWebkitBorderVerticalSpacing = 460,
    kWebkitBoxAlign = 461,
    kWebkitBoxDecorationBreak = 462,
    kWebkitBoxDirection = 463,
    kWebkitBoxFlex = 464,
    kWebkitBoxOrdinalGroup = 465,
    kWebkitBoxOrient = 466,
    kWebkitBoxPack = 467,
    kWebkitBoxReflect = 468,
    kWebkitLineBreak = 469,
    kWebkitLineClamp = 470,
    kWebkitMaskBoxImageOutset = 471,
    kWebkitMaskBoxImageRepeat = 472,
    kWebkitMaskBoxImageSlice = 473,
    kWebkitMaskBoxImageSource = 474,
    kWebkitMaskBoxImageWidth = 475,
    kWebkitMaskPositionX = 476,
    kWebkitMaskPositionY = 477,
    kWebkitPerspectiveOriginX = 478,
    kWebkitPerspectiveOriginY = 479,
    kWebkitPrintColorAdjust = 480,
    kWebkitRtlOrdering = 481,
    kWebkitRubyPosition = 482,
    kWebkitTapHighlightColor = 483,
    kWebkitTextCombine = 484,
    kWebkitTextDecorationsInEffect = 485,
    kWebkitTextFillColor = 486,
    kWebkitTextSecurity = 487,
    kWebkitTextStrokeColor = 488,
    kWebkitTextStrokeWidth = 489,
    kWebkitTransformOriginX = 490,
    kWebkitTransformOriginY = 491,
    kWebkitTransformOriginZ = 492,
    kWebkitUserDrag = 493,
    kWebkitUserModify = 494,
    kWhiteSpaceCollapse = 495,
    kWidows = 496,
    kWidth = 497,
    kWillChange = 498,
    kWordBreak = 499,
    kWordSpacing = 500,
    kX = 501,
    kY = 502,
    kZIndex = 503,
    kAlternativeAnimationWithTimeline = 504,
    kAlternativePositionTry = 505,
    kAnimation = 506,
    kAnimationRange = 507,
    kBackground = 508,
    kBackgroundPosition = 509,
    kBorder = 510,
    kBorderBlock = 511,
    kBorderBlockColor = 512,
    kBorderBlockEnd = 513,
    kBorderBlockStart = 514,
    kBorderBlockStyle = 515,
    kBorderBlockWidth = 516,
    kBorderBottom = 517,
    kBorderColor = 518,
    kBorderImage = 519,
    kBorderInline = 520,
    kBorderInlineColor = 521,
    kBorderInlineEnd = 522,
    kBorderInlineStart = 523,
    kBorderInlineStyle = 524,
    kBorderInlineWidth = 525,
    kBorderLeft = 526,
    kBorderRadius = 527,
    kBorderRight = 528,
    kBorderSpacing = 529,
    kBorderStyle = 530,
    kBorderTop = 531,
    kBorderWidth = 532,
    kColumnRule = 533,
    kColumns = 534,
    kContainIntrinsicSize = 535,
    kContainer = 536,
    kFlex = 537,
    kFlexFlow = 538,
    kFont = 539,
    kFontSynthesis = 540,
    kFontVariant = 541,
    kGap = 542,
    kGrid = 543,
    kGridArea = 544,
    kGridColumn = 545,
    kGridRow = 546,
    kGridTemplate = 547,
    kInset = 548,
    kInsetBlock = 549,
    kInsetInline = 550,
    kListStyle = 551,
    kMargin = 552,
    kMarginBlock = 553,
    kMarginInline = 554,
    kMarker = 555,
    kMask = 556,
    kMaskPosition = 557,
    kMasonryTrack = 558,
    kOffset = 559,
    kOutline = 560,
    kOverflow = 561,
    kOverscrollBehavior = 562,
    kPadding = 563,
    kPaddingBlock = 564,
    kPaddingInline = 565,
    kPageBreakAfter = 566,
    kPageBreakBefore = 567,
    kPageBreakInside = 568,
    kPlaceContent = 569,
    kPlaceItems = 570,
    kPlaceSelf = 571,
    kPositionTry = 572,
    kScrollMargin = 573,
    kScrollMarginBlock = 574,
    kScrollMarginInline = 575,
    kScrollPadding = 576,
    kScrollPaddingBlock = 577,
    kScrollPaddingInline = 578,
    kScrollStart = 579,
    kScrollStartTarget = 580,
    kScrollTimeline = 581,
    kTextBox = 582,
    kTextDecoration = 583,
    kTextEmphasis = 584,
    kTextSpacing = 585,
    kTextWrap = 586,
    kTransition = 587,
    kViewTimeline = 588,
    kWebkitColumnBreakAfter = 589,
    kWebkitColumnBreakBefore = 590,
    kWebkitColumnBreakInside = 591,
    kWebkitMaskBoxImage = 592,
    kWebkitTextStroke = 593,
    kWhiteSpace = 594,
    kAliasWebkitAppearance = 595,
    kAliasWebkitAppRegion = 596,
    kAliasWebkitMaskClip = 597,
    kAliasWebkitMaskComposite = 598,
    kAliasWebkitMaskImage = 599,
    kAliasWebkitMaskOrigin = 600,
    kAliasWebkitMaskRepeat = 601,
    kAliasWebkitMaskSize = 602,
    kAliasWebkitBorderEndColor = 603,
    kAliasWebkitBorderEndStyle = 604,
    kAliasWebkitBorderEndWidth = 605,
    kAliasWebkitBorderStartColor = 606,
    kAliasWebkitBorderStartStyle = 607,
    kAliasWebkitBorderStartWidth = 608,
    kAliasWebkitBorderBeforeColor = 609,
    kAliasWebkitBorderBeforeStyle = 610,
    kAliasWebkitBorderBeforeWidth = 611,
    kAliasWebkitBorderAfterColor = 612,
    kAliasWebkitBorderAfterStyle = 613,
    kAliasWebkitBorderAfterWidth = 614,
    kAliasWebkitMarginEnd = 615,
    kAliasWebkitMarginStart = 616,
    kAliasWebkitMarginBefore = 617,
    kAliasWebkitMarginAfter = 618,
    kAliasWebkitPaddingEnd = 619,
    kAliasWebkitPaddingStart = 620,
    kAliasWebkitPaddingBefore = 621,
    kAliasWebkitPaddingAfter = 622,
    kAliasWebkitLogicalWidth = 623,
    kAliasWebkitLogicalHeight = 624,
    kAliasWebkitMinLogicalWidth = 625,
    kAliasWebkitMinLogicalHeight = 626,
    kAliasWebkitMaxLogicalWidth = 627,
    kAliasWebkitMaxLogicalHeight = 628,
    kAliasWebkitBorderAfter = 629,
    kAliasWebkitBorderBefore = 630,
    kAliasWebkitBorderEnd = 631,
    kAliasWebkitBorderStart = 632,
    kAliasWebkitMask = 633,
    kAliasWebkitMaskPosition = 634,
    kAliasEpubCaptionSide = 635,
    kAliasEpubTextCombine = 636,
    kAliasEpubTextEmphasis = 637,
    kAliasEpubTextEmphasisColor = 638,
    kAliasEpubTextEmphasisStyle = 639,
    kAliasEpubTextOrientation = 640,
    kAliasEpubTextTransform = 641,
    kAliasEpubWordBreak = 642,
    kAliasEpubWritingMode = 643,
    kAliasWebkitAlignContent = 644,
    kAliasWebkitAlignItems = 645,
    kAliasWebkitAlignSelf = 646,
    kAliasWebkitAnimation = 647,
    kAliasWebkitAlternativeAnimationWithTimeline = 648,
    kAliasWebkitAnimationDelay = 649,
    kAliasWebkitAnimationDirection = 650,
    kAliasWebkitAnimationDuration = 651,
    kAliasWebkitAnimationFillMode = 652,
    kAliasWebkitAnimationIterationCount = 653,
    kAliasWebkitAnimationName = 654,
    kAliasWebkitAnimationPlayState = 655,
    kAliasWebkitAnimationTimingFunction = 656,
    kAliasWebkitBackfaceVisibility = 657,
    kAliasWebkitBackgroundClip = 658,
    kAliasWebkitBackgroundOrigin = 659,
    kAliasWebkitBackgroundSize = 660,
    kAliasWebkitBorderBottomLeftRadius = 661,
    kAliasWebkitBorderBottomRightRadius = 662,
    kAliasWebkitBorderRadius = 663,
    kAliasWebkitBorderTopLeftRadius = 664,
    kAliasWebkitBorderTopRightRadius = 665,
    kAliasWebkitBoxShadow = 666,
    kAliasWebkitBoxSizing = 667,
    kAliasWebkitClipPath = 668,
    kAliasWebkitColumnCount = 669,
    kAliasWebkitColumnGap = 670,
    kAliasWebkitColumnRule = 671,
    kAliasWebkitColumnRuleColor = 672,
    kAliasWebkitColumnRuleStyle = 673,
    kAliasWebkitColumnRuleWidth = 674,
    kAliasWebkitColumnSpan = 675,
    kAliasWebkitColumnWidth = 676,
    kAliasWebkitColumns = 677,
    kAliasWebkitFilter = 678,
    kAliasWebkitFlex = 679,
    kAliasWebkitFlexBasis = 680,
    kAliasWebkitFlexDirection = 681,
    kAliasWebkitFlexFlow = 682,
    kAliasWebkitFlexGrow = 683,
    kAliasWebkitFlexShrink = 684,
    kAliasWebkitFlexWrap = 685,
    kAliasWebkitFontFeatureSettings = 686,
    kAliasWebkitHyphenateCharacter = 687,
    kAliasWebkitJustifyContent = 688,
    kAliasWebkitOpacity = 689,
    kAliasWebkitOrder = 690,
    kAliasWebkitPerspective = 691,
    kAliasWebkitPerspectiveOrigin = 692,
    kAliasWebkitShapeImageThreshold = 693,
    kAliasWebkitShapeMargin = 694,
    kAliasWebkitShapeOutside = 695,
    kAliasWebkitTextEmphasis = 696,
    kAliasWebkitTextEmphasisColor = 697,
    kAliasWebkitTextEmphasisPosition = 698,
    kAliasWebkitTextEmphasisStyle = 699,
    kAliasWebkitTextSizeAdjust = 700,
    kAliasWebkitTransform = 701,
    kAliasWebkitTransformOrigin = 702,
    kAliasWebkitTransformStyle = 703,
    kAliasWebkitTransition = 704,
    kAliasWebkitTransitionDelay = 705,
    kAliasWebkitTransitionDuration = 706,
    kAliasWebkitTransitionProperty = 707,
    kAliasWebkitTransitionTimingFunction = 708,
    kAliasWebkitUserSelect = 709,
    kAliasWordWrap = 710,
    kAliasGridColumnGap = 711,
    kAliasGridRowGap = 712,
    kAliasGridGap = 713,
};

const CSSPropertyID kCSSPropertyAliasList[] = {
    CSSPropertyID::kAliasWebkitAppearance,
    CSSPropertyID::kAliasWebkitAppRegion,
    CSSPropertyID::kAliasWebkitMaskClip,
    CSSPropertyID::kAliasWebkitMaskComposite,
    CSSPropertyID::kAliasWebkitMaskImage,
    CSSPropertyID::kAliasWebkitMaskOrigin,
    CSSPropertyID::kAliasWebkitMaskRepeat,
    CSSPropertyID::kAliasWebkitMaskSize,
    CSSPropertyID::kAliasWebkitBorderEndColor,
    CSSPropertyID::kAliasWebkitBorderEndStyle,
    CSSPropertyID::kAliasWebkitBorderEndWidth,
    CSSPropertyID::kAliasWebkitBorderStartColor,
    CSSPropertyID::kAliasWebkitBorderStartStyle,
    CSSPropertyID::kAliasWebkitBorderStartWidth,
    CSSPropertyID::kAliasWebkitBorderBeforeColor,
    CSSPropertyID::kAliasWebkitBorderBeforeStyle,
    CSSPropertyID::kAliasWebkitBorderBeforeWidth,
    CSSPropertyID::kAliasWebkitBorderAfterColor,
    CSSPropertyID::kAliasWebkitBorderAfterStyle,
    CSSPropertyID::kAliasWebkitBorderAfterWidth,
    CSSPropertyID::kAliasWebkitMarginEnd,
    CSSPropertyID::kAliasWebkitMarginStart,
    CSSPropertyID::kAliasWebkitMarginBefore,
    CSSPropertyID::kAliasWebkitMarginAfter,
    CSSPropertyID::kAliasWebkitPaddingEnd,
    CSSPropertyID::kAliasWebkitPaddingStart,
    CSSPropertyID::kAliasWebkitPaddingBefore,
    CSSPropertyID::kAliasWebkitPaddingAfter,
    CSSPropertyID::kAliasWebkitLogicalWidth,
    CSSPropertyID::kAliasWebkitLogicalHeight,
    CSSPropertyID::kAliasWebkitMinLogicalWidth,
    CSSPropertyID::kAliasWebkitMinLogicalHeight,
    CSSPropertyID::kAliasWebkitMaxLogicalWidth,
    CSSPropertyID::kAliasWebkitMaxLogicalHeight,
    CSSPropertyID::kAliasWebkitBorderAfter,
    CSSPropertyID::kAliasWebkitBorderBefore,
    CSSPropertyID::kAliasWebkitBorderEnd,
    CSSPropertyID::kAliasWebkitBorderStart,
    CSSPropertyID::kAliasWebkitMask,
    CSSPropertyID::kAliasWebkitMaskPosition,
    CSSPropertyID::kAliasEpubCaptionSide,
    CSSPropertyID::kAliasEpubTextCombine,
    CSSPropertyID::kAliasEpubTextEmphasis,
    CSSPropertyID::kAliasEpubTextEmphasisColor,
    CSSPropertyID::kAliasEpubTextEmphasisStyle,
    CSSPropertyID::kAliasEpubTextOrientation,
    CSSPropertyID::kAliasEpubTextTransform,
    CSSPropertyID::kAliasEpubWordBreak,
    CSSPropertyID::kAliasEpubWritingMode,
    CSSPropertyID::kAliasWebkitAlignContent,
    CSSPropertyID::kAliasWebkitAlignItems,
    CSSPropertyID::kAliasWebkitAlignSelf,
    CSSPropertyID::kAliasWebkitAnimation,
    CSSPropertyID::kAliasWebkitAlternativeAnimationWithTimeline,
    CSSPropertyID::kAliasWebkitAnimationDelay,
    CSSPropertyID::kAliasWebkitAnimationDirection,
    CSSPropertyID::kAliasWebkitAnimationDuration,
    CSSPropertyID::kAliasWebkitAnimationFillMode,
    CSSPropertyID::kAliasWebkitAnimationIterationCount,
    CSSPropertyID::kAliasWebkitAnimationName,
    CSSPropertyID::kAliasWebkitAnimationPlayState,
    CSSPropertyID::kAliasWebkitAnimationTimingFunction,
    CSSPropertyID::kAliasWebkitBackfaceVisibility,
    CSSPropertyID::kAliasWebkitBackgroundClip,
    CSSPropertyID::kAliasWebkitBackgroundOrigin,
    CSSPropertyID::kAliasWebkitBackgroundSize,
    CSSPropertyID::kAliasWebkitBorderBottomLeftRadius,
    CSSPropertyID::kAliasWebkitBorderBottomRightRadius,
    CSSPropertyID::kAliasWebkitBorderRadius,
    CSSPropertyID::kAliasWebkitBorderTopLeftRadius,
    CSSPropertyID::kAliasWebkitBorderTopRightRadius,
    CSSPropertyID::kAliasWebkitBoxShadow,
    CSSPropertyID::kAliasWebkitBoxSizing,
    CSSPropertyID::kAliasWebkitClipPath,
    CSSPropertyID::kAliasWebkitColumnCount,
    CSSPropertyID::kAliasWebkitColumnGap,
    CSSPropertyID::kAliasWebkitColumnRule,
    CSSPropertyID::kAliasWebkitColumnRuleColor,
    CSSPropertyID::kAliasWebkitColumnRuleStyle,
    CSSPropertyID::kAliasWebkitColumnRuleWidth,
    CSSPropertyID::kAliasWebkitColumnSpan,
    CSSPropertyID::kAliasWebkitColumnWidth,
    CSSPropertyID::kAliasWebkitColumns,
    CSSPropertyID::kAliasWebkitFilter,
    CSSPropertyID::kAliasWebkitFlex,
    CSSPropertyID::kAliasWebkitFlexBasis,
    CSSPropertyID::kAliasWebkitFlexDirection,
    CSSPropertyID::kAliasWebkitFlexFlow,
    CSSPropertyID::kAliasWebkitFlexGrow,
    CSSPropertyID::kAliasWebkitFlexShrink,
    CSSPropertyID::kAliasWebkitFlexWrap,
    CSSPropertyID::kAliasWebkitFontFeatureSettings,
    CSSPropertyID::kAliasWebkitHyphenateCharacter,
    CSSPropertyID::kAliasWebkitJustifyContent,
    CSSPropertyID::kAliasWebkitOpacity,
    CSSPropertyID::kAliasWebkitOrder,
    CSSPropertyID::kAliasWebkitPerspective,
    CSSPropertyID::kAliasWebkitPerspectiveOrigin,
    CSSPropertyID::kAliasWebkitShapeImageThreshold,
    CSSPropertyID::kAliasWebkitShapeMargin,
    CSSPropertyID::kAliasWebkitShapeOutside,
    CSSPropertyID::kAliasWebkitTextEmphasis,
    CSSPropertyID::kAliasWebkitTextEmphasisColor,
    CSSPropertyID::kAliasWebkitTextEmphasisPosition,
    CSSPropertyID::kAliasWebkitTextEmphasisStyle,
    CSSPropertyID::kAliasWebkitTextSizeAdjust,
    CSSPropertyID::kAliasWebkitTransform,
    CSSPropertyID::kAliasWebkitTransformOrigin,
    CSSPropertyID::kAliasWebkitTransformStyle,
    CSSPropertyID::kAliasWebkitTransition,
    CSSPropertyID::kAliasWebkitTransitionDelay,
    CSSPropertyID::kAliasWebkitTransitionDuration,
    CSSPropertyID::kAliasWebkitTransitionProperty,
    CSSPropertyID::kAliasWebkitTransitionTimingFunction,
    CSSPropertyID::kAliasWebkitUserSelect,
    CSSPropertyID::kAliasWordWrap,
    CSSPropertyID::kAliasGridColumnGap,
    CSSPropertyID::kAliasGridRowGap,
    CSSPropertyID::kAliasGridGap,
};

const CSSPropertyID kCSSComputableProperties[] = {
    CSSPropertyID::kAccentColor,
    CSSPropertyID::kAlignContent,
    CSSPropertyID::kAlignItems,
    CSSPropertyID::kAlignSelf,
    CSSPropertyID::kAlignmentBaseline,
    CSSPropertyID::kAnchorName,
    CSSPropertyID::kAnchorScope,
    CSSPropertyID::kAnimationComposition,
    CSSPropertyID::kAnimationDelay,
    CSSPropertyID::kAnimationDirection,
    CSSPropertyID::kAnimationDuration,
    CSSPropertyID::kAnimationFillMode,
    CSSPropertyID::kAnimationIterationCount,
    CSSPropertyID::kAnimationName,
    CSSPropertyID::kAnimationPlayState,
    CSSPropertyID::kAnimationRangeEnd,
    CSSPropertyID::kAnimationRangeStart,
    CSSPropertyID::kAnimationTimeline,
    CSSPropertyID::kAnimationTimingFunction,
    CSSPropertyID::kAppRegion,
    CSSPropertyID::kAppearance,
    CSSPropertyID::kBackdropFilter,
    CSSPropertyID::kBackfaceVisibility,
    CSSPropertyID::kBackgroundAttachment,
    CSSPropertyID::kBackgroundBlendMode,
    CSSPropertyID::kBackgroundClip,
    CSSPropertyID::kBackgroundColor,
    CSSPropertyID::kBackgroundImage,
    CSSPropertyID::kBackgroundOrigin,
    CSSPropertyID::kBackgroundPosition,
    CSSPropertyID::kBackgroundRepeat,
    CSSPropertyID::kBackgroundSize,
    CSSPropertyID::kBaselineShift,
    CSSPropertyID::kBaselineSource,
    CSSPropertyID::kBlockSize,
    CSSPropertyID::kBorderBlockEndColor,
    CSSPropertyID::kBorderBlockEndStyle,
    CSSPropertyID::kBorderBlockEndWidth,
    CSSPropertyID::kBorderBlockStartColor,
    CSSPropertyID::kBorderBlockStartStyle,
    CSSPropertyID::kBorderBlockStartWidth,
    CSSPropertyID::kBorderBottomColor,
    CSSPropertyID::kBorderBottomLeftRadius,
    CSSPropertyID::kBorderBottomRightRadius,
    CSSPropertyID::kBorderBottomStyle,
    CSSPropertyID::kBorderBottomWidth,
    CSSPropertyID::kBorderCollapse,
    CSSPropertyID::kBorderEndEndRadius,
    CSSPropertyID::kBorderEndStartRadius,
    CSSPropertyID::kBorderImageOutset,
    CSSPropertyID::kBorderImageRepeat,
    CSSPropertyID::kBorderImageSlice,
    CSSPropertyID::kBorderImageSource,
    CSSPropertyID::kBorderImageWidth,
    CSSPropertyID::kBorderInlineEndColor,
    CSSPropertyID::kBorderInlineEndStyle,
    CSSPropertyID::kBorderInlineEndWidth,
    CSSPropertyID::kBorderInlineStartColor,
    CSSPropertyID::kBorderInlineStartStyle,
    CSSPropertyID::kBorderInlineStartWidth,
    CSSPropertyID::kBorderLeftColor,
    CSSPropertyID::kBorderLeftStyle,
    CSSPropertyID::kBorderLeftWidth,
    CSSPropertyID::kBorderRightColor,
    CSSPropertyID::kBorderRightStyle,
    CSSPropertyID::kBorderRightWidth,
    CSSPropertyID::kBorderStartEndRadius,
    CSSPropertyID::kBorderStartStartRadius,
    CSSPropertyID::kBorderTopColor,
    CSSPropertyID::kBorderTopLeftRadius,
    CSSPropertyID::kBorderTopRightRadius,
    CSSPropertyID::kBorderTopStyle,
    CSSPropertyID::kBorderTopWidth,
    CSSPropertyID::kBottom,
    CSSPropertyID::kBoxDecorationBreak,
    CSSPropertyID::kBoxShadow,
    CSSPropertyID::kBoxSizing,
    CSSPropertyID::kBreakAfter,
    CSSPropertyID::kBreakBefore,
    CSSPropertyID::kBreakInside,
    CSSPropertyID::kBufferedRendering,
    CSSPropertyID::kCaptionSide,
    CSSPropertyID::kCaretColor,
    CSSPropertyID::kClear,
    CSSPropertyID::kClip,
    CSSPropertyID::kClipPath,
    CSSPropertyID::kClipRule,
    CSSPropertyID::kColor,
    CSSPropertyID::kColorInterpolation,
    CSSPropertyID::kColorInterpolationFilters,
    CSSPropertyID::kColorRendering,
    CSSPropertyID::kColumnCount,
    CSSPropertyID::kColumnGap,
    CSSPropertyID::kColumnRuleColor,
    CSSPropertyID::kColumnRuleStyle,
    CSSPropertyID::kColumnRuleWidth,
    CSSPropertyID::kColumnSpan,
    CSSPropertyID::kColumnWidth,
    CSSPropertyID::kContainIntrinsicBlockSize,
    CSSPropertyID::kContainIntrinsicHeight,
    CSSPropertyID::kContainIntrinsicInlineSize,
    CSSPropertyID::kContainIntrinsicSize,
    CSSPropertyID::kContainIntrinsicWidth,
    CSSPropertyID::kContainerName,
    CSSPropertyID::kContainerType,
    CSSPropertyID::kContent,
    CSSPropertyID::kCursor,
    CSSPropertyID::kCx,
    CSSPropertyID::kCy,
    CSSPropertyID::kD,
    CSSPropertyID::kDirection,
    CSSPropertyID::kDisplay,
    CSSPropertyID::kDominantBaseline,
    CSSPropertyID::kDynamicRangeLimit,
    CSSPropertyID::kEmptyCells,
    CSSPropertyID::kFieldSizing,
    CSSPropertyID::kFill,
    CSSPropertyID::kFillOpacity,
    CSSPropertyID::kFillRule,
    CSSPropertyID::kFilter,
    CSSPropertyID::kFlexBasis,
    CSSPropertyID::kFlexDirection,
    CSSPropertyID::kFlexGrow,
    CSSPropertyID::kFlexShrink,
    CSSPropertyID::kFlexWrap,
    CSSPropertyID::kFloat,
    CSSPropertyID::kFloodColor,
    CSSPropertyID::kFloodOpacity,
    CSSPropertyID::kFontFamily,
    CSSPropertyID::kFontKerning,
    CSSPropertyID::kFontOpticalSizing,
    CSSPropertyID::kFontPalette,
    CSSPropertyID::kFontSize,
    CSSPropertyID::kFontSizeAdjust,
    CSSPropertyID::kFontStretch,
    CSSPropertyID::kFontStyle,
    CSSPropertyID::kFontSynthesisSmallCaps,
    CSSPropertyID::kFontSynthesisStyle,
    CSSPropertyID::kFontSynthesisWeight,
    CSSPropertyID::kFontVariant,
    CSSPropertyID::kFontVariantAlternates,
    CSSPropertyID::kFontVariantCaps,
    CSSPropertyID::kFontVariantEastAsian,
    CSSPropertyID::kFontVariantEmoji,
    CSSPropertyID::kFontVariantLigatures,
    CSSPropertyID::kFontVariantNumeric,
    CSSPropertyID::kFontVariantPosition,
    CSSPropertyID::kFontWeight,
    CSSPropertyID::kGridAutoColumns,
    CSSPropertyID::kGridAutoFlow,
    CSSPropertyID::kGridAutoRows,
    CSSPropertyID::kGridColumnEnd,
    CSSPropertyID::kGridColumnStart,
    CSSPropertyID::kGridRowEnd,
    CSSPropertyID::kGridRowStart,
    CSSPropertyID::kGridTemplateAreas,
    CSSPropertyID::kGridTemplateColumns,
    CSSPropertyID::kGridTemplateRows,
    CSSPropertyID::kHeight,
    CSSPropertyID::kHyphenateCharacter,
    CSSPropertyID::kHyphenateLimitChars,
    CSSPropertyID::kHyphens,
    CSSPropertyID::kImageOrientation,
    CSSPropertyID::kImageRendering,
    CSSPropertyID::kInitialLetter,
    CSSPropertyID::kInlineSize,
    CSSPropertyID::kInsetArea,
    CSSPropertyID::kInsetBlockEnd,
    CSSPropertyID::kInsetBlockStart,
    CSSPropertyID::kInsetInlineEnd,
    CSSPropertyID::kInsetInlineStart,
    CSSPropertyID::kInterpolateSize,
    CSSPropertyID::kIsolation,
    CSSPropertyID::kJustifyContent,
    CSSPropertyID::kJustifyItems,
    CSSPropertyID::kJustifySelf,
    CSSPropertyID::kLeft,
    CSSPropertyID::kLetterSpacing,
    CSSPropertyID::kLightingColor,
    CSSPropertyID::kLineBreak,
    CSSPropertyID::kLineClamp,
    CSSPropertyID::kLineHeight,
    CSSPropertyID::kListStyleImage,
    CSSPropertyID::kListStylePosition,
    CSSPropertyID::kListStyleType,
    CSSPropertyID::kMarginBlockEnd,
    CSSPropertyID::kMarginBlockStart,
    CSSPropertyID::kMarginBottom,
    CSSPropertyID::kMarginInlineEnd,
    CSSPropertyID::kMarginInlineStart,
    CSSPropertyID::kMarginLeft,
    CSSPropertyID::kMarginRight,
    CSSPropertyID::kMarginTop,
    CSSPropertyID::kMarkerEnd,
    CSSPropertyID::kMarkerMid,
    CSSPropertyID::kMarkerStart,
    CSSPropertyID::kMaskClip,
    CSSPropertyID::kMaskComposite,
    CSSPropertyID::kMaskImage,
    CSSPropertyID::kMaskMode,
    CSSPropertyID::kMaskOrigin,
    CSSPropertyID::kMaskPosition,
    CSSPropertyID::kMaskRepeat,
    CSSPropertyID::kMaskSize,
    CSSPropertyID::kMaskType,
    CSSPropertyID::kMasonrySlack,
    CSSPropertyID::kMasonryTemplateTracks,
    CSSPropertyID::kMasonryTrackEnd,
    CSSPropertyID::kMasonryTrackStart,
    CSSPropertyID::kMathDepth,
    CSSPropertyID::kMathShift,
    CSSPropertyID::kMathStyle,
    CSSPropertyID::kMaxBlockSize,
    CSSPropertyID::kMaxHeight,
    CSSPropertyID::kMaxInlineSize,
    CSSPropertyID::kMaxWidth,
    CSSPropertyID::kMinBlockSize,
    CSSPropertyID::kMinHeight,
    CSSPropertyID::kMinInlineSize,
    CSSPropertyID::kMinWidth,
    CSSPropertyID::kMixBlendMode,
    CSSPropertyID::kObjectFit,
    CSSPropertyID::kObjectPosition,
    CSSPropertyID::kObjectViewBox,
    CSSPropertyID::kOffsetAnchor,
    CSSPropertyID::kOffsetDistance,
    CSSPropertyID::kOffsetPath,
    CSSPropertyID::kOffsetPosition,
    CSSPropertyID::kOffsetRotate,
    CSSPropertyID::kOpacity,
    CSSPropertyID::kOrder,
    CSSPropertyID::kOrphans,
    CSSPropertyID::kOutlineColor,
    CSSPropertyID::kOutlineOffset,
    CSSPropertyID::kOutlineStyle,
    CSSPropertyID::kOutlineWidth,
    CSSPropertyID::kOverflowAnchor,
    CSSPropertyID::kOverflowBlock,
    CSSPropertyID::kOverflowClipMargin,
    CSSPropertyID::kOverflowInline,
    CSSPropertyID::kOverflowWrap,
    CSSPropertyID::kOverflowX,
    CSSPropertyID::kOverflowY,
    CSSPropertyID::kOverlay,
    CSSPropertyID::kOverscrollBehaviorBlock,
    CSSPropertyID::kOverscrollBehaviorInline,
    CSSPropertyID::kPaddingBlockEnd,
    CSSPropertyID::kPaddingBlockStart,
    CSSPropertyID::kPaddingBottom,
    CSSPropertyID::kPaddingInlineEnd,
    CSSPropertyID::kPaddingInlineStart,
    CSSPropertyID::kPaddingLeft,
    CSSPropertyID::kPaddingRight,
    CSSPropertyID::kPaddingTop,
    CSSPropertyID::kPaintOrder,
    CSSPropertyID::kPerspective,
    CSSPropertyID::kPerspectiveOrigin,
    CSSPropertyID::kPointerEvents,
    CSSPropertyID::kPopoverHideDelay,
    CSSPropertyID::kPopoverShowDelay,
    CSSPropertyID::kPosition,
    CSSPropertyID::kPositionAnchor,
    CSSPropertyID::kPositionArea,
    CSSPropertyID::kPositionTryFallbacks,
    CSSPropertyID::kPositionTryOptions,
    CSSPropertyID::kPositionTryOrder,
    CSSPropertyID::kPositionVisibility,
    CSSPropertyID::kR,
    CSSPropertyID::kReadingFlow,
    CSSPropertyID::kResize,
    CSSPropertyID::kRight,
    CSSPropertyID::kRotate,
    CSSPropertyID::kRowGap,
    CSSPropertyID::kRubyAlign,
    CSSPropertyID::kRubyPosition,
    CSSPropertyID::kRx,
    CSSPropertyID::kRy,
    CSSPropertyID::kScale,
    CSSPropertyID::kScrollBehavior,
    CSSPropertyID::kScrollMarginBlockEnd,
    CSSPropertyID::kScrollMarginBlockStart,
    CSSPropertyID::kScrollMarginInlineEnd,
    CSSPropertyID::kScrollMarginInlineStart,
    CSSPropertyID::kScrollMarkerGroup,
    CSSPropertyID::kScrollPaddingBlockEnd,
    CSSPropertyID::kScrollPaddingBlockStart,
    CSSPropertyID::kScrollPaddingInlineEnd,
    CSSPropertyID::kScrollPaddingInlineStart,
    CSSPropertyID::kScrollStartBlock,
    CSSPropertyID::kScrollStartInline,
    CSSPropertyID::kScrollStartTargetBlock,
    CSSPropertyID::kScrollStartTargetInline,
    CSSPropertyID::kScrollStartTargetX,
    CSSPropertyID::kScrollStartTargetY,
    CSSPropertyID::kScrollStartX,
    CSSPropertyID::kScrollStartY,
    CSSPropertyID::kScrollTimelineAxis,
    CSSPropertyID::kScrollTimelineName,
    CSSPropertyID::kScrollbarColor,
    CSSPropertyID::kScrollbarGutter,
    CSSPropertyID::kScrollbarWidth,
    CSSPropertyID::kShapeImageThreshold,
    CSSPropertyID::kShapeMargin,
    CSSPropertyID::kShapeOutside,
    CSSPropertyID::kShapeRendering,
    CSSPropertyID::kSpeak,
    CSSPropertyID::kStopColor,
    CSSPropertyID::kStopOpacity,
    CSSPropertyID::kStroke,
    CSSPropertyID::kStrokeDasharray,
    CSSPropertyID::kStrokeDashoffset,
    CSSPropertyID::kStrokeLinecap,
    CSSPropertyID::kStrokeLinejoin,
    CSSPropertyID::kStrokeMiterlimit,
    CSSPropertyID::kStrokeOpacity,
    CSSPropertyID::kStrokeWidth,
    CSSPropertyID::kTabSize,
    CSSPropertyID::kTableLayout,
    CSSPropertyID::kTextAlign,
    CSSPropertyID::kTextAlignLast,
    CSSPropertyID::kTextAnchor,
    CSSPropertyID::kTextAutospace,
    CSSPropertyID::kTextBoxEdge,
    CSSPropertyID::kTextBoxTrim,
    CSSPropertyID::kTextDecoration,
    CSSPropertyID::kTextDecorationColor,
    CSSPropertyID::kTextDecorationLine,
    CSSPropertyID::kTextDecorationSkipInk,
    CSSPropertyID::kTextDecorationStyle,
    CSSPropertyID::kTextEmphasisColor,
    CSSPropertyID::kTextEmphasisPosition,
    CSSPropertyID::kTextEmphasisStyle,
    CSSPropertyID::kTextIndent,
    CSSPropertyID::kTextOverflow,
    CSSPropertyID::kTextRendering,
    CSSPropertyID::kTextShadow,
    CSSPropertyID::kTextSizeAdjust,
    CSSPropertyID::kTextSpacingTrim,
    CSSPropertyID::kTextTransform,
    CSSPropertyID::kTextUnderlinePosition,
    CSSPropertyID::kTextWrapMode,
    CSSPropertyID::kTextWrapStyle,
    CSSPropertyID::kTimelineScope,
    CSSPropertyID::kTop,
    CSSPropertyID::kTouchAction,
    CSSPropertyID::kTransform,
    CSSPropertyID::kTransformOrigin,
    CSSPropertyID::kTransformStyle,
    CSSPropertyID::kTransitionBehavior,
    CSSPropertyID::kTransitionDelay,
    CSSPropertyID::kTransitionDuration,
    CSSPropertyID::kTransitionProperty,
    CSSPropertyID::kTransitionTimingFunction,
    CSSPropertyID::kTranslate,
    CSSPropertyID::kUnicodeBidi,
    CSSPropertyID::kUserSelect,
    CSSPropertyID::kVectorEffect,
    CSSPropertyID::kVerticalAlign,
    CSSPropertyID::kViewTimelineAxis,
    CSSPropertyID::kViewTimelineInset,
    CSSPropertyID::kViewTimelineName,
    CSSPropertyID::kViewTransitionClass,
    CSSPropertyID::kViewTransitionGroup,
    CSSPropertyID::kViewTransitionName,
    CSSPropertyID::kVisibility,
    CSSPropertyID::kWhiteSpaceCollapse,
    CSSPropertyID::kWidows,
    CSSPropertyID::kWidth,
    CSSPropertyID::kWillChange,
    CSSPropertyID::kWordBreak,
    CSSPropertyID::kWordSpacing,
    CSSPropertyID::kWritingMode,
    CSSPropertyID::kX,
    CSSPropertyID::kY,
    CSSPropertyID::kZIndex,
    CSSPropertyID::kZoom,
    CSSPropertyID::kWebkitBorderHorizontalSpacing,
    CSSPropertyID::kWebkitBorderImage,
    CSSPropertyID::kWebkitBorderVerticalSpacing,
    CSSPropertyID::kWebkitBoxAlign,
    CSSPropertyID::kWebkitBoxDecorationBreak,
    CSSPropertyID::kWebkitBoxDirection,
    CSSPropertyID::kWebkitBoxFlex,
    CSSPropertyID::kWebkitBoxOrdinalGroup,
    CSSPropertyID::kWebkitBoxOrient,
    CSSPropertyID::kWebkitBoxPack,
    CSSPropertyID::kWebkitBoxReflect,
    CSSPropertyID::kWebkitFontSmoothing,
    CSSPropertyID::kWebkitLineBreak,
    CSSPropertyID::kWebkitLineClamp,
    CSSPropertyID::kWebkitLocale,
    CSSPropertyID::kWebkitMaskBoxImage,
    CSSPropertyID::kWebkitMaskBoxImageOutset,
    CSSPropertyID::kWebkitMaskBoxImageRepeat,
    CSSPropertyID::kWebkitMaskBoxImageSlice,
    CSSPropertyID::kWebkitMaskBoxImageSource,
    CSSPropertyID::kWebkitMaskBoxImageWidth,
    CSSPropertyID::kWebkitPrintColorAdjust,
    CSSPropertyID::kWebkitRtlOrdering,
    CSSPropertyID::kWebkitTapHighlightColor,
    CSSPropertyID::kWebkitTextCombine,
    CSSPropertyID::kWebkitTextDecorationsInEffect,
    CSSPropertyID::kWebkitTextFillColor,
    CSSPropertyID::kWebkitTextOrientation,
    CSSPropertyID::kWebkitTextSecurity,
    CSSPropertyID::kWebkitTextStrokeColor,
    CSSPropertyID::kWebkitTextStrokeWidth,
    CSSPropertyID::kWebkitUserDrag,
    CSSPropertyID::kWebkitUserModify,
    CSSPropertyID::kWebkitWritingMode,
};

// The lowest CSSPropertyID excluding kInvalid and kVariable.
const int kIntFirstCSSProperty = 2;
const CSSPropertyID kFirstCSSProperty =
    static_cast<CSSPropertyID>(kIntFirstCSSProperty);

// The number of unresolved CSS properties excluding kInvalid and kVariable.
const int kNumCSSProperties = 712;

// The highest resolved CSSPropertyID.
const int kIntLastCSSProperty = 594;
const CSSPropertyID kLastCSSProperty =
    static_cast<CSSPropertyID>(kIntLastCSSProperty);

// The highest unresolved CSSPropertyID.
const CSSPropertyID kLastUnresolvedCSSProperty =
    static_cast<CSSPropertyID>(713);

const CSSPropertyID kFirstHighPriorityCSSProperty = kFirstCSSProperty;
const CSSPropertyID kLastHighPriorityCSSProperty = CSSPropertyID::kZoom;

// 1 + <The highest unresolved CSSPropertyID>.
const int kNumCSSPropertyIDs = static_cast<int>(kLastUnresolvedCSSProperty) + 1;

const size_t kMaxCSSPropertyNameLength = 48;
constexpr size_t kCSSPropertyIDBitLength = 10;
constexpr size_t kMaxShorthandExpansion = 18;

static_assert((static_cast<size_t>(1) << kCSSPropertyIDBitLength) >
                  static_cast<size_t>(kLastUnresolvedCSSProperty),
              "kCSSPropertyIDBitLength has enough bits");

// These are basically just change-detector tests, so that we do not
// accidentally add new high-priority properties or break the code generator.
static_assert(CSSPropertyID::kColorScheme == kFirstHighPriorityCSSProperty);
static_assert(CSSPropertyID::kZoom == kLastHighPriorityCSSProperty);
static_assert((static_cast<int>(kLastHighPriorityCSSProperty) -
               static_cast<int>(kFirstHighPriorityCSSProperty)) == 42,
              "There should a low number of high-priority properties");

inline int GetCSSPropertyIDIndex(CSSPropertyID id) {
    DCHECK_GE(id, kFirstCSSProperty);
    DCHECK_LE(id, kLastCSSProperty);
    return static_cast<int>(id) - kIntFirstCSSProperty;
}

constexpr bool IsHighPriority(CSSPropertyID id) {
  return id >= kFirstHighPriorityCSSProperty &&
      id <= kLastHighPriorityCSSProperty;
}

inline bool IsCSSPropertyIDWithName(CSSPropertyID id)
{
    return id >= kFirstCSSProperty && id <= kLastUnresolvedCSSProperty;
}

inline bool IsValidCSSPropertyID(CSSPropertyID id)
{
    return id != CSSPropertyID::kInvalid;
}

inline CSSPropertyID ConvertToCSSPropertyID(int value)
{
    DCHECK_GE(value, static_cast<int>(CSSPropertyID::kInvalid));
    DCHECK_LE(value, kIntLastCSSProperty);
    return static_cast<CSSPropertyID>(value);
}

int CORE_EXPORT ResolveCSSPropertyAlias(int value);

inline bool IsPropertyAlias(CSSPropertyID id) {
  return static_cast<int>(id) >= 595;
}

inline CSSPropertyID ResolveCSSPropertyID(CSSPropertyID id)
{
  int int_id = static_cast<int>(id);
  if (IsPropertyAlias(id))
    int_id = ResolveCSSPropertyAlias(int_id);
  return ConvertToCSSPropertyID(int_id);
}

CSSPropertyID CORE_EXPORT CssPropertyID(const ExecutionContext*,
                                        const WTF::String&);

class CSSPropertyIDList {
  STACK_ALLOCATED();

 public:
  class Iterator {
    STACK_ALLOCATED();
   public:
    Iterator(int id) : id_(id) {}
    CSSPropertyID operator*() const { return ConvertToCSSPropertyID(id_); }
    Iterator& operator++() {
      id_++;
      return *this;
    }
    bool operator!=(const Iterator& i) const { return id_ != i.id_; }

   private:
    int id_;
  };
  Iterator begin() const { return Iterator(kIntFirstCSSProperty); }
  Iterator end() const { return Iterator(kIntLastCSSProperty + 1); }
};

mojom::blink::CSSSampleId CORE_EXPORT GetCSSSampleId(CSSPropertyID id);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_CSS_PROPERTY_NAMES_H_
