// Copyright 2014 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
#define THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_

#include <string.h>
#include <stdint.h>

#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/css/parser/css_parser_mode.h"

namespace blink {

enum class CSSValueID {
  kInvalid = 0,
  kInherit = 1,
  kInitial = 2,
  kNone = 3,
  kHidden = 4,
  kInset = 5,
  kGroove = 6,
  kOutset = 7,
  kRidge = 8,
  kDotted = 9,
  kDashed = 10,
  kSolid = 11,
  kDouble = 12,
  kCaption = 13,
  kIcon = 14,
  kMenu = 15,
  kMessageBox = 16,
  kSmallCaption = 17,
  kWebkitMiniControl = 18,
  kWebkitSmallControl = 19,
  kWebkitControl = 20,
  kStatusBar = 21,
  kItalic = 22,
  kOblique = 23,
  kAll = 24,
  kCommonLigatures = 25,
  kNoCommonLigatures = 26,
  kDiscretionaryLigatures = 27,
  kNoDiscretionaryLigatures = 28,
  kHistoricalLigatures = 29,
  kNoHistoricalLigatures = 30,
  kContextual = 31,
  kNoContextual = 32,
  kSmallCaps = 33,
  kAllSmallCaps = 34,
  kPetiteCaps = 35,
  kAllPetiteCaps = 36,
  kUnicase = 37,
  kTitlingCaps = 38,
  kJis78 = 39,
  kJis83 = 40,
  kJis90 = 41,
  kJis04 = 42,
  kSimplified = 43,
  kTraditional = 44,
  kFullWidth = 45,
  kProportionalWidth = 46,
  kRuby = 47,
  kLiningNums = 48,
  kOldstyleNums = 49,
  kProportionalNums = 50,
  kTabularNums = 51,
  kDiagonalFractions = 52,
  kStackedFractions = 53,
  kOrdinal = 54,
  kSlashedZero = 55,
  kStylistic = 56,
  kHistoricalForms = 57,
  kStyleset = 58,
  kCharacterVariant = 59,
  kSwash = 60,
  kOrnaments = 61,
  kAnnotation = 62,
  kNormal = 63,
  kBold = 64,
  kBolder = 65,
  kLighter = 66,
  kWeight = 67,
  kUltraCondensed = 68,
  kExtraCondensed = 69,
  kCondensed = 70,
  kSemiCondensed = 71,
  kSemiExpanded = 72,
  kExpanded = 73,
  kExtraExpanded = 74,
  kUltraExpanded = 75,
  kXxSmall = 76,
  kXSmall = 77,
  kSmall = 78,
  kMedium = 79,
  kLarge = 80,
  kXLarge = 81,
  kXxLarge = 82,
  kXxxLarge = 83,
  kSmaller = 84,
  kLarger = 85,
  kWebkitXxxLarge = 86,
  kExHeight = 87,
  kCapHeight = 88,
  kChWidth = 89,
  kIcWidth = 90,
  kIcHeight = 91,
  kSerif = 92,
  kSansSerif = 93,
  kCursive = 94,
  kFantasy = 95,
  kMonospace = 96,
  kSystemUi = 97,
  kWebkitBody = 98,
  kMath = 99,
  kSwap = 100,
  kFallback = 101,
  kOptional = 102,
  kFontTech = 103,
  kFontFormat = 104,
  kEmoji = 105,
  kUnicode = 106,
  kPaletteMix = 107,
  kAqua = 108,
  kBlack = 109,
  kBlue = 110,
  kFuchsia = 111,
  kGray = 112,
  kGreen = 113,
  kLime = 114,
  kMaroon = 115,
  kNavy = 116,
  kOlive = 117,
  kOrange = 118,
  kPurple = 119,
  kRed = 120,
  kSilver = 121,
  kTeal = 122,
  kWhite = 123,
  kYellow = 124,
  kTransparent = 125,
  kWebkitLink = 126,
  kWebkitActivelink = 127,
  kAccentcolor = 128,
  kAccentcolortext = 129,
  kActiveborder = 130,
  kActivecaption = 131,
  kActivetext = 132,
  kAppworkspace = 133,
  kBackground = 134,
  kButtonborder = 135,
  kButtonface = 136,
  kButtonhighlight = 137,
  kButtonshadow = 138,
  kButtontext = 139,
  kCanvas = 140,
  kCanvastext = 141,
  kCaptiontext = 142,
  kField = 143,
  kFieldtext = 144,
  kGraytext = 145,
  kHighlight = 146,
  kHighlighttext = 147,
  kInactiveborder = 148,
  kInactivecaption = 149,
  kInactivecaptiontext = 150,
  kInfobackground = 151,
  kInfotext = 152,
  kLinktext = 153,
  kMark = 154,
  kMarktext = 155,
  kMenutext = 156,
  kSelecteditem = 157,
  kSelecteditemtext = 158,
  kScrollbar = 159,
  kThreeddarkshadow = 160,
  kThreedface = 161,
  kThreedhighlight = 162,
  kThreedlightshadow = 163,
  kThreedshadow = 164,
  kVisitedtext = 165,
  kWindow = 166,
  kWindowframe = 167,
  kWindowtext = 168,
  kInternalActiveListBoxSelection = 169,
  kInternalActiveListBoxSelectionText = 170,
  kInternalInactiveListBoxSelection = 171,
  kInternalInactiveListBoxSelectionText = 172,
  kWebkitFocusRingColor = 173,
  kCurrentcolor = 174,
  kGrey = 175,
  kInternalQuirkInherit = 176,
  kInternalSpellingErrorColor = 177,
  kInternalGrammarErrorColor = 178,
  kInternalSearchColor = 179,
  kInternalSearchTextColor = 180,
  kInternalCurrentSearchColor = 181,
  kInternalCurrentSearchTextColor = 182,
  kAliceblue = 183,
  kAntiquewhite = 184,
  kAquamarine = 185,
  kAzure = 186,
  kBeige = 187,
  kBisque = 188,
  kBlanchedalmond = 189,
  kBlueviolet = 190,
  kBrown = 191,
  kBurlywood = 192,
  kCadetblue = 193,
  kChartreuse = 194,
  kChocolate = 195,
  kCoral = 196,
  kCornflowerblue = 197,
  kCornsilk = 198,
  kCrimson = 199,
  kCyan = 200,
  kDarkblue = 201,
  kDarkcyan = 202,
  kDarkgoldenrod = 203,
  kDarkgray = 204,
  kDarkgreen = 205,
  kDarkgrey = 206,
  kDarkkhaki = 207,
  kDarkmagenta = 208,
  kDarkolivegreen = 209,
  kDarkorange = 210,
  kDarkorchid = 211,
  kDarkred = 212,
  kDarksalmon = 213,
  kDarkseagreen = 214,
  kDarkslateblue = 215,
  kDarkslategray = 216,
  kDarkslategrey = 217,
  kDarkturquoise = 218,
  kDarkviolet = 219,
  kDeeppink = 220,
  kDeepskyblue = 221,
  kDimgray = 222,
  kDimgrey = 223,
  kDodgerblue = 224,
  kFirebrick = 225,
  kFloralwhite = 226,
  kForestgreen = 227,
  kGainsboro = 228,
  kGhostwhite = 229,
  kGold = 230,
  kGoldenrod = 231,
  kGreenyellow = 232,
  kHoneydew = 233,
  kHotpink = 234,
  kIndianred = 235,
  kIndigo = 236,
  kIvory = 237,
  kKhaki = 238,
  kLavender = 239,
  kLavenderblush = 240,
  kLawngreen = 241,
  kLemonchiffon = 242,
  kLightblue = 243,
  kLightcoral = 244,
  kLightcyan = 245,
  kLightgoldenrodyellow = 246,
  kLightgray = 247,
  kLightgreen = 248,
  kLightgrey = 249,
  kLightpink = 250,
  kLightsalmon = 251,
  kLightseagreen = 252,
  kLightskyblue = 253,
  kLightslategray = 254,
  kLightslategrey = 255,
  kLightsteelblue = 256,
  kLightyellow = 257,
  kLimegreen = 258,
  kLinen = 259,
  kMagenta = 260,
  kMediumaquamarine = 261,
  kMediumblue = 262,
  kMediumorchid = 263,
  kMediumpurple = 264,
  kMediumseagreen = 265,
  kMediumslateblue = 266,
  kMediumspringgreen = 267,
  kMediumturquoise = 268,
  kMediumvioletred = 269,
  kMidnightblue = 270,
  kMintcream = 271,
  kMistyrose = 272,
  kMoccasin = 273,
  kNavajowhite = 274,
  kOldlace = 275,
  kOlivedrab = 276,
  kOrangered = 277,
  kOrchid = 278,
  kPalegoldenrod = 279,
  kPalegreen = 280,
  kPaleturquoise = 281,
  kPalevioletred = 282,
  kPapayawhip = 283,
  kPeachpuff = 284,
  kPeru = 285,
  kPink = 286,
  kPlum = 287,
  kPowderblue = 288,
  kRebeccapurple = 289,
  kRosybrown = 290,
  kRoyalblue = 291,
  kSaddlebrown = 292,
  kSalmon = 293,
  kSandybrown = 294,
  kSeagreen = 295,
  kSeashell = 296,
  kSienna = 297,
  kSkyblue = 298,
  kSlateblue = 299,
  kSlategray = 300,
  kSlategrey = 301,
  kSnow = 302,
  kSpringgreen = 303,
  kSteelblue = 304,
  kTan = 305,
  kThistle = 306,
  kTomato = 307,
  kTurquoise = 308,
  kViolet = 309,
  kWheat = 310,
  kWhitesmoke = 311,
  kYellowgreen = 312,
  kRepeat = 313,
  kRepeatX = 314,
  kRepeatY = 315,
  kNoRepeat = 316,
  kClear = 317,
  kCopy = 318,
  kSourceOver = 319,
  kSourceIn = 320,
  kSourceOut = 321,
  kSourceAtop = 322,
  kDestinationOver = 323,
  kDestinationIn = 324,
  kDestinationOut = 325,
  kDestinationAtop = 326,
  kXor = 327,
  kPlusLighter = 328,
  kSubtract = 329,
  kIntersect = 330,
  kExclude = 331,
  kBaseline = 332,
  kMiddle = 333,
  kSub = 334,
  kSuper = 335,
  kTextTop = 336,
  kTextBottom = 337,
  kTop = 338,
  kBottom = 339,
  kWebkitBaselineMiddle = 340,
  kWebkitAuto = 341,
  kLeft = 342,
  kRight = 343,
  kCenter = 344,
  kJustify = 345,
  kWebkitLeft = 346,
  kWebkitRight = 347,
  kWebkitCenter = 348,
  kWebkitMatchParent = 349,
  kInternalCenter = 350,
  kInlineStart = 351,
  kInlineEnd = 352,
  kOutside = 353,
  kInside = 354,
  kDisc = 355,
  kCircle = 356,
  kSquare = 357,
  kDisclosureOpen = 358,
  kDisclosureClosed = 359,
  kDecimal = 360,
  kInline = 361,
  kBlock = 362,
  kFlowRoot = 363,
  kFlow = 364,
  kTable = 365,
  kFlex = 366,
  kGrid = 367,
  kMasonry = 368,
  kContents = 369,
  kTableRowGroup = 370,
  kTableHeaderGroup = 371,
  kTableFooterGroup = 372,
  kTableRow = 373,
  kTableColumnGroup = 374,
  kTableColumn = 375,
  kTableCell = 376,
  kTableCaption = 377,
  kRubyText = 378,
  kInlineBlock = 379,
  kInlineTable = 380,
  kInlineFlex = 381,
  kInlineGrid = 382,
  kInlineMasonry = 383,
  kWebkitBox = 384,
  kWebkitInlineBox = 385,
  kWebkitFlex = 386,
  kWebkitInlineFlex = 387,
  kLayout = 388,
  kInlineLayout = 389,
  kListItem = 390,
  kAuto = 391,
  kCrosshair = 392,
  kDefault = 393,
  kPointer = 394,
  kMove = 395,
  kVerticalText = 396,
  kCell = 397,
  kContextMenu = 398,
  kAlias = 399,
  kProgress = 400,
  kNoDrop = 401,
  kNotAllowed = 402,
  kZoomIn = 403,
  kZoomOut = 404,
  kEResize = 405,
  kNeResize = 406,
  kNwResize = 407,
  kNResize = 408,
  kSeResize = 409,
  kSwResize = 410,
  kSResize = 411,
  kWResize = 412,
  kEwResize = 413,
  kNsResize = 414,
  kNeswResize = 415,
  kNwseResize = 416,
  kColResize = 417,
  kRowResize = 418,
  kText = 419,
  kWait = 420,
  kHelp = 421,
  kAllScroll = 422,
  kGrab = 423,
  kGrabbing = 424,
  kWebkitGrab = 425,
  kWebkitGrabbing = 426,
  kWebkitZoomIn = 427,
  kWebkitZoomOut = 428,
  kLtr = 429,
  kRtl = 430,
  kCapitalize = 431,
  kUppercase = 432,
  kLowercase = 433,
  kMathAuto = 434,
  kVisible = 435,
  kCollapse = 436,
  kInert = 437,
  kPreserve = 438,
  kPreserveBreaks = 439,
  kPretty = 440,
  kA3 = 441,
  kA4 = 442,
  kA5 = 443,
  kAbove = 444,
  kAbsolute = 445,
  kAlways = 446,
  kAvoid = 447,
  kB4 = 448,
  kB5 = 449,
  kBelow = 450,
  kBidiOverride = 451,
  kBlink = 452,
  kBoth = 453,
  kBreakSpaces = 454,
  kCloseQuote = 455,
  kEmbed = 456,
  kFixed = 457,
  kHand = 458,
  kHide = 459,
  kIsolate = 460,
  kIsolateOverride = 461,
  kPlaintext = 462,
  kWebkitIsolate = 463,
  kWebkitIsolateOverride = 464,
  kWebkitPlaintext = 465,
  kJisB5 = 466,
  kJisB4 = 467,
  kLandscape = 468,
  kLedger = 469,
  kLegal = 470,
  kLetter = 471,
  kLineThrough = 472,
  kLocal = 473,
  kNoCloseQuote = 474,
  kNoOpenQuote = 475,
  kNowrap = 476,
  kOpenQuote = 477,
  kOverlay = 478,
  kOverline = 479,
  kPortrait = 480,
  kPre = 481,
  kPreLine = 482,
  kPreWrap = 483,
  kRelative = 484,
  kScroll = 485,
  kSeparate = 486,
  kShow = 487,
  kStatic = 488,
  kThick = 489,
  kThin = 490,
  kUnderline = 491,
  kView = 492,
  kWavy = 493,
  kCompact = 494,
  kStretch = 495,
  kStart = 496,
  kEnd = 497,
  kClone = 498,
  kSlice = 499,
  kReverse = 500,
  kHorizontal = 501,
  kVertical = 502,
  kInlineAxis = 503,
  kBlockAxis = 504,
  kFlexStart = 505,
  kFlexEnd = 506,
  kSpaceBetween = 507,
  kSpaceAround = 508,
  kSpaceEvenly = 509,
  kUnsafe = 510,
  kSafe = 511,
  kAnchorCenter = 512,
  kRow = 513,
  kRowReverse = 514,
  kColumn = 515,
  kColumnReverse = 516,
  kWrap = 517,
  kWrapReverse = 518,
  kAutoFlow = 519,
  kDense = 520,
  kReadOnly = 521,
  kReadWrite = 522,
  kReadWritePlaintextOnly = 523,
  kElement = 524,
  kWebkitMinContent = 525,
  kWebkitMaxContent = 526,
  kWebkitFillAvailable = 527,
  kWebkitFitContent = 528,
  kMinContent = 529,
  kMaxContent = 530,
  kFitContent = 531,
  kNoAutospace = 532,
  kCap = 533,
  kEx = 534,
  kTrimBoth = 535,
  kTrimEnd = 536,
  kTrimStart = 537,
  kClip = 538,
  kEllipsis = 539,
  kSpellingError = 540,
  kGrammarError = 541,
  kFromFont = 542,
  kSpaceAll = 543,
  kSpaceFirst = 544,
  kBreakAll = 545,
  kKeepAll = 546,
  kAutoPhrase = 547,
  kBreakWord = 548,
  kSpace = 549,
  kLoose = 550,
  kStrict = 551,
  kAfterWhiteSpace = 552,
  kAnywhere = 553,
  kManual = 554,
  kCheckbox = 555,
  kRadio = 556,
  kButton = 557,
  kListbox = 558,
  kInternalMediaControl = 559,
  kMenulist = 560,
  kMenulistButton = 561,
  kMeter = 562,
  kProgressBar = 563,
  kSearchfield = 564,
  kTextfield = 565,
  kTextarea = 566,
  kSliderVertical = 567,
  kRound = 568,
  kBaseSelect = 569,
  kInternalAppearanceAutoBaseSelect = 570,
  kBorder = 571,
  kBorderBox = 572,
  kContent = 573,
  kContentBox = 574,
  kPadding = 575,
  kPaddingBox = 576,
  kMarginBox = 577,
  kNoClip = 578,
  kContain = 579,
  kCover = 580,
  kLogical = 581,
  kVisual = 582,
  kReplace = 583,
  kAccumulate = 584,
  kAlternate = 585,
  kAlternateReverse = 586,
  kForwards = 587,
  kBackwards = 588,
  kInfinite = 589,
  kRunning = 590,
  kPaused = 591,
  kFlat = 592,
  kPreserve3d = 593,
  kFillBox = 594,
  kViewBox = 595,
  kEase = 596,
  kLinear = 597,
  kEaseIn = 598,
  kEaseOut = 599,
  kEaseInOut = 600,
  kJumpBoth = 601,
  kJumpEnd = 602,
  kJumpNone = 603,
  kJumpStart = 604,
  kStepStart = 605,
  kStepEnd = 606,
  kSteps = 607,
  kFrames = 608,
  kCubicBezier = 609,
  kDocument = 610,
  kReset = 611,
  kZoom = 612,
  kVisiblepainted = 613,
  kVisiblefill = 614,
  kVisiblestroke = 615,
  kPainted = 616,
  kFill = 617,
  kStroke = 618,
  kBoundingBox = 619,
  kSpellOut = 620,
  kDigits = 621,
  kLiteralPunctuation = 622,
  kNoPunctuation = 623,
  kAntialiased = 624,
  kSubpixelAntialiased = 625,
  kOptimizespeed = 626,
  kOptimizelegibility = 627,
  kGeometricprecision = 628,
  kCrispedges = 629,
  kEconomy = 630,
  kExact = 631,
  kLr = 632,
  kRl = 633,
  kTb = 634,
  kLrTb = 635,
  kRlTb = 636,
  kTbRl = 637,
  kHorizontalTb = 638,
  kVerticalRl = 639,
  kVerticalLr = 640,
  kSidewaysRl = 641,
  kSidewaysLr = 642,
  kAfter = 643,
  kBefore = 644,
  kInterCharacter = 645,
  kOver = 646,
  kUnder = 647,
  kFilled = 648,
  kOpen = 649,
  kDot = 650,
  kDoubleCircle = 651,
  kTriangle = 652,
  kSesame = 653,
  kEllipse = 654,
  kClosestSide = 655,
  kClosestCorner = 656,
  kFarthestSide = 657,
  kFarthestCorner = 658,
  kMixed = 659,
  kSideways = 660,
  kSidewaysRight = 661,
  kUpright = 662,
  kVerticalRight = 663,
  kOn = 664,
  kOff = 665,
  kOptimizequality = 666,
  kPixelated = 667,
  kWebkitOptimizeContrast = 668,
  kFromImage = 669,
  kRotateLeft = 670,
  kRotateRight = 671,
  kNonzero = 672,
  kEvenodd = 673,
  kAt = 674,
  kAlphabetic = 675,
  kBorderless = 676,
  kFullscreen = 677,
  kStandalone = 678,
  kMinimalUi = 679,
  kBrowser = 680,
  kWindowControlsOverlay = 681,
  kTabbed = 682,
  kPictureInPicture = 683,
  kMinimized = 684,
  kMaximized = 685,
  kPaged = 686,
  kSlow = 687,
  kFast = 688,
  kSticky = 689,
  kCoarse = 690,
  kFine = 691,
  kOnDemand = 692,
  kHover = 693,
  kMultiply = 694,
  kScreen = 695,
  kDarken = 696,
  kLighten = 697,
  kColorDodge = 698,
  kColorBurn = 699,
  kHardLight = 700,
  kSoftLight = 701,
  kDifference = 702,
  kExclusion = 703,
  kHue = 704,
  kSaturation = 705,
  kColor = 706,
  kLuminosity = 707,
  kScaleDown = 708,
  kBalance = 709,
  kDrag = 710,
  kNoDrag = 711,
  kSpan = 712,
  kMinmax = 713,
  kSubgrid = 714,
  kProgressive = 715,
  kInterlace = 716,
  kMarkers = 717,
  kAlpha = 718,
  kLuminance = 719,
  kMatchSource = 720,
  kSRGB = 721,
  kLinearrgb = 722,
  kButt = 723,
  kMiter = 724,
  kBevel = 725,
  kBeforeEdge = 726,
  kAfterEdge = 727,
  kCentral = 728,
  kTextBeforeEdge = 729,
  kTextAfterEdge = 730,
  kIdeographic = 731,
  kHanging = 732,
  kMathematical = 733,
  kUseScript = 734,
  kNoChange = 735,
  kResetSize = 736,
  kDynamic = 737,
  kNonScalingStroke = 738,
  kInternalExtendToZoom = 739,
  kPanX = 740,
  kPanY = 741,
  kPanLeft = 742,
  kPanRight = 743,
  kPanUp = 744,
  kPanDown = 745,
  kManipulation = 746,
  kPinchZoom = 747,
  kLastBaseline = 748,
  kFirstBaseline = 749,
  kFirst = 750,
  kLast = 751,
  kSelfStart = 752,
  kSelfEnd = 753,
  kLegacy = 754,
  kSmooth = 755,
  kScrollPosition = 756,
  kRevert = 757,
  kRevertLayer = 758,
  kUnset = 759,
  kLinearGradient = 760,
  kRadialGradient = 761,
  kConicGradient = 762,
  kRepeatingLinearGradient = 763,
  kRepeatingRadialGradient = 764,
  kRepeatingConicGradient = 765,
  kPaint = 766,
  kCrossFade = 767,
  kWebkitCrossFade = 768,
  kWebkitGradient = 769,
  kWebkitLinearGradient = 770,
  kWebkitRadialGradient = 771,
  kWebkitRepeatingLinearGradient = 772,
  kWebkitRepeatingRadialGradient = 773,
  kWebkitImageSet = 774,
  kImageSet = 775,
  kType = 776,
  kTo = 777,
  kColorStop = 778,
  kRadial = 779,
  kAttr = 780,
  kCounter = 781,
  kCounters = 782,
  kRect = 783,
  kPolygon = 784,
  kFormat = 785,
  kCollection = 786,
  kEmbeddedOpentype = 787,
  kOpentype = 788,
  kSVG = 789,
  kTruetype = 790,
  kWoff = 791,
  kWoff2 = 792,
  kTech = 793,
  kFeaturesOpentype = 794,
  kFeaturesAat = 795,
  kFeaturesGraphite = 796,
  kColorCOLRv0 = 797,
  kColorCOLRv1 = 798,
  kColorSVG = 799,
  kColorSbix = 800,
  kColorCBDT = 801,
  kVariations = 802,
  kPalettes = 803,
  kIncremental = 804,
  kInvert = 805,
  kGrayscale = 806,
  kSepia = 807,
  kSaturate = 808,
  kHueRotate = 809,
  kOpacity = 810,
  kBrightness = 811,
  kContrast = 812,
  kBlur = 813,
  kDropShadow = 814,
  kUrl = 815,
  kRgb = 816,
  kRgba = 817,
  kHsl = 818,
  kHsla = 819,
  kHwb = 820,
  kLab = 821,
  kOklab = 822,
  kLch = 823,
  kOklch = 824,
  kLightDark = 825,
  kSRGBLinear = 826,
  kDisplayP3 = 827,
  kA98Rgb = 828,
  kProphotoRgb = 829,
  kXyz = 830,
  kXyzD50 = 831,
  kXyzD65 = 832,
  kShorter = 833,
  kLonger = 834,
  kDecreasing = 835,
  kIncreasing = 836,
  kIn = 837,
  kColorMix = 838,
  kFrom = 839,
  kR = 840,
  kG = 841,
  kB = 842,
  kH = 843,
  kS = 844,
  kL = 845,
  kW = 846,
  kA = 847,
  kC = 848,
  kMatrix = 849,
  kMatrix3d = 850,
  kPerspective = 851,
  kRotate = 852,
  kRotateX = 853,
  kRotateY = 854,
  kRotateZ = 855,
  kRotate3d = 856,
  kScale = 857,
  kScaleX = 858,
  kScaleY = 859,
  kScaleZ = 860,
  kScale3d = 861,
  kSkew = 862,
  kSkewX = 863,
  kSkewY = 864,
  kTranslate = 865,
  kTranslateX = 866,
  kTranslateY = 867,
  kTranslateZ = 868,
  kTranslate3d = 869,
  kX = 870,
  kY = 871,
  kZ = 872,
  kPath = 873,
  kRay = 874,
  kSides = 875,
  kStrokeBox = 876,
  kCalc = 877,
  kWebkitCalc = 878,
  kMin = 879,
  kMax = 880,
  kClamp = 881,
  kCalcSize = 882,
  kAny = 883,
  kSin = 884,
  kCos = 885,
  kAsin = 886,
  kAtan = 887,
  kAtan2 = 888,
  kAcos = 889,
  kMod = 890,
  kRem = 891,
  kUp = 892,
  kDown = 893,
  kToZero = 894,
  kSign = 895,
  kAbs = 896,
  kPow = 897,
  kSqrt = 898,
  kHypot = 899,
  kLog = 900,
  kExp = 901,
  kInfinity = 902,
  kNegativeInfinity = 903,
  kNan = 904,
  kPi = 905,
  kE = 906,
  kMandatory = 907,
  kProximity = 908,
  kStyle = 909,
  kSize = 910,
  kBlockSize = 911,
  kInlineSize = 912,
  kScrollState = 913,
  kBlockStart = 914,
  kBlockEnd = 915,
  kAutoFill = 916,
  kAutoFit = 917,
  kVar = 918,
  kInternalVariableValue = 919,
  kEnv = 920,
  kArg = 921,
  kAvoidPage = 922,
  kPage = 923,
  kRecto = 924,
  kVerso = 925,
  kAvoidColumn = 926,
  kP3 = 927,
  kRec2020 = 928,
  kAdd = 929,
  kAutoAdd = 930,
  kTrue = 931,
  kFalse = 932,
  kNoPreference = 933,
  kDark = 934,
  kLight = 935,
  kOnly = 936,
  kReduce = 937,
  kActive = 938,
  kPreserveParentColor = 939,
  kBackButton = 940,
  kFabricated = 941,
  kSelector = 942,
  kContinuous = 943,
  kFolded = 944,
  kStable = 945,
  kBothEdges = 946,
  kMore = 947,
  kLess = 948,
  kCustom = 949,
  kCyclic = 950,
  kSymbolic = 951,
  kNumeric = 952,
  kAdditive = 953,
  kExtends = 954,
  kInternalSimpChineseInformal = 955,
  kInternalSimpChineseFormal = 956,
  kInternalTradChineseInformal = 957,
  kInternalTradChineseFormal = 958,
  kInternalKoreanHangulFormal = 959,
  kInternalKoreanHanjaInformal = 960,
  kInternalKoreanHanjaFormal = 961,
  kInternalHebrew = 962,
  kInternalLowerArmenian = 963,
  kInternalUpperArmenian = 964,
  kInternalEthiopicNumeric = 965,
  kBullets = 966,
  kNumbers = 967,
  kWords = 968,
  kStandard = 969,
  kHigh = 970,
  kConstrainedHigh = 971,
  kDynamicRangeLimitMix = 972,
  kLayer = 973,
  kSupports = 974,
  kColorContrast = 975,
  kVs = 976,
  kAA = 977,
  kAALarge = 978,
  kAAA = 979,
  kAAALarge = 980,
  kDrop = 981,
  kRaise = 982,
  kXywh = 983,
  kAnchor = 984,
  kAnchorSize = 985,
  kWidth = 986,
  kHeight = 987,
  kSelfBlock = 988,
  kSelfInline = 989,
  kEntry = 990,
  kExit = 991,
  kEntryCrossing = 992,
  kExitCrossing = 993,
  kRoot = 994,
  kNearest = 995,
  kSelf = 996,
  kAllowDiscrete = 997,
  kInverted = 998,
  kEnabled = 999,
  kInitialOnly = 1000,
  kSpanLeft = 1001,
  kSpanRight = 1002,
  kXStart = 1003,
  kXEnd = 1004,
  kSpanXStart = 1005,
  kSpanXEnd = 1006,
  kXSelfStart = 1007,
  kXSelfEnd = 1008,
  kSpanXSelfStart = 1009,
  kSpanXSelfEnd = 1010,
  kSpanAll = 1011,
  kSpanTop = 1012,
  kSpanBottom = 1013,
  kYStart = 1014,
  kYEnd = 1015,
  kSpanYStart = 1016,
  kSpanYEnd = 1017,
  kYSelfStart = 1018,
  kYSelfEnd = 1019,
  kSpanYSelfStart = 1020,
  kSpanYSelfEnd = 1021,
  kSpanBlockStart = 1022,
  kSpanBlockEnd = 1023,
  kSelfBlockStart = 1024,
  kSelfBlockEnd = 1025,
  kSpanSelfBlockStart = 1026,
  kSpanSelfBlockEnd = 1027,
  kSpanInlineStart = 1028,
  kSpanInlineEnd = 1029,
  kSelfInlineStart = 1030,
  kSelfInlineEnd = 1031,
  kSpanSelfInlineStart = 1032,
  kSpanSelfInlineEnd = 1033,
  kSpanStart = 1034,
  kSpanEnd = 1035,
  kSpanSelfStart = 1036,
  kSpanSelfEnd = 1037,
  kPositionArea = 1038,
  kInternalTextareaAuto = 1039,
  kMostWidth = 1040,
  kMostHeight = 1041,
  kMostBlockSize = 1042,
  kMostInlineSize = 1043,
  kFlipBlock = 1044,
  kFlipInline = 1045,
  kFlipStart = 1046,
  kAnchorsVisible = 1047,
  kNoOverflow = 1048,
  kFlexVisual = 1049,
  kFlexFlow = 1050,
  kGridRows = 1051,
  kGridColumns = 1052,
  kGridOrder = 1053,
  kContextFill = 1054,
  kContextStroke = 1055,
  kMediaProgress = 1056,
  kContainerProgress = 1057,
  kOf = 1058,
  kBlinkFeature = 1059,
  kNumericOnly = 1060,
  kAllowKeywords = 1061,
};

const int numCSSValueKeywords = 1062;
const size_t maxCSSValueKeywordLength = 42;

inline bool IsValidCSSValueID(CSSValueID id)
{
    return id != CSSValueID::kInvalid;
}

CORE_EXPORT const char* getValueName(CSSValueID);
bool isValueAllowedInMode(CSSValueID id, CSSParserMode mode);

}  // namespace blink

#endif  // THIRD_PARTY_BLINK_RENDERER_CORE_CSS_VALUE_KEYWORDS_H_
