// This file is generated by TypeBuilder_h.template.

// Copyright 2016 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef blink_protocol_Database_h
#define blink_protocol_Database_h

#include "third_party/blink/renderer/core/core_export.h"
#include "third_party/blink/renderer/core/inspector/protocol/protocol.h"
// For each imported domain we generate a ValueConversions struct instead of a full domain definition
// and include Domain::API version from there.

namespace blink {
namespace protocol {
namespace Database {
using DatabaseId = String;
class Database;
class Error;

// ------------- Forward and enum declarations.

// ------------- Type and builder declarations.

class CORE_EXPORT Database : public ::crdtp::ProtocolObject<Database> {
public:
    ~Database() override { }

    String getId() { return m_id; }
    void setId(const String& value) { m_id = value; }

    String getDomain() { return m_domain; }
    void setDomain(const String& value) { m_domain = value; }

    String getName() { return m_name; }
    void setName(const String& value) { m_name = value; }

    String getVersion() { return m_version; }
    void setVersion(const String& value) { m_version = value; }

    template<int STATE>
    class DatabaseBuilder {
    public:
        enum {
            NoFieldsSet = 0,
            IdSet = 1 << 1,
            DomainSet = 1 << 2,
            NameSet = 1 << 3,
            VersionSet = 1 << 4,
            AllFieldsSet = (IdSet | DomainSet | NameSet | VersionSet | 0)};


        DatabaseBuilder<STATE | IdSet>& setId(const String& value)
        {
            static_assert(!(STATE & IdSet), "property id should not be set yet");
            m_result->setId(value);
            return castState<IdSet>();
        }

        DatabaseBuilder<STATE | DomainSet>& setDomain(const String& value)
        {
            static_assert(!(STATE & DomainSet), "property domain should not be set yet");
            m_result->setDomain(value);
            return castState<DomainSet>();
        }

        DatabaseBuilder<STATE | NameSet>& setName(const String& value)
        {
            static_assert(!(STATE & NameSet), "property name should not be set yet");
            m_result->setName(value);
            return castState<NameSet>();
        }

        DatabaseBuilder<STATE | VersionSet>& setVersion(const String& value)
        {
            static_assert(!(STATE & VersionSet), "property version should not be set yet");
            m_result->setVersion(value);
            return castState<VersionSet>();
        }

        std::unique_ptr<Database> build()
        {
            static_assert(STATE == AllFieldsSet, "state should be AllFieldsSet");
            return std::move(m_result);
        }

    private:
        friend class Database;
        DatabaseBuilder() : m_result(new Database()) { }

        template<int STEP> DatabaseBuilder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<DatabaseBuilder<STATE | STEP>*>(this);
        }

        std::unique_ptr<protocol::Database::Database> m_result;
    };

    static DatabaseBuilder<0> create()
    {
        return DatabaseBuilder<0>();
    }

private:
    DECLARE_SERIALIZATION_SUPPORT();

    Database()
    {
    }

    String m_id;
    String m_domain;
    String m_name;
    String m_version;
};


class CORE_EXPORT Error : public ::crdtp::ProtocolObject<Error> {
public:
    ~Error() override { }

    String getMessage() { return m_message; }
    void setMessage(const String& value) { m_message = value; }

    int getCode() { return m_code; }
    void setCode(int value) { m_code = value; }

    template<int STATE>
    class ErrorBuilder {
    public:
        enum {
            NoFieldsSet = 0,
            MessageSet = 1 << 1,
            CodeSet = 1 << 2,
            AllFieldsSet = (MessageSet | CodeSet | 0)};


        ErrorBuilder<STATE | MessageSet>& setMessage(const String& value)
        {
            static_assert(!(STATE & MessageSet), "property message should not be set yet");
            m_result->setMessage(value);
            return castState<MessageSet>();
        }

        ErrorBuilder<STATE | CodeSet>& setCode(int value)
        {
            static_assert(!(STATE & CodeSet), "property code should not be set yet");
            m_result->setCode(value);
            return castState<CodeSet>();
        }

        std::unique_ptr<Error> build()
        {
            static_assert(STATE == AllFieldsSet, "state should be AllFieldsSet");
            return std::move(m_result);
        }

    private:
        friend class Error;
        ErrorBuilder() : m_result(new Error()) { }

        template<int STEP> ErrorBuilder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<ErrorBuilder<STATE | STEP>*>(this);
        }

        std::unique_ptr<protocol::Database::Error> m_result;
    };

    static ErrorBuilder<0> create()
    {
        return ErrorBuilder<0>();
    }

private:
    DECLARE_SERIALIZATION_SUPPORT();

    Error()
    {
          m_code = 0;
    }

    String m_message;
    int m_code;
};


// ------------- Backend interface.

class CORE_EXPORT Backend {
public:
    virtual ~Backend() { }

    virtual DispatchResponse disable() = 0;
    virtual DispatchResponse enable() = 0;
    class CORE_EXPORT ExecuteSQLCallback {
    public:
        virtual void sendSuccess(Maybe<protocol::Array<String>> columnNames, Maybe<protocol::Array<protocol::Value>> values, Maybe<protocol::Database::Error> sqlError) = 0;
        virtual void sendFailure(const DispatchResponse&) = 0;
        virtual void fallThrough() = 0;
        virtual ~ExecuteSQLCallback() { }
    };
    virtual void executeSQL(const String& in_databaseId, const String& in_query, std::unique_ptr<ExecuteSQLCallback> callback) = 0;
    virtual DispatchResponse getDatabaseTableNames(const String& in_databaseId, std::unique_ptr<protocol::Array<String>>* out_tableNames) = 0;

};

// ------------- Frontend interface.

class CORE_EXPORT Frontend {
public:
  explicit Frontend(FrontendChannel* frontend_channel) : frontend_channel_(frontend_channel) {}
    void addDatabase(std::unique_ptr<protocol::Database::Database> database);

  void flush();
  void sendRawNotification(std::unique_ptr<Serializable>);
 private:
  FrontendChannel* frontend_channel_;
};

// ------------- Dispatcher.

class CORE_EXPORT Dispatcher {
public:
    static void wire(UberDispatcher*, Backend*);

private:
    Dispatcher() { }
};

// ------------- Metainfo.

class CORE_EXPORT Metainfo {
public:
    using BackendClass = Backend;
    using FrontendClass = Frontend;
    using DispatcherClass = Dispatcher;
    static const char domainName[];
    static const char commandPrefix[];
    static const char version[];
};

} // namespace Database
} // namespace blink
} // namespace protocol

#endif // !defined(blink_protocol_Database_h)
