// Copyright 2017 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// vector_icons.cc.template is used to generate vector_icons.cc. Edit the former
// rather than the latter.

#include "chrome/app/vector_icons/vector_icons.h"

#include "components/vector_icons/cc_macros.h"
#include "ui/gfx/animation/tween.h"
#include "ui/gfx/vector_icon_types.h"

#define DECLARE_VECTOR_COMMAND(x) using gfx::x;
DECLARE_VECTOR_COMMANDS

VECTOR_ICON_REP_TEMPLATE(kAccountAddChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 11.87f, 8.63f,
V_LINE_TO, 7.26f,
R_H_LINE_TO, -1.37f,
V_LINE_TO, 6,
R_H_LINE_TO, 1.37f,
V_LINE_TO, 4.63f,
R_H_LINE_TO, 1.27f,
R_V_LINE_TO, 1.37f,
R_H_LINE_TO, 1.37f,
R_V_LINE_TO, 1.27f,
R_H_LINE_TO, -1.37f,
R_V_LINE_TO, 1.37f,
CLOSE,
R_MOVE_TO, -5.46f, -0.69f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -1.8f, -0.74f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -0.74f, -1.8f,
R_CUBIC_TO, 0, -0.71f, 0.25f, -1.31f, 0.74f, -1.8f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, 1.8f, -0.74f,
R_CUBIC_TO, 0.71f, 0, 1.31f, 0.25f, 1.8f, 0.74f,
R_CUBIC_TO, 0.49f, 0.49f, 0.74f, 1.09f, 0.74f, 1.8f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -0.74f, 1.8f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -1.8f, 0.74f,
CLOSE,
R_MOVE_TO, -4.93f, 5.07f,
R_V_LINE_TO, -1.68f,
R_CUBIC_TO, 0, -0.31f, 0.08f, -0.6f, 0.23f, -0.86f,
R_CUBIC_TO, 0.15f, -0.26f, 0.36f, -0.47f, 0.63f, -0.62f,
R_ARC_TO, 8.32f, 8.32f, 0, 0, 1, 1.96f, -0.82f,
R_ARC_TO, 7.86f, 7.86f, 0, 0, 1, 2.11f, -0.29f,
R_ARC_TO, 8.04f, 8.04f, 0, 0, 1, 4.08f, 1.11f,
R_CUBIC_TO, 0.27f, 0.16f, 0.48f, 0.36f, 0.63f, 0.62f,
R_CUBIC_TO, 0.15f, 0.26f, 0.23f, 0.55f, 0.23f, 0.87f,
R_V_LINE_TO, 1.68f,
CLOSE,
R_MOVE_TO, 1.38f, -1.38f,
R_H_LINE_TO, 7.11f,
R_V_LINE_TO, -0.27f,
R_ARC_TO, 0.36f, 0.36f, 0, 0, 0, -0.05f, -0.19f,
R_ARC_TO, 0.39f, 0.39f, 0, 0, 0, -0.13f, -0.13f,
R_ARC_TO, 7.14f, 7.14f, 0, 0, 0, -1.61f, -0.67f,
R_ARC_TO, 6.62f, 6.62f, 0, 0, 0, -1.76f, -0.23f,
R_CUBIC_TO, -0.59f, 0, -1.18f, 0.08f, -1.75f, 0.24f,
R_ARC_TO, 6.93f, 6.93f, 0, 0, 0, -1.61f, 0.67f,
R_ARC_TO, 0.39f, 0.39f, 0, 0, 0, -0.13f, 0.13f,
R_ARC_TO, 0.36f, 0.36f, 0, 0, 0, -0.05f, 0.19f,
CLOSE,
R_MOVE_TO, 3.56f, -5.07f,
R_CUBIC_TO, 0.32f, 0, 0.59f, -0.11f, 0.81f, -0.34f,
R_CUBIC_TO, 0.23f, -0.23f, 0.34f, -0.5f, 0.34f, -0.82f,
R_CUBIC_TO, 0, -0.32f, -0.11f, -0.59f, -0.33f, -0.81f,
R_ARC_TO, 1.12f, 1.12f, 0, 0, 0, -0.82f, -0.33f,
R_CUBIC_TO, -0.32f, 0, -0.59f, 0.11f, -0.81f, 0.34f,
R_ARC_TO, 1.1f, 1.1f, 0, 0, 0, -0.34f, 0.81f,
R_CUBIC_TO, 0, 0.32f, 0.11f, 0.59f, 0.34f, 0.82f,
R_CUBIC_TO, 0.23f, 0.22f, 0.5f, 0.34f, 0.82f, 0.34f,
CLOSE,
MOVE_TO, 6.41f, 5.38f,
CLOSE,
R_MOVE_TO, 0, 6.24f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAccountAddChromeRefreshRepList, kAccountAddChromeRefreshIcon, {kAccountAddChromeRefreshPath, std::size(kAccountAddChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kAccountChildCirclePath, CANVAS_DIMENSIONS, 24,
CIRCLE, 12, 13.49f, 1.5f,
MOVE_TO, 12, 2,
CUBIC_TO, 6.48f, 2, 2, 6.48f, 2, 12,
R_CUBIC_TO, 0, 5.52f, 4.48f, 10, 10, 10,
R_CUBIC_TO, 5.52f, 0, 10, -4.48f, 10, -10,
CUBIC_TO, 22, 6.48f, 17.52f, 2, 12, 2,
CLOSE,
R_MOVE_TO, 0, 2,
R_CUBIC_TO, 1.38f, 0, 2.5f, 1.12f, 2.5f, 2.5f,
CUBIC_TO, 14.5f, 7.88f, 13.38f, 9, 12, 9,
CUBIC_TO, 10.62f, 9, 9.5f, 7.88f, 9.5f, 6.5f,
CUBIC_TO, 9.5f, 5.12f, 10.62f, 4, 12, 4,
CLOSE,
R_MOVE_TO, 5, 13.22f,
R_CUBIC_TO, 0, 0.89f, -0.84f, 1.66f, -2, 2.17f,
R_V_LINE_TO, -1.72f,
R_CUBIC_TO, 0, -0.73f, -0.93f, -1.32f, -2, -1.55f,
R_CUBIC_TO, -0.33f, -0.08f, -0.67f, -0.12f, -1, -0.12f,
R_CUBIC_TO, -1.29f, 0, -2.7f, 0.59f, -2.95f, 1.4f,
R_CUBIC_TO, 0.88f, 0.38f, 1.94f, 0.6f, 2.95f, 0.6f,
R_CUBIC_TO, 0.33f, 0, 0.67f, -0.03f, 1, -0.08f,
R_V_LINE_TO, 2,
R_CUBIC_TO, -0.33f, 0.05f, -0.67f, 0.08f, -1, 0.08f,
R_CUBIC_TO, -1.03f, 0, -2.11f, -0.23f, -3, -0.61f,
R_CUBIC_TO, -1.16f, -0.5f, -2, -1.27f, -2, -2.17f,
R_V_LINE_TO, -4.44f,
CUBIC_TO, 7, 11.2f, 9.63f, 10, 12, 10,
R_CUBIC_TO, 2.37f, 0, 5, 1.2f, 5, 2.78f,
R_V_LINE_TO, 4.44f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAccountChildCircleRepList, kAccountChildCircleIcon, {kAccountChildCirclePath, std::size(kAccountChildCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kAccountCirclePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12, 2,
CUBIC_TO, 6.48f, 2, 2, 6.48f, 2, 12,
R_CUBIC_TO, 0, 5.52f, 4.48f, 10, 10, 10,
R_CUBIC_TO, 5.52f, 0, 10, -4.48f, 10, -10,
CUBIC_TO_SHORTHAND, 17.52f, 2, 12, 2,
CLOSE,
R_MOVE_TO, 6.14f, 15.12f,
R_ARC_TO, 9.95f, 9.95f, 0, 0, 0, -12.28f, 0,
ARC_TO, 7.96f, 7.96f, 0, 0, 1, 4, 12,
R_CUBIC_TO, 0, -4.42f, 3.58f, -8, 8, -8,
R_CUBIC_TO, 4.42f, 0, 8, 3.58f, 8, 8,
R_CUBIC_TO, 0, 1.95f, -0.7f, 3.73f, -1.86f, 5.12f,
CLOSE,
NEW_PATH,
MOVE_TO, 12, 6,
R_CUBIC_TO, -1.93f, 0, -3.5f, 1.57f, -3.5f, 3.5f,
CUBIC_TO_SHORTHAND, 10.07f, 13, 12, 13,
R_CUBIC_TO, 1.93f, 0, 3.5f, -1.57f, 3.5f, -3.5f,
CUBIC_TO_SHORTHAND, 13.93f, 6, 12, 6,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAccountCircleRepList, kAccountCircleIcon, {kAccountCirclePath, std::size(kAccountCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kAccountCircleChromeRefreshPath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 5.85f, 17.1f,
R_ARC_TO, 10.32f, 10.32f, 0, 0, 1, 2.85f, -1.54f,
ARC_TO, 9.75f, 9.75f, 0, 0, 1, 12, 15,
R_CUBIC_TO, 1.15f, 0, 2.25f, 0.19f, 3.3f, 0.56f,
R_CUBIC_TO, 1.05f, 0.37f, 2, 0.89f, 2.85f, 1.54f,
R_ARC_TO, 7.67f, 7.67f, 0, 0, 0, 1.36f, -2.33f,
ARC_TO, 7.83f, 7.83f, 0, 0, 0, 20, 12,
R_CUBIC_TO, 0, -2.21f, -0.78f, -4.11f, -2.34f, -5.66f,
CUBIC_TO, 16.11f, 4.78f, 14.21f, 4, 12, 4,
R_CUBIC_TO, -2.21f, 0, -4.11f, 0.78f, -5.66f, 2.34f,
CUBIC_TO, 4.78f, 7.9f, 4, 9.79f, 4, 12,
R_CUBIC_TO, 0, 0.98f, 0.16f, 1.91f, 0.49f, 2.77f,
R_ARC_TO, 7.67f, 7.67f, 0, 0, 0, 1.36f, 2.33f,
CLOSE,
MOVE_TO, 12, 13,
R_CUBIC_TO, -0.98f, 0, -1.81f, -0.34f, -2.49f, -1.01f,
CUBIC_TO, 8.84f, 11.31f, 8.5f, 10.48f, 8.5f, 9.5f,
R_CUBIC_TO, 0, -0.98f, 0.34f, -1.81f, 1.01f, -2.49f,
CUBIC_TO, 10.19f, 6.34f, 11.02f, 6, 12, 6,
R_CUBIC_TO, 0.98f, 0, 1.81f, 0.34f, 2.49f, 1.01f,
R_CUBIC_TO, 0.68f, 0.68f, 1.01f, 1.5f, 1.01f, 2.49f,
R_CUBIC_TO, 0, 0.99f, -0.34f, 1.81f, -1.01f, 2.49f,
CUBIC_TO, 13.81f, 12.66f, 12.98f, 13, 12, 13,
CLOSE,
R_MOVE_TO, 0, 9,
R_ARC_TO, 9.69f, 9.69f, 0, 0, 1, -3.9f, -0.79f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, -3.18f, -2.14f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, -2.14f, -3.18f,
ARC_TO, 9.69f, 9.69f, 0, 0, 1, 2, 12,
R_CUBIC_TO, 0, -1.38f, 0.26f, -2.68f, 0.79f, -3.9f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, 2.14f, -3.18f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, 3.18f, -2.14f,
ARC_TO, 9.69f, 9.69f, 0, 0, 1, 12, 2,
R_CUBIC_TO, 1.38f, 0, 2.68f, 0.26f, 3.9f, 0.79f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, 3.18f, 2.14f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, 2.14f, 3.18f,
ARC_TO, 9.69f, 9.69f, 0, 0, 1, 22, 12,
R_ARC_TO, 9.69f, 9.69f, 0, 0, 1, -0.79f, 3.9f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, -2.14f, 3.18f,
R_ARC_TO, 9.99f, 9.99f, 0, 0, 1, -3.18f, 2.14f,
ARC_TO, 9.69f, 9.69f, 0, 0, 1, 12, 22,
CLOSE,
R_MOVE_TO, 0, -2,
R_CUBIC_TO, 0.88f, 0, 1.72f, -0.13f, 2.5f, -0.39f,
R_ARC_TO, 7.44f, 7.44f, 0, 0, 0, 2.15f, -1.11f,
R_ARC_TO, 7.44f, 7.44f, 0, 0, 0, -2.15f, -1.11f,
ARC_TO, 7.96f, 7.96f, 0, 0, 0, 12, 17,
R_CUBIC_TO, -0.88f, 0, -1.71f, 0.13f, -2.5f, 0.39f,
ARC_TO, 7.44f, 7.44f, 0, 0, 0, 7.35f, 18.5f,
ARC_TO, 7.44f, 7.44f, 0, 0, 0, 9.5f, 19.61f,
ARC_TO, 7.96f, 7.96f, 0, 0, 0, 12, 20,
CLOSE,
R_MOVE_TO, 0, -9,
R_CUBIC_TO, 0.43f, 0, 0.79f, -0.14f, 1.07f, -0.43f,
R_CUBIC_TO, 0.28f, -0.28f, 0.43f, -0.64f, 0.43f, -1.07f,
R_CUBIC_TO, 0, -0.43f, -0.14f, -0.79f, -0.43f, -1.07f,
CUBIC_TO, 12.79f, 8.14f, 12.43f, 8, 12, 8,
R_CUBIC_TO, -0.43f, 0, -0.79f, 0.14f, -1.07f, 0.43f,
R_CUBIC_TO, -0.28f, 0.28f, -0.43f, 0.64f, -0.43f, 1.07f,
R_CUBIC_TO, 0, 0.43f, 0.14f, 0.79f, 0.43f, 1.07f,
R_CUBIC_TO, 0.28f, 0.28f, 0.64f, 0.43f, 1.07f, 0.43f,
CLOSE,
R_MOVE_TO, 0, -1.5f,
CLOSE,
R_MOVE_TO, 0, 9,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAccountCircleChromeRefreshRepList, kAccountCircleChromeRefreshIcon, {kAccountCircleChromeRefreshPath, std::size(kAccountCircleChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kAccountManageChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 6.4f, 7.93f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -1.8f, -0.74f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -0.74f, -1.8f,
R_CUBIC_TO, 0, -0.71f, 0.25f, -1.31f, 0.74f, -1.8f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, 1.8f, -0.74f,
R_CUBIC_TO, 0.71f, 0, 1.31f, 0.25f, 1.8f, 0.74f,
R_CUBIC_TO, 0.49f, 0.49f, 0.74f, 1.09f, 0.74f, 1.8f,
R_CUBIC_TO, 0, 0.7f, -0.24f, 1.31f, -0.73f, 1.8f,
R_ARC_TO, 2.45f, 2.45f, 0, 0, 1, -1.8f, 0.74f,
CLOSE,
R_MOVE_TO, -4.93f, 5.07f,
R_V_LINE_TO, -1.68f,
R_CUBIC_TO, 0, -0.33f, 0.08f, -0.62f, 0.25f, -0.88f,
R_ARC_TO, 1.84f, 1.84f, 0, 0, 1, 0.61f, -0.6f,
R_ARC_TO, 8.2f, 8.2f, 0, 0, 1, 1.96f, -0.82f,
R_ARC_TO, 7.81f, 7.81f, 0, 0, 1, 2.11f, -0.29f,
R_CUBIC_TO, 0.13f, 0, 0.25f, 0, 0.38f, 0.01f,
R_CUBIC_TO, 0.12f, 0, 0.26f, 0.01f, 0.39f, 0.02f,
R_CUBIC_TO, -0.09f, 0.19f, -0.16f, 0.41f, -0.21f, 0.66f,
R_ARC_TO, 7.18f, 7.18f, 0, 0, 0, -0.13f, 0.7f,
R_LINE_TO, -0.41f, -0.02f,
R_CUBIC_TO, -0.58f, 0, -1.16f, 0.07f, -1.73f, 0.22f,
R_ARC_TO, 6.36f, 6.36f, 0, 0, 0, -1.64f, 0.69f,
R_ARC_TO, 0.38f, 0.38f, 0, 0, 0, -0.13f, 0.13f,
R_ARC_TO, 0.34f, 0.34f, 0, 0, 0, -0.05f, 0.19f,
R_V_LINE_TO, 0.27f,
R_H_LINE_TO, 4.13f,
R_CUBIC_TO, 0.06f, 0.23f, 0.14f, 0.47f, 0.26f, 0.72f,
R_CUBIC_TO, 0.11f, 0.24f, 0.24f, 0.47f, 0.38f, 0.67f,
CLOSE,
R_MOVE_TO, 9.25f, 0.73f,
R_LINE_TO, -0.21f, -0.95f,
R_ARC_TO, 2.64f, 2.64f, 0, 0, 1, -0.79f, -0.44f,
R_LINE_TO, -0.93f, 0.29f,
R_LINE_TO, -0.58f, -1,
R_LINE_TO, 0.7f, -0.68f,
R_ARC_TO, 1.36f, 1.36f, 0, 0, 1, -0.06f, -0.46f,
R_ARC_TO, 2.43f, 2.43f, 0, 0, 1, 0.06f, -0.46f,
LINE_TO, 8.2f, 9.37f,
R_LINE_TO, 0.58f, -1.01f,
R_LINE_TO, 0.92f, 0.27f,
R_CUBIC_TO, 0.11f, -0.1f, 0.24f, -0.19f, 0.38f, -0.28f,
R_CUBIC_TO, 0.14f, -0.08f, 0.28f, -0.14f, 0.43f, -0.18f,
R_LINE_TO, 0.22f, -0.95f,
R_H_LINE_TO, 1.17f,
R_LINE_TO, 0.22f, 0.95f,
R_CUBIC_TO, 0.14f, 0.05f, 0.29f, 0.11f, 0.43f, 0.19f,
R_CUBIC_TO, 0.14f, 0.08f, 0.27f, 0.17f, 0.38f, 0.27f,
R_LINE_TO, 0.92f, -0.26f,
R_LINE_TO, 0.59f, 1,
R_LINE_TO, -0.69f, 0.65f,
R_CUBIC_TO, 0.02f, 0.16f, 0.04f, 0.32f, 0.04f, 0.48f,
R_CUBIC_TO, 0, 0.16f, -0.02f, 0.31f, -0.05f, 0.46f,
R_LINE_TO, 0.7f, 0.66f,
R_LINE_TO, -0.59f, 1,
R_LINE_TO, -0.93f, -0.27f,
R_CUBIC_TO, -0.11f, 0.09f, -0.24f, 0.18f, -0.37f, 0.26f,
R_CUBIC_TO, -0.13f, 0.08f, -0.27f, 0.14f, -0.42f, 0.18f,
R_LINE_TO, -0.24f, 0.95f,
CLOSE,
R_MOVE_TO, 0.61f, -2.09f,
R_CUBIC_TO, 0.32f, 0, 0.59f, -0.12f, 0.82f, -0.35f,
R_CUBIC_TO, 0.23f, -0.23f, 0.34f, -0.51f, 0.34f, -0.83f,
R_CUBIC_TO, 0, -0.32f, -0.12f, -0.59f, -0.35f, -0.82f,
R_ARC_TO, 1.13f, 1.13f, 0, 0, 0, -0.82f, -0.34f,
R_CUBIC_TO, -0.32f, 0, -0.6f, 0.11f, -0.82f, 0.34f,
R_ARC_TO, 1.13f, 1.13f, 0, 0, 0, -0.34f, 0.83f,
R_CUBIC_TO, 0, 0.32f, 0.11f, 0.59f, 0.35f, 0.82f,
R_CUBIC_TO, 0.23f, 0.23f, 0.51f, 0.34f, 0.83f, 0.34f,
CLOSE,
MOVE_TO, 6.4f, 6.55f,
R_CUBIC_TO, 0.32f, 0, 0.59f, -0.11f, 0.81f, -0.34f,
R_CUBIC_TO, 0.23f, -0.23f, 0.34f, -0.5f, 0.34f, -0.82f,
R_CUBIC_TO, 0, -0.32f, -0.11f, -0.59f, -0.34f, -0.81f,
R_ARC_TO, 1.11f, 1.11f, 0, 0, 0, -0.81f, -0.33f,
R_CUBIC_TO, -0.32f, 0, -0.59f, 0.11f, -0.82f, 0.34f,
R_ARC_TO, 1.12f, 1.12f, 0, 0, 0, -0.34f, 0.81f,
R_CUBIC_TO, 0, 0.32f, 0.11f, 0.59f, 0.34f, 0.82f,
R_CUBIC_TO, 0.23f, 0.22f, 0.5f, 0.34f, 0.82f, 0.34f,
CLOSE,
R_MOVE_TO, 0, -1.15f,
CLOSE,
R_MOVE_TO, 0.58f, 6.22f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAccountManageChromeRefreshRepList, kAccountManageChromeRefreshIcon, {kAccountManageChromeRefreshPath, std::size(kAccountManageChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kAddPath, CANVAS_DIMENSIONS, 16,
STROKE, 2.f,
CAP_SQUARE,
MOVE_TO, 3, 8,
R_H_LINE_TO, 10,
MOVE_TO, 8, 3,
R_V_LINE_TO, 10)
VECTOR_ICON_TEMPLATE_CC(kAddRepList, kAddIcon, {kAddPath, std::size(kAddPath)})
VECTOR_ICON_REP_TEMPLATE(kAddChromeRefreshPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 7.18f, 12.22f,
V_LINE_TO, 8.82f,
H_LINE_TO, 3.79f,
V_LINE_TO, 7.18f,
R_H_LINE_TO, 3.4f,
V_LINE_TO, 3.79f,
R_H_LINE_TO, 1.63f,
R_V_LINE_TO, 3.4f,
R_H_LINE_TO, 3.4f,
R_V_LINE_TO, 1.63f,
H_LINE_TO, 8.82f,
R_V_LINE_TO, 3.4f,
CLOSE,
R_MOVE_TO, 0, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAddChromeRefreshRepList, kAddChromeRefreshIcon, {kAddChromeRefreshPath, std::size(kAddChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kAutoTabGroupsPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 8, 4,
R_V_LINE_TO, 9,
V_LINE_TO, 16,
CLOSE,
MOVE_TO, 4, 22,
R_ARC_TO, 1.99f, 1.99f, 0, 0, 1, -1.42f, -0.57f,
ARC_TO, 1.99f, 1.99f, 0, 0, 1, 2, 20,
V_LINE_TO, 6,
R_H_LINE_TO, 2,
R_V_LINE_TO, 14,
R_H_LINE_TO, 14,
R_V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 4, -4,
R_ARC_TO, 1.99f, 1.99f, 0, 0, 1, -1.42f, -0.57f,
ARC_TO, 1.99f, 1.99f, 0, 0, 1, 6, 16,
V_LINE_TO, 4,
R_CUBIC_TO, 0, -0.55f, 0.19f, -1.02f, 0.57f, -1.4f,
R_CUBIC_TO, 0.4f, -0.4f, 0.88f, -0.6f, 1.43f, -0.6f,
R_H_LINE_TO, 4.83f,
R_ARC_TO, 5.7f, 5.7f, 0, 0, 0, -0.75f, 0.93f,
ARC_TO, 7.33f, 7.33f, 0, 0, 0, 11.5f, 4,
H_LINE_TO, 8,
R_V_LINE_TO, 12,
R_H_LINE_TO, 12,
R_V_LINE_TO, -3.5f,
R_ARC_TO, 7.33f, 7.33f, 0, 0, 0, 1.08f, -0.57f,
R_CUBIC_TO, 0.33f, -0.22f, 0.64f, -0.47f, 0.93f, -0.75f,
V_LINE_TO, 16,
R_CUBIC_TO, 0, 0.55f, -0.2f, 1.02f, -0.6f, 1.43f,
R_CUBIC_TO, -0.38f, 0.38f, -0.85f, 0.57f, -1.4f, 0.57f,
CLOSE,
R_MOVE_TO, 9.5f, -6,
R_CUBIC_TO, 0, -1.53f, -0.53f, -2.83f, -1.6f, -3.9f,
R_CUBIC_TO, -1.07f, -1.07f, -2.37f, -1.6f, -3.9f, -1.6f,
R_CUBIC_TO, 1.53f, 0, 2.83f, -0.53f, 3.9f, -1.6f,
R_CUBIC_TO, 1.07f, -1.07f, 1.6f, -2.37f, 1.6f, -3.9f,
R_CUBIC_TO, 0, 1.53f, 0.53f, 2.83f, 1.6f, 3.9f,
R_CUBIC_TO, 1.07f, 1.07f, 2.37f, 1.6f, 3.9f, 1.6f,
R_CUBIC_TO, -1.53f, 0, -2.83f, 0.53f, -3.9f, 1.6f,
R_CUBIC_TO, -1.07f, 1.07f, -1.6f, 2.37f, -1.6f, 3.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kAutoTabGroupsRepList, kAutoTabGroupsIcon, {kAutoTabGroupsPath, std::size(kAutoTabGroupsPath)})
VECTOR_ICON_REP_TEMPLATE(kLocalOfferFlippedRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10.57f, 17.72f,
R_ARC_TO, 1.66f, 1.66f, 0, 0, 1, -1.22f, 0.51f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.51f,
R_LINE_TO, -5.86f, -5.86f,
R_ARC_TO, 1.66f, 1.66f, 0, 0, 1, -0.51f, -1.22f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.51f, -1.23f,
LINE_TO, 9.43f, 2.27f,
R_CUBIC_TO, 0.17f, -0.17f, 0.36f, -0.3f, 0.56f, -0.38f,
R_CUBIC_TO, 0.21f, -0.08f, 0.42f, -0.12f, 0.65f, -0.12f,
H_LINE_TO, 16.5f,
R_CUBIC_TO, 0.48f, 0, 0.88f, 0.17f, 1.22f, 0.51f,
R_CUBIC_TO, 0.34f, 0.34f, 0.51f, 0.75f, 0.51f, 1.22f,
R_V_LINE_TO, 5.88f,
R_ARC_TO, 1.66f, 1.66f, 0, 0, 1, -0.51f, 1.2f,
CLOSE,
MOVE_TO, 14.5f, 6.75f,
R_CUBIC_TO, 0.35f, 0, 0.64f, -0.12f, 0.89f, -0.36f,
R_CUBIC_TO, 0.24f, -0.25f, 0.36f, -0.54f, 0.36f, -0.89f,
R_CUBIC_TO, 0, -0.35f, -0.12f, -0.64f, -0.36f, -0.89f,
R_ARC_TO, 1.22f, 1.22f, 0, 0, 0, -0.89f, -0.36f,
R_CUBIC_TO, -0.35f, 0, -0.64f, 0.12f, -0.89f, 0.36f,
R_CUBIC_TO, -0.24f, 0.25f, -0.36f, 0.54f, -0.36f, 0.89f,
R_CUBIC_TO, 0, 0.35f, 0.12f, 0.64f, 0.36f, 0.89f,
R_CUBIC_TO, 0.25f, 0.24f, 0.54f, 0.36f, 0.89f, 0.36f,
CLOSE,
MOVE_TO, 9.35f, 16.5f,
LINE_TO, 16.5f, 9.38f,
V_LINE_TO, 3.5f,
R_H_LINE_TO, -5.85f,
R_LINE_TO, -7.15f, 7.15f,
CLOSE,
MOVE_TO, 16.48f, 3.58f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kLocalOfferFlippedRefreshRepList, kLocalOfferFlippedRefreshIcon, {kLocalOfferFlippedRefreshPath, std::size(kLocalOfferFlippedRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kWebauthnDialogHeaderPath, CANVAS_DIMENSIONS, 448,
NEW_PATH,
PATH_COLOR_ARGB, 0x0A, 0x00, 0x00, 0x00,
MOVE_TO, 0, 0,
R_H_LINE_TO, 448,
R_V_LINE_TO, 119,
H_LINE_TO, 0,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
MOVE_TO, 318.49f, 67.44f,
H_LINE_TO, 275,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 302.36f, 52,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
MOVE_TO, 372.49f, 33.44f,
H_LINE_TO, 329,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 356.36f, 18,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
MOVE_TO, 60.49f, 72.44f,
H_LINE_TO, 17,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 44.36f, 57,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 128, 87, 37.058,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
CIRCLE, 128, 87, 37.058,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
MOVE_TO, 140.08f, 47.81f,
ARC_TO, 40.99f, 40.99f, 0, 0, 0, 128, 46,
R_CUBIC_TO, -5.24f, 0, -10.25f, 0.98f, -14.85f, 2.77f,
R_ARC_TO, 41.07f, 41.07f, 0, 0, 0, -15.34f, 10.49f,
MOVE_TO, 87, 87,
R_CUBIC_TO, 0, 4.36f, 0.68f, 8.56f, 1.94f, 12.5f,
R_MOVE_TO, 12.64f, 18.86f,
ARC_TO, 40.85f, 40.85f, 0, 0, 0, 128, 128,
R_CUBIC_TO, 11.43f, 0, 21.77f, -4.68f, 29.21f, -12.23f,
ARC_TO, 40.96f, 40.96f, 0, 0, 0, 167.45f, 98.2f,
MOVE_TO, 169, 87,
R_ARC_TO, 40.84f, 40.84f, 0, 0, 0, -3.94f, -17.57f,
ARC_TO, 41.14f, 41.14f, 0, 0, 0, 154.73f, 55.91f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x1A, 0x73, 0xE8,
MOVE_TO, 115.96f, 76.75f,
H_LINE_TO, 144.63f,
V_LINE_TO, 73.17f,
H_LINE_TO, 115.96f,
CUBIC_TO, 113.99f, 73.17f, 112.38f, 74.78f, 112.38f, 76.75f,
V_LINE_TO, 96.46f,
H_LINE_TO, 108.79f,
V_LINE_TO, 101.83f,
H_LINE_TO, 128.5f,
V_LINE_TO, 96.46f,
H_LINE_TO, 115.96f,
V_LINE_TO, 76.75f,
CLOSE,
MOVE_TO, 144.63f, 80.33f,
H_LINE_TO, 133.88f,
CUBIC_TO, 132.89f, 80.33f, 132.08f, 81.14f, 132.08f, 82.13f,
V_LINE_TO, 100.04f,
CUBIC_TO, 132.08f, 101.03f, 132.89f, 101.83f, 133.88f, 101.83f,
H_LINE_TO, 144.63f,
CUBIC_TO, 145.61f, 101.83f, 146.42f, 101.03f, 146.42f, 100.04f,
V_LINE_TO, 82.13f,
CUBIC_TO, 146.42f, 81.14f, 145.61f, 80.33f, 144.63f, 80.33f,
CLOSE,
MOVE_TO, 135.67f, 96.46f,
H_LINE_TO, 142.83f,
V_LINE_TO, 83.92f,
H_LINE_TO, 135.67f,
V_LINE_TO, 96.46f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebauthnDialogHeaderRepList, kWebauthnDialogHeaderIcon, {kWebauthnDialogHeaderPath, std::size(kWebauthnDialogHeaderPath)})
VECTOR_ICON_REP_TEMPLATE(kWebauthnDialogHeaderDarkPath, CANVAS_DIMENSIONS, 448,
PATH_COLOR_ARGB, 0x18, 0x00, 0x00, 0x00,
MOVE_TO, 318.49f, 67.44f,
H_LINE_TO, 275,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 302.36f, 52,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
MOVE_TO, 372.49f, 33.44f,
H_LINE_TO, 329,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 356.36f, 18,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
MOVE_TO, 60.49f, 72.44f,
H_LINE_TO, 17,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 44.36f, 57,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
MOVE_TO, 60.49f, 72.44f,
H_LINE_TO, 17,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 44.36f, 57,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x41, 0x44, 0x47,
CIRCLE, 128, 87, 37.058,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 128, 87, 37.058,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 140.08f, 47.81f,
ARC_TO, 40.99f, 40.99f, 0, 0, 0, 128, 46,
R_CUBIC_TO, -5.24f, 0, -10.25f, 0.98f, -14.85f, 2.77f,
R_ARC_TO, 41.07f, 41.07f, 0, 0, 0, -15.34f, 10.49f,
MOVE_TO, 87, 87,
R_CUBIC_TO, 0, 4.36f, 0.68f, 8.56f, 1.94f, 12.5f,
R_MOVE_TO, 12.64f, 18.86f,
ARC_TO, 40.85f, 40.85f, 0, 0, 0, 128, 128,
R_CUBIC_TO, 11.43f, 0, 21.77f, -4.68f, 29.21f, -12.23f,
ARC_TO, 40.96f, 40.96f, 0, 0, 0, 167.45f, 98.2f,
MOVE_TO, 169, 87,
R_ARC_TO, 40.84f, 40.84f, 0, 0, 0, -3.94f, -17.57f,
ARC_TO, 41.14f, 41.14f, 0, 0, 0, 154.73f, 55.91f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x8A, 0xB4, 0xF8,
MOVE_TO, 115.96f, 76.75f,
H_LINE_TO, 144.63f,
V_LINE_TO, 73.17f,
H_LINE_TO, 115.96f,
CUBIC_TO, 113.99f, 73.17f, 112.38f, 74.78f, 112.38f, 76.75f,
V_LINE_TO, 96.46f,
H_LINE_TO, 108.79f,
V_LINE_TO, 101.83f,
H_LINE_TO, 128.5f,
V_LINE_TO, 96.46f,
H_LINE_TO, 115.96f,
V_LINE_TO, 76.75f,
CLOSE,
MOVE_TO, 144.63f, 80.33f,
H_LINE_TO, 133.88f,
CUBIC_TO, 132.89f, 80.33f, 132.08f, 81.14f, 132.08f, 82.13f,
V_LINE_TO, 100.04f,
CUBIC_TO, 132.08f, 101.03f, 132.89f, 101.83f, 133.88f, 101.83f,
H_LINE_TO, 144.63f,
CUBIC_TO, 145.61f, 101.83f, 146.42f, 101.03f, 146.42f, 100.04f,
V_LINE_TO, 82.13f,
CUBIC_TO, 146.42f, 81.14f, 145.61f, 80.33f, 144.63f, 80.33f,
CLOSE,
MOVE_TO, 135.67f, 96.46f,
H_LINE_TO, 142.83f,
V_LINE_TO, 83.92f,
H_LINE_TO, 135.67f,
V_LINE_TO, 96.46f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebauthnDialogHeaderDarkRepList, kWebauthnDialogHeaderDarkIcon, {kWebauthnDialogHeaderDarkPath, std::size(kWebauthnDialogHeaderDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kBackArrowTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12.91f, 18.59f,
R_LINE_TO, 0.09f, -0.09f,
LINE_TO, 7.49f, 13,
H_LINE_TO, 19,
R_ARC_TO, 1, 1, 0, 0, 0, 0, -2,
H_LINE_TO, 7.49f,
LINE_TO, 13, 5.49f,
R_LINE_TO, -0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 0, -1.32f, -1.32f,
R_H_LINE_TO, 0,
LINE_TO, 11.51f, 4,
LINE_TO, 4, 11.51f,
R_LINE_TO, 0.09f, 0.09f,
R_ARC_TO, 1, 1, 0, 0, 0, 0, 0.82f,
LINE_TO, 4, 12.5f,
LINE_TO, 11.51f, 20,
R_LINE_TO, 0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 0, 1.32f, -1.32f,
R_V_LINE_TO, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBackArrowTouchRepList, kBackArrowTouchIcon, {kBackArrowTouchPath, std::size(kBackArrowTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kBackspacePath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 4.48f, 3,
CUBIC_TO, 4.18f, 3, 3.89f, 3.14f, 3.7f, 3.38f,
LINE_TO, 0, 8,
LINE_TO, 3.7f, 12.62f,
CUBIC_TO, 3.89f, 12.86f, 4.18f, 13, 4.48f, 13,
H_LINE_TO, 12.5f,
CUBIC_TO, 13.33f, 13, 14, 12.33f, 14, 11.5f,
V_LINE_TO, 4.5f,
CUBIC_TO, 14, 3.67f, 13.33f, 3, 12.5f, 3,
H_LINE_TO, 4.48f,
CLOSE,
MOVE_TO, 5, 6,
LINE_TO, 6, 5,
LINE_TO, 8, 7,
LINE_TO, 10, 5,
LINE_TO, 11, 6,
LINE_TO, 9, 8,
LINE_TO, 11, 10,
LINE_TO, 10, 11,
LINE_TO, 8, 9,
LINE_TO, 6, 11,
LINE_TO, 5, 10,
LINE_TO, 7, 8,
LINE_TO, 5, 6,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBackspaceRepList, kBackspaceIcon, {kBackspacePath, std::size(kBackspacePath)})
VECTOR_ICON_REP_TEMPLATE(kBatterySaverRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 9.67f, 15.49f,
R_CUBIC_TO, 0.67f, 0, 1.29f, -0.12f, 1.88f, -0.37f,
R_ARC_TO, 4.73f, 4.73f, 0, 0, 0, 1.56f, -1.09f,
ARC_TO, 7.52f, 7.52f, 0, 0, 0, 15, 10.69f,
R_CUBIC_TO, 0.34f, -1.27f, 0.51f, -2.56f, 0.51f, -3.88f,
R_CUBIC_TO, 0, -0.37f, -0.02f, -0.73f, -0.06f, -1.09f,
R_CUBIC_TO, -0.04f, -0.36f, -0.07f, -0.71f, -0.09f, -1.08f,
R_CUBIC_TO, -0.35f, -0.04f, -0.7f, -0.07f, -1.03f, -0.11f,
R_CUBIC_TO, -0.34f, -0.03f, -0.68f, -0.05f, -1.03f, -0.05f,
R_CUBIC_TO, -1.36f, 0, -2.69f, 0.17f, -3.99f, 0.51f,
R_ARC_TO, 7.44f, 7.44f, 0, 0, 0, -3.4f, 1.95f,
R_CUBIC_TO, -0.45f, 0.45f, -0.8f, 0.97f, -1.04f, 1.56f,
R_ARC_TO, 4.76f, 4.76f, 0, 0, 0, -0.37f, 1.85f,
ARC_TO, 5.14f, 5.14f, 0, 0, 0, 6, 14,
R_CUBIC_TO, 0.48f, 0.48f, 1.04f, 0.85f, 1.68f, 1.11f,
R_ARC_TO, 5.25f, 5.25f, 0, 0, 0, 1.99f, 0.38f,
CLOSE,
R_MOVE_TO, -0.1f, 1.68f,
R_CUBIC_TO, -0.75f, 0, -1.47f, -0.12f, -2.16f, -0.37f,
R_ARC_TO, 7.2f, 7.2f, 0, 0, 1, -1.93f, -1.04f,
R_LINE_TO, -1.41f, 1.41f,
R_LINE_TO, -1.23f, -1.23f,
R_LINE_TO, 1.41f, -1.41f,
R_CUBIC_TO, -0.45f, -0.59f, -0.8f, -1.23f, -1.04f, -1.91f,
R_ARC_TO, 6.32f, 6.32f, 0, 0, 1, -0.37f, -2.16f,
R_CUBIC_TO, 0, -0.89f, 0.17f, -1.75f, 0.51f, -2.57f,
R_CUBIC_TO, 0.34f, -0.82f, 0.84f, -1.54f, 1.47f, -2.16f,
ARC_TO, 9.13f, 9.13f, 0, 0, 1, 8.72f, 3.47f,
R_ARC_TO, 16.37f, 16.37f, 0, 0, 1, 4.55f, -0.64f,
R_CUBIC_TO, 0.61f, 0, 1.2f, 0.02f, 1.79f, 0.08f,
R_CUBIC_TO, 0.59f, 0.05f, 1.18f, 0.14f, 1.77f, 0.25f,
R_CUBIC_TO, 0.13f, 0.6f, 0.22f, 1.21f, 0.27f, 1.82f,
R_CUBIC_TO, 0.05f, 0.61f, 0.07f, 1.22f, 0.07f, 1.83f,
R_CUBIC_TO, 0, 1.54f, -0.21f, 3.05f, -0.64f, 4.52f,
R_ARC_TO, 9.06f, 9.06f, 0, 0, 1, -2.25f, 3.87f,
R_ARC_TO, 6.31f, 6.31f, 0, 0, 1, -2.15f, 1.47f,
R_ARC_TO, 6.65f, 6.65f, 0, 0, 1, -2.57f, 0.5f,
CLOSE,
R_MOVE_TO, -0.61f, -3.18f,
R_LINE_TO, 3.95f, -3.69f,
R_ARC_TO, 0.5f, 0.5f, 0, 0, 0, 0.15f, -0.56f,
R_CUBIC_TO, -0.07f, -0.22f, -0.23f, -0.34f, -0.46f, -0.35f,
LINE_TO, 9.57f, 9.21f,
R_LINE_TO, 2.07f, -2.64f,
R_ARC_TO, 0.52f, 0.52f, 0, 0, 0, 0.07f, -0.28f,
R_ARC_TO, 0.36f, 0.36f, 0, 0, 0, -0.11f, -0.25f,
R_ARC_TO, 0.51f, 0.51f, 0, 0, 0, -0.29f, -0.12f,
R_ARC_TO, 0.33f, 0.33f, 0, 0, 0, -0.26f, 0.09f,
LINE_TO, 7.09f, 9.7f,
R_CUBIC_TO, -0.19f, 0.16f, -0.23f, 0.34f, -0.15f, 0.56f,
R_CUBIC_TO, 0.09f, 0.22f, 0.25f, 0.34f, 0.49f, 0.35f,
R_LINE_TO, 3, 0.18f,
R_LINE_TO, -2.07f, 2.65f,
R_ARC_TO, 0.34f, 0.34f, 0, 0, 0, -0.09f, 0.28f,
R_ARC_TO, 0.5f, 0.5f, 0, 0, 0, 0.13f, 0.27f,
R_CUBIC_TO, 0.07f, 0.07f, 0.16f, 0.11f, 0.27f, 0.12f,
R_CUBIC_TO, 0.11f, 0, 0.2f, -0.04f, 0.28f, -0.1f,
CLOSE,
MOVE_TO, 10, 10,
CLOSE,
R_MOVE_TO, 0, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBatterySaverRefreshRepList, kBatterySaverRefreshIcon, {kBatterySaverRefreshPath, std::size(kBatterySaverRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kBookmarkAllTabsChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 11.23f, 9.52f,
R_LINE_TO, 2.43f, -2.08f,
R_LINE_TO, 2.02f, 0.18f,
R_LINE_TO, -2.91f, 2.49f,
R_LINE_TO, 0.89f, 3.71f,
R_LINE_TO, -1.64f, -1,
CLOSE,
MOVE_TO, 9.7f, 4.84f,
R_LINE_TO, -0.69f, -1.59f,
R_LINE_TO, 0.86f, -1.98f,
LINE_TO, 11.46f, 4.98f,
CLOSE,
R_MOVE_TO, -4.55f, 6.17f,
R_LINE_TO, 2.06f, -1.22f,
R_LINE_TO, 2.07f, 1.22f,
R_LINE_TO, -0.55f, -2.31f,
R_LINE_TO, 1.79f, -1.53f,
R_LINE_TO, -2.36f, -0.21f,
R_LINE_TO, -0.95f, -2.2f,
R_LINE_TO, -0.94f, 2.2f,
R_LINE_TO, -2.36f, 0.2f,
R_LINE_TO, 1.8f, 1.52f,
CLOSE,
R_MOVE_TO, -2.09f, 2.87f,
R_LINE_TO, 1.11f, -4.66f,
R_LINE_TO, -3.68f, -3.14f,
LINE_TO, 5.31f, 5.66f,
R_LINE_TO, 1.89f, -4.4f,
R_LINE_TO, 1.89f, 4.41f,
R_LINE_TO, 4.83f, 0.4f,
R_LINE_TO, -3.68f, 3.14f,
R_LINE_TO, 1.11f, 4.66f,
R_LINE_TO, -4.15f, -2.47f,
CLOSE,
R_MOVE_TO, 4.15f, -5.65f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarkAllTabsChromeRefreshRepList, kBookmarkAllTabsChromeRefreshIcon, {kBookmarkAllTabsChromeRefreshPath, std::size(kBookmarkAllTabsChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kBookmarkbarOverflowRefreshPath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 9.57f, 12,
LINE_TO, 5, 7.4f,
LINE_TO, 6.4f, 6,
R_LINE_TO, 6, 6,
R_LINE_TO, -6, 6,
LINE_TO, 5, 16.6f,
CLOSE,
R_MOVE_TO, 6.6f, 0,
R_LINE_TO, -4.57f, -4.6f,
LINE_TO, 13, 6,
R_LINE_TO, 6, 6,
R_LINE_TO, -6, 6,
R_LINE_TO, -1.4f, -1.4f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kBookmarkbarOverflowRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 6.2f, 8,
LINE_TO, 3.05f, 4.85f,
R_LINE_TO, 0.98f, -0.98f,
LINE_TO, 8.15f, 8,
R_LINE_TO, -4.12f, 4.13f,
R_LINE_TO, -0.98f, -0.98f,
CLOSE,
MOVE_TO, 11, 8,
LINE_TO, 7.85f, 4.85f,
R_LINE_TO, 0.98f, -0.98f,
LINE_TO, 12.95f, 8,
R_LINE_TO, -4.13f, 4.13f,
R_LINE_TO, -0.98f, -0.98f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarkbarOverflowRefreshRepList, kBookmarkbarOverflowRefreshIcon, {kBookmarkbarOverflowRefreshPath, std::size(kBookmarkbarOverflowRefreshPath)}, {kBookmarkbarOverflowRefresh16Path, std::size(kBookmarkbarOverflowRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kBookmarksListsMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.94f, 11.01f,
LINE_TO, 8, 9.79f,
R_LINE_TO, 2.07f, 1.22f,
R_LINE_TO, -0.55f, -2.31f,
R_LINE_TO, 1.79f, -1.53f,
R_LINE_TO, -2.36f, -0.21f,
LINE_TO, 8, 4.76f,
LINE_TO, 7.05f, 6.97f,
R_LINE_TO, -2.36f, 0.2f,
R_LINE_TO, 1.8f, 1.52f,
CLOSE,
R_MOVE_TO, -2.09f, 2.87f,
R_LINE_TO, 1.11f, -4.66f,
R_LINE_TO, -3.68f, -3.14f,
LINE_TO, 6.11f, 5.66f,
LINE_TO, 8, 1.26f,
R_LINE_TO, 1.89f, 4.41f,
R_LINE_TO, 4.83f, 0.4f,
R_LINE_TO, -3.68f, 3.14f,
R_LINE_TO, 1.11f, 4.66f,
LINE_TO, 8, 11.4f,
CLOSE,
MOVE_TO, 8, 8.06f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarksListsMenuRepList, kBookmarksListsMenuIcon, {kBookmarksListsMenuPath, std::size(kBookmarksListsMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kBookmarksManagerPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 3.5f, 14.5f,
R_V_LINE_TO, -9,
CLOSE,
R_MOVE_TO, 0, 1.5f,
R_ARC_TO, 1.45f, 1.45f, 0, 0, 1, -1.05f, -0.44f,
CUBIC_TO, 2.15f, 15.27f, 2, 14.91f, 2, 14.5f,
V_LINE_TO, 5.49f,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.76f, 0.44f, -1.05f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 3.5f, 4,
H_LINE_TO, 8,
R_LINE_TO, 2, 2,
R_H_LINE_TO, 6.5f,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.44f,
R_CUBIC_TO, 0.29f, 0.29f, 0.44f, 0.65f, 0.44f, 1.06f,
R_V_LINE_TO, 2.19f,
R_ARC_TO, 2.83f, 2.83f, 0, 0, 0, -0.74f, -0.18f,
R_ARC_TO, 1.92f, 1.92f, 0, 0, 0, -0.76f, 0.06f,
V_LINE_TO, 7.5f,
H_LINE_TO, 9.38f,
R_LINE_TO, -2, -2,
H_LINE_TO, 3.5f,
R_V_LINE_TO, 9,
R_H_LINE_TO, 7.52f,
R_LINE_TO, -0.52f, 0.52f,
V_LINE_TO, 16,
CLOSE,
R_MOVE_TO, 8.5f, 2,
R_V_LINE_TO, -2.35f,
R_LINE_TO, 4.38f, -4.35f,
R_ARC_TO, 0.84f, 0.84f, 0, 0, 1, 0.34f, -0.22f,
R_CUBIC_TO, 0.13f, -0.05f, 0.25f, -0.07f, 0.37f, -0.07f,
R_ARC_TO, 1, 1, 0, 0, 1, 0.72f, 0.29f,
R_LINE_TO, 0.91f, 0.94f,
R_CUBIC_TO, 0.09f, 0.1f, 0.16f, 0.21f, 0.22f, 0.34f,
R_ARC_TO, 0.96f, 0.96f, 0, 0, 1, 0.01f, 0.74f,
R_ARC_TO, 0.81f, 0.81f, 0, 0, 1, -0.22f, 0.34f,
LINE_TO, 14.36f, 18,
CLOSE,
R_MOVE_TO, 6, -5.06f,
LINE_TO, 17.06f, 12,
CLOSE,
MOVE_TO, 13, 17,
R_H_LINE_TO, 0.94f,
R_LINE_TO, 2.39f, -2.39f,
R_LINE_TO, -0.46f, -0.48f,
R_LINE_TO, -0.46f, -0.46f,
LINE_TO, 13, 16.06f,
CLOSE,
R_MOVE_TO, 2.88f, -2.87f,
R_LINE_TO, -0.46f, -0.46f,
R_LINE_TO, 0.91f, 0.94f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarksManagerRepList, kBookmarksManagerIcon, {kBookmarksManagerPath, std::size(kBookmarksManagerPath)})
VECTOR_ICON_REP_TEMPLATE(kBookmarksSidePanelPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 14, 12.9f,
R_LINE_TO, 3, -2.6f,
R_LINE_TO, 2.5f, 0.2f,
LINE_TO, 16, 13.6f,
R_LINE_TO, 1.1f, 4.6f,
LINE_TO, 15, 17,
CUBIC_TO, 15, 17, 14, 12.9f, 14, 12.9f,
CLOSE,
MOVE_TO, 12.1f, 7,
R_LINE_TO, -0.9f, -2,
R_LINE_TO, 1.1f, -2.5f,
R_LINE_TO, 2, 4.6f,
LINE_TO, 12.1f, 7,
CLOSE,
MOVE_TO, 6.4f, 14.8f,
LINE_TO, 9, 13.2f,
R_LINE_TO, 2.6f, 1.5f,
R_LINE_TO, -0.7f, -2.9f,
R_LINE_TO, 2.2f, -1.9f,
R_LINE_TO, -3, -0.3f,
LINE_TO, 9, 7,
LINE_TO, 7.8f, 9.7f,
R_LINE_TO, -3, 0.3f,
R_LINE_TO, 2.3f, 1.9f,
CUBIC_TO, 7.1f, 11.9f, 6.4f, 14.8f, 6.4f, 14.8f,
CLOSE,
MOVE_TO, 3.8f, 18.3f,
R_LINE_TO, 1.4f, -5.8f,
LINE_TO, 0.6f, 8.6f,
R_LINE_TO, 6, -0.5f,
LINE_TO, 9, 2.6f,
R_LINE_TO, 2.4f, 5.5f,
R_LINE_TO, 6, 0.5f,
R_LINE_TO, -4.6f, 3.9f,
R_LINE_TO, 1.4f, 5.8f,
LINE_TO, 9, 15.3f,
LINE_TO, 3.8f, 18.3f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarksSidePanelRepList, kBookmarksSidePanelIcon, {kBookmarksSidePanelPath, std::size(kBookmarksSidePanelPath)})
VECTOR_ICON_REP_TEMPLATE(kBookmarksSidePanelRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
R_MOVE_TO, 13.75f, 11.81f,
R_LINE_TO, 3.04f, -2.61f,
LINE_TO, 19, 9.4f,
R_LINE_TO, -3.58f, 3.06f,
LINE_TO, 16.5f, 17,
R_LINE_TO, -1.77f, -1.06f,
CLOSE,
R_MOVE_TO, -1.81f, -5.52f,
R_LINE_TO, -0.89f, -2.06f,
LINE_TO, 12, 2,
R_LINE_TO, 1.92f, 4.46f,
CLOSE,
R_MOVE_TO, -5.6f, 7.6f,
LINE_TO, 9, 12.31f,
R_LINE_TO, 2.69f, 1.58f,
R_LINE_TO, -0.71f, -3,
R_LINE_TO, 2.31f, -1.98f,
R_LINE_TO, -3.06f, -0.27f,
LINE_TO, 9, 5.79f,
LINE_TO, 7.77f, 8.65f,
R_LINE_TO, -3.06f, 0.27f,
R_LINE_TO, 2.34f, 1.98f,
CLOSE,
MOVE_TO, 4.06f, 17,
R_LINE_TO, 1.31f, -5.54f,
LINE_TO, 1, 7.73f,
R_LINE_TO, 5.75f, -0.5f,
LINE_TO, 9, 2,
R_LINE_TO, 2.25f, 5.25f,
R_LINE_TO, 5.75f, 0.48f,
R_LINE_TO, -4.37f, 3.73f,
LINE_TO, 13.94f, 17,
LINE_TO, 9, 14.06f,
CLOSE,
MOVE_TO, 9, 10.27f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBookmarksSidePanelRefreshRepList, kBookmarksSidePanelRefreshIcon, {kBookmarksSidePanelRefreshPath, std::size(kBookmarksSidePanelRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserLogoPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 5.5f, 8.02f,
R_CUBIC_TO, 0, 0.7f, 0.24f, 1.29f, 0.73f, 1.77f,
R_ARC_TO, 2.41f, 2.41f, 0, 0, 0, 1.77f, 0.73f,
R_CUBIC_TO, 0.7f, 0, 1.29f, -0.24f, 1.77f, -0.73f,
R_ARC_TO, 2.41f, 2.41f, 0, 0, 0, 0.73f, -1.77f,
R_CUBIC_TO, 0, -0.69f, -0.24f, -1.28f, -0.73f, -1.77f,
ARC_TO, 2.41f, 2.41f, 0, 0, 0, 8, 5.52f,
R_CUBIC_TO, -0.69f, 0, -1.28f, 0.24f, -1.77f, 0.73f,
R_ARC_TO, 2.41f, 2.41f, 0, 0, 0, -0.73f, 1.77f,
CLOSE,
MOVE_TO, 8, 11.77f,
R_CUBIC_TO, 0.13f, 0, 0.25f, 0, 0.39f, -0.01f,
R_CUBIC_TO, 0.14f, -0.01f, 0.26f, -0.03f, 0.39f, -0.06f,
LINE_TO, 7.09f, 14.59f,
R_CUBIC_TO, -1.66f, -0.18f, -3.03f, -0.9f, -4.11f, -2.16f,
R_CUBIC_TO, -1.08f, -1.26f, -1.62f, -2.73f, -1.62f, -4.41f,
R_ARC_TO, 6.54f, 6.54f, 0, 0, 1, 0.66f, -2.86f,
R_LINE_TO, 2.77f, 4.73f,
R_CUBIC_TO, 0.33f, 0.59f, 0.77f, 1.05f, 1.35f, 1.38f,
R_CUBIC_TO, 0.58f, 0.33f, 1.2f, 0.5f, 1.87f, 0.5f,
CLOSE,
MOVE_TO, 8, 4.27f,
R_CUBIC_TO, -0.82f, 0, -1.55f, 0.24f, -2.19f, 0.73f,
ARC_TO, 3.79f, 3.79f, 0, 0, 0, 4.46f, 6.87f,
LINE_TO, 2.77f, 3.94f,
R_ARC_TO, 6.2f, 6.2f, 0, 0, 1, 2.29f, -1.9f,
ARC_TO, 6.64f, 6.64f, 0, 0, 1, 8, 1.38f,
R_CUBIC_TO, 1.09f, 0, 2.13f, 0.26f, 3.09f, 0.77f,
R_ARC_TO, 6.72f, 6.72f, 0, 0, 1, 2.38f, 2.12f,
CLOSE,
R_MOVE_TO, 6.16f, 1.25f,
R_CUBIC_TO, 0.17f, 0.41f, 0.3f, 0.82f, 0.37f, 1.23f,
R_CUBIC_TO, 0.08f, 0.42f, 0.12f, 0.84f, 0.12f, 1.27f,
R_CUBIC_TO, 0, 1.79f, -0.58f, 3.29f, -1.73f, 4.5f,
R_CUBIC_TO, -1.15f, 1.22f, -2.62f, 1.93f, -4.41f, 2.12f,
R_LINE_TO, 2.71f, -4.75f,
R_CUBIC_TO, 0.16f, -0.29f, 0.29f, -0.59f, 0.38f, -0.9f,
R_ARC_TO, 3.34f, 3.34f, 0, 0, 0, 0.14f, -0.97f,
R_CUBIC_TO, 0, -0.46f, -0.09f, -0.91f, -0.26f, -1.34f,
R_ARC_TO, 4.12f, 4.12f, 0, 0, 0, -0.71f, -1.16f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserLogoRepList, kBrowserLogoIcon, {kBrowserLogoPath, std::size(kBrowserLogoPath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 16, 9,
CUBIC_TO, 17.66f, 9, 19, 7.66f, 19, 6,
CUBIC_TO, 19, 4.34f, 17.66f, 3, 16, 3,
CUBIC_TO, 14.34f, 3, 13, 4.34f, 13, 6,
CUBIC_TO, 13, 7.66f, 14.34f, 9, 16, 9,
CLOSE,
MOVE_TO, 16, 19,
CUBIC_TO, 17.66f, 19, 19, 17.66f, 19, 16,
CUBIC_TO, 19, 14.34f, 17.66f, 13, 16, 13,
CUBIC_TO, 14.34f, 13, 13, 14.34f, 13, 16,
CUBIC_TO, 13, 17.66f, 14.34f, 19, 16, 19,
CLOSE,
MOVE_TO, 16, 29,
CUBIC_TO, 17.66f, 29, 19, 27.66f, 19, 26,
CUBIC_TO, 19, 24.34f, 17.66f, 23, 16, 23,
CUBIC_TO, 14.34f, 23, 13, 24.34f, 13, 26,
CUBIC_TO, 13, 27.66f, 14.34f, 29, 16, 29,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kBrowserTools16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 7, 3.5f,
CUBIC_TO, 7, 2.67f, 7.67f, 2, 8.5f, 2,
CUBIC_TO, 9.33f, 2, 10, 2.67f, 10, 3.5f,
CUBIC_TO, 10, 4.33f, 9.33f, 5, 8.5f, 5,
CUBIC_TO, 7.67f, 5, 7, 4.33f, 7, 3.5f,
CLOSE,
MOVE_TO, 7, 8.5f,
CUBIC_TO, 7, 7.67f, 7.67f, 7, 8.5f, 7,
CUBIC_TO, 9.33f, 7, 10, 7.67f, 10, 8.5f,
CUBIC_TO, 10, 9.33f, 9.33f, 10, 8.5f, 10,
CUBIC_TO, 7.67f, 10, 7, 9.33f, 7, 8.5f,
CLOSE,
MOVE_TO, 7, 13.5f,
CUBIC_TO, 7, 12.67f, 7.67f, 12, 8.5f, 12,
CUBIC_TO, 9.33f, 12, 10, 12.67f, 10, 13.5f,
CUBIC_TO, 10, 14.33f, 9.33f, 15, 8.5f, 15,
CUBIC_TO, 7.67f, 15, 7, 14.33f, 7, 13.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsRepList, kBrowserToolsIcon, {kBrowserToolsPath, std::size(kBrowserToolsPath)}, {kBrowserTools16Path, std::size(kBrowserTools16Path)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10, 6,
R_CUBIC_TO, 0.83f, 0, 1.5f, -0.68f, 1.5f, -1.5f,
R_CUBIC_TO, 0, -0.82f, -0.67f, -1.5f, -1.5f, -1.5f,
R_CUBIC_TO, -0.83f, 0, -1.5f, 0.68f, -1.5f, 1.5f,
R_CUBIC_TO, 0, 0.82f, 0.67f, 1.5f, 1.5f, 1.5f,
CLOSE,
R_MOVE_TO, 0, 2.5f,
R_CUBIC_TO, -0.83f, 0, -1.5f, 0.67f, -1.5f, 1.5f,
R_CUBIC_TO, 0, 0.83f, 0.67f, 1.5f, 1.5f, 1.5f,
R_CUBIC_TO, 0.83f, 0, 1.5f, -0.67f, 1.5f, -1.5f,
R_CUBIC_TO, 0, -0.83f, -0.67f, -1.5f, -1.5f, -1.5f,
CLOSE,
R_MOVE_TO, 0, 5.5f,
R_CUBIC_TO, -0.83f, 0, -1.5f, 0.67f, -1.5f, 1.5f,
R_CUBIC_TO, 0, 0.83f, 0.67f, 1.5f, 1.5f, 1.5f,
R_CUBIC_TO, 0.83f, 0, 1.5f, -0.67f, 1.5f, -1.5f,
R_CUBIC_TO, 0, -0.83f, -0.67f, -1.5f, -1.5f, -1.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsChromeRefreshRepList, kBrowserToolsChromeRefreshIcon, {kBrowserToolsChromeRefreshPath, std::size(kBrowserToolsChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsErrorPath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 8,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 9, 8,
H_LINE_TO, 7,
V_LINE_TO, 4,
R_H_LINE_TO, 2,
R_V_LINE_TO, 4,
CLOSE,
R_MOVE_TO, 0, 4,
H_LINE_TO, 7,
R_V_LINE_TO, -2,
R_H_LINE_TO, 2,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsErrorRepList, kBrowserToolsErrorIcon, {kBrowserToolsErrorPath, std::size(kBrowserToolsErrorPath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12, 8,
R_ARC_TO, 2, 2, 0, 1, 0, 0, -4,
R_ARC_TO, 2, 2, 0, 0, 0, 0, 4,
CLOSE,
R_MOVE_TO, 0, 2,
R_ARC_TO, 2, 2, 0, 1, 0, 0, 4,
R_ARC_TO, 2, 2, 0, 0, 0, 0, -4,
CLOSE,
R_MOVE_TO, 0, 6,
R_ARC_TO, 2, 2, 0, 1, 0, 0, 4,
R_ARC_TO, 2, 2, 0, 0, 0, 0, -4,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsTouchRepList, kBrowserToolsTouchIcon, {kBrowserToolsTouchPath, std::size(kBrowserToolsTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsUpdatePath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 8,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 6, 12,
R_H_LINE_TO, 4,
V_LINE_TO, 9,
R_H_LINE_TO, 2,
LINE_TO, 8, 4,
LINE_TO, 4, 9,
R_H_LINE_TO, 2,
R_V_LINE_TO, 3,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsUpdateRepList, kBrowserToolsUpdateIcon, {kBrowserToolsUpdatePath, std::size(kBrowserToolsUpdatePath)})
VECTOR_ICON_REP_TEMPLATE(kBrowserToolsUpdateChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 4.06f, 6.75f,
R_LINE_TO, 1.21f, 0.4f,
R_CUBIC_TO, 0.15f, -0.28f, 0.31f, -0.55f, 0.47f, -0.82f,
R_CUBIC_TO, 0.16f, -0.27f, 0.33f, -0.54f, 0.52f, -0.8f,
R_LINE_TO, -0.86f, -0.1f,
CLOSE,
R_MOVE_TO, 2.36f, 1.15f,
R_LINE_TO, 1.66f, 1.67f,
R_ARC_TO, 17.77f, 17.77f, 0, 0, 0, 1.52f, -0.92f,
R_ARC_TO, 9.17f, 9.17f, 0, 0, 0, 1.32f, -1.1f,
R_ARC_TO, 9.62f, 9.62f, 0, 0, 0, 1.64f, -2.17f,
R_CUBIC_TO, 0.43f, -0.79f, 0.66f, -1.66f, 0.67f, -2.61f,
R_CUBIC_TO, -0.96f, 0.01f, -1.84f, 0.23f, -2.63f, 0.66f,
R_CUBIC_TO, -0.79f, 0.43f, -1.51f, 0.97f, -2.16f, 1.63f,
R_CUBIC_TO, -0.42f, 0.42f, -0.79f, 0.86f, -1.11f, 1.33f,
R_ARC_TO, 13.83f, 13.83f, 0, 0, 0, -0.91f, 1.52f,
CLOSE,
R_MOVE_TO, 2.88f, -1.19f,
R_ARC_TO, 1.13f, 1.13f, 0, 0, 1, -0.34f, -0.83f,
R_CUBIC_TO, 0, -0.32f, 0.11f, -0.6f, 0.34f, -0.83f,
R_CUBIC_TO, 0.23f, -0.23f, 0.5f, -0.34f, 0.83f, -0.34f,
R_CUBIC_TO, 0.32f, 0, 0.6f, 0.12f, 0.83f, 0.34f,
R_CUBIC_TO, 0.23f, 0.23f, 0.34f, 0.5f, 0.34f, 0.83f,
R_CUBIC_TO, 0, 0.33f, -0.12f, 0.61f, -0.34f, 0.83f,
R_CUBIC_TO, -0.23f, 0.23f, -0.5f, 0.34f, -0.83f, 0.34f,
R_CUBIC_TO, -0.32f, 0, -0.6f, -0.11f, -0.83f, -0.34f,
CLOSE,
R_MOVE_TO, -0.05f, 5.23f,
R_LINE_TO, 1.34f, -1.34f,
R_LINE_TO, -0.1f, -0.86f,
R_CUBIC_TO, -0.27f, 0.18f, -0.54f, 0.36f, -0.8f, 0.52f,
R_CUBIC_TO, -0.27f, 0.16f, -0.54f, 0.32f, -0.82f, 0.47f,
CLOSE,
R_MOVE_TO, 5.24f, -10.43f,
R_CUBIC_TO, 0.28f, 1.24f, 0.17f, 2.47f, -0.33f, 3.7f,
R_ARC_TO, 9.47f, 9.47f, 0, 0, 1, -2.1f, 3.19f,
R_CUBIC_TO, 0, 0.01f, -0.05f, 0.06f, -0.15f, 0.16f,
R_LINE_TO, -0.15f, 0.16f,
R_LINE_TO, 0.2f, 1.7f,
R_CUBIC_TO, 0.03f, 0.22f, 0.01f, 0.43f, -0.06f, 0.63f,
R_CUBIC_TO, -0.07f, 0.2f, -0.18f, 0.38f, -0.34f, 0.54f,
R_LINE_TO, -2.89f, 2.89f,
R_LINE_TO, -1.22f, -3.59f,
LINE_TO, 5.12f, 8.57f,
LINE_TO, 1.52f, 7.35f,
R_LINE_TO, 2.9f, -2.91f,
R_CUBIC_TO, 0.16f, -0.16f, 0.34f, -0.27f, 0.54f, -0.34f,
R_CUBIC_TO, 0.2f, -0.07f, 0.41f, -0.09f, 0.63f, -0.06f,
R_LINE_TO, 1.69f, 0.22f,
R_ARC_TO, 39.7f, 39.7f, 0, 0, 1, 0.35f, -0.33f,
R_ARC_TO, 9.59f, 9.59f, 0, 0, 1, 3.17f, -2.1f,
R_CUBIC_TO, 1.22f, -0.5f, 2.44f, -0.61f, 3.68f, -0.32f,
CLOSE,
MOVE_TO, 2.81f, 10.51f,
R_ARC_TO, 1.85f, 1.85f, 0, 0, 1, 1.34f, -0.53f,
R_CUBIC_TO, 0.51f, 0, 0.96f, 0.2f, 1.34f, 0.58f,
R_CUBIC_TO, 0.36f, 0.36f, 0.54f, 0.79f, 0.55f, 1.29f,
R_CUBIC_TO, 0.01f, 0.5f, -0.16f, 0.95f, -0.5f, 1.33f,
R_CUBIC_TO, -0.5f, 0.57f, -1.14f, 0.93f, -1.91f, 1.08f,
R_CUBIC_TO, -0.77f, 0.15f, -1.52f, 0.28f, -2.26f, 0.4f,
R_ARC_TO, 58.28f, 58.28f, 0, 0, 1, 0.39f, -2.23f,
R_ARC_TO, 3.5f, 3.5f, 0, 0, 1, 1.04f, -1.92f,
CLOSE,
R_MOVE_TO, 0.88f, 1.08f,
R_CUBIC_TO, -0.18f, 0.18f, -0.3f, 0.39f, -0.35f, 0.61f,
R_ARC_TO, 6.42f, 6.42f, 0, 0, 1, -0.19f, 0.7f,
R_ARC_TO, 49.4f, 49.4f, 0, 0, 1, 0.69f, -0.19f,
R_CUBIC_TO, 0.22f, -0.06f, 0.42f, -0.18f, 0.59f, -0.36f,
R_ARC_TO, 0.78f, 0.78f, 0, 0, 0, 0.2f, -0.42f,
R_ARC_TO, 0.43f, 0.43f, 0, 0, 0, -0.13f, -0.4f,
R_ARC_TO, 0.44f, 0.44f, 0, 0, 0, -0.39f, -0.14f,
R_ARC_TO, 0.78f, 0.78f, 0, 0, 0, -0.41f, 0.2f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kBrowserToolsUpdateChromeRefreshRepList, kBrowserToolsUpdateChromeRefreshIcon, {kBrowserToolsUpdateChromeRefreshPath, std::size(kBrowserToolsUpdateChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kCastChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 8, 8,
CLOSE,
R_MOVE_TO, 5.15f, 4.93f,
H_LINE_TO, 9.25f,
R_CUBIC_TO, 0, -0.23f, -0.01f, -0.46f, -0.03f, -0.69f,
R_ARC_TO, 8.41f, 8.41f, 0, 0, 0, -0.1f, -0.69f,
R_H_LINE_TO, 4.04f,
V_LINE_TO, 4.45f,
H_LINE_TO, 2.85f,
R_V_LINE_TO, 0.84f,
R_ARC_TO, 7.02f, 7.02f, 0, 0, 0, -1.38f, -0.13f,
V_LINE_TO, 4.45f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 10.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 7.11f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, -11.69f, 0,
R_V_LINE_TO, -1.66f,
R_CUBIC_TO, 0.46f, 0, 0.86f, 0.16f, 1.18f, 0.48f,
R_CUBIC_TO, 0.32f, 0.32f, 0.48f, 0.72f, 0.48f, 1.18f,
CLOSE,
R_MOVE_TO, 2.86f, 0,
R_CUBIC_TO, 0, -0.79f, -0.28f, -1.47f, -0.84f, -2.03f,
R_ARC_TO, 2.78f, 2.78f, 0, 0, 0, -2.03f, -0.84f,
V_LINE_TO, 8.81f,
R_CUBIC_TO, 1.14f, 0, 2.12f, 0.4f, 2.92f, 1.2f,
R_CUBIC_TO, 0.8f, 0.8f, 1.2f, 1.77f, 1.2f, 2.92f,
CLOSE,
R_MOVE_TO, 2.46f, 0,
R_CUBIC_TO, 0, -0.73f, -0.14f, -1.42f, -0.42f, -2.07f,
R_ARC_TO, 5.43f, 5.43f, 0, 0, 0, -1.14f, -1.69f,
R_ARC_TO, 5.42f, 5.42f, 0, 0, 0, -1.69f, -1.14f,
R_ARC_TO, 5.19f, 5.19f, 0, 0, 0, -2.07f, -0.41f,
V_LINE_TO, 6.35f,
R_CUBIC_TO, 0.91f, 0, 1.77f, 0.17f, 2.57f, 0.52f,
R_ARC_TO, 6.64f, 6.64f, 0, 0, 1, 2.09f, 1.41f,
R_ARC_TO, 6.58f, 6.58f, 0, 0, 1, 1.41f, 2.09f,
R_ARC_TO, 6.41f, 6.41f, 0, 0, 1, 0.52f, 2.57f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCastChromeRefreshRepList, kCastChromeRefreshIcon, {kCastChromeRefreshPath, std::size(kCastChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kChevronRightPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 7.59f, 18.59f,
LINE_TO, 9, 20,
R_LINE_TO, 8, -8,
R_LINE_TO, -8, -8,
R_LINE_TO, -1.41f, 1.41f,
LINE_TO, 14.17f, 12,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kChevronRightRepList, kChevronRightIcon, {kChevronRightPath, std::size(kChevronRightPath)})
VECTOR_ICON_REP_TEMPLATE(kChevronRightChromeRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 8, 15.16f,
R_LINE_TO, -1.22f, -1.22f,
LINE_TO, 10.72f, 10,
LINE_TO, 6.78f, 6.06f,
LINE_TO, 8, 4.84f,
LINE_TO, 13.16f, 10,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kChevronRightChromeRefreshRepList, kChevronRightChromeRefreshIcon, {kChevronRightChromeRefreshPath, std::size(kChevronRightChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kChromiumMinimizePath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 4, 20,
R_V_LINE_TO, -2,
R_H_LINE_TO, 16,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kChromiumMinimizeRepList, kChromiumMinimizeIcon, {kChromiumMinimizePath, std::size(kChromiumMinimizePath)})
VECTOR_ICON_REP_TEMPLATE(kClickToCallIllustrationPath, CANVAS_DIMENSIONS, 320,
PATH_COLOR_ARGB, 0xFF, 0xF8, 0xF9, 0xFA,
MOVE_TO, -12, -5,
R_H_LINE_TO, 358,
R_V_LINE_TO, 94,
H_LINE_TO, -12,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
MOVE_TO, 223.44f, 14.8f,
R_CUBIC_TO, 0, -13.05f, -3.67f, -23.31f, -10.9f, -30.5f,
R_CUBIC_TO, -11.35f, -11.3f, -27.96f, -11.35f, -28.12f, -11.35f,
R_LINE_TO, -0.05f, -6.38f,
R_CUBIC_TO, 0.78f, 0, 19.34f, 0, 32.62f, 13.16f,
R_CUBIC_TO, 8.5f, 8.43f, 12.82f, 20.23f, 12.82f, 35.07f,
R_H_LINE_TO, -6.37f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
STROKE, 0.8,
MOVE_TO, 240.5f, 14.36f,
R_H_LINE_TO, 2.98f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, 8, 8,
V_LINE_TO, 80,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE5, 0x44, 0x40,
STROKE, 0.8,
MOVE_TO, 251.5f, 86.25f,
R_V_LINE_TO, 4.8f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, -8, 8,
R_H_LINE_TO, -6.18f,
R_MOVE_TO, 14.18f, -12.8f,
R_V_LINE_TO, 4.8f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, -8, 8,
R_H_LINE_TO, -6.18f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
STROKE, 0.8,
MOVE_TO, 229.56f, 14.36f,
R_H_LINE_TO, -17.82f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, -7.98f, 8,
R_V_LINE_TO, 68.7f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, 7.98f, 7.98f,
R_H_LINE_TO, 15.6f,
R_MOVE_TO, 2.22f, -84.68f,
R_H_LINE_TO, -17.82f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, -7.98f, 8,
R_V_LINE_TO, 68.7f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, 7.98f, 7.98f,
R_H_LINE_TO, 15.6f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
MOVE_TO, 101.18f, 64.25f,
R_LINE_TO, -4.5f, -4.5f,
R_CUBIC_TO, 0.3f, -0.33f, 7.9f, -7.81f, 19.76f, -11.17f,
R_CUBIC_TO, 11.04f, -3.13f, 27.9f, -3.14f, 45.6f, 14.54f,
R_LINE_TO, -4.52f, 4.5f,
R_CUBIC_TO, -12.44f, -12.43f, -25.68f, -16.78f, -39.34f, -12.91f,
R_ARC_TO, 42.99f, 42.99f, 0, 0, 0, -17, 9.53f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xF4, 0xF4, 0xF4,
MOVE_TO, 123.86f, 27.94f,
R_H_LINE_TO, -85.8f,
R_ARC_TO, 4.5f, 4.5f, 0, 0, 0, -4.52f, 4.43f,
R_V_LINE_TO, 61.97f,
R_H_LINE_TO, 94.83f,
V_LINE_TO, 32.37f,
R_ARC_TO, 4.48f, 4.48f, 0, 0, 0, -4.5f, -4.43f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
STROKE, 0.8,
MOVE_TO, 123.86f, 27.94f,
R_H_LINE_TO, -85.8f,
R_ARC_TO, 4.5f, 4.5f, 0, 0, 0, -4.52f, 4.43f,
R_V_LINE_TO, 61.97f,
R_H_LINE_TO, 94.83f,
V_LINE_TO, 32.37f,
R_ARC_TO, 4.48f, 4.48f, 0, 0, 0, -4.5f, -4.43f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
STROKE, 0.8,
MOVE_TO, 14.72f, 99.04f,
R_H_LINE_TO, 131.95f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 214.44f, 20.75f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, 4.28f,
R_V_LINE_TO, 63.34f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, 4.27f,
R_H_LINE_TO, 26.57f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, -4.28f,
V_LINE_TO, 25.03f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, -4.28f,
R_H_LINE_TO, -26.57f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
STROKE, 0.8,
MOVE_TO, 214.44f, 20.75f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, 4.28f,
R_V_LINE_TO, 63.34f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, 4.27f,
R_H_LINE_TO, 26.57f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, -4.28f,
V_LINE_TO, 25.03f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, -4.28f,
R_H_LINE_TO, -26.57f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
MOVE_TO, 161.91f, 62.98f,
R_CUBIC_TO, 9.24f, 9.23f, 19.1f, 13.9f, 29.29f, 13.87f,
R_CUBIC_TO, 16.02f, -0.04f, 27.8f, -11.74f, 27.91f, -11.86f,
R_LINE_TO, 4.57f, 4.47f,
R_CUBIC_TO, -0.55f, 0.56f, -13.66f, 13.68f, -32.36f, 13.76f,
R_CUBIC_TO, -12, 0.05f, -23.4f, -5.24f, -33.9f, -15.74f,
R_LINE_TO, 4.5f, -4.5f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x06,
MOVE_TO, 262.1f, 8.96f,
R_CUBIC_TO, -0.27f, 1.02f, -0.61f, 1.73f, -1.03f, 2.15f,
R_CUBIC_TO, -0.48f, 0.48f, -1.44f, 0.94f, -2.88f, 1.39f,
R_CUBIC_TO, 1.21f, 0.31f, 2.1f, 0.74f, 2.66f, 1.31f,
R_CUBIC_TO, 0.56f, 0.56f, 1, 1.45f, 1.31f, 2.66f,
R_CUBIC_TO, 0.45f, -1.44f, 0.9f, -2.4f, 1.39f, -2.88f,
R_CUBIC_TO, 0.42f, -0.42f, 1.14f, -0.76f, 2.15f, -1.03f,
R_CUBIC_TO, -1.06f, -0.3f, -1.85f, -0.71f, -2.37f, -1.23f,
R_CUBIC_TO, -0.52f, -0.52f, -0.92f, -1.31f, -1.23f, -2.37f,
CLOSE,
MOVE_TO, 132.61f, 17.72f,
R_CUBIC_TO, -0.37f, 1.62f, -0.86f, 2.74f, -1.48f, 3.35f,
R_CUBIC_TO, -0.68f, 0.68f, -2.11f, 1.33f, -4.3f, 1.98f,
R_CUBIC_TO, 1.86f, 0.42f, 3.2f, 1.03f, 4.01f, 1.84f,
R_CUBIC_TO, 0.81f, 0.81f, 1.42f, 2.15f, 1.84f, 4.01f,
R_CUBIC_TO, 0.64f, -2.19f, 1.29f, -3.62f, 1.98f, -4.3f,
R_CUBIC_TO, 0.62f, -0.62f, 1.73f, -1.1f, 3.35f, -1.48f,
R_CUBIC_TO, -1.67f, -0.41f, -2.89f, -0.99f, -3.65f, -1.75f,
R_CUBIC_TO, -0.76f, -0.76f, -1.34f, -1.97f, -1.75f, -3.65f,
CLOSE,
MOVE_TO, 144.45f, 12.85f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 274.52f, 4.46f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 270.89f, 45.28f,
R_CUBIC_TO, -0.14f, 0.39f, -0.31f, 0.68f, -0.51f, 0.88f,
R_CUBIC_TO, -0.25f, 0.25f, -0.68f, 0.49f, -1.31f, 0.71f,
R_CUBIC_TO, 0.51f, 0.18f, 0.9f, 0.4f, 1.18f, 0.68f,
R_CUBIC_TO, 0.28f, 0.28f, 0.5f, 0.67f, 0.68f, 1.18f,
R_CUBIC_TO, 0.23f, -0.62f, 0.46f, -1.06f, 0.71f, -1.31f,
R_CUBIC_TO, 0.2f, -0.2f, 0.5f, -0.37f, 0.88f, -0.51f,
R_CUBIC_TO, -0.43f, -0.17f, -0.77f, -0.37f, -1.01f, -0.62f,
R_CUBIC_TO, -0.25f, -0.25f, -0.45f, -0.58f, -0.62f, -1.01f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x06,
STROKE, 0.8,
MOVE_TO, 262.1f, 8.96f,
R_CUBIC_TO, -0.27f, 1.02f, -0.61f, 1.73f, -1.03f, 2.15f,
R_CUBIC_TO, -0.48f, 0.48f, -1.44f, 0.94f, -2.88f, 1.39f,
R_CUBIC_TO, 1.21f, 0.31f, 2.1f, 0.74f, 2.66f, 1.31f,
R_CUBIC_TO, 0.56f, 0.56f, 1, 1.45f, 1.31f, 2.66f,
R_CUBIC_TO, 0.45f, -1.44f, 0.9f, -2.4f, 1.39f, -2.88f,
R_CUBIC_TO, 0.42f, -0.42f, 1.14f, -0.76f, 2.15f, -1.03f,
R_CUBIC_TO, -1.06f, -0.3f, -1.85f, -0.71f, -2.37f, -1.23f,
R_CUBIC_TO, -0.52f, -0.52f, -0.92f, -1.31f, -1.23f, -2.37f,
CLOSE,
MOVE_TO, 132.61f, 17.72f,
R_CUBIC_TO, -0.37f, 1.62f, -0.86f, 2.74f, -1.48f, 3.35f,
R_CUBIC_TO, -0.68f, 0.68f, -2.11f, 1.33f, -4.3f, 1.98f,
R_CUBIC_TO, 1.86f, 0.42f, 3.2f, 1.03f, 4.01f, 1.84f,
R_CUBIC_TO, 0.81f, 0.81f, 1.42f, 2.15f, 1.84f, 4.01f,
R_CUBIC_TO, 0.64f, -2.19f, 1.29f, -3.62f, 1.98f, -4.3f,
R_CUBIC_TO, 0.62f, -0.62f, 1.73f, -1.1f, 3.35f, -1.48f,
R_CUBIC_TO, -1.67f, -0.41f, -2.89f, -0.99f, -3.65f, -1.75f,
R_CUBIC_TO, -0.76f, -0.76f, -1.34f, -1.97f, -1.75f, -3.65f,
CLOSE,
MOVE_TO, 144.45f, 12.85f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 274.52f, 4.46f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 270.89f, 45.28f,
R_CUBIC_TO, -0.14f, 0.39f, -0.31f, 0.68f, -0.51f, 0.88f,
R_CUBIC_TO, -0.25f, 0.25f, -0.68f, 0.49f, -1.31f, 0.71f,
R_CUBIC_TO, 0.51f, 0.18f, 0.9f, 0.4f, 1.18f, 0.68f,
R_CUBIC_TO, 0.28f, 0.28f, 0.5f, 0.67f, 0.68f, 1.18f,
R_CUBIC_TO, 0.23f, -0.62f, 0.46f, -1.06f, 0.71f, -1.31f,
R_CUBIC_TO, 0.2f, -0.2f, 0.5f, -0.37f, 0.88f, -0.51f,
R_CUBIC_TO, -0.43f, -0.17f, -0.77f, -0.37f, -1.01f, -0.62f,
R_CUBIC_TO, -0.25f, -0.25f, -0.45f, -0.58f, -0.62f, -1.01f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xF8, 0xF9, 0xFA,
MOVE_TO, 302.43f, 39.28f,
R_H_LINE_TO, -34.75f,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.36f,
R_LINE_TO, 0.38f, 0.13f,
R_LINE_TO, 0.25f, -0.3f,
R_ARC_TO, 12.86f, 12.86f, 0, 0, 1, 10.03f, -4.8f,
R_CUBIC_TO, 6.93f, 0, 12.6f, 5.48f, 12.9f, 12.33f,
CLOSE,
R_MOVE_TO, 37.32f, 42.06f,
H_LINE_TO, 305,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.37f,
R_LINE_TO, 0.37f, 0.14f,
R_LINE_TO, 0.25f, -0.3f,
ARC_TO, 12.86f, 12.86f, 0, 0, 1, 326.86f, 69,
R_ARC_TO, 12.9f, 12.9f, 0, 0, 1, 12.89f, 12.34f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
STROKE, 0.8,
MOVE_TO, 302.43f, 39.28f,
R_H_LINE_TO, -34.75f,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.36f,
R_LINE_TO, 0.38f, 0.13f,
R_LINE_TO, 0.25f, -0.3f,
R_ARC_TO, 12.86f, 12.86f, 0, 0, 1, 10.03f, -4.8f,
R_CUBIC_TO, 6.93f, 0, 12.6f, 5.48f, 12.9f, 12.33f,
CLOSE,
R_MOVE_TO, 37.32f, 42.06f,
H_LINE_TO, 305,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.37f,
R_LINE_TO, 0.37f, 0.14f,
R_LINE_TO, 0.25f, -0.3f,
ARC_TO, 12.86f, 12.86f, 0, 0, 1, 326.86f, 69,
R_ARC_TO, 12.9f, 12.9f, 0, 0, 1, 12.89f, 12.34f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC0, 0xC5,
MOVE_TO, 89.97f, 67.82f,
R_V_LINE_TO, 4.15f,
R_ARC_TO, 1, 1, 0, 0, 1, -1.03f, 1,
R_ARC_TO, 17.92f, 17.92f, 0, 0, 1, -8, -2.43f,
R_ARC_TO, 18.08f, 18.08f, 0, 0, 1, -6.5f, -6.5f,
ARC_TO, 17.92f, 17.92f, 0, 0, 1, 72, 56.03f,
ARC_TO, 1, 1, 0, 0, 1, 73, 55,
R_H_LINE_TO, 4.15f,
R_ARC_TO, 1, 1, 0, 0, 1, 0.98f, 0.8f,
R_LINE_TO, 0.65f, 3.26f,
R_ARC_TO, 0.99f, 0.99f, 0, 0, 1, -0.27f, 0.9f,
R_LINE_TO, -2.62f, 2.62f,
R_CUBIC_TO, 1.5f, 2.75f, 3.75f, 5, 6.5f, 6.5f,
R_LINE_TO, 2.6f, -2.62f,
R_ARC_TO, 0.98f, 0.98f, 0, 0, 1, 0.91f, -0.27f,
R_LINE_TO, 3.26f, 0.65f,
R_CUBIC_TO, 0.47f, 0.1f, 0.8f, 0.5f, 0.8f, 0.98f,
CLOSE,
MOVE_TO, 83, 58,
R_H_LINE_TO, 2.6f,
R_LINE_TO, -4.32f, 4.31f,
R_LINE_TO, 1.41f, 1.41f,
LINE_TO, 87, 59.41f,
V_LINE_TO, 62,
R_H_LINE_TO, 2,
R_V_LINE_TO, -6,
R_H_LINE_TO, -6,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kClickToCallIllustrationRepList, kClickToCallIllustrationIcon, {kClickToCallIllustrationPath, std::size(kClickToCallIllustrationPath)})
VECTOR_ICON_REP_TEMPLATE(kClickToCallIllustrationDarkPath, CANVAS_DIMENSIONS, 320,
PATH_COLOR_ARGB, 0xFF, 0x20, 0x21, 0x24,
MOVE_TO, -12, -5,
R_H_LINE_TO, 358,
R_V_LINE_TO, 94,
H_LINE_TO, -12,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 223.44f, 14.8f,
R_CUBIC_TO, 0, -13.05f, -3.67f, -23.31f, -10.9f, -30.5f,
R_CUBIC_TO, -11.35f, -11.3f, -27.96f, -11.35f, -28.12f, -11.35f,
R_LINE_TO, -0.05f, -6.38f,
R_CUBIC_TO, 0.78f, 0, 19.34f, 0, 32.62f, 13.16f,
R_CUBIC_TO, 8.5f, 8.43f, 12.82f, 20.23f, 12.82f, 35.07f,
R_H_LINE_TO, -6.37f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 240.5f, 14.36f,
R_H_LINE_TO, 2.98f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, 8, 8,
V_LINE_TO, 80,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xF6, 0xAE, 0xA9,
STROKE, 0.8,
MOVE_TO, 251.5f, 86.25f,
R_V_LINE_TO, 4.8f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, -8, 8,
R_H_LINE_TO, -6.18f,
R_MOVE_TO, 14.18f, -12.8f,
R_V_LINE_TO, 4.8f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 1, -8, 8,
R_H_LINE_TO, -6.18f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 229.56f, 14.36f,
R_H_LINE_TO, -17.82f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, -7.98f, 8,
R_V_LINE_TO, 68.7f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, 7.98f, 7.98f,
R_H_LINE_TO, 15.6f,
R_MOVE_TO, 2.22f, -84.68f,
R_H_LINE_TO, -17.82f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, -7.98f, 8,
R_V_LINE_TO, 68.7f,
R_ARC_TO, 7.99f, 7.99f, 0, 0, 0, 7.98f, 7.98f,
R_H_LINE_TO, 15.6f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 101.18f, 64.25f,
R_LINE_TO, -4.5f, -4.5f,
R_CUBIC_TO, 0.3f, -0.33f, 7.9f, -7.81f, 19.76f, -11.17f,
R_CUBIC_TO, 11.04f, -3.13f, 27.9f, -3.14f, 45.6f, 14.54f,
R_LINE_TO, -4.52f, 4.5f,
R_CUBIC_TO, -12.44f, -12.43f, -25.68f, -16.78f, -39.34f, -12.91f,
R_ARC_TO, 42.99f, 42.99f, 0, 0, 0, -17, 9.53f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x20, 0x21, 0x24,
MOVE_TO, 123.86f, 27.94f,
R_H_LINE_TO, -85.8f,
R_ARC_TO, 4.5f, 4.5f, 0, 0, 0, -4.52f, 4.43f,
R_V_LINE_TO, 61.97f,
R_H_LINE_TO, 94.83f,
V_LINE_TO, 32.37f,
R_ARC_TO, 4.48f, 4.48f, 0, 0, 0, -4.5f, -4.43f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 123.86f, 27.94f,
R_H_LINE_TO, -85.8f,
R_ARC_TO, 4.5f, 4.5f, 0, 0, 0, -4.52f, 4.43f,
R_V_LINE_TO, 61.97f,
R_H_LINE_TO, 94.83f,
V_LINE_TO, 32.37f,
R_ARC_TO, 4.48f, 4.48f, 0, 0, 0, -4.5f, -4.43f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 14.72f, 99.04f,
R_H_LINE_TO, 131.95f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x20, 0x21, 0x24,
MOVE_TO, 214.44f, 20.75f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, 4.28f,
R_V_LINE_TO, 63.34f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, 4.27f,
R_H_LINE_TO, 26.57f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, -4.28f,
V_LINE_TO, 25.03f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, -4.28f,
R_H_LINE_TO, -26.57f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 214.44f, 20.75f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, 4.28f,
R_V_LINE_TO, 63.34f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, 4.27f,
R_H_LINE_TO, 26.57f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, 4.3f, -4.28f,
V_LINE_TO, 25.03f,
R_ARC_TO, 4.3f, 4.3f, 0, 0, 0, -4.3f, -4.28f,
R_H_LINE_TO, -26.57f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x8A, 0xB4, 0xF8,
MOVE_TO, 161.91f, 62.98f,
R_CUBIC_TO, 9.24f, 9.23f, 19.1f, 13.9f, 29.29f, 13.87f,
R_CUBIC_TO, 16.02f, -0.04f, 27.8f, -11.74f, 27.91f, -11.86f,
R_LINE_TO, 4.57f, 4.47f,
R_CUBIC_TO, -0.55f, 0.56f, -13.66f, 13.68f, -32.36f, 13.76f,
R_CUBIC_TO, -12, 0.05f, -23.4f, -5.24f, -33.9f, -15.74f,
R_LINE_TO, 4.5f, -4.5f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFD, 0xD6, 0x63,
MOVE_TO, 262.1f, 8.96f,
R_CUBIC_TO, -0.27f, 1.02f, -0.61f, 1.73f, -1.03f, 2.15f,
R_CUBIC_TO, -0.48f, 0.48f, -1.44f, 0.94f, -2.88f, 1.39f,
R_CUBIC_TO, 1.21f, 0.31f, 2.1f, 0.74f, 2.66f, 1.31f,
R_CUBIC_TO, 0.56f, 0.56f, 1, 1.45f, 1.31f, 2.66f,
R_CUBIC_TO, 0.45f, -1.44f, 0.9f, -2.4f, 1.39f, -2.88f,
R_CUBIC_TO, 0.42f, -0.42f, 1.14f, -0.76f, 2.15f, -1.03f,
R_CUBIC_TO, -1.06f, -0.3f, -1.85f, -0.71f, -2.37f, -1.23f,
R_CUBIC_TO, -0.52f, -0.52f, -0.92f, -1.31f, -1.23f, -2.37f,
CLOSE,
MOVE_TO, 132.61f, 17.72f,
R_CUBIC_TO, -0.37f, 1.62f, -0.86f, 2.74f, -1.48f, 3.35f,
R_CUBIC_TO, -0.68f, 0.68f, -2.11f, 1.33f, -4.3f, 1.98f,
R_CUBIC_TO, 1.86f, 0.42f, 3.2f, 1.03f, 4.01f, 1.84f,
R_CUBIC_TO, 0.81f, 0.81f, 1.42f, 2.15f, 1.84f, 4.01f,
R_CUBIC_TO, 0.64f, -2.19f, 1.29f, -3.62f, 1.98f, -4.3f,
R_CUBIC_TO, 0.62f, -0.62f, 1.73f, -1.1f, 3.35f, -1.48f,
R_CUBIC_TO, -1.67f, -0.41f, -2.89f, -0.99f, -3.65f, -1.75f,
R_CUBIC_TO, -0.76f, -0.76f, -1.34f, -1.97f, -1.75f, -3.65f,
CLOSE,
MOVE_TO, 144.45f, 12.85f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 274.52f, 4.46f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 270.89f, 45.28f,
R_CUBIC_TO, -0.14f, 0.39f, -0.31f, 0.68f, -0.51f, 0.88f,
R_CUBIC_TO, -0.25f, 0.25f, -0.68f, 0.49f, -1.31f, 0.71f,
R_CUBIC_TO, 0.51f, 0.18f, 0.9f, 0.4f, 1.18f, 0.68f,
R_CUBIC_TO, 0.28f, 0.28f, 0.5f, 0.67f, 0.68f, 1.18f,
R_CUBIC_TO, 0.23f, -0.62f, 0.46f, -1.06f, 0.71f, -1.31f,
R_CUBIC_TO, 0.2f, -0.2f, 0.5f, -0.37f, 0.88f, -0.51f,
R_CUBIC_TO, -0.43f, -0.17f, -0.77f, -0.37f, -1.01f, -0.62f,
R_CUBIC_TO, -0.25f, -0.25f, -0.45f, -0.58f, -0.62f, -1.01f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFD, 0xD6, 0x63,
STROKE, 0.8,
MOVE_TO, 262.1f, 8.96f,
R_CUBIC_TO, -0.27f, 1.02f, -0.61f, 1.73f, -1.03f, 2.15f,
R_CUBIC_TO, -0.48f, 0.48f, -1.44f, 0.94f, -2.88f, 1.39f,
R_CUBIC_TO, 1.21f, 0.31f, 2.1f, 0.74f, 2.66f, 1.31f,
R_CUBIC_TO, 0.56f, 0.56f, 1, 1.45f, 1.31f, 2.66f,
R_CUBIC_TO, 0.45f, -1.44f, 0.9f, -2.4f, 1.39f, -2.88f,
R_CUBIC_TO, 0.42f, -0.42f, 1.14f, -0.76f, 2.15f, -1.03f,
R_CUBIC_TO, -1.06f, -0.3f, -1.85f, -0.71f, -2.37f, -1.23f,
R_CUBIC_TO, -0.52f, -0.52f, -0.92f, -1.31f, -1.23f, -2.37f,
CLOSE,
MOVE_TO, 132.61f, 17.72f,
R_CUBIC_TO, -0.37f, 1.62f, -0.86f, 2.74f, -1.48f, 3.35f,
R_CUBIC_TO, -0.68f, 0.68f, -2.11f, 1.33f, -4.3f, 1.98f,
R_CUBIC_TO, 1.86f, 0.42f, 3.2f, 1.03f, 4.01f, 1.84f,
R_CUBIC_TO, 0.81f, 0.81f, 1.42f, 2.15f, 1.84f, 4.01f,
R_CUBIC_TO, 0.64f, -2.19f, 1.29f, -3.62f, 1.98f, -4.3f,
R_CUBIC_TO, 0.62f, -0.62f, 1.73f, -1.1f, 3.35f, -1.48f,
R_CUBIC_TO, -1.67f, -0.41f, -2.89f, -0.99f, -3.65f, -1.75f,
R_CUBIC_TO, -0.76f, -0.76f, -1.34f, -1.97f, -1.75f, -3.65f,
CLOSE,
MOVE_TO, 144.45f, 12.85f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 274.52f, 4.46f,
R_CUBIC_TO, -0.18f, 0.55f, -0.39f, 0.95f, -0.65f, 1.21f,
R_CUBIC_TO, -0.31f, 0.32f, -0.88f, 0.61f, -1.72f, 0.89f,
R_CUBIC_TO, 0.69f, 0.22f, 1.21f, 0.5f, 1.57f, 0.85f,
R_CUBIC_TO, 0.36f, 0.36f, 0.64f, 0.88f, 0.85f, 1.57f,
R_CUBIC_TO, 0.28f, -0.84f, 0.58f, -1.41f, 0.89f, -1.72f,
R_CUBIC_TO, 0.26f, -0.26f, 0.66f, -0.48f, 1.21f, -0.65f,
R_CUBIC_TO, -0.59f, -0.2f, -1.04f, -0.46f, -1.36f, -0.78f,
R_CUBIC_TO, -0.32f, -0.32f, -0.58f, -0.77f, -0.78f, -1.36f,
CLOSE,
MOVE_TO, 270.89f, 45.28f,
R_CUBIC_TO, -0.14f, 0.39f, -0.31f, 0.68f, -0.51f, 0.88f,
R_CUBIC_TO, -0.25f, 0.25f, -0.68f, 0.49f, -1.31f, 0.71f,
R_CUBIC_TO, 0.51f, 0.18f, 0.9f, 0.4f, 1.18f, 0.68f,
R_CUBIC_TO, 0.28f, 0.28f, 0.5f, 0.67f, 0.68f, 1.18f,
R_CUBIC_TO, 0.23f, -0.62f, 0.46f, -1.06f, 0.71f, -1.31f,
R_CUBIC_TO, 0.2f, -0.2f, 0.5f, -0.37f, 0.88f, -0.51f,
R_CUBIC_TO, -0.43f, -0.17f, -0.77f, -0.37f, -1.01f, -0.62f,
R_CUBIC_TO, -0.25f, -0.25f, -0.45f, -0.58f, -0.62f, -1.01f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x29, 0x2A, 0x2D,
MOVE_TO, 302.43f, 39.28f,
R_H_LINE_TO, -34.75f,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.36f,
R_LINE_TO, 0.38f, 0.13f,
R_LINE_TO, 0.25f, -0.3f,
R_ARC_TO, 12.86f, 12.86f, 0, 0, 1, 10.03f, -4.8f,
R_CUBIC_TO, 6.93f, 0, 12.6f, 5.48f, 12.9f, 12.33f,
CLOSE,
R_MOVE_TO, 37.32f, 42.06f,
H_LINE_TO, 305,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.37f,
R_LINE_TO, 0.37f, 0.14f,
R_LINE_TO, 0.25f, -0.3f,
ARC_TO, 12.86f, 12.86f, 0, 0, 1, 326.86f, 69,
R_ARC_TO, 12.9f, 12.9f, 0, 0, 1, 12.89f, 12.34f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
STROKE, 0.8,
MOVE_TO, 302.43f, 39.28f,
R_H_LINE_TO, -34.75f,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.36f,
R_LINE_TO, 0.38f, 0.13f,
R_LINE_TO, 0.25f, -0.3f,
R_ARC_TO, 12.86f, 12.86f, 0, 0, 1, 10.03f, -4.8f,
R_CUBIC_TO, 6.93f, 0, 12.6f, 5.48f, 12.9f, 12.33f,
CLOSE,
R_MOVE_TO, 37.32f, 42.06f,
H_LINE_TO, 305,
R_ARC_TO, 8.42f, 8.42f, 0, 0, 1, 11.2f, -7.37f,
R_LINE_TO, 0.37f, 0.14f,
R_LINE_TO, 0.25f, -0.3f,
ARC_TO, 12.86f, 12.86f, 0, 0, 1, 326.86f, 69,
R_ARC_TO, 12.9f, 12.9f, 0, 0, 1, 12.89f, 12.34f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 89.97f, 67.82f,
R_V_LINE_TO, 4.15f,
R_ARC_TO, 1, 1, 0, 0, 1, -1.03f, 1,
R_ARC_TO, 17.92f, 17.92f, 0, 0, 1, -8, -2.43f,
R_ARC_TO, 18.08f, 18.08f, 0, 0, 1, -6.5f, -6.5f,
ARC_TO, 17.92f, 17.92f, 0, 0, 1, 72, 56.03f,
ARC_TO, 1, 1, 0, 0, 1, 73, 55,
R_H_LINE_TO, 4.15f,
R_ARC_TO, 1, 1, 0, 0, 1, 0.98f, 0.8f,
R_LINE_TO, 0.65f, 3.26f,
R_ARC_TO, 0.99f, 0.99f, 0, 0, 1, -0.27f, 0.9f,
R_LINE_TO, -2.62f, 2.62f,
R_CUBIC_TO, 1.5f, 2.75f, 3.75f, 5, 6.5f, 6.5f,
R_LINE_TO, 2.6f, -2.62f,
R_ARC_TO, 0.98f, 0.98f, 0, 0, 1, 0.91f, -0.27f,
R_LINE_TO, 3.26f, 0.65f,
R_CUBIC_TO, 0.47f, 0.1f, 0.8f, 0.5f, 0.8f, 0.98f,
CLOSE,
MOVE_TO, 83, 58,
R_H_LINE_TO, 2.6f,
R_LINE_TO, -4.32f, 4.31f,
R_LINE_TO, 1.41f, 1.41f,
LINE_TO, 87, 59.41f,
V_LINE_TO, 62,
R_H_LINE_TO, 2,
R_V_LINE_TO, -6,
R_H_LINE_TO, -6,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kClickToCallIllustrationDarkRepList, kClickToCallIllustrationDarkIcon, {kClickToCallIllustrationDarkPath, std::size(kClickToCallIllustrationDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kCloseChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 6.06f, 15.16f,
LINE_TO, 4.84f, 13.94f,
LINE_TO, 8.78f, 10,
LINE_TO, 4.84f, 6.06f,
LINE_TO, 6.06f, 4.84f,
LINE_TO, 10, 8.78f,
R_LINE_TO, 3.94f, -3.94f,
R_LINE_TO, 1.22f, 1.22f,
LINE_TO, 11.22f, 10,
R_LINE_TO, 3.94f, 3.94f,
R_LINE_TO, -1.22f, 1.22f,
LINE_TO, 10, 11.22f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCloseChromeRefreshRepList, kCloseChromeRefreshIcon, {kCloseChromeRefreshPath, std::size(kCloseChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kCloseGroupRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 9.56f, 11.5f,
LINE_TO, 11.5f, 9.56f,
R_LINE_TO, 1.94f, 1.94f,
R_LINE_TO, 1.06f, -1.06f,
LINE_TO, 12.56f, 8.5f,
LINE_TO, 14.5f, 6.56f,
LINE_TO, 13.44f, 5.5f,
LINE_TO, 11.5f, 7.44f,
LINE_TO, 9.56f, 5.5f,
LINE_TO, 8.5f, 6.56f,
LINE_TO, 10.44f, 8.5f,
LINE_TO, 8.5f, 10.44f,
CLOSE,
MOVE_TO, 6.5f, 15,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 5, 13.5f,
R_V_LINE_TO, -10,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.77f, 0.44f, -1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 6.5f, 2,
R_H_LINE_TO, 10,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.44f,
R_CUBIC_TO, 0.29f, 0.29f, 0.44f, 0.65f, 0.44f, 1.06f,
R_V_LINE_TO, 10,
R_CUBIC_TO, 0, 0.41f, -0.15f, 0.77f, -0.44f, 1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 16.5f, 15,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_H_LINE_TO, 10,
R_V_LINE_TO, -10,
R_H_LINE_TO, -10,
CLOSE,
R_MOVE_TO, -3, 4.5f,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 2, 16.5f,
V_LINE_TO, 5,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 11.5f,
H_LINE_TO, 15,
V_LINE_TO, 18,
CLOSE,
R_MOVE_TO, 3, -14.5f,
R_V_LINE_TO, 10,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kCloseGroupRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 7.74f, 9.24f,
R_LINE_TO, 1.55f, -1.55f,
R_LINE_TO, 1.55f, 1.55f,
R_LINE_TO, 0.98f, -0.98f,
R_LINE_TO, -1.55f, -1.55f,
R_LINE_TO, 1.55f, -1.55f,
R_LINE_TO, -0.98f, -0.98f,
R_LINE_TO, -1.55f, 1.55f,
R_LINE_TO, -1.55f, -1.55f,
R_LINE_TO, -0.98f, 0.98f,
R_LINE_TO, 1.55f, 1.55f,
R_LINE_TO, -1.55f, 1.55f,
CLOSE,
MOVE_TO, 5.34f, 12.04f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 2.76f,
R_CUBIC_TO, 0, -0.39f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 7.9f,
R_CUBIC_TO, 0.39f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.59f, 0.41f, 0.98f,
R_V_LINE_TO, 7.9f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.71f, -0.41f, 0.98f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_H_LINE_TO, 7.9f,
V_LINE_TO, 2.76f,
H_LINE_TO, 5.34f,
CLOSE,
MOVE_TO, 2.76f, 14.63f,
R_CUBIC_TO, -0.39f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 3.96f,
R_H_LINE_TO, 1.39f,
R_V_LINE_TO, 9.29f,
R_H_LINE_TO, 9.29f,
R_V_LINE_TO, 1.39f,
CLOSE,
MOVE_TO, 5.34f, 2.76f,
R_V_LINE_TO, 7.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCloseGroupRefreshRepList, kCloseGroupRefreshIcon, {kCloseGroupRefreshPath, std::size(kCloseGroupRefreshPath)}, {kCloseGroupRefresh16Path, std::size(kCloseGroupRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kCloseTabChromeRefreshPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 10.94f, 4,
LINE_TO, 4, 10.94f,
LINE_TO, 5.06f, 12,
LINE_TO, 12, 5.07f,
LINE_TO, 10.94f, 4,
CLOSE,
NEW_PATH,
MOVE_TO, 5.06f, 4,
LINE_TO, 4, 5.06f,
LINE_TO, 10.94f, 12,
LINE_TO, 12, 10.93f,
LINE_TO, 5.06f, 4,
CLOSE,
NEW_PATH)
VECTOR_ICON_TEMPLATE_CC(kCloseTabChromeRefreshRepList, kCloseTabChromeRefreshIcon, {kCloseTabChromeRefreshPath, std::size(kCloseTabChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kComputerWithCircleBackgroundPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17.6f, 16.2f,
R_CUBIC_TO, 0.77f, 0, 1.39f, -0.63f, 1.39f, -1.4f,
R_LINE_TO, 0.01f, -7,
R_CUBIC_TO, 0, -0.77f, -0.63f, -1.4f, -1.4f, -1.4f,
H_LINE_TO, 6.4f,
R_CUBIC_TO, -0.77f, 0, -1.4f, 0.63f, -1.4f, 1.4f,
R_V_LINE_TO, 7,
R_CUBIC_TO, 0, 0.77f, 0.63f, 1.4f, 1.4f, 1.4f,
R_H_LINE_TO, 11.2f,
CLOSE,
R_MOVE_TO, -11.2f, -8.4f,
R_H_LINE_TO, 11.2f,
R_V_LINE_TO, 7,
H_LINE_TO, 6.4f,
R_V_LINE_TO, -7,
CLOSE,
R_MOVE_TO, -2.1f, 9.1f,
R_H_LINE_TO, 15.4f,
R_V_LINE_TO, 1.4f,
H_LINE_TO, 4.3f,
R_V_LINE_TO, -1.4f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 50, 0x9A, 0xA0, 0xA6,
ROUND_RECT, 0, 0, 24, 24, 12,)
VECTOR_ICON_TEMPLATE_CC(kComputerWithCircleBackgroundRepList, kComputerWithCircleBackgroundIcon, {kComputerWithCircleBackgroundPath, std::size(kComputerWithCircleBackgroundPath)})
VECTOR_ICON_REP_TEMPLATE(kCopyPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 16, 1,
H_LINE_TO, 4,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 14,
R_H_LINE_TO, 2,
V_LINE_TO, 3,
R_H_LINE_TO, 12,
V_LINE_TO, 1,
CLOSE,
R_MOVE_TO, 3, 4,
H_LINE_TO, 8,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 14,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 11,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 7,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
CLOSE,
R_MOVE_TO, 0, 16,
H_LINE_TO, 8,
V_LINE_TO, 7,
R_H_LINE_TO, 11,
R_V_LINE_TO, 14,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCopyRepList, kCopyIcon, {kCopyPath, std::size(kCopyPath)})
VECTOR_ICON_REP_TEMPLATE(kCopyMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 6.14f, 12.04f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 2.76f,
R_CUBIC_TO, 0, -0.39f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 6.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.59f, 0.41f, 0.98f,
R_V_LINE_TO, 7.9f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.71f, -0.41f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.59f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_H_LINE_TO, 6.31f,
V_LINE_TO, 2.76f,
H_LINE_TO, 6.14f,
CLOSE,
MOVE_TO, 3.56f, 14.63f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 3.96f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 9.29f,
R_H_LINE_TO, 7.69f,
R_V_LINE_TO, 1.39f,
CLOSE,
MOVE_TO, 6.14f, 10.66f,
V_LINE_TO, 2.76f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCopyMenuRepList, kCopyMenuIcon, {kCopyMenuPath, std::size(kCopyMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kCrashedTabPath, STROKE, 3,
CAP_SQUARE,
MOVE_TO, 1.5, 8.5,
R_V_LINE_TO, 34,
R_H_LINE_TO, 45,
R_V_LINE_TO, -28,
R_MOVE_TO, -3, -3,
R_H_LINE_TO, -10,
R_V_LINE_TO, -3,
R_MOVE_TO, -3, -3,
R_H_LINE_TO, -10,
R_MOVE_TO, 15, 6,
R_H_LINE_TO, -18,
R_V_LINE_TO, -3,
R_MOVE_TO, -3, -3,
R_H_LINE_TO, -10,
NEW_PATH,
STROKE, 2,
CAP_SQUARE,
MOVE_TO, 12, 35,
R_H_LINE_TO, 2,
R_MOVE_TO, 2, -2,
R_H_LINE_TO, 12,
R_MOVE_TO, 2, 2,
R_H_LINE_TO, 3,
R_MOVE_TO, 2, 2,
R_H_LINE_TO, 3,
MOVE_TO, 11, 21,
R_LINE_TO, 0, 0,
R_MOVE_TO, 0, 4,
R_LINE_TO, 0, 0,
R_MOVE_TO, 4, 0,
R_LINE_TO, 0, 0,
R_MOVE_TO, 0, -4,
R_LINE_TO, 0, 0,
R_MOVE_TO, -2, 2,
R_LINE_TO, 0, 0,
MOVE_TO, 33, 21,
R_LINE_TO, 0, 0,
R_MOVE_TO, 0, 4,
R_LINE_TO, 0, 0,
R_MOVE_TO, 4, 0,
R_LINE_TO, 0, 0,
R_MOVE_TO, 0, -4,
R_LINE_TO, 0, 0,
R_MOVE_TO, -2, 2,
R_LINE_TO, 0, 0)
VECTOR_ICON_TEMPLATE_CC(kCrashedTabRepList, kCrashedTabIcon, {kCrashedTabPath, std::size(kCrashedTabPath)})
VECTOR_ICON_REP_TEMPLATE(kCreateNewTabGroupPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10.86f, 11.89f,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, -2.75f,
R_H_LINE_TO, 2.75f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, -2.75f,
R_V_LINE_TO, -2.75f,
R_H_LINE_TO, -1.5f,
R_V_LINE_TO, 2.75f,
R_H_LINE_TO, -2.75f,
R_V_LINE_TO, 1.5f,
R_H_LINE_TO, 2.75f,
CLOSE,
R_MOVE_TO, -4.19f, 3.17f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.23f, -0.5f,
R_ARC_TO, 1.68f, 1.68f, 0, 0, 1, -0.5f, -1.23f,
V_LINE_TO, 3.45f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.5f, -1.22f,
R_CUBIC_TO, 0.34f, -0.34f, 0.75f, -0.51f, 1.23f, -0.51f,
R_H_LINE_TO, 9.88f,
R_CUBIC_TO, 0.48f, 0, 0.89f, 0.17f, 1.22f, 0.51f,
R_CUBIC_TO, 0.34f, 0.34f, 0.51f, 0.74f, 0.51f, 1.22f,
R_V_LINE_TO, 9.88f,
R_CUBIC_TO, 0, 0.48f, -0.17f, 0.89f, -0.51f, 1.23f,
R_ARC_TO, 1.66f, 1.66f, 0, 0, 1, -1.22f, 0.5f,
CLOSE,
R_MOVE_TO, 0, -1.73f,
R_H_LINE_TO, 9.88f,
R_V_LINE_TO, -9.88f,
R_H_LINE_TO, -9.88f,
CLOSE,
R_MOVE_TO, -3.23f, 4.96f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.51f,
R_ARC_TO, 1.65f, 1.65f, 0, 0, 1, -0.51f, -1.22f,
V_LINE_TO, 4.95f,
R_H_LINE_TO, 1.73f,
R_V_LINE_TO, 11.61f,
R_H_LINE_TO, 11.61f,
R_V_LINE_TO, 1.73f,
CLOSE,
R_MOVE_TO, 3.23f, -14.84f,
R_V_LINE_TO, 9.88f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCreateNewTabGroupRepList, kCreateNewTabGroupIcon, {kCreateNewTabGroupPath, std::size(kCreateNewTabGroupPath)})
VECTOR_ICON_REP_TEMPLATE(kCreditCardPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 27, 5,
H_LINE_TO, 5,
CUBIC_TO, 3.25f, 5, 2.01f, 6.22f, 2, 8,
R_V_LINE_TO, 16,
R_CUBIC_TO, 0, 1.78f, 1.25f, 3, 3, 3,
R_H_LINE_TO, 22,
R_CUBIC_TO, 1.75f, 0, 3, -1.22f, 3, -3,
V_LINE_TO, 8,
R_CUBIC_TO, 0, -1.78f, -1.25f, -3, -3, -3,
CLOSE,
R_MOVE_TO, 0, 19,
H_LINE_TO, 5,
R_V_LINE_TO, -8,
R_H_LINE_TO, 22,
R_V_LINE_TO, 8,
CLOSE,
R_MOVE_TO, 0, -13,
H_LINE_TO, 5,
V_LINE_TO, 8,
R_H_LINE_TO, 22,
R_V_LINE_TO, 3,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kCreditCard16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 13.6f, 2,
LINE_TO, 2.4f, 2,
CUBIC_TO, 1.63f, 2, 1.01f, 2.62f, 1.01f, 3.38f,
LINE_TO, 1, 12.12f,
CUBIC_TO, 1, 12.88f, 1.63f, 13.5f, 2.4f, 13.5f,
LINE_TO, 13.6f, 13.5f,
CUBIC_TO, 14.37f, 13.5f, 15, 12.88f, 15, 12.12f,
LINE_TO, 15, 3.38f,
CUBIC_TO, 15, 2.62f, 14.37f, 2, 13.6f, 2,
CLOSE,
MOVE_TO, 13.5f, 12,
LINE_TO, 2.5f, 12,
LINE_TO, 2.5f, 7.5f,
LINE_TO, 13.5f, 7.5f,
LINE_TO, 13.5f, 12,
CLOSE,
MOVE_TO, 13.5f, 5,
LINE_TO, 2.5f, 5,
LINE_TO, 2.5f, 3.5f,
LINE_TO, 13.5f, 3.5f,
LINE_TO, 13.5f, 5,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCreditCardRepList, kCreditCardIcon, {kCreditCardPath, std::size(kCreditCardPath)}, {kCreditCard16Path, std::size(kCreditCard16Path)})
VECTOR_ICON_REP_TEMPLATE(kCreditCardChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 18, 5.5f,
R_V_LINE_TO, 9,
R_CUBIC_TO, 0, 0.4f, -0.15f, 0.75f, -0.44f, 1.05f,
R_CUBIC_TO, -0.29f, 0.3f, -0.64f, 0.45f, -1.06f, 0.45f,
R_H_LINE_TO, -13,
R_CUBIC_TO, -0.4f, 0, -0.75f, -0.15f, -1.05f, -0.45f,
R_CUBIC_TO, -0.3f, -0.3f, -0.45f, -0.65f, -0.45f, -1.05f,
R_V_LINE_TO, -9,
R_CUBIC_TO, 0, -0.4f, 0.15f, -0.75f, 0.45f, -1.05f,
R_CUBIC_TO, 0.3f, -0.3f, 0.65f, -0.45f, 1.05f, -0.45f,
R_H_LINE_TO, 13,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.45f,
R_CUBIC_TO, 0.29f, 0.3f, 0.44f, 0.65f, 0.44f, 1.05f,
CLOSE,
MOVE_TO, 3.5f, 7,
R_H_LINE_TO, 13,
V_LINE_TO, 5.5f,
R_H_LINE_TO, -13,
CLOSE,
R_MOVE_TO, 0, 3,
R_V_LINE_TO, 4.5f,
R_H_LINE_TO, 13,
V_LINE_TO, 10,
CLOSE,
R_MOVE_TO, 0, 4.5f,
R_V_LINE_TO, -9,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCreditCardChromeRefreshRepList, kCreditCardChromeRefreshIcon, {kCreditCardChromeRefreshPath, std::size(kCreditCardChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kCutMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 12.36f, 13.66f,
LINE_TO, 7.98f, 9.3f,
LINE_TO, 6.58f, 10.72f,
R_CUBIC_TO, 0.08f, 0.18f, 0.14f, 0.35f, 0.18f, 0.54f,
R_CUBIC_TO, 0.04f, 0.18f, 0.06f, 0.36f, 0.06f, 0.54f,
R_CUBIC_TO, 0, 0.74f, -0.25f, 1.36f, -0.76f, 1.87f,
R_CUBIC_TO, -0.51f, 0.51f, -1.13f, 0.77f, -1.87f, 0.77f,
R_ARC_TO, 2.54f, 2.54f, 0, 0, 1, -1.86f, -0.77f,
R_ARC_TO, 2.53f, 2.53f, 0, 0, 1, -0.77f, -1.86f,
R_CUBIC_TO, 0, -0.73f, 0.26f, -1.36f, 0.77f, -1.87f,
R_CUBIC_TO, 0.51f, -0.51f, 1.13f, -0.77f, 1.87f, -0.77f,
R_CUBIC_TO, 0.18f, 0, 0.36f, 0.02f, 0.54f, 0.07f,
R_CUBIC_TO, 0.18f, 0.04f, 0.36f, 0.11f, 0.54f, 0.18f,
R_LINE_TO, 1.42f, -1.4f,
R_LINE_TO, -1.42f, -1.42f,
R_CUBIC_TO, -0.18f, 0.08f, -0.36f, 0.14f, -0.54f, 0.18f,
R_CUBIC_TO, -0.18f, 0.04f, -0.36f, 0.05f, -0.54f, 0.05f,
R_CUBIC_TO, -0.74f, 0, -1.36f, -0.25f, -1.87f, -0.77f,
ARC_TO, 2.55f, 2.55f, 0, 0, 1, 1.55f, 4.2f,
R_CUBIC_TO, 0, -0.73f, 0.26f, -1.35f, 0.77f, -1.86f,
R_ARC_TO, 2.53f, 2.53f, 0, 0, 1, 1.86f, -0.77f,
R_CUBIC_TO, 0.73f, 0, 1.36f, 0.26f, 1.87f, 0.77f,
R_ARC_TO, 2.55f, 2.55f, 0, 0, 1, 0.77f, 1.87f,
R_CUBIC_TO, 0, 0.18f, -0.02f, 0.36f, -0.06f, 0.54f,
R_ARC_TO, 2.26f, 2.26f, 0, 0, 1, -0.18f, 0.53f,
R_LINE_TO, 7.86f, 7.83f,
R_V_LINE_TO, 0.55f,
CLOSE,
MOVE_TO, 9.99f, 7.33f,
R_LINE_TO, -1.32f, -1.32f,
R_LINE_TO, 3.69f, -3.68f,
R_H_LINE_TO, 2.09f,
R_V_LINE_TO, 0.55f,
CLOSE,
R_MOVE_TO, -5.8f, -1.77f,
R_CUBIC_TO, 0.38f, 0, 0.7f, -0.13f, 0.97f, -0.4f,
R_CUBIC_TO, 0.27f, -0.27f, 0.4f, -0.59f, 0.4f, -0.97f,
R_ARC_TO, 1.3f, 1.3f, 0, 0, 0, -0.4f, -0.96f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 0, -0.97f, -0.4f,
R_ARC_TO, 1.3f, 1.3f, 0, 0, 0, -0.96f, 0.4f,
R_CUBIC_TO, -0.27f, 0.27f, -0.4f, 0.59f, -0.4f, 0.97f,
R_CUBIC_TO, 0, 0.38f, 0.13f, 0.7f, 0.4f, 0.96f,
R_CUBIC_TO, 0.27f, 0.26f, 0.59f, 0.4f, 0.97f, 0.4f,
CLOSE,
R_MOVE_TO, 3.8f, 2.93f,
R_CUBIC_TO, 0.13f, 0, 0.25f, -0.05f, 0.34f, -0.14f,
R_ARC_TO, 0.48f, 0.48f, 0, 0, 0, 0, -0.7f,
R_ARC_TO, 0.48f, 0.48f, 0, 0, 0, -0.34f, -0.14f,
R_ARC_TO, 0.49f, 0.49f, 0, 0, 0, -0.35f, 0.84f,
R_ARC_TO, 0.49f, 0.49f, 0, 0, 0, 0.35f, 0.14f,
CLOSE,
R_MOVE_TO, -3.8f, 4.68f,
R_CUBIC_TO, 0.38f, 0, 0.7f, -0.13f, 0.97f, -0.4f,
R_CUBIC_TO, 0.27f, -0.27f, 0.4f, -0.59f, 0.4f, -0.97f,
R_ARC_TO, 1.3f, 1.3f, 0, 0, 0, -0.4f, -0.96f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 0, -0.97f, -0.4f,
R_ARC_TO, 1.3f, 1.3f, 0, 0, 0, -0.96f, 0.4f,
R_CUBIC_TO, -0.27f, 0.27f, -0.4f, 0.59f, -0.4f, 0.97f,
R_CUBIC_TO, 0, 0.38f, 0.13f, 0.7f, 0.4f, 0.97f,
R_CUBIC_TO, 0.27f, 0.27f, 0.59f, 0.4f, 0.97f, 0.4f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kCutMenuRepList, kCutMenuIcon, {kCutMenuPath, std::size(kCutMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kDefaultTouchFaviconPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 0, 0,
LINE_TO, 16, 0,
LINE_TO, 16, 16,
LINE_TO, 0, 16,
CLOSE,
MOVE_TO, 5, 5,
LINE_TO, 8, 5,
LINE_TO, 11, 8,
LINE_TO, 11, 15,
LINE_TO, 5, 15,
CLOSE,
MOVE_TO, 10, 9,
LINE_TO, 7, 6,
LINE_TO, 7, 9,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDefaultTouchFaviconRepList, kDefaultTouchFaviconIcon, {kDefaultTouchFaviconPath, std::size(kDefaultTouchFaviconPath)})
VECTOR_ICON_REP_TEMPLATE(kDefaultTouchFaviconMaskPath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 6,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDefaultTouchFaviconMaskRepList, kDefaultTouchFaviconMaskIcon, {kDefaultTouchFaviconMaskPath, std::size(kDefaultTouchFaviconMaskPath)})
VECTOR_ICON_REP_TEMPLATE(kDesktopWindowsChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.54f, 13.72f,
R_V_LINE_TO, -1.38f,
R_H_LINE_TO, 1.2f,
R_V_LINE_TO, -1.2f,
R_H_LINE_TO, -3.89f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 3.64f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 10.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 6.12f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.41f, -0.98f, 0.41f,
H_LINE_TO, 9.26f,
R_V_LINE_TO, 1.2f,
R_H_LINE_TO, 1.2f,
R_V_LINE_TO, 1.38f,
CLOSE,
MOVE_TO, 2.85f, 9.75f,
R_H_LINE_TO, 10.3f,
V_LINE_TO, 3.64f,
H_LINE_TO, 2.85f,
CLOSE,
R_MOVE_TO, 0, 0,
V_LINE_TO, 3.64f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDesktopWindowsChromeRefreshRepList, kDesktopWindowsChromeRefreshIcon, {kDesktopWindowsChromeRefreshPath, std::size(kDesktopWindowsChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kDeveloperToolsPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 5.54f, 12.13f,
LINE_TO, 1.41f, 8,
R_LINE_TO, 4.13f, -4.13f,
R_LINE_TO, 0.98f, 0.98f,
R_LINE_TO, -3.15f, 3.15f,
R_LINE_TO, 3.15f, 3.15f,
CLOSE,
R_MOVE_TO, 4.92f, 0,
R_LINE_TO, -0.98f, -0.98f,
R_LINE_TO, 3.15f, -3.15f,
R_LINE_TO, -3.15f, -3.15f,
R_LINE_TO, 0.98f, -0.98f,
LINE_TO, 14.59f, 8,
CLOSE,
R_MOVE_TO, 0, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDeveloperToolsRepList, kDeveloperToolsIcon, {kDeveloperToolsPath, std::size(kDeveloperToolsPath)})
VECTOR_ICON_REP_TEMPLATE(kDevicesPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 2.5f, 4.21f,
R_H_LINE_TO, 11.25f,
V_LINE_TO, 3,
H_LINE_TO, 2.5f,
R_CUBIC_TO, -0.69f, 0, -1.25f, 0.54f, -1.25f, 1.21f,
R_V_LINE_TO, 6.63f,
H_LINE_TO, 0,
R_V_LINE_TO, 1.81f,
R_H_LINE_TO, 8.75f,
R_V_LINE_TO, -1.81f,
H_LINE_TO, 2.5f,
R_V_LINE_TO, -6.63f,
CLOSE,
R_MOVE_TO, 11.88f, 1.21f,
R_H_LINE_TO, -3.75f,
R_ARC_TO, 0.62f, 0.62f, 0, 0, 0, -0.62f, 0.6f,
R_V_LINE_TO, 6.03f,
R_CUBIC_TO, 0, 0.33f, 0.28f, 0.6f, 0.63f, 0.6f,
R_H_LINE_TO, 3.75f,
ARC_TO, 0.62f, 0.62f, 0, 0, 0, 15, 12.04f,
V_LINE_TO, 6.01f,
R_ARC_TO, 0.62f, 0.62f, 0, 0, 0, -0.62f, -0.6f,
CLOSE,
R_MOVE_TO, -0.62f, 5.42f,
R_H_LINE_TO, -2.5f,
V_LINE_TO, 6.62f,
R_H_LINE_TO, 2.5f,
R_V_LINE_TO, 4.22f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDevicesRepList, kDevicesIcon, {kDevicesPath, std::size(kDevicesPath)})
VECTOR_ICON_REP_TEMPLATE(kDevicesChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 2, 16,
R_V_LINE_TO, -2,
R_H_LINE_TO, 2,
V_LINE_TO, 5.5f,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.77f, 0.44f, -1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 5.5f, 4,
H_LINE_TO, 17,
R_V_LINE_TO, 1.5f,
H_LINE_TO, 5.5f,
V_LINE_TO, 14,
H_LINE_TO, 10,
R_V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 10.76f, 0,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, -0.54f, -0.21f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, -0.22f, -0.53f,
V_LINE_TO, 7.76f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, 0.22f, -0.54f,
ARC_TO, 0.73f, 0.73f, 0, 0, 1, 12.75f, 7,
R_H_LINE_TO, 4.49f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, 0.54f, 0.22f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, 0.22f, 0.54f,
R_V_LINE_TO, 7.49f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, -0.21f, 0.54f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, -0.53f, 0.22f,
CLOSE,
R_MOVE_TO, 0.74f, -2,
R_H_LINE_TO, 3,
V_LINE_TO, 8.5f,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, 0, 0,
R_H_LINE_TO, 3,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kDevicesChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 1.47f, 12.93f,
R_V_LINE_TO, -1.78f,
R_H_LINE_TO, 1.6f,
V_LINE_TO, 4.45f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 9.29f,
V_LINE_TO, 4.45f,
H_LINE_TO, 4.45f,
R_V_LINE_TO, 6.7f,
R_H_LINE_TO, 3.5f,
R_V_LINE_TO, 1.78f,
CLOSE,
R_MOVE_TO, 8.79f, 0,
R_ARC_TO, 0.68f, 0.68f, 0, 0, 1, -0.7f, -0.69f,
V_LINE_TO, 6.34f,
R_CUBIC_TO, 0, -0.19f, 0.07f, -0.36f, 0.2f, -0.49f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 1, 0.49f, -0.2f,
R_H_LINE_TO, 3.6f,
R_CUBIC_TO, 0.19f, 0, 0.36f, 0.07f, 0.49f, 0.2f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 1, 0.2f, 0.49f,
R_V_LINE_TO, 5.9f,
R_ARC_TO, 0.68f, 0.68f, 0, 0, 1, -0.69f, 0.7f,
CLOSE,
R_MOVE_TO, 0.61f, -1.78f,
R_H_LINE_TO, 2.38f,
V_LINE_TO, 6.95f,
R_H_LINE_TO, -2.37f,
CLOSE,
R_MOVE_TO, 0, 0,
R_H_LINE_TO, 2.38f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDevicesChromeRefreshRepList, kDevicesChromeRefreshIcon, {kDevicesChromeRefreshPath, std::size(kDevicesChromeRefreshPath)}, {kDevicesChromeRefresh16Path, std::size(kDevicesChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kDownloadInProgressChromeRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 10, 15,
R_LINE_TO, -5, -5,
R_LINE_TO, 1.4f, -1.45f,
R_LINE_TO, 2.6f, 2.6f,
V_LINE_TO, 5,
R_H_LINE_TO, 2,
R_V_LINE_TO, 6.15f,
R_LINE_TO, 2.6f, -2.6f,
LINE_TO, 15, 10,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadInProgressChromeRefreshRepList, kDownloadInProgressChromeRefreshIcon, {kDownloadInProgressChromeRefreshPath, std::size(kDownloadInProgressChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadInProgressTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 16.67f, 10.99f,
LINE_TO, 18.5f, 12.75f,
LINE_TO, 12, 19,
LINE_TO, 5.5f, 12.75f,
LINE_TO, 7.33f, 10.99f,
LINE_TO, 10.82f, 14.38f,
V_LINE_TO, 5,
H_LINE_TO, 13.18f,
V_LINE_TO, 14.38f,
LINE_TO, 16.67f, 10.99f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadInProgressTouchRepList, kDownloadInProgressTouchIcon, {kDownloadInProgressTouchPath, std::size(kDownloadInProgressTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 8, 10.35f,
LINE_TO, 4.61f, 6.96f,
R_LINE_TO, 0.98f, -0.97f,
R_LINE_TO, 1.72f, 1.72f,
V_LINE_TO, 2.27f,
H_LINE_TO, 8.69f,
V_LINE_TO, 7.71f,
R_LINE_TO, 1.72f, -1.72f,
R_LINE_TO, 0.98f, 0.97f,
CLOSE,
R_MOVE_TO, -3.56f, 2.58f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.35f, 1.35f, 0, 0, 1, -0.4f, -0.98f,
R_V_LINE_TO, -1.2f,
H_LINE_TO, 4.45f,
R_V_LINE_TO, 1.2f,
R_H_LINE_TO, 7.1f,
R_V_LINE_TO, -1.2f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 1.2f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadMenuRepList, kDownloadMenuIcon, {kDownloadMenuPath, std::size(kDownloadMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadToolbarButtonPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 14, 11,
V_LINE_TO, 14,
H_LINE_TO, 2,
V_LINE_TO, 11,
H_LINE_TO, 0,
V_LINE_TO, 14,
CUBIC_TO, 0, 15.1f, 0.9f, 16, 2, 16,
H_LINE_TO, 14,
CUBIC_TO, 15.1f, 16, 16, 15.1f, 16, 14,
V_LINE_TO, 11,
H_LINE_TO, 14,
CLOSE,
MOVE_TO, 13, 7,
LINE_TO, 11.59f, 5.59f,
LINE_TO, 9, 8.17f,
V_LINE_TO, 0,
H_LINE_TO, 7,
V_LINE_TO, 8.17f,
LINE_TO, 4.41f, 5.59f,
LINE_TO, 3, 7,
LINE_TO, 8, 12,
LINE_TO, 13, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadToolbarButtonRepList, kDownloadToolbarButtonIcon, {kDownloadToolbarButtonPath, std::size(kDownloadToolbarButtonPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadToolbarButtonChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 4, 18,
R_ARC_TO, 1.99f, 1.99f, 0, 0, 1, -1.42f, -0.57f,
ARC_TO, 1.99f, 1.99f, 0, 0, 1, 2, 16,
R_V_LINE_TO, -3,
R_H_LINE_TO, 2,
R_V_LINE_TO, 3,
R_H_LINE_TO, 12,
R_V_LINE_TO, -3,
R_H_LINE_TO, 2,
R_V_LINE_TO, 3,
R_CUBIC_TO, 0, 0.55f, -0.2f, 1.02f, -0.6f, 1.43f,
R_CUBIC_TO, -0.38f, 0.38f, -0.85f, 0.57f, -1.4f, 0.57f,
CLOSE,
R_MOVE_TO, 6, -4,
LINE_TO, 5, 9,
R_LINE_TO, 1.4f, -1.45f,
R_LINE_TO, 2.6f, 2.6f,
V_LINE_TO, 2,
R_H_LINE_TO, 2,
R_V_LINE_TO, 8.15f,
R_LINE_TO, 2.6f, -2.6f,
LINE_TO, 15, 9,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadToolbarButtonChromeRefreshRepList, kDownloadToolbarButtonChromeRefreshIcon, {kDownloadToolbarButtonChromeRefreshPath, std::size(kDownloadToolbarButtonChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadToolbarButtonTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 18.88f, 18,
V_LINE_TO, 14,
H_LINE_TO, 21,
V_LINE_TO, 18,
CUBIC_TO, 21, 19.21f, 20.04f, 20.25f, 18.88f, 20.25f,
H_LINE_TO, 5.13f,
CUBIC_TO, 3.96f, 20.25f, 3, 19.21f, 3, 18,
V_LINE_TO, 14,
H_LINE_TO, 5.13f,
V_LINE_TO, 18,
H_LINE_TO, 18.88f,
CLOSE,
NEW_PATH,
MOVE_TO, 15.95f, 9.06f,
LINE_TO, 17.5f, 10.58f,
LINE_TO, 12, 16,
LINE_TO, 6.5f, 10.58f,
LINE_TO, 8.05f, 9.06f,
LINE_TO, 11, 12,
V_LINE_TO, 3,
H_LINE_TO, 13,
V_LINE_TO, 12,
LINE_TO, 15.95f, 9.06f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadToolbarButtonTouchRepList, kDownloadToolbarButtonTouchIcon, {kDownloadToolbarButtonTouchPath, std::size(kDownloadToolbarButtonTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kDownloadWarningPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 1, 17,
LINE_TO, 10, 2,
LINE_TO, 19, 17,
CLOSE,
MOVE_TO, 10, 14.5f,
CUBIC_TO, 10.21f, 14.5f, 10.39f, 14.43f, 10.53f, 14.29f,
CUBIC_TO, 10.68f, 14.14f, 10.75f, 13.96f, 10.75f, 13.75f,
CUBIC_TO, 10.75f, 13.54f, 10.68f, 13.36f, 10.54f, 13.22f,
CUBIC_TO, 10.39f, 13.07f, 10.21f, 13, 10, 13,
CUBIC_TO, 9.79f, 13, 9.61f, 13.07f, 9.47f, 13.21f,
CUBIC_TO, 9.32f, 13.36f, 9.25f, 13.54f, 9.25f, 13.75f,
CUBIC_TO, 9.25f, 13.96f, 9.32f, 14.14f, 9.46f, 14.28f,
CUBIC_TO, 9.61f, 14.43f, 9.79f, 14.5f, 10, 14.5f,
CLOSE,
MOVE_TO, 9.25f, 12,
LINE_TO, 10.75f, 12,
LINE_TO, 10.75f, 8,
LINE_TO, 9.25f, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDownloadWarningRepList, kDownloadWarningIcon, {kDownloadWarningPath, std::size(kDownloadWarningPath)})
VECTOR_ICON_REP_TEMPLATE(kDragHandlePath, CANVAS_DIMENSIONS, 17,
MOVE_TO, 6, 0,
LINE_TO, 8, 0,
V_LINE_TO, 16,
H_LINE_TO, 6,
V_LINE_TO, 0,
CLOSE,
NEW_PATH,
MOVE_TO, 10, 0,
LINE_TO, 12, 0,
V_LINE_TO, 16,
H_LINE_TO, 10,
V_LINE_TO, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDragHandleRepList, kDragHandleIcon, {kDragHandlePath, std::size(kDragHandlePath)})
VECTOR_ICON_REP_TEMPLATE(kDriveShortcutChromeRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 10.49f, 14.54f,
R_CUBIC_TO, -1.49f, -0.41f, -2.7f, -1.22f, -3.64f, -2.43f,
R_ARC_TO, 6.55f, 6.55f, 0, 0, 1, -1.4f, -4.11f,
R_CUBIC_TO, 0, -0.99f, 0.21f, -1.93f, 0.63f, -2.83f,
R_ARC_TO, 6.6f, 6.6f, 0, 0, 1, 1.78f, -2.31f,
H_LINE_TO, 5.51f,
V_LINE_TO, 1.47f,
R_H_LINE_TO, 4.98f,
R_V_LINE_TO, 4.98f,
H_LINE_TO, 9.11f,
V_LINE_TO, 3.63f,
R_ARC_TO, 5.56f, 5.56f, 0, 0, 0, -1.66f, 1.91f,
R_ARC_TO, 5.12f, 5.12f, 0, 0, 0, -0.61f, 2.44f,
R_CUBIC_TO, 0, 1.16f, 0.34f, 2.2f, 1.01f, 3.13f,
R_ARC_TO, 5.37f, 5.37f, 0, 0, 0, 2.65f, 1.96f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kDriveShortcutChromeRefreshRepList, kDriveShortcutChromeRefreshIcon, {kDriveShortcutChromeRefreshPath, std::size(kDriveShortcutChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kEditChromeRefreshPath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 5, 19,
R_H_LINE_TO, 1.43f,
R_LINE_TO, 9.77f, -9.77f,
LINE_TO, 14.77f, 7.8f,
LINE_TO, 5, 17.57f,
CLOSE,
R_MOVE_TO, -2, 2,
R_V_LINE_TO, -4.25f,
LINE_TO, 16.2f, 3.57f,
R_CUBIC_TO, 0.2f, -0.18f, 0.42f, -0.32f, 0.66f, -0.43f,
ARC_TO, 2.02f, 2.02f, 0, 0, 1, 17.63f, 3,
R_CUBIC_TO, 0.27f, 0, 0.52f, 0.05f, 0.77f, 0.15f,
R_CUBIC_TO, 0.25f, 0.1f, 0.47f, 0.25f, 0.65f, 0.45f,
LINE_TO, 20.43f, 5,
R_CUBIC_TO, 0.2f, 0.18f, 0.34f, 0.4f, 0.44f, 0.65f,
R_ARC_TO, 2.18f, 2.18f, 0, 0, 1, 0, 1.52f,
R_CUBIC_TO, -0.09f, 0.24f, -0.24f, 0.46f, -0.44f, 0.66f,
LINE_TO, 7.25f, 21,
CLOSE,
MOVE_TO, 19, 6.4f,
LINE_TO, 17.6f, 5,
CLOSE,
R_MOVE_TO, -3.52f, 2.13f,
R_LINE_TO, -0.7f, -0.72f,
LINE_TO, 16.2f, 9.23f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kEditChromeRefreshRepList, kEditChromeRefreshIcon, {kEditChromeRefreshPath, std::size(kEditChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kExitMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.65f, 13.73f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
R_V_LINE_TO, -2.66f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 2.66f,
R_H_LINE_TO, 8.7f,
V_LINE_TO, 3.65f,
H_LINE_TO, 3.65f,
R_V_LINE_TO, 2.66f,
H_LINE_TO, 2.27f,
R_V_LINE_TO, -2.66f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 8.7f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 8.7f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 3.63f, -2.37f,
R_LINE_TO, -0.98f, -0.97f,
LINE_TO, 7.99f, 8.69f,
H_LINE_TO, 2.27f,
V_LINE_TO, 7.31f,
R_H_LINE_TO, 5.72f,
LINE_TO, 6.29f, 5.61f,
R_LINE_TO, 0.98f, -0.97f,
LINE_TO, 10.63f, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kExitMenuRepList, kExitMenuIcon, {kExitMenuPath, std::size(kExitMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kExtensionCrashedPath, MOVE_TO, 37, 23,
LINE_TO, 40, 23,
CUBIC_TO, 42.76f, 23, 45, 25.24f, 45, 28,
CUBIC_TO, 45, 30.76f, 42.76f, 33, 40, 33,
LINE_TO, 37, 33,
LINE_TO, 37, 41,
CUBIC_TO, 37, 43.2f, 35.2f, 45, 33, 45,
LINE_TO, 25.4f, 45,
LINE_TO, 25.4f, 42,
CUBIC_TO, 25.4f, 39.02f, 22.98f, 36.6f, 20, 36.6f,
CUBIC_TO, 17.02f, 36.6f, 14.6f, 39.02f, 14.6f, 42,
LINE_TO, 14.6f, 45,
LINE_TO, 7, 45,
CUBIC_TO, 4.8f, 45, 3, 43.2f, 3, 41,
LINE_TO, 3, 33.4f,
LINE_TO, 6, 33.4f,
CUBIC_TO, 8.98f, 33.4f, 11.4f, 30.98f, 11.4f, 28,
CUBIC_TO, 11.4f, 25.02f, 8.98f, 22.6f, 6, 22.6f,
LINE_TO, 3.02f, 22.6f,
LINE_TO, 3.02f, 15,
CUBIC_TO, 3.02f, 12.8f, 4.8f, 11, 7, 11,
LINE_TO, 15, 11,
LINE_TO, 15, 8,
CUBIC_TO, 15, 5.24f, 17.24f, 3, 20, 3,
CUBIC_TO, 22.76f, 3, 25, 5.24f, 25, 8,
LINE_TO, 25, 11,
LINE_TO, 29.5f, 11,
LINE_TO, 16, 31.5f,
LINE_TO, 37, 17.5f,
LINE_TO, 37, 23,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kExtensionCrashedRepList, kExtensionCrashedIcon, {kExtensionCrashedPath, std::size(kExtensionCrashedPath)})
VECTOR_ICON_REP_TEMPLATE(kFileDownloadShelfPath, CANVAS_DIMENSIONS, 72,
PATH_COLOR_ARGB, 0xE6, 0xFF, 0xFF, 0xFF,
CIRCLE, 36, 36, 36,
NEW_PATH,
MOVE_TO, 32, 22,
R_H_LINE_TO, 8,
R_V_LINE_TO, 10,
R_H_LINE_TO, 6,
R_LINE_TO, -10, 10,
R_LINE_TO, -10, -10,
R_H_LINE_TO, 6,
R_V_LINE_TO, -10,
CLOSE,
MOVE_TO, 20, 56,
R_H_LINE_TO, 32,
R_V_LINE_TO, 4,
R_H_LINE_TO, -32,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFileDownloadShelfRepList, kFileDownloadShelfIcon, {kFileDownloadShelfPath, std::size(kFileDownloadShelfPath)})
VECTOR_ICON_REP_TEMPLATE(kFileSavePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 11, 17,
H_LINE_TO, 6,
V_LINE_TO, 4,
R_H_LINE_TO, 5,
R_V_LINE_TO, 4,
R_H_LINE_TO, 4,
R_V_LINE_TO, 2,
R_H_LINE_TO, 2,
V_LINE_TO, 7,
R_LINE_TO, -5, -5,
H_LINE_TO, 6,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 13,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 5,
R_V_LINE_TO, -2,
CLOSE,
R_MOVE_TO, 9, 5,
R_H_LINE_TO, -8,
R_V_LINE_TO, -2,
R_H_LINE_TO, 8,
R_V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 0, -7,
R_LINE_TO, -4, 4,
R_LINE_TO, -4, -4,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, 2,
R_V_LINE_TO, 3,
R_H_LINE_TO, 3,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFileSaveRepList, kFileSaveIcon, {kFileSavePath, std::size(kFileSavePath)})
VECTOR_ICON_REP_TEMPLATE(kFileSaveChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
R_MOVE_TO, 15.25f, 17.5f,
R_LINE_TO, 3, -3,
R_LINE_TO, -1.06f, -1.06f,
LINE_TO, 16, 14.63f,
V_LINE_TO, 11.5f,
R_H_LINE_TO, -1.5f,
R_V_LINE_TO, 3.13f,
R_LINE_TO, -1.19f, -1.19f,
LINE_TO, 12.25f, 14.5f,
CLOSE,
R_MOVE_TO, -3, 2.5f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 6,
V_LINE_TO, 20,
CLOSE,
MOVE_TO, 5.5f, 17,
R_CUBIC_TO, -0.4f, 0, -0.75f, -0.15f, -1.05f, -0.45f,
R_CUBIC_TO, -0.3f, -0.3f, -0.45f, -0.65f, -0.45f, -1.05f,
R_V_LINE_TO, -12,
R_CUBIC_TO, 0, -0.4f, 0.15f, -0.75f, 0.45f, -1.05f,
R_CUBIC_TO, 0.3f, -0.3f, 0.65f, -0.45f, 1.05f, -0.45f,
H_LINE_TO, 12,
R_LINE_TO, 4, 4,
R_V_LINE_TO, 4,
R_H_LINE_TO, -1.5f,
V_LINE_TO, 7,
H_LINE_TO, 11,
V_LINE_TO, 3.5f,
H_LINE_TO, 5.5f,
R_V_LINE_TO, 12,
H_LINE_TO, 11,
V_LINE_TO, 17,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_V_LINE_TO, -12,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kFileSaveChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 12.2f, 14,
R_LINE_TO, 2.41f, -2.41f,
R_LINE_TO, -0.85f, -0.85f,
R_LINE_TO, -0.96f, 0.96f,
V_LINE_TO, 9.2f,
R_H_LINE_TO, -1.2f,
R_V_LINE_TO, 2.5f,
R_LINE_TO, -0.96f, -0.96f,
R_LINE_TO, -0.85f, 0.85f,
CLOSE,
R_MOVE_TO, -2.4f, 2,
R_V_LINE_TO, -1.2f,
R_H_LINE_TO, 4.8f,
V_LINE_TO, 16,
CLOSE,
R_MOVE_TO, -5.31f, -2.31f,
R_CUBIC_TO, -0.37f, 0, -0.7f, -0.14f, -0.97f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.97f,
V_LINE_TO, 2.89f,
R_CUBIC_TO, 0, -0.37f, 0.14f, -0.7f, 0.41f, -0.97f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, 0.97f, -0.41f,
H_LINE_TO, 9.7f,
R_LINE_TO, 3.19f, 3.19f,
V_LINE_TO, 8,
R_H_LINE_TO, -1.38f,
V_LINE_TO, 5.63f,
H_LINE_TO, 8.77f,
V_LINE_TO, 2.89f,
H_LINE_TO, 4.49f,
R_V_LINE_TO, 9.42f,
R_H_LINE_TO, 4.31f,
R_V_LINE_TO, 1.38f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
V_LINE_TO, 2.89f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFileSaveChromeRefreshRepList, kFileSaveChromeRefreshIcon, {kFileSaveChromeRefreshPath, std::size(kFileSaveChromeRefreshPath)}, {kFileSaveChromeRefresh16Path, std::size(kFileSaveChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kFileSaveOffChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
R_MOVE_TO, 17.44f, 15.31f,
R_LINE_TO, -1.06f, -1.06f,
R_LINE_TO, 0.81f, -0.81f,
LINE_TO, 18.25f, 14.5f,
CLOSE,
MOVE_TO, 16, 13.87f,
R_LINE_TO, -1.5f, -1.5f,
V_LINE_TO, 11.5f,
H_LINE_TO, 16,
CLOSE,
MOVE_TO, 12.25f, 20,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 4.13f,
LINE_TO, 5.5f, 7.63f,
V_LINE_TO, 15.5f,
R_H_LINE_TO, 5.52f,
V_LINE_TO, 17,
H_LINE_TO, 5.5f,
R_CUBIC_TO, -0.4f, 0, -0.75f, -0.15f, -1.05f, -0.45f,
R_CUBIC_TO, -0.3f, -0.3f, -0.45f, -0.65f, -0.45f, -1.05f,
V_LINE_TO, 6.13f,
LINE_TO, 1.17f, 3.27f,
LINE_TO, 2.23f, 2.23f,
LINE_TO, 18.5f, 18.5f,
V_LINE_TO, 20,
CLOSE,
R_MOVE_TO, 2.25f, -10,
H_LINE_TO, 16,
V_LINE_TO, 6,
R_LINE_TO, -4, -4,
H_LINE_TO, 5.5f,
R_CUBIC_TO, -0.18f, 0, -0.36f, 0.04f, -0.55f, 0.11f,
R_CUBIC_TO, -0.19f, 0.07f, -0.34f, 0.16f, -0.45f, 0.27f,
LINE_TO, 5.63f, 3.5f,
H_LINE_TO, 11,
V_LINE_TO, 7,
R_H_LINE_TO, 3.5f,
CLOSE,
R_MOVE_TO, -5.04f, 1.58f,
CLOSE,
R_MOVE_TO, 2, -2.46f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFileSaveOffChromeRefreshRepList, kFileSaveOffChromeRefreshIcon, {kFileSaveOffChromeRefreshPath, std::size(kFileSaveOffChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kFingerprintPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17.81f, 4.47f,
R_CUBIC_TO, -0.08f, 0, -0.16f, -0.02f, -0.23f, -0.06f,
CUBIC_TO, 15.66f, 3.42f, 14, 3, 12.01f, 3,
R_CUBIC_TO, -1.98f, 0, -3.86f, 0.47f, -5.57f, 1.41f,
R_CUBIC_TO, -0.24f, 0.13f, -0.54f, 0.04f, -0.68f, -0.2f,
R_CUBIC_TO, -0.13f, -0.24f, -0.04f, -0.55f, 0.2f, -0.68f,
CUBIC_TO, 7.82f, 2.52f, 9.86f, 2, 12.01f, 2,
R_CUBIC_TO, 2.13f, 0, 3.99f, 0.47f, 6.03f, 1.52f,
R_CUBIC_TO, 0.25f, 0.13f, 0.34f, 0.43f, 0.21f, 0.67f,
R_CUBIC_TO, -0.09f, 0.18f, -0.26f, 0.28f, -0.44f, 0.28f,
CLOSE,
MOVE_TO, 3.5f, 9.72f,
R_CUBIC_TO, -0.1f, 0, -0.2f, -0.03f, -0.29f, -0.09f,
R_CUBIC_TO, -0.23f, -0.16f, -0.28f, -0.47f, -0.12f, -0.7f,
R_CUBIC_TO, 0.99f, -1.4f, 2.25f, -2.5f, 3.75f, -3.27f,
CUBIC_TO, 9.98f, 4.04f, 14, 4.03f, 17.15f, 5.65f,
R_CUBIC_TO, 1.5f, 0.77f, 2.76f, 1.86f, 3.75f, 3.25f,
R_CUBIC_TO, 0.16f, 0.22f, 0.11f, 0.54f, -0.12f, 0.7f,
R_CUBIC_TO, -0.23f, 0.16f, -0.54f, 0.11f, -0.7f, -0.12f,
R_CUBIC_TO, -0.9f, -1.26f, -2.04f, -2.25f, -3.39f, -2.94f,
R_CUBIC_TO, -2.87f, -1.47f, -6.54f, -1.47f, -9.4f, 0.01f,
R_CUBIC_TO, -1.36f, 0.7f, -2.5f, 1.7f, -3.4f, 2.96f,
R_CUBIC_TO, -0.08f, 0.14f, -0.23f, 0.21f, -0.39f, 0.21f,
CLOSE,
R_MOVE_TO, 6.25f, 12.07f,
R_CUBIC_TO, -0.13f, 0, -0.26f, -0.05f, -0.35f, -0.15f,
R_CUBIC_TO, -0.87f, -0.87f, -1.34f, -1.43f, -2.01f, -2.64f,
R_CUBIC_TO, -0.69f, -1.23f, -1.05f, -2.73f, -1.05f, -4.34f,
R_CUBIC_TO, 0, -2.97f, 2.54f, -5.39f, 5.66f, -5.39f,
R_CUBIC_TO, 3.12f, 0, 5.66f, 2.42f, 5.66f, 5.39f,
R_CUBIC_TO, 0, 0.28f, -0.22f, 0.5f, -0.5f, 0.5f,
R_CUBIC_TO, -0.28f, 0, -0.5f, -0.22f, -0.5f, -0.5f,
R_CUBIC_TO, 0, -2.42f, -2.09f, -4.39f, -4.66f, -4.39f,
R_CUBIC_TO, -2.57f, 0, -4.66f, 1.97f, -4.66f, 4.39f,
R_CUBIC_TO, 0, 1.44f, 0.32f, 2.77f, 0.93f, 3.85f,
R_CUBIC_TO, 0.64f, 1.15f, 1.08f, 1.64f, 1.85f, 2.42f,
R_CUBIC_TO, 0.19f, 0.2f, 0.19f, 0.51f, 0, 0.71f,
R_CUBIC_TO, -0.11f, 0.1f, -0.24f, 0.15f, -0.37f, 0.15f,
CLOSE,
R_MOVE_TO, 7.17f, -1.85f,
R_CUBIC_TO, -1.19f, 0, -2.24f, -0.3f, -3.1f, -0.89f,
R_CUBIC_TO, -1.49f, -1.01f, -2.38f, -2.65f, -2.38f, -4.39f,
R_CUBIC_TO, 0, -0.28f, 0.22f, -0.5f, 0.5f, -0.5f,
R_CUBIC_TO, 0.28f, 0, 0.5f, 0.22f, 0.5f, 0.5f,
R_CUBIC_TO, 0, 1.41f, 0.72f, 2.74f, 1.94f, 3.56f,
R_CUBIC_TO, 0.71f, 0.48f, 1.54f, 0.71f, 2.54f, 0.71f,
R_CUBIC_TO, 0.24f, 0, 0.64f, -0.03f, 1.04f, -0.1f,
R_CUBIC_TO, 0.27f, -0.05f, 0.53f, 0.13f, 0.58f, 0.41f,
R_CUBIC_TO, 0.05f, 0.27f, -0.13f, 0.53f, -0.41f, 0.58f,
R_CUBIC_TO, -0.57f, 0.11f, -1.07f, 0.12f, -1.21f, 0.12f,
CLOSE,
MOVE_TO, 14.91f, 22,
R_CUBIC_TO, -0.04f, 0, -0.09f, -0.01f, -0.13f, -0.02f,
R_CUBIC_TO, -1.59f, -0.44f, -2.63f, -1.03f, -3.72f, -2.1f,
R_CUBIC_TO, -1.4f, -1.39f, -2.17f, -3.24f, -2.17f, -5.22f,
R_CUBIC_TO, 0, -1.62f, 1.38f, -2.94f, 3.08f, -2.94f,
R_CUBIC_TO, 1.7f, 0, 3.08f, 1.32f, 3.08f, 2.94f,
R_CUBIC_TO, 0, 1.07f, 0.93f, 1.94f, 2.08f, 1.94f,
R_CUBIC_TO, 1.15f, 0, 2.08f, -0.87f, 2.08f, -1.94f,
R_CUBIC_TO, 0, -3.77f, -3.25f, -6.83f, -7.25f, -6.83f,
R_CUBIC_TO, -2.84f, 0, -5.44f, 1.58f, -6.61f, 4.03f,
R_CUBIC_TO, -0.39f, 0.81f, -0.59f, 1.76f, -0.59f, 2.8f,
R_CUBIC_TO, 0, 0.78f, 0.07f, 2.01f, 0.67f, 3.61f,
R_CUBIC_TO, 0.1f, 0.26f, -0.03f, 0.55f, -0.29f, 0.64f,
R_CUBIC_TO, -0.26f, 0.1f, -0.55f, -0.04f, -0.64f, -0.29f,
R_CUBIC_TO, -0.49f, -1.31f, -0.73f, -2.61f, -0.73f, -3.96f,
R_CUBIC_TO, 0, -1.2f, 0.23f, -2.29f, 0.68f, -3.24f,
R_CUBIC_TO, 1.33f, -2.79f, 4.28f, -4.6f, 7.51f, -4.6f,
R_CUBIC_TO, 4.55f, 0, 8.25f, 3.51f, 8.25f, 7.83f,
R_CUBIC_TO, 0, 1.62f, -1.38f, 2.94f, -3.08f, 2.94f,
R_CUBIC_TO, -1.7f, 0, -3.08f, -1.32f, -3.08f, -2.94f,
R_CUBIC_TO, 0, -1.07f, -0.93f, -1.94f, -2.08f, -1.94f,
R_CUBIC_TO, -1.15f, 0, -2.08f, 0.87f, -2.08f, 1.94f,
R_CUBIC_TO, 0, 1.71f, 0.66f, 3.31f, 1.87f, 4.51f,
R_CUBIC_TO, 0.95f, 0.94f, 1.86f, 1.46f, 3.27f, 1.85f,
R_CUBIC_TO, 0.27f, 0.07f, 0.42f, 0.35f, 0.35f, 0.61f,
R_CUBIC_TO, -0.05f, 0.23f, -0.26f, 0.38f, -0.47f, 0.38f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFingerprintRepList, kFingerprintIcon, {kFingerprintPath, std::size(kFingerprintPath)})
VECTOR_ICON_REP_TEMPLATE(kForwardArrowTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 11.09f, 18.59f,
LINE_TO, 11, 18.51f,
LINE_TO, 16.51f, 13,
H_LINE_TO, 5,
R_ARC_TO, 1, 1, 0, 0, 1, 0, -2,
R_H_LINE_TO, 11.51f,
LINE_TO, 11, 5.49f,
R_LINE_TO, 0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, 1.32f, -1.32f,
R_H_LINE_TO, 0,
LINE_TO, 12.49f, 4,
LINE_TO, 20, 11.51f,
R_LINE_TO, -0.09f, 0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, 0, 0.82f,
R_LINE_TO, 0.09f, 0.09f,
LINE_TO, 12.49f, 20,
R_LINE_TO, -0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, -1.32f, -1.32f,
R_V_LINE_TO, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kForwardArrowTouchRepList, kForwardArrowTouchIcon, {kForwardArrowTouchPath, std::size(kForwardArrowTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kFullscreenPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 7, 14,
H_LINE_TO, 5,
R_V_LINE_TO, 5,
R_H_LINE_TO, 5,
R_V_LINE_TO, -2,
H_LINE_TO, 7,
R_V_LINE_TO, -3,
CLOSE,
R_MOVE_TO, -2, -4,
R_H_LINE_TO, 2,
V_LINE_TO, 7,
R_H_LINE_TO, 3,
V_LINE_TO, 5,
H_LINE_TO, 5,
R_V_LINE_TO, 5,
CLOSE,
R_MOVE_TO, 12, 7,
R_H_LINE_TO, -3,
R_V_LINE_TO, 2,
R_H_LINE_TO, 5,
R_V_LINE_TO, -5,
R_H_LINE_TO, -2,
R_V_LINE_TO, 3,
CLOSE,
MOVE_TO, 14, 5,
R_V_LINE_TO, 2,
R_H_LINE_TO, 3,
R_V_LINE_TO, 3,
R_H_LINE_TO, 2,
V_LINE_TO, 5,
R_H_LINE_TO, -5,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFullscreenRepList, kFullscreenIcon, {kFullscreenPath, std::size(kFullscreenPath)})
VECTOR_ICON_REP_TEMPLATE(kFullscreenRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 2.27f, 13.73f,
R_V_LINE_TO, -3.44f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 2.06f,
R_H_LINE_TO, 2.06f,
R_V_LINE_TO, 1.38f,
CLOSE,
R_MOVE_TO, 8.04f, 0,
R_V_LINE_TO, -1.38f,
R_H_LINE_TO, 2.05f,
R_V_LINE_TO, -2.06f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 3.44f,
CLOSE,
MOVE_TO, 2.27f, 5.7f,
V_LINE_TO, 2.27f,
R_H_LINE_TO, 3.44f,
R_V_LINE_TO, 1.38f,
H_LINE_TO, 3.65f,
V_LINE_TO, 5.7f,
CLOSE,
R_MOVE_TO, 10.09f, 0,
V_LINE_TO, 3.65f,
H_LINE_TO, 10.3f,
V_LINE_TO, 2.27f,
R_H_LINE_TO, 3.43f,
R_V_LINE_TO, 3.43f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kFullscreenRefreshRepList, kFullscreenRefreshIcon, {kFullscreenRefreshPath, std::size(kFullscreenRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kGenericStopPath, CANVAS_DIMENSIONS, 20,
ROUND_RECT, 7, 7, 6, 6, 0,
NEW_PATH,
STROKE, 2,
CIRCLE, 10, 10, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kGenericStopRepList, kGenericStopIcon, {kGenericStopPath, std::size(kGenericStopPath)})
VECTOR_ICON_REP_TEMPLATE(kGlobePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12, 22.8f,
CUBIC_TO, 10.51f, 22.8f, 9.1f, 22.52f, 7.79f, 21.95f,
CUBIC_TO, 6.47f, 21.38f, 5.33f, 20.61f, 4.36f, 19.64f,
CUBIC_TO, 3.39f, 18.67f, 2.62f, 17.53f, 2.05f, 16.21f,
CUBIC_TO, 1.48f, 14.9f, 1.2f, 13.49f, 1.2f, 12,
CUBIC_TO, 1.2f, 10.5f, 1.48f, 9.09f, 2.05f, 7.78f,
CUBIC_TO, 2.62f, 6.46f, 3.39f, 5.31f, 4.36f, 4.34f,
CUBIC_TO, 5.34f, 3.36f, 6.48f, 2.6f, 7.79f, 2.04f,
CUBIC_TO, 9.11f, 1.48f, 10.51f, 1.2f, 12, 1.2f,
CUBIC_TO, 13.5f, 1.2f, 14.91f, 1.48f, 16.22f, 2.04f,
CUBIC_TO, 17.54f, 2.6f, 18.69f, 3.36f, 19.66f, 4.34f,
CUBIC_TO, 20.64f, 5.31f, 21.4f, 6.46f, 21.96f, 7.78f,
CUBIC_TO, 22.52f, 9.09f, 22.8f, 10.5f, 22.8f, 12,
CUBIC_TO, 22.8f, 13.5f, 22.52f, 14.9f, 21.96f, 16.21f,
CUBIC_TO, 21.4f, 17.52f, 20.64f, 18.66f, 19.66f, 19.64f,
CUBIC_TO, 18.69f, 20.61f, 17.54f, 21.38f, 16.22f, 21.95f,
CUBIC_TO, 14.91f, 22.52f, 13.5f, 22.8f, 12, 22.8f,
CLOSE,
MOVE_TO, 12, 20.15f,
CUBIC_TO, 14.08f, 20.15f, 15.88f, 19.48f, 17.39f, 18.14f,
CUBIC_TO, 18.89f, 16.8f, 19.79f, 15.13f, 20.07f, 13.13f,
CUBIC_TO, 19.88f, 13.44f, 19.61f, 13.7f, 19.26f, 13.89f,
CUBIC_TO, 18.92f, 14.08f, 18.55f, 14.18f, 18.15f, 14.18f,
LINE_TO, 15.85f, 14.18f,
CUBIC_TO, 15.22f, 14.18f, 14.68f, 13.95f, 14.23f, 13.5f,
CUBIC_TO, 13.77f, 13.05f, 13.55f, 12.51f, 13.55f, 11.88f,
LINE_TO, 13.55f, 10.73f,
LINE_TO, 9.2f, 10.73f,
LINE_TO, 9.2f, 8.2f,
CUBIC_TO, 9.2f, 7.57f, 9.43f, 7.03f, 9.88f, 6.57f,
CUBIC_TO, 10.32f, 6.13f, 10.87f, 5.9f, 11.5f, 5.9f,
LINE_TO, 12.4f, 5.9f,
CUBIC_TO, 12.4f, 5.48f, 12.5f, 5.11f, 12.69f, 4.77f,
CUBIC_TO, 12.88f, 4.44f, 13.13f, 4.18f, 13.45f, 3.98f,
CUBIC_TO, 13.22f, 3.94f, 12.98f, 3.91f, 12.74f, 3.88f,
CUBIC_TO, 12.5f, 3.86f, 12.26f, 3.85f, 12, 3.85f,
CUBIC_TO, 9.73f, 3.85f, 7.8f, 4.64f, 6.22f, 6.22f,
CUBIC_TO, 4.64f, 7.8f, 3.85f, 9.73f, 3.85f, 12,
LINE_TO, 3.85f, 12.38f,
CUBIC_TO, 3.85f, 12.49f, 3.86f, 12.62f, 3.88f, 12.75f,
LINE_TO, 7.3f, 12.75f,
CUBIC_TO, 8.58f, 12.75f, 9.67f, 13.2f, 10.56f, 14.09f,
CUBIC_TO, 11.45f, 14.98f, 11.9f, 16.06f, 11.9f, 17.34f,
LINE_TO, 11.9f, 18.5f,
LINE_TO, 8.45f, 18.5f,
LINE_TO, 8.45f, 19.35f,
CUBIC_TO, 8.98f, 19.62f, 9.55f, 19.82f, 10.14f, 19.95f,
CUBIC_TO, 10.74f, 20.08f, 11.36f, 20.15f, 12, 20.15f,
CLOSE,
MOVE_TO, 12, 20.15f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kGlobeRepList, kGlobeIcon, {kGlobePath, std::size(kGlobePath)})
VECTOR_ICON_REP_TEMPLATE(kGuardianPath, CANVAS_DIMENSIONS, 32,
FILL_RULE_NONZERO,
MOVE_TO, 16, 27.23f,
R_CUBIC_TO, -3.17f, 0, -5.83f, -0.59f, -8, -1.77f,
R_CUBIC_TO, -2.17f, -1.18f, -3.25f, -2.65f, -3.25f, -4.4f,
R_CUBIC_TO, 0, -1.18f, 0.54f, -2.27f, 1.63f, -3.25f,
R_CUBIC_TO, 1.08f, -0.98f, 2.58f, -1.72f, 4.47f, -2.22f,
R_V_LINE_TO, 2.3f,
R_CUBIC_TO, -1.18f, 0.38f, -2.12f, 0.87f, -2.8f, 1.45f,
R_CUBIC_TO, -0.68f, 0.57f, -1.02f, 1.14f, -1.02f, 1.73f,
R_CUBIC_TO, 0, 1.05f, 0.89f, 1.97f, 2.68f, 2.75f,
R_CUBIC_TO, 1.8f, 0.77f, 3.9f, 1.15f, 6.3f, 1.15f,
R_CUBIC_TO, 2.4f, 0, 4.49f, -0.38f, 6.28f, -1.15f,
R_CUBIC_TO, 1.8f, -0.78f, 2.7f, -1.7f, 2.7f, -2.75f,
R_CUBIC_TO, 0, -0.58f, -0.35f, -1.16f, -1.05f, -1.72f,
R_CUBIC_TO, -0.68f, -0.58f, -1.61f, -1.07f, -2.77f, -1.45f,
R_V_LINE_TO, -2.3f,
R_CUBIC_TO, 1.9f, 0.5f, 3.39f, 1.24f, 4.47f, 2.23f,
R_CUBIC_TO, 1.08f, 0.98f, 1.63f, 2.07f, 1.63f, 3.25f,
R_CUBIC_TO, 0, 1.75f, -1.08f, 3.22f, -3.25f, 4.4f,
R_CUBIC_TO, -2.17f, 1.18f, -4.83f, 1.78f, -8, 1.78f,
CLOSE,
R_MOVE_TO, -3.15f, -4.3f,
V_LINE_TO, 12.35f,
H_LINE_TO, 7.8f,
R_V_LINE_TO, -2.27f,
R_H_LINE_TO, 16.4f,
R_V_LINE_TO, 2.28f,
R_H_LINE_TO, -5.05f,
R_V_LINE_TO, 10.58f,
R_H_LINE_TO, -2.27f,
V_LINE_TO, 18.05f,
R_H_LINE_TO, -1.75f,
R_V_LINE_TO, 4.88f,
H_LINE_TO, 12.85f,
CLOSE,
MOVE_TO, 16, 9.07f,
R_ARC_TO, 2.13f, 2.13f, 0, 0, 1, -1.52f, -0.62f,
R_ARC_TO, 2.08f, 2.08f, 0, 0, 1, -0.62f, -1.52f,
R_CUBIC_TO, 0, -0.58f, 0.21f, -1.08f, 0.63f, -1.5f,
ARC_TO, 2.13f, 2.13f, 0, 0, 1, 16, 4.8f,
R_CUBIC_TO, 0.58f, 0, 1.08f, 0.21f, 1.5f, 0.63f,
R_CUBIC_TO, 0.43f, 0.42f, 0.65f, 0.92f, 0.65f, 1.5f,
R_CUBIC_TO, 0, 0.6f, -0.22f, 1.11f, -0.65f, 1.53f,
R_ARC_TO, 2.05f, 2.05f, 0, 0, 1, -1.5f, 0.63f,
CLOSE,
NEW_PATH)
VECTOR_ICON_TEMPLATE_CC(kGuardianRepList, kGuardianIcon, {kGuardianPath, std::size(kGuardianPath)})
VECTOR_ICON_REP_TEMPLATE(kGuestMenuArtPath, CANVAS_DIMENSIONS, 288,
STROKE, 1.7,
MOVE_TO, 210.06f, 65.04f,
LINE_TO, 212.21f, 56.9f,
CUBIC_TO, 212.4f, 56.06f, 213.05f, 55.41f, 213.9f, 55.22f,
LINE_TO, 222.04f, 53.07f,
CUBIC_TO, 222.88f, 52.88f, 223.72f, 53.07f, 224.38f, 53.72f,
LINE_TO, 230.28f, 59.62f,
CUBIC_TO, 230.93f, 60.27f, 231.12f, 61.11f, 230.93f, 61.96f,
LINE_TO, 228.78f, 70.1f,
CUBIC_TO, 228.59f, 70.94f, 227.94f, 71.59f, 227.09f, 71.78f,
LINE_TO, 218.95f, 73.93f,
CUBIC_TO, 218.11f, 74.12f, 217.27f, 73.93f, 216.61f, 73.28f,
LINE_TO, 210.71f, 67.38f,
CUBIC_TO, 210.15f, 66.82f, 209.87f, 65.89f, 210.06f, 65.04f,
CLOSE,
NEW_PATH,
MOVE_TO, 184.1f, 6.4f,
CUBIC_TO, 184.1f, 15.3f, 191.3f, 22.5f, 200.2f, 22.5f,
CUBIC_TO, 209.1f, 22.5f, 216.3f, 15.3f, 216.3f, 6.4f,
CUBIC_TO, 216.3f, 4.1f, 215.8f, 1.9f, 215, 0,
H_LINE_TO, 185.5f,
CUBIC_TO, 184.6f, 2, 184.1f, 4.1f, 184.1f, 6.4f,
CLOSE,
MOVE_TO, 110.5f, 22.7f,
CUBIC_TO, 113.92f, 22.7f, 116.7f, 19.92f, 116.7f, 16.5f,
CUBIC_TO, 116.7f, 13.08f, 113.92f, 10.3f, 110.5f, 10.3f,
CUBIC_TO, 107.08f, 10.3f, 104.3f, 13.08f, 104.3f, 16.5f,
CUBIC_TO, 104.3f, 19.92f, 107.08f, 22.7f, 110.5f, 22.7f,
CLOSE,
MOVE_TO, 64.1f, 4.9f,
CUBIC_TO, 66.3f, 8, 65.6f, 12.4f, 62.4f, 14.6f,
CUBIC_TO, 60.9f, 15.7f, 59, 16.1f, 57.2f, 15.8f,
CUBIC_TO, 55.4f, 15.5f, 53.8f, 14.5f, 52.7f, 13,
LINE_TO, 43.9f, 0.4f,
CUBIC_TO, 43.8f, 0.3f, 43.7f, 0.1f, 43.7f, 0,
H_LINE_TO, 42,
CUBIC_TO, 42.2f, 0.4f, 42.4f, 0.8f, 42.7f, 1.2f,
LINE_TO, 51.6f, 13.8f,
CUBIC_TO, 52.9f, 15.6f, 54.8f, 16.9f, 57.1f, 17.3f,
CUBIC_TO, 57.6f, 17.4f, 58.1f, 17.4f, 58.6f, 17.4f,
CUBIC_TO, 60.3f, 17.4f, 62, 16.9f, 63.5f, 15.9f,
CUBIC_TO, 67.3f, 13.2f, 68.2f, 7.9f, 65.5f, 4.1f,
LINE_TO, 62.5f, 0,
H_LINE_TO, 60.7f,
LINE_TO, 64.1f, 4.9f,
CLOSE,
MOVE_TO, 288, 27.2f,
CUBIC_TO, 286.8f, 27.1f, 285.7f, 27.3f, 284.6f, 27.7f,
CUBIC_TO, 283.3f, 28.2f, 281.9f, 28.3f, 280.6f, 27.8f,
LINE_TO, 280, 27.6f,
CUBIC_TO, 278.7f, 27.1f, 277.7f, 26.1f, 277.2f, 24.8f,
CUBIC_TO, 276.4f, 22.9f, 274.8f, 21.4f, 272.6f, 20.8f,
CUBIC_TO, 269.4f, 19.9f, 265.8f, 21.3f, 264.2f, 24.3f,
CUBIC_TO, 262.1f, 28.2f, 263.9f, 32.9f, 267.9f, 34.4f,
CUBIC_TO, 269.7f, 35.1f, 271.7f, 35.1f, 273.4f, 34.4f,
CUBIC_TO, 274.7f, 33.9f, 276.1f, 33.8f, 277.3f, 34.3f,
LINE_TO, 277.9f, 34.5f,
CUBIC_TO, 279.2f, 35, 280.2f, 36, 280.7f, 37.3f,
CUBIC_TO, 281.4f, 39, 282.8f, 40.4f, 284.6f, 41.1f,
CUBIC_TO, 285.7f, 41.5f, 286.8f, 41.7f, 287.9f, 41.6f,
V_LINE_TO, 27.2f,
H_LINE_TO, 288,
CLOSE,
MOVE_TO, 0, 34.1f,
V_LINE_TO, 56.9f,
LINE_TO, 25, 48.3f,
CUBIC_TO, 25.8f, 48, 26.2f, 47.1f, 25.9f, 46.3f,
CUBIC_TO, 21.6f, 36.2f, 10.5f, 31, 0, 34.1f,
CLOSE,
MOVE_TO, 77.95f, 56.77f,
LINE_TO, 55.57f, 59.54f,
CUBIC_TO, 54.42f, 59.69f, 53.82f, 60.99f, 54.52f, 61.9f,
LINE_TO, 67.82f, 79.47f,
CUBIC_TO, 68.53f, 80.39f, 69.94f, 80.2f, 70.44f, 79.16f,
LINE_TO, 79.49f, 58.83f,
CUBIC_TO, 79.94f, 57.77f, 79.11f, 56.63f, 77.95f, 56.77f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kGuestMenuArtRepList, kGuestMenuArtIcon, {kGuestMenuArtPath, std::size(kGuestMenuArtPath)})
VECTOR_ICON_REP_TEMPLATE(kHardwareComputerPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 20, 18,
R_CUBIC_TO, 1.1f, 0, 1.99f, -0.9f, 1.99f, -2,
LINE_TO, 22, 6,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
H_LINE_TO, 4,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 10,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
H_LINE_TO, 0,
R_V_LINE_TO, 2,
R_H_LINE_TO, 24,
R_V_LINE_TO, -2,
R_H_LINE_TO, -4,
CLOSE,
MOVE_TO, 4, 6,
R_H_LINE_TO, 16,
R_V_LINE_TO, 10,
H_LINE_TO, 4,
V_LINE_TO, 6,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kHardwareComputerRepList, kHardwareComputerIcon, {kHardwareComputerPath, std::size(kHardwareComputerPath)})
VECTOR_ICON_REP_TEMPLATE(kHardwareComputerSmallPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17.6f, 16.2f,
R_CUBIC_TO, 0.77f, 0, 1.39f, -0.63f, 1.39f, -1.4f,
R_LINE_TO, 0.01f, -7,
R_CUBIC_TO, 0, -0.77f, -0.63f, -1.4f, -1.4f, -1.4f,
H_LINE_TO, 6.4f,
R_CUBIC_TO, -0.77f, 0, -1.4f, 0.63f, -1.4f, 1.4f,
R_V_LINE_TO, 7,
R_CUBIC_TO, 0, 0.77f, 0.63f, 1.4f, 1.4f, 1.4f,
R_H_LINE_TO, 11.2f,
CLOSE,
R_MOVE_TO, -11.2f, -8.4f,
R_H_LINE_TO, 11.2f,
R_V_LINE_TO, 7,
H_LINE_TO, 6.4f,
R_V_LINE_TO, -7,
CLOSE,
R_MOVE_TO, -2.1f, 9.1f,
R_H_LINE_TO, 15.4f,
R_V_LINE_TO, 1.4f,
H_LINE_TO, 4.3f,
R_V_LINE_TO, -1.4f,
CLOSE,)
VECTOR_ICON_TEMPLATE_CC(kHardwareComputerSmallRepList, kHardwareComputerSmallIcon, {kHardwareComputerSmallPath, std::size(kHardwareComputerSmallPath)})
VECTOR_ICON_REP_TEMPLATE(kHardwareSmartphonePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17, 1.01f,
LINE_TO, 7, 1,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 18,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 10,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 3,
R_CUBIC_TO, 0, -1.1f, -0.9f, -1.99f, -2, -1.99f,
CLOSE,
MOVE_TO, 17, 19,
H_LINE_TO, 7,
V_LINE_TO, 5,
R_H_LINE_TO, 10,
R_V_LINE_TO, 14,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kHardwareSmartphoneRepList, kHardwareSmartphoneIcon, {kHardwareSmartphonePath, std::size(kHardwareSmartphonePath)})
VECTOR_ICON_REP_TEMPLATE(kHelpMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 7.99f, 12,
R_ARC_TO, 0.82f, 0.82f, 0, 0, 0, 0.6f, -0.25f,
R_ARC_TO, 0.82f, 0.82f, 0, 0, 0, 0.25f, -0.6f,
R_ARC_TO, 0.8f, 0.8f, 0, 0, 0, -0.25f, -0.59f,
R_ARC_TO, 0.82f, 0.82f, 0, 0, 0, -0.6f, -0.25f,
R_ARC_TO, 0.81f, 0.81f, 0, 0, 0, -0.6f, 0.25f,
R_ARC_TO, 0.8f, 0.8f, 0, 0, 0, -0.25f, 0.59f,
R_CUBIC_TO, 0, 0.24f, 0.08f, 0.43f, 0.25f, 0.6f,
R_ARC_TO, 0.81f, 0.81f, 0, 0, 0, 0.6f, 0.25f,
CLOSE,
R_MOVE_TO, -0.64f, -2.55f,
H_LINE_TO, 8.64f,
R_CUBIC_TO, 0, -0.4f, 0.04f, -0.69f, 0.11f, -0.86f,
R_CUBIC_TO, 0.07f, -0.18f, 0.24f, -0.39f, 0.51f, -0.64f,
R_CUBIC_TO, 0.38f, -0.37f, 0.64f, -0.69f, 0.79f, -0.96f,
R_CUBIC_TO, 0.15f, -0.27f, 0.22f, -0.57f, 0.22f, -0.9f,
R_CUBIC_TO, 0, -0.62f, -0.2f, -1.12f, -0.61f, -1.51f,
R_CUBIC_TO, -0.41f, -0.39f, -0.94f, -0.58f, -1.59f, -0.58f,
R_CUBIC_TO, -0.58f, 0, -1.08f, 0.16f, -1.51f, 0.47f,
R_CUBIC_TO, -0.43f, 0.31f, -0.73f, 0.73f, -0.9f, 1.26f,
R_LINE_TO, 1.17f, 0.48f,
R_CUBIC_TO, 0.11f, -0.32f, 0.26f, -0.56f, 0.47f, -0.73f,
R_CUBIC_TO, 0.21f, -0.17f, 0.45f, -0.25f, 0.72f, -0.25f,
R_CUBIC_TO, 0.28f, 0, 0.52f, 0.08f, 0.7f, 0.25f,
ARC_TO, 0.83f, 0.83f, 0, 0, 1, 9, 6.12f,
R_CUBIC_TO, 0, 0.24f, -0.08f, 0.46f, -0.25f, 0.66f,
R_ARC_TO, 6.56f, 6.56f, 0, 0, 1, -0.54f, 0.56f,
R_CUBIC_TO, -0.38f, 0.35f, -0.62f, 0.65f, -0.71f, 0.88f,
R_CUBIC_TO, -0.1f, 0.24f, -0.15f, 0.65f, -0.15f, 1.24f,
CLOSE,
MOVE_TO, 8, 14.53f,
R_ARC_TO, 6.32f, 6.32f, 0, 0, 1, -2.54f, -0.51f,
R_ARC_TO, 6.52f, 6.52f, 0, 0, 1, -2.08f, -1.4f,
R_ARC_TO, 6.52f, 6.52f, 0, 0, 1, -1.4f, -2.08f,
ARC_TO, 6.32f, 6.32f, 0, 0, 1, 1.47f, 8,
R_ARC_TO, 6.3f, 6.3f, 0, 0, 1, 0.51f, -2.55f,
ARC_TO, 6.53f, 6.53f, 0, 0, 1, 3.38f, 3.38f,
ARC_TO, 6.52f, 6.52f, 0, 0, 1, 5.46f, 1.98f,
ARC_TO, 6.32f, 6.32f, 0, 0, 1, 8, 1.47f,
R_ARC_TO, 6.3f, 6.3f, 0, 0, 1, 2.55f, 0.51f,
R_CUBIC_TO, 0.79f, 0.34f, 1.48f, 0.8f, 2.07f, 1.4f,
R_ARC_TO, 6.53f, 6.53f, 0, 0, 1, 1.4f, 2.07f,
ARC_TO, 6.3f, 6.3f, 0, 0, 1, 14.54f, 8,
R_CUBIC_TO, 0, 0.9f, -0.17f, 1.75f, -0.51f, 2.54f,
R_ARC_TO, 6.52f, 6.52f, 0, 0, 1, -1.4f, 2.08f,
R_ARC_TO, 6.53f, 6.53f, 0, 0, 1, -2.07f, 1.4f,
ARC_TO, 6.3f, 6.3f, 0, 0, 1, 8, 14.54f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_CUBIC_TO, 1.43f, 0, 2.65f, -0.5f, 3.65f, -1.5f,
R_CUBIC_TO, 1, -1, 1.5f, -2.22f, 1.5f, -3.65f,
R_CUBIC_TO, 0, -1.43f, -0.5f, -2.65f, -1.5f, -3.65f,
CUBIC_TO_SHORTHAND, 9.43f, 2.85f, 8, 2.85f,
R_CUBIC_TO, -1.43f, 0, -2.65f, 0.5f, -3.65f, 1.5f,
CUBIC_TO_SHORTHAND, 2.85f, 6.57f, 2.85f, 8,
R_CUBIC_TO, 0, 1.43f, 0.5f, 2.65f, 1.5f, 3.65f,
R_CUBIC_TO, 1, 1, 2.22f, 1.5f, 3.65f, 1.5f,
CLOSE,
MOVE_TO, 8, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kHelpMenuRepList, kHelpMenuIcon, {kHelpMenuPath, std::size(kHelpMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kHistoryPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 7.99f, 13.73f,
R_CUBIC_TO, -1.59f, 0, -2.94f, -0.55f, -4.06f, -1.67f,
CUBIC_TO, 2.81f, 10.95f, 2.26f, 9.59f, 2.26f, 8,
H_LINE_TO, 3.64f,
R_CUBIC_TO, 0, 1.2f, 0.43f, 2.22f, 1.28f, 3.07f,
R_CUBIC_TO, 0.85f, 0.85f, 1.88f, 1.28f, 3.07f, 1.28f,
R_CUBIC_TO, 1.2f, 0, 2.22f, -0.43f, 3.07f, -1.28f,
R_CUBIC_TO, 0.85f, -0.85f, 1.28f, -1.87f, 1.28f, -3.07f,
R_CUBIC_TO, 0, -1.2f, -0.43f, -2.22f, -1.28f, -3.07f,
R_CUBIC_TO, -0.85f, -0.85f, -1.87f, -1.28f, -3.07f, -1.28f,
R_CUBIC_TO, -0.66f, 0, -1.26f, 0.13f, -1.83f, 0.39f,
ARC_TO, 4.16f, 4.16f, 0, 0, 0, 4.73f, 5.13f,
R_H_LINE_TO, 1.67f,
R_V_LINE_TO, 1.26f,
H_LINE_TO, 2.3f,
V_LINE_TO, 2.3f,
R_H_LINE_TO, 1.26f,
R_V_LINE_TO, 2.11f,
ARC_TO, 5.75f, 5.75f, 0, 0, 1, 5.49f, 2.84f,
R_ARC_TO, 5.51f, 5.51f, 0, 0, 1, 2.5f, -0.57f,
R_CUBIC_TO, 0.79f, 0, 1.54f, 0.15f, 2.24f, 0.45f,
R_CUBIC_TO, 0.7f, 0.3f, 1.3f, 0.71f, 1.82f, 1.23f,
R_ARC_TO, 5.77f, 5.77f, 0, 0, 1, 1.23f, 1.82f,
R_CUBIC_TO, 0.31f, 0.7f, 0.45f, 1.44f, 0.45f, 2.24f,
R_CUBIC_TO, 0, 0.79f, -0.15f, 1.54f, -0.45f, 2.23f,
R_CUBIC_TO, -0.3f, 0.7f, -0.71f, 1.31f, -1.23f, 1.82f,
R_ARC_TO, 5.77f, 5.77f, 0, 0, 1, -1.82f, 1.23f,
R_ARC_TO, 5.51f, 5.51f, 0, 0, 1, -2.23f, 0.45f,
CLOSE,
R_MOVE_TO, 1.65f, -3.44f,
LINE_TO, 7.37f, 8.03f,
R_V_LINE_TO, -3.23f,
R_H_LINE_TO, 1.26f,
V_LINE_TO, 7.5f,
R_LINE_TO, 1.9f, 1.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kHistoryRepList, kHistoryIcon, {kHistoryPath, std::size(kHistoryPath)})
VECTOR_ICON_REP_TEMPLATE(kIncognitoPath, CANVAS_DIMENSIONS, 25,
CIRCLE, 12.5, 12.5, 12.5,
MOVE_TO, 16.41f, 13.37f,
R_ARC_TO, 3.04f, 3.04f, 0, 0, 1, 3.04f, 3.04f,
R_ARC_TO, 3.04f, 3.04f, 0, 0, 1, -3.04f, 3.04f,
R_CUBIC_TO, -1.59f, 0, -2.87f, -1.22f, -3.01f, -2.78f,
R_CUBIC_TO, -0.84f, -0.54f, -1.53f, -0.2f, -1.79f, -0.02f,
R_ARC_TO, 3.02f, 3.02f, 0, 0, 1, -3.01f, 2.8f,
R_ARC_TO, 3.04f, 3.04f, 0, 0, 1, -3.04f, -3.04f,
R_ARC_TO, 3.04f, 3.04f, 0, 0, 1, 3.04f, -3.04f,
R_ARC_TO, 3.04f, 3.04f, 0, 0, 1, 2.95f, 2.34f,
R_ARC_TO, 2.38f, 2.38f, 0, 0, 1, 1.91f, 0.01f,
R_ARC_TO, 3.03f, 3.03f, 0, 0, 1, 2.95f, -2.35f,
CLOSE,
R_MOVE_TO, -7.81f, 0.87f,
R_ARC_TO, 2.17f, 2.17f, 0, 1, 0, 0, 4.34f,
R_ARC_TO, 2.17f, 2.17f, 0, 0, 0, 0, -4.34f,
CLOSE,
R_MOVE_TO, 7.81f, 0,
R_ARC_TO, 2.17f, 2.17f, 0, 1, 0, 0, 4.34f,
R_ARC_TO, 2.17f, 2.17f, 0, 0, 0, 0, -4.34f,
CLOSE,
MOVE_TO, 20.31f, 11,
R_V_LINE_TO, 1,
H_LINE_TO, 4.69f,
R_V_LINE_TO, -1,
R_H_LINE_TO, 15.63f,
CLOSE,
R_MOVE_TO, -4.97f, -5.64f,
R_LINE_TO, 0.05f, 0.11f,
LINE_TO, 17.05f, 10,
H_LINE_TO, 7.95f,
R_LINE_TO, 1.65f, -4.53f,
R_ARC_TO, 0.87f, 0.87f, 0, 0, 1, 0.96f, -0.55f,
R_LINE_TO, 0.12f, 0.03f,
R_LINE_TO, 1.81f, 0.61f,
R_LINE_TO, 1.81f, -0.61f,
R_ARC_TO, 0.87f, 0.87f, 0, 0, 1, 1.03f, 0.41f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kIncognito24Path, CANVAS_DIMENSIONS, 24,
CIRCLE, 12, 12, 12,
MOVE_TO, 15.75f, 12.83f,
R_ARC_TO, 2.92f, 2.92f, 0, 0, 1, 2.92f, 2.92f,
R_ARC_TO, 2.92f, 2.92f, 0, 0, 1, -2.92f, 2.92f,
R_CUBIC_TO, -1.52f, 0, -2.76f, -1.17f, -2.89f, -2.67f,
R_CUBIC_TO, -0.81f, -0.52f, -1.47f, -0.19f, -1.72f, -0.02f,
R_ARC_TO, 2.9f, 2.9f, 0, 0, 1, -2.89f, 2.68f,
R_ARC_TO, 2.92f, 2.92f, 0, 0, 1, -2.92f, -2.92f,
R_ARC_TO, 2.92f, 2.92f, 0, 0, 1, 2.92f, -2.92f,
R_ARC_TO, 2.92f, 2.92f, 0, 0, 1, 2.83f, 2.25f,
R_ARC_TO, 2.29f, 2.29f, 0, 0, 1, 1.83f, 0.01f,
R_ARC_TO, 2.91f, 2.91f, 0, 0, 1, 2.83f, -2.26f,
CLOSE,
R_MOVE_TO, -7.5f, 0.83f,
R_ARC_TO, 2.08f, 2.08f, 0, 1, 0, 0, 4.17f,
R_ARC_TO, 2.08f, 2.08f, 0, 0, 0, 0, -4.17f,
CLOSE,
R_MOVE_TO, 7.5f, 0,
R_ARC_TO, 2.08f, 2.08f, 0, 1, 0, 0, 4.17f,
R_ARC_TO, 2.08f, 2.08f, 0, 0, 0, 0, -4.17f,
CLOSE,
MOVE_TO, 19.5f, 11,
R_V_LINE_TO, 1,
R_H_LINE_TO, -15,
R_V_LINE_TO, -1,
R_H_LINE_TO, 15,
CLOSE,
R_MOVE_TO, -4.77f, -5.85f,
R_LINE_TO, 0.05f, 0.1f,
R_LINE_TO, 1.6f, 4.25f,
R_LINE_TO, -8.74f, -0.01f,
LINE_TO, 9.22f, 5.25f,
R_ARC_TO, 0.84f, 0.84f, 0, 0, 1, 0.94f, -0.53f,
R_LINE_TO, 0.11f, 0.03f,
LINE_TO, 12, 5.33f,
R_LINE_TO, 1.73f, -0.58f,
R_CUBIC_TO, 0.39f, -0.13f, 0.81f, 0.05f, 1, 0.4f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kIncognito20Path, CANVAS_DIMENSIONS, 20,
CIRCLE, 10, 10, 10,
MOVE_TO, 13.13f, 10.69f,
R_CUBIC_TO, 1.34f, 0, 2.43f, 1.09f, 2.43f, 2.43f,
R_CUBIC_TO, 0, 1.34f, -1.09f, 2.43f, -2.43f, 2.43f,
R_CUBIC_TO, -1.27f, 0, -2.3f, -0.98f, -2.41f, -2.22f,
R_CUBIC_TO, -0.67f, -0.43f, -1.22f, -0.16f, -1.43f, -0.01f,
R_ARC_TO, 2.42f, 2.42f, 0, 0, 1, -2.41f, 2.24f,
R_CUBIC_TO, -1.34f, 0, -2.43f, -1.09f, -2.43f, -2.43f,
R_CUBIC_TO, 0, -1.34f, 1.09f, -2.43f, 2.43f, -2.43f,
R_CUBIC_TO, 1.15f, 0, 2.1f, 0.8f, 2.36f, 1.87f,
R_ARC_TO, 1.91f, 1.91f, 0, 0, 1, 1.53f, 0.01f,
R_ARC_TO, 2.43f, 2.43f, 0, 0, 1, 2.36f, -1.88f,
CLOSE,
R_MOVE_TO, -6.25f, 0.7f,
R_ARC_TO, 1.74f, 1.74f, 0, 1, 0, 0, 3.47f,
R_ARC_TO, 1.74f, 1.74f, 0, 0, 0, 0, -3.47f,
CLOSE,
R_MOVE_TO, 6.25f, 0,
R_ARC_TO, 1.74f, 1.74f, 0, 1, 0, 0, 3.47f,
R_ARC_TO, 1.74f, 1.74f, 0, 0, 0, 0, -3.47f,
CLOSE,
MOVE_TO, 16.25f, 9,
R_V_LINE_TO, 1,
H_LINE_TO, 3.75f,
V_LINE_TO, 9,
R_H_LINE_TO, 12.5f,
CLOSE,
R_MOVE_TO, -3.98f, -4.72f,
R_LINE_TO, 0.04f, 0.09f,
LINE_TO, 13.64f, 8,
H_LINE_TO, 6.36f,
LINE_TO, 7.68f, 4.38f,
R_CUBIC_TO, 0.12f, -0.32f, 0.45f, -0.5f, 0.77f, -0.44f,
R_LINE_TO, 0.1f, 0.02f,
LINE_TO, 10, 4.44f,
R_LINE_TO, 1.44f, -0.49f,
R_ARC_TO, 0.7f, 0.7f, 0, 0, 1, 0.83f, 0.33f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kIncognitoRepList, kIncognitoIcon, {kIncognitoPath, std::size(kIncognitoPath)}, {kIncognito24Path, std::size(kIncognito24Path)}, {kIncognito20Path, std::size(kIncognito20Path)})
VECTOR_ICON_REP_TEMPLATE(kIncognitoMenuArtPath, CANVAS_DIMENSIONS, 288,
MOVE_TO, 217, -1,
H_LINE_TO, 288,
V_LINE_TO, 80,
H_LINE_TO, 274.16f,
LINE_TO, 272.69f, 67.98f,
CUBIC_TO, 270.89f, 68.5f, 268.99f, 68.82f, 266.99f, 68.82f,
CUBIC_TO, 256.97f, 68.82f, 248.85f, 61.12f, 247.9f, 51.2f,
CUBIC_TO, 242.52f, 47.83f, 238.19f, 49.94f, 236.61f, 51.1f,
CUBIC_TO, 235.87f, 61.01f, 227.65f, 68.82f, 217.52f, 68.82f,
CUBIC_TO, 206.87f, 68.82f, 198.32f, 60.17f, 198.32f, 49.62f,
CUBIC_TO, 198.32f, 38.97f, 206.97f, 30.42f, 217.52f, 30.42f,
CUBIC_TO, 226.59f, 30.42f, 234.19f, 36.75f, 236.19f, 45.3f,
CUBIC_TO, 239.35f, 43.93f, 243.68f, 43.4f, 248.32f, 45.4f,
CUBIC_TO, 250.32f, 36.86f, 257.92f, 30.53f, 266.99f, 30.53f,
H_LINE_TO, 268.26f,
LINE_TO, 267.62f, 24.94f,
H_LINE_TO, 192.73f,
H_LINE_TO, 100.12f,
LINE_TO, 106.13f, 17.24f,
H_LINE_TO, 182.08f,
LINE_TO, 175.2f, -1,
H_LINE_TO, 200.5f,
LINE_TO, 194.63f, 19.35f,
H_LINE_TO, 266.99f,
LINE_TO, 266.36f, 13.86f,
H_LINE_TO, 211.3f,
LINE_TO, 217, -1,
CLOSE,
MOVE_TO, 70.9f, -1,
H_LINE_TO, 0,
V_LINE_TO, 80,
H_LINE_TO, 81.13f,
CUBIC_TO, 79.02f, 76.1f, 77.76f, 71.67f, 77.76f, 66.92f,
CUBIC_TO, 77.76f, 52.15f, 89.78f, 40.13f, 104.55f, 40.13f,
CUBIC_TO, 117.21f, 40.13f, 127.76f, 48.99f, 130.6f, 60.8f,
CUBIC_TO, 135.03f, 58.9f, 141.05f, 58.17f, 147.48f, 60.91f,
CUBIC_TO, 150.22f, 48.99f, 160.88f, 40.13f, 173.53f, 40.13f,
CUBIC_TO, 178.6f, 40.13f, 183.34f, 41.61f, 187.35f, 44.03f,
LINE_TO, 190.73f, 32.53f,
H_LINE_TO, 94.11f,
LINE_TO, 85.04f, 43.93f,
CUBIC_TO, 85.56f, 42.34f, 85.78f, 40.76f, 85.78f, 39.07f,
CUBIC_TO, 85.78f, 29.69f, 78.08f, 21.99f, 68.69f, 21.99f,
CUBIC_TO, 60.57f, 21.99f, 53.82f, 27.68f, 52.02f, 35.17f,
CUBIC_TO, 47.91f, 33.48f, 44.11f, 33.91f, 41.26f, 35.17f,
CUBIC_TO, 39.58f, 28.32f, 33.88f, 23.04f, 26.81f, 22.09f,
LINE_TO, 28.71f, 17.03f,
H_LINE_TO, 90.73f,
V_LINE_TO, 12.18f,
H_LINE_TO, 30.5f,
LINE_TO, 32.3f, 7.32f,
H_LINE_TO, 74.07f,
LINE_TO, 70.9f, -1,
CLOSE,
MOVE_TO, 90.52f, 80,
H_LINE_TO, 118.37f,
CUBIC_TO, 121.53f, 76.52f, 123.54f, 71.98f, 123.54f, 66.92f,
CUBIC_TO, 123.54f, 56.37f, 114.99f, 47.83f, 104.45f, 47.83f,
CUBIC_TO, 93.9f, 47.83f, 85.35f, 56.37f, 85.35f, 66.92f,
CUBIC_TO, 85.35f, 71.98f, 87.36f, 76.62f, 90.52f, 80,
CLOSE,
MOVE_TO, 127.65f, 80,
H_LINE_TO, 149.91f,
CUBIC_TO, 148.11f, 76.73f, 146.95f, 73.14f, 146.64f, 69.24f,
CUBIC_TO, 139.25f, 64.49f, 133.24f, 67.55f, 130.92f, 69.14f,
CUBIC_TO, 130.6f, 73.04f, 129.44f, 76.73f, 127.65f, 80,
CLOSE,
MOVE_TO, 159.19f, 80,
H_LINE_TO, 176.91f,
LINE_TO, 184.72f, 51.73f,
CUBIC_TO, 181.45f, 49.31f, 177.44f, 47.83f, 173.11f, 47.83f,
CUBIC_TO, 162.56f, 47.83f, 154.02f, 56.37f, 154.02f, 66.92f,
CUBIC_TO, 154.02f, 71.98f, 156.03f, 76.62f, 159.19f, 80,
CLOSE,
MOVE_TO, 80.92f, 39.07f,
CUBIC_TO, 80.92f, 45.83f, 75.45f, 51.31f, 68.69f, 51.31f,
CUBIC_TO, 61.93f, 51.31f, 56.45f, 45.83f, 56.45f, 39.07f,
CUBIC_TO, 56.45f, 32.32f, 61.93f, 26.84f, 68.69f, 26.84f,
CUBIC_TO, 75.45f, 26.84f, 80.92f, 32.32f, 80.92f, 39.07f,
CLOSE,
MOVE_TO, 51.71f, 40.55f,
CUBIC_TO, 46.96f, 37.49f, 43.06f, 39.39f, 41.58f, 40.45f,
CUBIC_TO, 40.95f, 49.2f, 33.56f, 56.16f, 24.6f, 56.16f,
CUBIC_TO, 21.12f, 56.16f, 17.85f, 55.11f, 15.1f, 53.31f,
LINE_TO, 5.29f, 80,
H_LINE_TO, 56.45f,
LINE_TO, 77.13f, 53.95f,
CUBIC_TO, 74.59f, 55.42f, 71.75f, 56.16f, 68.69f, 56.16f,
CUBIC_TO, 59.72f, 56.16f, 52.44f, 49.31f, 51.71f, 40.55f,
CLOSE,
MOVE_TO, 36.83f, 39.07f,
CUBIC_TO, 36.83f, 32.43f, 31.56f, 27.05f, 25.02f, 26.84f,
LINE_TO, 16.9f, 48.57f,
CUBIC_TO, 19.01f, 50.25f, 21.64f, 51.31f, 24.6f, 51.31f,
CUBIC_TO, 31.35f, 51.31f, 36.83f, 45.82f, 36.83f, 39.07f,
CLOSE,
MOVE_TO, 231.23f, 49.62f,
CUBIC_TO, 231.23f, 57.2f, 225.09f, 63.33f, 217.52f, 63.33f,
CUBIC_TO, 209.95f, 63.33f, 203.81f, 57.2f, 203.81f, 49.62f,
CUBIC_TO, 203.81f, 42.05f, 209.95f, 35.91f, 217.52f, 35.91f,
CUBIC_TO, 225.09f, 35.91f, 231.23f, 42.05f, 231.23f, 49.62f,
CLOSE,
MOVE_TO, 253.28f, 49.62f,
CUBIC_TO, 253.28f, 57.22f, 259.39f, 63.33f, 266.99f, 63.33f,
CUBIC_TO, 268.78f, 63.33f, 270.47f, 63.02f, 272.05f, 62.39f,
LINE_TO, 268.89f, 36.02f,
CUBIC_TO, 268.26f, 35.91f, 267.62f, 35.91f, 266.88f, 35.91f,
CUBIC_TO, 259.39f, 35.8f, 253.28f, 42.03f, 253.28f, 49.62f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kIncognitoMenuArtRepList, kIncognitoMenuArtIcon, {kIncognitoMenuArtPath, std::size(kIncognitoMenuArtPath)})
VECTOR_ICON_REP_TEMPLATE(kIncognitoProfilePath, CANVAS_DIMENSIONS, 40,
CIRCLE, 20, 20, 20,
MOVE_TO, 25.5f, 21.64f,
CUBIC_TO, 27.86f, 21.64f, 29.78f, 23.52f, 29.78f, 25.82f,
CUBIC_TO, 29.78f, 28.13f, 27.86f, 30, 25.5f, 30,
CUBIC_TO, 23.26f, 30, 21.45f, 28.32f, 21.26f, 26.18f,
CUBIC_TO, 20.07f, 25.44f, 19.11f, 25.9f, 18.74f, 26.16f,
CUBIC_TO, 18.57f, 28.3f, 16.75f, 30, 14.5f, 30,
CUBIC_TO, 12.14f, 30, 10.22f, 28.13f, 10.22f, 25.82f,
CUBIC_TO, 10.22f, 23.52f, 12.14f, 21.64f, 14.5f, 21.64f,
CUBIC_TO, 16.52f, 21.64f, 18.2f, 23.02f, 18.66f, 24.87f,
CUBIC_TO, 19.36f, 24.57f, 20.32f, 24.45f, 21.34f, 24.88f,
CUBIC_TO, 21.78f, 23.03f, 23.47f, 21.64f, 25.5f, 21.64f,
CLOSE,
MOVE_TO, 14.5f, 22.84f,
CUBIC_TO, 12.81f, 22.84f, 11.44f, 24.17f, 11.44f, 25.82f,
CUBIC_TO, 11.44f, 27.47f, 12.81f, 28.81f, 14.5f, 28.81f,
CUBIC_TO, 16.19f, 28.81f, 17.56f, 27.47f, 17.56f, 25.82f,
CUBIC_TO, 17.56f, 24.17f, 16.19f, 22.84f, 14.5f, 22.84f,
CLOSE,
MOVE_TO, 25.5f, 22.84f,
CUBIC_TO, 23.81f, 22.84f, 22.44f, 24.17f, 22.44f, 25.82f,
CUBIC_TO, 22.44f, 27.47f, 23.81f, 28.81f, 25.5f, 28.81f,
CUBIC_TO, 27.19f, 28.81f, 28.56f, 27.47f, 28.56f, 25.82f,
CUBIC_TO, 28.56f, 24.17f, 27.19f, 22.84f, 25.5f, 22.84f,
CLOSE,
MOVE_TO, 31, 19.02f,
LINE_TO, 31, 20.5f,
LINE_TO, 9, 20.5f,
LINE_TO, 9, 19.02f,
LINE_TO, 31, 19.02f,
CLOSE,
MOVE_TO, 24.02f, 10.66f,
LINE_TO, 24.07f, 10.78f,
LINE_TO, 26.5f, 17,
LINE_TO, 13.5f, 17,
LINE_TO, 15.92f, 10.78f,
CUBIC_TO, 16.13f, 10.22f, 16.73f, 9.9f, 17.32f, 10.03f,
LINE_TO, 17.45f, 10.06f,
LINE_TO, 20, 10.9f,
LINE_TO, 22.54f, 10.06f,
CUBIC_TO, 23.12f, 9.87f, 23.75f, 10.14f, 24.02f, 10.66f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kIncognitoProfileRepList, kIncognitoProfileIcon, {kIncognitoProfilePath, std::size(kIncognitoProfilePath)})
VECTOR_ICON_REP_TEMPLATE(kIncognitoRefreshMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.16f, 12.83f,
R_ARC_TO, 1.95f, 1.95f, 0, 0, 1, -1.44f, -0.59f,
R_ARC_TO, 1.96f, 1.96f, 0, 0, 1, -0.59f, -1.44f,
R_CUBIC_TO, 0, -0.57f, 0.2f, -1.05f, 0.59f, -1.44f,
R_CUBIC_TO, 0.39f, -0.39f, 0.87f, -0.59f, 1.44f, -0.59f,
R_CUBIC_TO, 0.42f, 0, 0.8f, 0.12f, 1.14f, 0.36f,
R_CUBIC_TO, 0.34f, 0.24f, 0.59f, 0.55f, 0.75f, 0.93f,
R_CUBIC_TO, 0.14f, -0.06f, 0.29f, -0.11f, 0.44f, -0.13f,
R_CUBIC_TO, 0.16f, -0.03f, 0.32f, -0.04f, 0.47f, -0.04f,
R_ARC_TO, 2.43f, 2.43f, 0, 0, 1, 0.92f, 0.18f,
R_ARC_TO, 2.1f, 2.1f, 0, 0, 1, 0.74f, -0.93f,
R_CUBIC_TO, 0.34f, -0.24f, 0.72f, -0.36f, 1.14f, -0.36f,
R_CUBIC_TO, 0.57f, 0, 1.04f, 0.2f, 1.44f, 0.59f,
R_CUBIC_TO, 0.4f, 0.4f, 0.59f, 0.88f, 0.59f, 1.44f,
R_CUBIC_TO, 0, 0.57f, -0.2f, 1.05f, -0.59f, 1.44f,
R_CUBIC_TO, -0.39f, 0.4f, -0.87f, 0.59f, -1.44f, 0.59f,
R_CUBIC_TO, -0.43f, 0, -0.82f, -0.12f, -1.16f, -0.36f,
R_ARC_TO, 1.89f, 1.89f, 0, 0, 1, -0.73f, -0.95f,
R_ARC_TO, 1.25f, 1.25f, 0, 0, 0, -1.41f, -0.28f,
R_CUBIC_TO, -0.16f, 0.07f, -0.29f, 0.16f, -0.41f, 0.28f,
R_CUBIC_TO, -0.14f, 0.39f, -0.38f, 0.71f, -0.73f, 0.95f,
R_ARC_TO, 2, 2, 0, 0, 1, -1.16f, 0.36f,
CLOSE,
R_MOVE_TO, 0, -1.27f,
R_CUBIC_TO, 0.21f, 0, 0.4f, -0.07f, 0.54f, -0.22f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, 0.23f, -0.54f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, -0.23f, -0.54f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 0, -0.54f, -0.23f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 0, -0.54f, 0.23f,
R_ARC_TO, 0.75f, 0.75f, 0, 0, 0, -0.23f, 0.54f,
R_CUBIC_TO, 0, 0.21f, 0.08f, 0.39f, 0.23f, 0.54f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, 0.54f, 0.22f,
CLOSE,
R_MOVE_TO, 5.6f, 0,
R_ARC_TO, 0.75f, 0.75f, 0, 0, 0, 0.54f, -0.22f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, 0.23f, -0.54f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 0, -0.23f, -0.54f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, -0.54f, -0.23f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 0, -0.54f, 0.23f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 0, -0.23f, 0.54f,
R_CUBIC_TO, 0, 0.21f, 0.08f, 0.39f, 0.23f, 0.54f,
R_ARC_TO, 0.74f, 0.74f, 0, 0, 0, 0.54f, 0.22f,
CLOSE,
MOVE_TO, 2.3f, 8,
V_LINE_TO, 6.68f,
R_H_LINE_TO, 2.04f,
LINE_TO, 5.59f, 2.78f,
R_CUBIC_TO, 0.1f, -0.24f, 0.25f, -0.41f, 0.45f, -0.52f,
R_CUBIC_TO, 0.2f, -0.11f, 0.42f, -0.14f, 0.68f, -0.08f,
R_LINE_TO, 1.25f, 0.33f,
R_LINE_TO, 1.25f, -0.33f,
R_ARC_TO, 0.91f, 0.91f, 0, 0, 1, 0.67f, 0.08f,
R_CUBIC_TO, 0.2f, 0.11f, 0.35f, 0.28f, 0.45f, 0.52f,
R_LINE_TO, 1.26f, 3.9f,
R_H_LINE_TO, 2.12f,
V_LINE_TO, 8,
CLOSE,
R_MOVE_TO, 3.5f, -1.32f,
R_H_LINE_TO, 4.33f,
R_LINE_TO, -0.99f, -3.05f,
R_LINE_TO, -1.18f, 0.31f,
R_LINE_TO, -1.18f, -0.31f,
CLOSE,
R_MOVE_TO, 2.18f, -0.07f,
R_H_LINE_TO, 2.19f,
R_H_LINE_TO, -4.39f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kIncognitoRefreshMenuRepList, kIncognitoRefreshMenuIcon, {kIncognitoRefreshMenuPath, std::size(kIncognitoRefreshMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kInfoPath, CANVAS_DIMENSIONS, 960,
FILL_RULE_NONZERO,
MOVE_TO, 440, 680,
R_H_LINE_TO, 80,
R_V_LINE_TO, -240,
R_H_LINE_TO, -80,
R_V_LINE_TO, 240,
CLOSE,
R_MOVE_TO, 40, -320,
R_QUADRATIC_TO, 17, 0, 28.5f, -11.5f,
QUADRATIC_TO_SHORTHAND, 520, 320,
R_QUADRATIC_TO, 0, -17, -11.5f, -28.5f,
QUADRATIC_TO_SHORTHAND, 480, 280,
R_QUADRATIC_TO, -17, 0, -28.5f, 11.5f,
QUADRATIC_TO_SHORTHAND, 440, 320,
R_QUADRATIC_TO, 0, 17, 11.5f, 28.5f,
QUADRATIC_TO_SHORTHAND, 480, 360,
CLOSE,
R_MOVE_TO, 0, 520,
R_QUADRATIC_TO, -83, 0, -156, -31.5f,
QUADRATIC_TO_SHORTHAND, 197, 763,
R_QUADRATIC_TO, -54, -54, -85.5f, -127,
QUADRATIC_TO_SHORTHAND, 80, 480,
R_QUADRATIC_TO, 0, -83, 31.5f, -156,
QUADRATIC_TO_SHORTHAND, 197, 197,
R_QUADRATIC_TO, 54, -54, 127, -85.5f,
QUADRATIC_TO_SHORTHAND, 480, 80,
R_QUADRATIC_TO, 83, 0, 156, 31.5f,
QUADRATIC_TO_SHORTHAND, 763, 197,
R_QUADRATIC_TO, 54, 54, 85.5f, 127,
QUADRATIC_TO_SHORTHAND, 880, 480,
R_QUADRATIC_TO, 0, 83, -31.5f, 156,
QUADRATIC_TO_SHORTHAND, 763, 763,
R_QUADRATIC_TO, -54, 54, -127, 85.5f,
QUADRATIC_TO_SHORTHAND, 480, 880,
CLOSE,
R_MOVE_TO, 0, -80,
R_QUADRATIC_TO, 134, 0, 227, -93,
R_QUADRATIC_TO, 93, -93, 93, -227,
R_QUADRATIC_TO, 0, -134, -93, -227,
R_QUADRATIC_TO, -93, -93, -227, -93,
R_QUADRATIC_TO, -134, 0, -227, 93,
R_QUADRATIC_TO, -93, 93, -93, 227,
R_QUADRATIC_TO, 0, 134, 93, 227,
R_QUADRATIC_TO, 93, 93, 227, 93,
CLOSE,
R_MOVE_TO, 0, -320,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kInfoRepList, kInfoIcon, {kInfoPath, std::size(kInfoPath)})
VECTOR_ICON_REP_TEMPLATE(kInputPath, CANVAS_DIMENSIONS, 960,
FILL_RULE_NONZERO,
MOVE_TO, 160, 800,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 80, 720,
R_V_LINE_TO, -120,
R_H_LINE_TO, 80,
R_V_LINE_TO, 120,
R_H_LINE_TO, 640,
R_V_LINE_TO, -480,
H_LINE_TO, 160,
R_V_LINE_TO, 120,
H_LINE_TO, 80,
R_V_LINE_TO, -120,
R_QUADRATIC_TO, 0, -33, 23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 160, 160,
R_H_LINE_TO, 640,
R_QUADRATIC_TO, 33, 0, 56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 880, 240,
R_V_LINE_TO, 480,
R_QUADRATIC_TO, 0, 33, -23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 800, 800,
H_LINE_TO, 160,
CLOSE,
R_MOVE_TO, 300, -140,
R_LINE_TO, -56, -58,
R_LINE_TO, 83, -82,
H_LINE_TO, 80,
R_V_LINE_TO, -80,
R_H_LINE_TO, 407,
R_LINE_TO, -83, -82,
R_LINE_TO, 56, -58,
R_LINE_TO, 180, 180,
R_LINE_TO, -180, 180,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kInputRepList, kInputIcon, {kInputPath, std::size(kInputPath)})
VECTOR_ICON_REP_TEMPLATE(kInstallDesktopChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 7, 17,
R_V_LINE_TO, -2,
H_LINE_TO, 3.5f,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.45f, 1.45f, 0, 0, 1, 2, 13.5f,
V_LINE_TO, 4.49f,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.76f, 0.44f, -1.05f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 3.5f, 3,
H_LINE_TO, 11,
R_V_LINE_TO, 1.5f,
H_LINE_TO, 3.5f,
R_V_LINE_TO, 9,
R_H_LINE_TO, 13,
V_LINE_TO, 11,
H_LINE_TO, 18,
R_V_LINE_TO, 2.5f,
R_CUBIC_TO, 0, 0.42f, -0.15f, 0.77f, -0.44f, 1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 16.5f, 15,
H_LINE_TO, 13,
R_V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 7.48f, -6,
LINE_TO, 11, 7.52f,
R_LINE_TO, 1.06f, -1.06f,
R_LINE_TO, 1.69f, 1.69f,
V_LINE_TO, 3,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 5.11f,
R_LINE_TO, 1.69f, -1.69f,
LINE_TO, 18, 7.48f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kInstallDesktopChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.53f, 13.73f,
R_V_LINE_TO, -1.6f,
R_H_LINE_TO, -2.68f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 5.95f,
R_V_LINE_TO, 1.38f,
H_LINE_TO, 2.85f,
R_V_LINE_TO, 7.11f,
R_H_LINE_TO, 10.3f,
V_LINE_TO, 8.68f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 2.07f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_ARC_TO, 1.35f, 1.35f, 0, 0, 1, -0.98f, 0.4f,
R_H_LINE_TO, -2.68f,
R_V_LINE_TO, 1.6f,
CLOSE,
R_MOVE_TO, 6.06f, -5,
LINE_TO, 8.76f, 5.91f,
R_LINE_TO, 0.89f, -0.89f,
R_LINE_TO, 1.32f, 1.32f,
V_LINE_TO, 2.27f,
R_H_LINE_TO, 1.26f,
R_V_LINE_TO, 4.04f,
R_LINE_TO, 1.32f, -1.32f,
R_LINE_TO, 0.9f, 0.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kInstallDesktopChromeRefreshRepList, kInstallDesktopChromeRefreshIcon, {kInstallDesktopChromeRefreshPath, std::size(kInstallDesktopChromeRefreshPath)}, {kInstallDesktopChromeRefresh16Path, std::size(kInstallDesktopChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kJourneysPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 19, 15,
R_CUBIC_TO, -1.3f, 0, -2.4f, 0.84f, -2.82f, 2,
H_LINE_TO, 11,
R_CUBIC_TO, -1.1f, 0, -2, -0.9f, -2, -2,
R_CUBIC_TO, 0, -1.1f, 0.9f, -2, 2, -2,
R_H_LINE_TO, 2,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
H_LINE_TO, 7.82f,
CUBIC_TO, 7.4f, 3.84f, 6.3f, 3, 5, 3,
CUBIC_TO, 3.34f, 3, 2, 4.34f, 2, 6,
R_CUBIC_TO, 0, 1.66f, 1.34f, 3, 3, 3,
R_CUBIC_TO, 1.3f, 0, 2.4f, -0.84f, 2.82f, -2,
H_LINE_TO, 13,
R_CUBIC_TO, 1.1f, 0, 2, 0.9f, 2, 2,
R_CUBIC_TO, 0, 1.1f, -0.9f, 2, -2, 2,
R_H_LINE_TO, -2,
R_CUBIC_TO, -2.21f, 0, -4, 1.79f, -4, 4,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 5.18f,
ARC_TO, 3, 3, 0, 0, 0, 22, 18,
R_CUBIC_TO, 0, -1.66f, -1.34f, -3, -3, -3,
CLOSE,
MOVE_TO, 5, 7,
R_CUBIC_TO, -0.55f, 0, -1, -0.45f, -1, -1,
R_CUBIC_TO, 0, -0.55f, 0.45f, -1, 1, -1,
R_CUBIC_TO, 0.55f, 0, 1, 0.45f, 1, 1,
R_CUBIC_TO, 0, 0.55f, -0.45f, 1, -1, 1,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kJourneysRepList, kJourneysIcon, {kJourneysPath, std::size(kJourneysPath)})
VECTOR_ICON_REP_TEMPLATE(kKeepPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 10, 18.31f,
R_LINE_TO, -0.86f, -0.87f,
R_V_LINE_TO, -4.29f,
H_LINE_TO, 4.33f,
V_LINE_TO, 11.28f,
R_LINE_TO, 1.5f, -2.1f,
V_LINE_TO, 3.54f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.51f, -1.22f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, 1.22f, -0.5f,
R_H_LINE_TO, 4.88f,
R_CUBIC_TO, 0.47f, 0, 0.88f, 0.17f, 1.22f, 0.5f,
R_CUBIC_TO, 0.34f, 0.34f, 0.51f, 0.75f, 0.51f, 1.22f,
R_V_LINE_TO, 5.64f,
R_LINE_TO, 1.5f, 2.11f,
R_V_LINE_TO, 1.86f,
R_H_LINE_TO, -4.8f,
R_V_LINE_TO, 4.29f,
CLOSE,
R_MOVE_TO, -3.64f, -6.89f,
R_H_LINE_TO, 7.27f,
LINE_TO, 12.44f, 9.72f,
R_V_LINE_TO, -6.18f,
H_LINE_TO, 7.56f,
R_V_LINE_TO, 6.18f,
CLOSE,
R_MOVE_TO, 3.64f, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeepRepList, kKeepIcon, {kKeepPath, std::size(kKeepPath)})
VECTOR_ICON_REP_TEMPLATE(kKeepFilledPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10, 18.3f,
LINE_TO, 9.14f, 17.44f,
LINE_TO, 9.14f, 13.14f,
LINE_TO, 4.33f, 13.14f,
LINE_TO, 4.33f, 11.28f,
LINE_TO, 5.83f, 9.18f,
LINE_TO, 5.83f, 3.54f,
CUBIC_TO, 5.83f, 3.06f, 6, 2.65f, 6.34f, 2.31f,
CUBIC_TO, 6.68f, 1.98f, 7.09f, 1.81f, 7.56f, 1.81f,
LINE_TO, 12.44f, 1.81f,
CUBIC_TO, 12.91f, 1.81f, 13.32f, 1.98f, 13.66f, 2.31f,
CUBIC_TO, 14, 2.65f, 14.17f, 3.06f, 14.17f, 3.54f,
LINE_TO, 14.17f, 9.18f,
LINE_TO, 15.67f, 11.28f,
LINE_TO, 15.67f, 13.14f,
LINE_TO, 10.86f, 13.14f,
LINE_TO, 10.86f, 17.44f,
CLOSE,
MOVE_TO, 10, 18.3f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeepFilledRepList, kKeepFilledIcon, {kKeepFilledPath, std::size(kKeepFilledPath)})
VECTOR_ICON_REP_TEMPLATE(kKeepOffPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 11.2f, 2.4f,
R_V_LINE_TO, 1.2f,
R_H_LINE_TO, -0.8f,
R_V_LINE_TO, 5.12f,
R_LINE_TO, -1.2f, -1.2f,
V_LINE_TO, 3.6f,
H_LINE_TO, 6.8f,
R_V_LINE_TO, 1.52f,
R_LINE_TO, -1.5f, -1.5f,
R_LINE_TO, -0.5f, -0.52f,
R_V_LINE_TO, -0.7f,
R_H_LINE_TO, 6.4f,
CLOSE,
MOVE_TO, 8, 15.2f,
R_LINE_TO, -0.6f, -0.6f,
R_V_LINE_TO, -3.8f,
H_LINE_TO, 4,
V_LINE_TO, 9.6f,
LINE_TO, 5.6f, 8,
R_V_LINE_TO, -0.7f,
LINE_TO, 1.5f, 3.2f,
R_LINE_TO, 0.85f, -0.85f,
R_LINE_TO, 11.3f, 11.32f,
R_LINE_TO, -0.85f, 0.85f,
LINE_TO, 9.1f, 10.8f,
R_H_LINE_TO, -0.5f,
R_V_LINE_TO, 3.8f,
R_LINE_TO, -0.6f, 0.6f,
CLOSE,
MOVE_TO, 5.7f, 9.6f,
R_H_LINE_TO, 2.2f,
LINE_TO, 6.8f, 8.5f,
LINE_TO, 5.7f, 9.6f,
CLOSE,
NEW_PATH)
VECTOR_ICON_TEMPLATE_CC(kKeepOffRepList, kKeepOffIcon, {kKeepOffPath, std::size(kKeepOffPath)})
VECTOR_ICON_REP_TEMPLATE(kKeyPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 17.42f, 12,
H_LINE_TO, 30,
R_V_LINE_TO, 5,
R_H_LINE_TO, -2,
R_V_LINE_TO, 5,
R_H_LINE_TO, -4,
R_V_LINE_TO, -5,
R_H_LINE_TO, -6.25f,
CUBIC_TO, 16.86f, 20.45f, 13.73f, 23, 10, 23,
R_CUBIC_TO, -4.42f, 0, -8, -3.58f, -8, -8,
R_CUBIC_TO, 0, -4.42f, 3.58f, -8, 8, -8,
R_CUBIC_TO, 3.36f, 0, 6.23f, 2.07f, 7.42f, 5,
CLOSE,
MOVE_TO, 10, 18,
R_CUBIC_TO, 1.66f, 0, 3, -1.34f, 3, -3,
R_CUBIC_TO, 0, -1.66f, -1.34f, -3, -3, -3,
R_CUBIC_TO, -1.66f, 0, -3, 1.34f, -3, 3,
R_CUBIC_TO, 0, 1.66f, 1.34f, 3, 3, 3,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kKey16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8.41f, 6,
CUBIC_TO, 7.89f, 4.44f, 6.48f, 3.33f, 4.82f, 3.33f,
CUBIC_TO, 2.71f, 3.33f, 1, 5.12f, 1, 7.33f,
CUBIC_TO, 1, 9.54f, 2.71f, 11.33f, 4.82f, 11.33f,
CUBIC_TO, 6.48f, 11.33f, 7.89f, 10.22f, 8.41f, 8.66f,
LINE_TO, 11.18f, 8.66f,
LINE_TO, 11.18f, 11.33f,
LINE_TO, 13.73f, 11.33f,
LINE_TO, 13.73f, 8.66f,
LINE_TO, 15, 8.66f,
LINE_TO, 15, 6,
LINE_TO, 8.41f, 6,
CLOSE,
MOVE_TO, 4.82f, 8.67f,
CUBIC_TO, 4.12f, 8.67f, 3.55f, 8.08f, 3.55f, 7.34f,
CUBIC_TO, 3.55f, 6.6f, 4.12f, 6.01f, 4.82f, 6.01f,
CUBIC_TO, 5.52f, 6.01f, 6.09f, 6.6f, 6.09f, 7.34f,
CUBIC_TO, 6.09f, 8.08f, 5.52f, 8.67f, 4.82f, 8.67f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyRepList, kKeyIcon, {kKeyPath, std::size(kKeyPath)}, {kKey16Path, std::size(kKey16Path)})
VECTOR_ICON_REP_TEMPLATE(kKeyboardArrowDownPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 7.41f, 8.59f,
LINE_TO, 12, 13.17f,
R_LINE_TO, 4.59f, -4.58f,
LINE_TO, 18, 10,
R_LINE_TO, -6, 6,
R_LINE_TO, -6, -6,
R_LINE_TO, 1.41f, -1.41f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyboardArrowDownRepList, kKeyboardArrowDownIcon, {kKeyboardArrowDownPath, std::size(kKeyboardArrowDownPath)})
VECTOR_ICON_REP_TEMPLATE(kKeyboardArrowDownChromeRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 10, 13.22f,
R_LINE_TO, -5.16f, -5.16f,
LINE_TO, 6.06f, 6.84f,
LINE_TO, 10, 10.78f,
R_LINE_TO, 3.94f, -3.94f,
R_LINE_TO, 1.22f, 1.22f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyboardArrowDownChromeRefreshRepList, kKeyboardArrowDownChromeRefreshIcon, {kKeyboardArrowDownChromeRefreshPath, std::size(kKeyboardArrowDownChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kKeyboardArrowRightPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 9.71f, 18.71f,
R_LINE_TO, -1.42f, -1.42f,
R_LINE_TO, 5.3f, -5.29f,
R_LINE_TO, -5.3f, -5.29f,
R_LINE_TO, 1.42f, -1.42f,
R_LINE_TO, 6.7f, 6.71f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyboardArrowRightRepList, kKeyboardArrowRightIcon, {kKeyboardArrowRightPath, std::size(kKeyboardArrowRightPath)})
VECTOR_ICON_REP_TEMPLATE(kKeyboardArrowUpPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 7.41f, 15.41f,
LINE_TO, 12, 10.83f,
R_LINE_TO, 4.59f, 4.58f,
LINE_TO, 18, 14,
R_LINE_TO, -6, -6,
R_LINE_TO, -6, 6,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyboardArrowUpRepList, kKeyboardArrowUpIcon, {kKeyboardArrowUpPath, std::size(kKeyboardArrowUpPath)})
VECTOR_ICON_REP_TEMPLATE(kKeyboardArrowUpChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 6.06f, 13.16f,
LINE_TO, 4.84f, 11.94f,
LINE_TO, 10, 6.78f,
R_LINE_TO, 5.16f, 5.16f,
R_LINE_TO, -1.22f, 1.22f,
LINE_TO, 10, 9.22f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kKeyboardArrowUpChromeRefreshRepList, kKeyboardArrowUpChromeRefreshIcon, {kKeyboardArrowUpChromeRefreshPath, std::size(kKeyboardArrowUpChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kLaptopPath, MOVE_TO, 40, 36,
R_CUBIC_TO, 2.2f, 0, 4, -1.8f, 4, -4,
V_LINE_TO, 12,
R_CUBIC_TO, 0, -2.2f, -1.8f, -4, -4, -4,
H_LINE_TO, 8,
R_CUBIC_TO, -2.2f, 0, -4, 1.8f, -4, 4,
R_V_LINE_TO, 20,
R_CUBIC_TO, 0, 2.2f, 1.8f, 4, 4, 4,
H_LINE_TO, 0,
R_V_LINE_TO, 4,
R_H_LINE_TO, 48,
R_V_LINE_TO, -4,
R_H_LINE_TO, -8,
CLOSE,
MOVE_TO, 8, 12,
R_H_LINE_TO, 32,
R_V_LINE_TO, 20,
H_LINE_TO, 8,
V_LINE_TO, 12,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kLaptopRepList, kLaptopIcon, {kLaptopPath, std::size(kLaptopPath)})
VECTOR_ICON_REP_TEMPLATE(kLeadingScrollPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 7.03f, 8,
LINE_TO, 12, 3.4f,
LINE_TO, 10.49f, 2,
LINE_TO, 4, 8,
R_LINE_TO, 6.49f, 6,
LINE_TO, 12, 12.6f,
LINE_TO, 7.03f, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kLeadingScrollRepList, kLeadingScrollIcon, {kLeadingScrollPath, std::size(kLeadingScrollPath)})
VECTOR_ICON_REP_TEMPLATE(kLeftPanelClosePath, CANVAS_DIMENSIONS, 20,
FLIPS_IN_RTL,
MOVE_TO, 13.5f, 13.25f,
V_LINE_TO, 6.75f,
LINE_TO, 10, 10,
LINE_TO, 13.5f, 13.25f,
CLOSE,
MOVE_TO, 4.4f, 17,
CUBIC_TO, 4.01f, 17, 3.68f, 16.86f, 3.41f, 16.59f,
CUBIC_TO, 3.14f, 16.32f, 3, 15.99f, 3, 15.6f,
V_LINE_TO, 4.4f,
CUBIC_TO, 3, 4.01f, 3.14f, 3.68f, 3.41f, 3.41f,
CUBIC_TO, 3.68f, 3.14f, 4.01f, 3, 4.4f, 3,
H_LINE_TO, 15.6f,
CUBIC_TO, 15.99f, 3, 16.32f, 3.14f, 16.59f, 3.41f,
CUBIC_TO, 16.86f, 3.68f, 17, 4.01f, 17, 4.4f,
V_LINE_TO, 15.6f,
CUBIC_TO, 17, 15.99f, 16.86f, 16.32f, 16.59f, 16.59f,
CUBIC_TO, 16.32f, 16.86f, 15.99f, 17, 15.6f, 17,
H_LINE_TO, 4.4f,
CLOSE,
MOVE_TO, 7, 15.5f,
V_LINE_TO, 4.5f,
H_LINE_TO, 4.5f,
V_LINE_TO, 15.5f,
H_LINE_TO, 7,
CLOSE,
MOVE_TO, 8.5f, 15.5f,
H_LINE_TO, 15.5f,
V_LINE_TO, 4.5f,
H_LINE_TO, 8.5f,
V_LINE_TO, 15.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kLeftPanelCloseRepList, kLeftPanelCloseIcon, {kLeftPanelClosePath, std::size(kLeftPanelClosePath)})
VECTOR_ICON_REP_TEMPLATE(kLinkChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 9, 14,
H_LINE_TO, 6,
R_CUBIC_TO, -1.1f, 0, -2.05f, -0.39f, -2.83f, -1.17f,
CUBIC_TO, 2.39f, 12.05f, 2, 11.1f, 2, 10,
R_CUBIC_TO, 0, -1.11f, 0.39f, -2.05f, 1.17f, -2.83f,
CUBIC_TO, 3.95f, 6.39f, 4.89f, 6, 6, 6,
R_H_LINE_TO, 3,
R_V_LINE_TO, 1.5f,
H_LINE_TO, 6,
R_CUBIC_TO, -0.69f, 0, -1.28f, 0.24f, -1.77f, 0.73f,
ARC_TO, 2.4f, 2.4f, 0, 0, 0, 3.5f, 10,
R_CUBIC_TO, 0, 0.7f, 0.24f, 1.29f, 0.73f, 1.77f,
R_CUBIC_TO, 0.49f, 0.49f, 1.08f, 0.73f, 1.77f, 0.73f,
R_H_LINE_TO, 3,
CLOSE,
R_MOVE_TO, -2, -3.25f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 6,
R_V_LINE_TO, 1.5f,
CLOSE,
MOVE_TO, 11, 14,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 3,
R_CUBIC_TO, 0.7f, 0, 1.29f, -0.24f, 1.77f, -0.73f,
R_CUBIC_TO, 0.49f, -0.48f, 0.73f, -1.07f, 0.73f, -1.77f,
R_CUBIC_TO, 0, -0.69f, -0.24f, -1.28f, -0.73f, -1.77f,
ARC_TO, 2.4f, 2.4f, 0, 0, 0, 14, 7.5f,
R_H_LINE_TO, -3,
V_LINE_TO, 6,
R_H_LINE_TO, 3,
R_CUBIC_TO, 1.11f, 0, 2.05f, 0.39f, 2.83f, 1.17f,
CUBIC_TO, 17.61f, 7.95f, 18, 8.9f, 18, 10,
R_CUBIC_TO, 0, 1.11f, -0.39f, 2.05f, -1.17f, 2.83f,
CUBIC_TO, 16.05f, 13.61f, 15.11f, 14, 14, 14,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kLinkChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 7.14f, 11.34f,
H_LINE_TO, 4.8f,
R_CUBIC_TO, -0.92f, 0, -1.71f, -0.32f, -2.36f, -0.98f,
R_ARC_TO, 3.21f, 3.21f, 0, 0, 1, -0.98f, -2.36f,
R_CUBIC_TO, 0, -0.92f, 0.32f, -1.71f, 0.98f, -2.36f,
R_ARC_TO, 3.22f, 3.22f, 0, 0, 1, 2.36f, -0.97f,
R_H_LINE_TO, 2.34f,
R_V_LINE_TO, 1.38f,
H_LINE_TO, 4.8f,
R_CUBIC_TO, -0.54f, 0, -1, 0.19f, -1.38f, 0.57f,
R_CUBIC_TO, -0.38f, 0.38f, -0.57f, 0.84f, -0.57f, 1.38f,
R_CUBIC_TO, 0, 0.54f, 0.19f, 1, 0.57f, 1.38f,
R_CUBIC_TO, 0.38f, 0.38f, 0.84f, 0.57f, 1.38f, 0.57f,
R_H_LINE_TO, 2.34f,
CLOSE,
MOVE_TO, 5.51f, 8.63f,
V_LINE_TO, 7.37f,
R_H_LINE_TO, 4.98f,
R_V_LINE_TO, 1.27f,
CLOSE,
R_MOVE_TO, 3.35f, 2.7f,
V_LINE_TO, 9.95f,
R_H_LINE_TO, 2.34f,
R_CUBIC_TO, 0.54f, 0, 1, -0.19f, 1.38f, -0.57f,
R_CUBIC_TO, 0.38f, -0.38f, 0.57f, -0.84f, 0.57f, -1.38f,
R_CUBIC_TO, 0, -0.54f, -0.19f, -1, -0.57f, -1.38f,
R_ARC_TO, 1.88f, 1.88f, 0, 0, 0, -1.38f, -0.57f,
H_LINE_TO, 8.86f,
V_LINE_TO, 4.66f,
R_H_LINE_TO, 2.34f,
R_CUBIC_TO, 0.92f, 0, 1.71f, 0.32f, 2.36f, 0.98f,
R_CUBIC_TO, 0.65f, 0.65f, 0.98f, 1.44f, 0.98f, 2.36f,
R_CUBIC_TO, 0, 0.92f, -0.32f, 1.71f, -0.98f, 2.36f,
R_ARC_TO, 3.22f, 3.22f, 0, 0, 1, -2.36f, 0.97f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kLinkChromeRefreshRepList, kLinkChromeRefreshIcon, {kLinkChromeRefreshPath, std::size(kLinkChromeRefreshPath)}, {kLinkChromeRefresh16Path, std::size(kLinkChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kMediaControlsArrowDropDownPath, CANVAS_DIMENSIONS, 24,
NEW_PATH,
MOVE_TO, 7, 10,
R_LINE_TO, 5, 5,
R_LINE_TO, 5, -5,
H_LINE_TO, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMediaControlsArrowDropDownRepList, kMediaControlsArrowDropDownIcon, {kMediaControlsArrowDropDownPath, std::size(kMediaControlsArrowDropDownPath)})
VECTOR_ICON_REP_TEMPLATE(kMediaControlsArrowDropUpPath, CANVAS_DIMENSIONS, 24,
NEW_PATH,
MOVE_TO, 7, 14,
R_LINE_TO, 5, -5,
R_LINE_TO, 5, 5,
H_LINE_TO, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMediaControlsArrowDropUpRepList, kMediaControlsArrowDropUpIcon, {kMediaControlsArrowDropUpPath, std::size(kMediaControlsArrowDropUpPath)})
VECTOR_ICON_REP_TEMPLATE(kMediaToolbarButtonChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 13, 15,
R_ARC_TO, 2.4f, 2.4f, 0, 0, 1, -1.77f, -0.73f,
R_ARC_TO, 2.4f, 2.4f, 0, 0, 1, -0.73f, -1.77f,
R_CUBIC_TO, 0, -0.69f, 0.24f, -1.28f, 0.73f, -1.77f,
ARC_TO, 2.4f, 2.4f, 0, 0, 1, 13, 10,
R_ARC_TO, 2.46f, 2.46f, 0, 0, 1, 1, 0.21f,
V_LINE_TO, 4,
R_H_LINE_TO, 4,
R_V_LINE_TO, 1.5f,
R_H_LINE_TO, -2.5f,
R_V_LINE_TO, 7,
R_CUBIC_TO, 0, 0.7f, -0.24f, 1.29f, -0.73f, 1.77f,
ARC_TO, 2.4f, 2.4f, 0, 0, 1, 13, 15,
CLOSE,
MOVE_TO, 3, 11.5f,
V_LINE_TO, 10,
R_H_LINE_TO, 6,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 0, -3,
V_LINE_TO, 7,
R_H_LINE_TO, 9,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 0, -3,
V_LINE_TO, 4,
R_H_LINE_TO, 9,
R_V_LINE_TO, 1.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMediaToolbarButtonChromeRefreshRepList, kMediaToolbarButtonChromeRefreshIcon, {kMediaToolbarButtonChromeRefreshPath, std::size(kMediaToolbarButtonChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kMediaToolbarButtonTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 23, 3,
R_ARC_TO, 1, 1, 0, 0, 1, 0, 2,
R_H_LINE_TO, -3,
LINE_TO, 20, 17,
R_CUBIC_TO, -0.04f, 2.27f, -1.43f, 4, -3.5f, 4,
R_ARC_TO, 3.75f, 3.75f, 0, 1, 1, 1.5f, -7.19f,
LINE_TO, 18, 4.13f,
R_LINE_TO, 0, -0.07f,
ARC_TO, 1, 1, 0, 0, 1, 19, 3,
R_H_LINE_TO, 4,
CLOSE,
MOVE_TO, 10.33f, 15,
R_ARC_TO, 4.71f, 4.71f, 0, 0, 0, -0.57f, 2,
H_LINE_TO, 1,
R_ARC_TO, 1, 1, 0, 0, 1, 0, -2,
R_H_LINE_TO, 9.33f,
CLOSE,
MOVE_TO, 14, 9,
R_ARC_TO, 1, 1, 0, 0, 1, 0, 2,
H_LINE_TO, 1,
R_ARC_TO, 1, 1, 0, 1, 1, 0, -2,
R_H_LINE_TO, 13,
CLOSE,
R_MOVE_TO, 0, -6,
R_ARC_TO, 1, 1, 0, 0, 1, 0, 2,
H_LINE_TO, 1,
R_ARC_TO, 1, 1, 0, 1, 1, 0, -2,
R_H_LINE_TO, 13,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMediaToolbarButtonTouchRepList, kMediaToolbarButtonTouchIcon, {kMediaToolbarButtonTouchPath, std::size(kMediaToolbarButtonTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kMenuBookChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 11.75f, 7.96f,
V_LINE_TO, 6.38f,
R_CUBIC_TO, 0.43f, -0.14f, 0.88f, -0.24f, 1.33f, -0.3f,
R_CUBIC_TO, 0.46f, -0.06f, 0.93f, -0.09f, 1.42f, -0.09f,
R_CUBIC_TO, 0.32f, 0, 0.64f, 0.02f, 0.97f, 0.05f,
R_CUBIC_TO, 0.33f, 0.04f, 0.67f, 0.09f, 1.03f, 0.16f,
V_LINE_TO, 7.73f,
R_ARC_TO, 11.87f, 11.87f, 0, 0, 0, -1.1f, -0.21f,
R_ARC_TO, 7.39f, 7.39f, 0, 0, 0, -0.89f, -0.06f,
R_CUBIC_TO, -0.47f, 0, -0.94f, 0.04f, -1.39f, 0.13f,
R_CUBIC_TO, -0.46f, 0.09f, -0.91f, 0.21f, -1.35f, 0.38f,
CLOSE,
R_MOVE_TO, 0, 4.92f,
R_V_LINE_TO, -1.58f,
R_CUBIC_TO, 0.39f, -0.12f, 0.81f, -0.23f, 1.25f, -0.3f,
R_ARC_TO, 9.16f, 9.16f, 0, 0, 1, 1.5f, -0.11f,
R_CUBIC_TO, 0.38f, 0, 0.73f, 0.02f, 1.05f, 0.06f,
R_CUBIC_TO, 0.33f, 0.04f, 0.65f, 0.1f, 0.95f, 0.17f,
R_V_LINE_TO, 1.54f,
R_ARC_TO, 11.73f, 11.73f, 0, 0, 0, -1.1f, -0.21f,
R_ARC_TO, 6.68f, 6.68f, 0, 0, 0, -0.89f, -0.06f,
R_CUBIC_TO, -0.47f, 0, -0.94f, 0.04f, -1.39f, 0.13f,
R_CUBIC_TO, -0.46f, 0.08f, -0.91f, 0.21f, -1.35f, 0.38f,
CLOSE,
R_MOVE_TO, 0, -2.46f,
V_LINE_TO, 8.83f,
R_ARC_TO, 9.45f, 9.45f, 0, 0, 1, 1.36f, -0.31f,
R_CUBIC_TO, 0.47f, -0.07f, 0.93f, -0.1f, 1.39f, -0.1f,
R_CUBIC_TO, 0.38f, 0, 0.73f, 0.02f, 1.05f, 0.06f,
R_CUBIC_TO, 0.33f, 0.04f, 0.65f, 0.1f, 0.95f, 0.17f,
R_V_LINE_TO, 1.54f,
R_ARC_TO, 7.19f, 7.19f, 0, 0, 0, -1.03f, -0.21f,
R_ARC_TO, 7.42f, 7.42f, 0, 0, 0, -2.31f, 0.06f,
R_ARC_TO, 9.42f, 9.42f, 0, 0, 0, -1.41f, 0.38f,
CLOSE,
MOVE_TO, 5.5f, 14,
R_CUBIC_TO, 0.65f, 0, 1.29f, 0.08f, 1.92f, 0.25f,
R_CUBIC_TO, 0.63f, 0.17f, 1.23f, 0.38f, 1.83f, 0.63f,
V_LINE_TO, 5.42f,
R_ARC_TO, 7.79f, 7.79f, 0, 0, 0, -1.81f, -0.69f,
ARC_TO, 8.15f, 8.15f, 0, 0, 0, 5.5f, 4.5f,
R_CUBIC_TO, -0.52f, 0, -1.02f, 0.05f, -1.53f, 0.14f,
R_ARC_TO, 6.44f, 6.44f, 0, 0, 0, -1.47f, 0.45f,
V_LINE_TO, 14.5f,
R_CUBIC_TO, 0.48f, -0.18f, 0.98f, -0.31f, 1.48f, -0.39f,
CUBIC_TO, 4.48f, 14.04f, 4.98f, 14, 5.5f, 14,
CLOSE,
R_MOVE_TO, 5.25f, 0.88f,
R_ARC_TO, 9.92f, 9.92f, 0, 0, 1, 1.83f, -0.64f,
ARC_TO, 8.08f, 8.08f, 0, 0, 1, 14.5f, 14,
R_CUBIC_TO, 0.52f, 0, 1.02f, 0.03f, 1.53f, 0.09f,
R_CUBIC_TO, 0.51f, 0.06f, 1, 0.2f, 1.47f, 0.41f,
V_LINE_TO, 5.08f,
R_CUBIC_TO, -0.48f, -0.18f, -0.98f, -0.32f, -1.48f, -0.43f,
ARC_TO, 7.32f, 7.32f, 0, 0, 0, 14.5f, 4.5f,
R_CUBIC_TO, -0.67f, 0, -1.31f, 0.08f, -1.94f, 0.23f,
R_ARC_TO, 7.79f, 7.79f, 0, 0, 0, -1.81f, 0.69f,
CLOSE,
MOVE_TO, 10, 17,
R_ARC_TO, 10.62f, 10.62f, 0, 0, 0, -2.14f, -1.08f,
ARC_TO, 6.74f, 6.74f, 0, 0, 0, 5.5f, 15.5f,
R_CUBIC_TO, -0.53f, 0, -1.05f, 0.05f, -1.58f, 0.16f,
R_ARC_TO, 7.82f, 7.82f, 0, 0, 0, -1.52f, 0.47f,
R_ARC_TO, 0.97f, 0.97f, 0, 0, 1, -0.95f, -0.07f,
R_ARC_TO, 0.91f, 0.91f, 0, 0, 1, -0.45f, -0.82f,
V_LINE_TO, 4.75f,
R_CUBIC_TO, 0, -0.19f, 0.05f, -0.37f, 0.16f, -0.54f,
R_CUBIC_TO, 0.11f, -0.16f, 0.25f, -0.29f, 0.43f, -0.37f,
ARC_TO, 9.82f, 9.82f, 0, 0, 1, 3.5f, 3.21f,
R_ARC_TO, 9.73f, 9.73f, 0, 0, 1, 4.32f, 0.07f,
R_CUBIC_TO, 0.75f, 0.19f, 1.48f, 0.47f, 2.18f, 0.84f,
R_ARC_TO, 10.63f, 10.63f, 0, 0, 1, 2.19f, -0.83f,
ARC_TO, 9.13f, 9.13f, 0, 0, 1, 14.5f, 3,
R_CUBIC_TO, 0.68f, 0, 1.35f, 0.07f, 2, 0.21f,
R_CUBIC_TO, 0.65f, 0.14f, 1.29f, 0.35f, 1.92f, 0.63f,
R_CUBIC_TO, 0.18f, 0.09f, 0.32f, 0.21f, 0.44f, 0.38f,
R_CUBIC_TO, 0.11f, 0.17f, 0.17f, 0.35f, 0.17f, 0.54f,
R_V_LINE_TO, 10.48f,
R_CUBIC_TO, 0, 0.35f, -0.11f, 0.63f, -0.32f, 0.83f,
R_CUBIC_TO, -0.22f, 0.21f, -0.44f, 0.26f, -0.68f, 0.14f,
R_ARC_TO, 7.5f, 7.5f, 0, 0, 0, -1.72f, -0.54f,
R_ARC_TO, 9.54f, 9.54f, 0, 0, 0, -1.8f, -0.17f,
R_CUBIC_TO, -0.82f, 0, -1.6f, 0.14f, -2.35f, 0.42f,
R_CUBIC_TO, -0.75f, 0.28f, -1.46f, 0.64f, -2.14f, 1.08f,
CLOSE,
MOVE_TO, 5.9f, 9.69f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMenuBookChromeRefreshRepList, kMenuBookChromeRefreshIcon, {kMenuBookChromeRefreshPath, std::size(kMenuBookChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kMixedContentPath, MOVE_TO, 26, 4,
LINE_TO, 10, 11.27f,
LINE_TO, 10, 22.18f,
CUBIC_TO, 10, 32.27f, 16.83f, 41.71f, 26, 44,
CUBIC_TO, 35.17f, 41.71f, 42, 32.27f, 42, 22.18f,
LINE_TO, 42, 11.27f,
LINE_TO, 26, 4,
LINE_TO, 26, 4,
CLOSE,
MOVE_TO, 26, 8,
LINE_TO, 39, 14,
LINE_TO, 39, 22.11f,
CUBIC_TO, 39, 29.93f, 33.34f, 38.22f, 26, 40,
LINE_TO, 26, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMixedContentRepList, kMixedContentIcon, {kMixedContentPath, std::size(kMixedContentPath)})
VECTOR_ICON_REP_TEMPLATE(kMoreToolsMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 8, 9.23f,
CLOSE,
R_MOVE_TO, -5.73f, 3.7f,
V_LINE_TO, 6.91f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_ARC_TO, 1.35f, 1.35f, 0, 0, 1, 0.98f, -0.4f,
R_H_LINE_TO, 1.42f,
V_LINE_TO, 4.45f,
R_CUBIC_TO, 0, -0.39f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 3.1f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 1.08f,
R_H_LINE_TO, 1.42f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.59f, 0.4f, 0.98f,
R_V_LINE_TO, 6.02f,
CLOSE,
MOVE_TO, 6.15f, 9.91f,
R_V_LINE_TO, 0.6f,
H_LINE_TO, 4.77f,
V_LINE_TO, 9.91f,
H_LINE_TO, 3.65f,
R_V_LINE_TO, 1.64f,
R_H_LINE_TO, 8.7f,
V_LINE_TO, 9.91f,
H_LINE_TO, 11.23f,
R_V_LINE_TO, 0.6f,
H_LINE_TO, 9.85f,
V_LINE_TO, 9.91f,
CLOSE,
R_MOVE_TO, -2.5f, -3,
R_V_LINE_TO, 1.64f,
H_LINE_TO, 4.77f,
R_V_LINE_TO, -0.6f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 0.6f,
R_H_LINE_TO, 3.7f,
R_V_LINE_TO, -0.6f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 0.6f,
R_H_LINE_TO, 1.12f,
V_LINE_TO, 6.91f,
CLOSE,
MOVE_TO, 6.45f, 5.53f,
R_H_LINE_TO, 3.1f,
V_LINE_TO, 4.45f,
H_LINE_TO, 6.45f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMoreToolsMenuRepList, kMoreToolsMenuIcon, {kMoreToolsMenuPath, std::size(kMoreToolsMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kMoveGroupToNewWindowRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 6.5f, 15,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 5, 13.5f,
R_V_LINE_TO, -2,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 2,
R_H_LINE_TO, 10,
V_LINE_TO, 5,
R_H_LINE_TO, -10,
R_V_LINE_TO, 2,
H_LINE_TO, 5,
V_LINE_TO, 3.5f,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.77f, 0.44f, -1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 6.5f, 2,
R_H_LINE_TO, 10,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.44f,
R_CUBIC_TO, 0.29f, 0.29f, 0.44f, 0.65f, 0.44f, 1.06f,
R_V_LINE_TO, 10,
R_CUBIC_TO, 0, 0.41f, -0.15f, 0.77f, -0.44f, 1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 16.5f, 15,
CLOSE,
R_MOVE_TO, -3, 3,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 2, 16.5f,
V_LINE_TO, 5,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 11.5f,
H_LINE_TO, 15,
V_LINE_TO, 18,
CLOSE,
R_MOVE_TO, 7.25f, -5.5f,
R_LINE_TO, -1.06f, -1.06f,
LINE_TO, 11.13f, 10,
H_LINE_TO, 5,
V_LINE_TO, 8.5f,
R_H_LINE_TO, 6.13f,
LINE_TO, 9.69f, 7.06f,
LINE_TO, 10.75f, 6,
LINE_TO, 14, 9.25f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kMoveGroupToNewWindowRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.34f, 12.04f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 9.14f,
H_LINE_TO, 5.34f,
R_V_LINE_TO, 1.52f,
R_H_LINE_TO, 7.9f,
V_LINE_TO, 3.91f,
H_LINE_TO, 5.34f,
R_V_LINE_TO, 1.52f,
H_LINE_TO, 3.96f,
V_LINE_TO, 2.76f,
R_CUBIC_TO, 0, -0.39f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 7.9f,
R_CUBIC_TO, 0.39f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.59f, 0.41f, 0.98f,
R_V_LINE_TO, 7.9f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.71f, -0.41f, 0.98f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, -2.58f, 2.59f,
R_CUBIC_TO, -0.39f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 3.96f,
R_H_LINE_TO, 1.39f,
R_V_LINE_TO, 9.29f,
R_H_LINE_TO, 9.29f,
R_V_LINE_TO, 1.39f,
CLOSE,
R_MOVE_TO, 6, -4.67f,
R_LINE_TO, -0.92f, -0.92f,
R_LINE_TO, 1.1f, -1.1f,
R_H_LINE_TO, -4.98f,
V_LINE_TO, 6.63f,
R_H_LINE_TO, 4.98f,
LINE_TO, 7.84f, 5.54f,
R_LINE_TO, 0.92f, -0.92f,
R_LINE_TO, 2.67f, 2.67f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMoveGroupToNewWindowRefreshRepList, kMoveGroupToNewWindowRefreshIcon, {kMoveGroupToNewWindowRefreshPath, std::size(kMoveGroupToNewWindowRefreshPath)}, {kMoveGroupToNewWindowRefresh16Path, std::size(kMoveGroupToNewWindowRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kMyLocationPath, MOVE_TO, 24, 16,
R_CUBIC_TO, -4.42f, 0, -8, 3.58f, -8, 8,
R_CUBIC_TO, 0, 4.42f, 3.58f, 8, 8, 8,
R_CUBIC_TO, 4.42f, 0, 8, -3.58f, 8, -8,
R_CUBIC_TO, 0, -4.42f, -3.58f, -8, -8, -8,
CLOSE,
R_MOVE_TO, 17.88f, 6,
CUBIC_TO, 40.96f, 13.66f, 34.34f, 7.04f, 26, 6.12f,
V_LINE_TO, 2,
R_H_LINE_TO, -4,
R_V_LINE_TO, 4.12f,
CUBIC_TO, 13.66f, 7.04f, 7.04f, 13.66f, 6.12f, 22,
H_LINE_TO, 2,
R_V_LINE_TO, 4,
R_H_LINE_TO, 4.12f,
R_CUBIC_TO, 0.92f, 8.34f, 7.54f, 14.96f, 15.88f, 15.88f,
V_LINE_TO, 46,
R_H_LINE_TO, 4,
R_V_LINE_TO, -4.12f,
R_CUBIC_TO, 8.34f, -0.92f, 14.96f, -7.54f, 15.88f, -15.88f,
H_LINE_TO, 46,
R_V_LINE_TO, -4,
R_H_LINE_TO, -4.12f,
CLOSE,
MOVE_TO, 24, 38,
R_CUBIC_TO, -7.73f, 0, -14, -6.27f, -14, -14,
R_CUBIC_TO, 0, -7.73f, 6.27f, -14, 14, -14,
R_CUBIC_TO, 7.73f, 0, 14, 6.27f, 14, 14,
R_CUBIC_TO, 0, 7.73f, -6.27f, 14, -14, 14,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kMyLocationRepList, kMyLocationIcon, {kMyLocationPath, std::size(kMyLocationPath)})
VECTOR_ICON_REP_TEMPLATE(kNameWindowPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 2.85f, 12.93f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 4.44f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 10.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 7.11f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_H_LINE_TO, 10.3f,
V_LINE_TO, 5.73f,
H_LINE_TO, 2.85f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNameWindowRepList, kNameWindowIcon, {kNameWindowPath, std::size(kNameWindowPath)})
VECTOR_ICON_REP_TEMPLATE(kNavigateHomePath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 9, 14.46f,
LINE_TO, 16, 7.63f,
LINE_TO, 23, 14.46f,
LINE_TO, 23, 25.13f,
LINE_TO, 23, 25.13f,
LINE_TO, 23, 26,
LINE_TO, 22.13f, 26,
LINE_TO, 9.73f, 26,
LINE_TO, 9, 26,
LINE_TO, 9, 25.27f,
LINE_TO, 9, 14.46f,
LINE_TO, 9, 14.46f,
CLOSE,
MOVE_TO, 6, 17.39f,
LINE_TO, 5.86f, 17.52f,
CUBIC_TO, 5.21f, 18.16f, 4.15f, 18.16f, 3.49f, 17.52f,
CUBIC_TO, 2.84f, 16.88f, 2.84f, 15.85f, 3.49f, 15.21f,
LINE_TO, 13.63f, 5.31f,
LINE_TO, 16, 3,
LINE_TO, 18.37f, 5.31f,
LINE_TO, 28.51f, 15.21f,
CUBIC_TO, 29.16f, 15.85f, 29.16f, 16.88f, 28.51f, 17.52f,
CUBIC_TO, 27.85f, 18.16f, 26.79f, 18.16f, 26.14f, 17.52f,
LINE_TO, 26, 17.39f,
LINE_TO, 26, 27,
CUBIC_TO, 26, 28.1f, 25.1f, 29, 24, 29,
LINE_TO, 8, 29,
CUBIC_TO, 6.9f, 29, 6, 28.1f, 6, 27,
LINE_TO, 6, 17.39f,
LINE_TO, 6, 17.39f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kNavigateHome16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 3.35f, 7,
LINE_TO, 8, 2.5f,
LINE_TO, 12.65f, 7,
LINE_TO, 12, 7,
LINE_TO, 12, 14,
LINE_TO, 4, 14,
LINE_TO, 4, 7,
LINE_TO, 3.35f, 7,
LINE_TO, 3.35f, 7,
CLOSE,
MOVE_TO, 2, 8.3f,
LINE_TO, 1.54f, 8.74f,
CUBIC_TO, 1.19f, 9.09f, 0.62f, 9.09f, 0.26f, 8.74f,
CUBIC_TO, -0.09f, 8.4f, -0.09f, 7.85f, 0.26f, 7.51f,
LINE_TO, 8, 0,
LINE_TO, 15.74f, 7.51f,
CUBIC_TO, 16.09f, 7.85f, 16.09f, 8.4f, 15.74f, 8.74f,
CUBIC_TO, 15.38f, 9.09f, 14.81f, 9.09f, 14.46f, 8.74f,
LINE_TO, 14, 8.3f,
LINE_TO, 14, 14.5f,
CUBIC_TO, 14, 15.5f, 13.5f, 16, 12.5f, 16,
LINE_TO, 3.5f, 16,
CUBIC_TO, 2.5f, 16, 2, 15.5f, 2, 14.5f,
LINE_TO, 2, 8.3f,
LINE_TO, 2, 8.3f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateHomeRepList, kNavigateHomeIcon, {kNavigateHomePath, std::size(kNavigateHomePath)}, {kNavigateHome16Path, std::size(kNavigateHome16Path)})
VECTOR_ICON_REP_TEMPLATE(kNavigateHomeChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 5.5f, 15.5f,
R_H_LINE_TO, 2,
R_V_LINE_TO, -5,
R_H_LINE_TO, 5,
R_V_LINE_TO, 5,
R_H_LINE_TO, 2,
V_LINE_TO, 8.25f,
LINE_TO, 10, 4.88f,
LINE_TO, 5.5f, 8.25f,
CLOSE,
MOVE_TO, 4, 17,
V_LINE_TO, 7.5f,
LINE_TO, 10, 3,
R_LINE_TO, 6, 4.5f,
V_LINE_TO, 17,
R_H_LINE_TO, -5,
R_V_LINE_TO, -5,
H_LINE_TO, 9,
R_V_LINE_TO, 5,
CLOSE,
R_MOVE_TO, 6, -6.81f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateHomeChromeRefreshRepList, kNavigateHomeChromeRefreshIcon, {kNavigateHomeChromeRefreshPath, std::size(kNavigateHomeChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kNavigateHomeTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12.87f, 3.54f,
CUBIC_TO, 12.67f, 3.33f, 12.33f, 3, 12, 3,
R_CUBIC_TO, -0.33f, 0, -0.67f, 0.33f, -0.87f, 0.54f,
R_LINE_TO, -7.89f, 7.84f,
ARC_TO, 0.98f, 0.98f, 0, 0, 0, 3, 12,
R_ARC_TO, 1, 1, 0, 0, 0, 1, 1,
R_CUBIC_TO, 0.24f, 0, 0.45f, -0.09f, 0.62f, -0.24f,
R_LINE_TO, 0.38f, -0.35f,
V_LINE_TO, 19,
R_ARC_TO, 2, 2, 0, 0, 0, 2, 2,
R_H_LINE_TO, 10,
R_ARC_TO, 2, 2, 0, 0, 0, 2, -2,
R_V_LINE_TO, -6.59f,
R_LINE_TO, 0.38f, 0.35f,
ARC_TO, 0.98f, 0.98f, 0, 0, 0, 20, 13,
R_ARC_TO, 1, 1, 0, 0, 0, 1, -1,
R_ARC_TO, 0.98f, 0.98f, 0, 0, 0, -0.24f, -0.62f,
R_CUBIC_TO, 0, 0, -5.91f, -5.88f, -7.89f, -7.84f,
CLOSE,
MOVE_TO, 7, 19,
R_V_LINE_TO, -8.85f,
LINE_TO, 12, 5,
R_LINE_TO, 5, 5.15f,
V_LINE_TO, 19,
H_LINE_TO, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateHomeTouchRepList, kNavigateHomeTouchIcon, {kNavigateHomeTouchPath, std::size(kNavigateHomeTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kNavigateStopPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 16, 18.37f,
LINE_TO, 6.86f, 27.51f,
CUBIC_TO, 6.21f, 28.16f, 5.15f, 28.16f, 4.49f, 27.51f,
CUBIC_TO, 3.84f, 26.85f, 3.84f, 25.79f, 4.49f, 25.14f,
LINE_TO, 13.63f, 16,
LINE_TO, 4.49f, 6.86f,
CUBIC_TO, 3.84f, 6.21f, 3.84f, 5.15f, 4.49f, 4.49f,
CUBIC_TO, 5.15f, 3.84f, 6.21f, 3.84f, 6.86f, 4.49f,
LINE_TO, 16, 13.63f,
LINE_TO, 25.14f, 4.49f,
CUBIC_TO, 25.79f, 3.84f, 26.85f, 3.84f, 27.51f, 4.49f,
CUBIC_TO, 28.16f, 5.15f, 28.16f, 6.21f, 27.51f, 6.86f,
LINE_TO, 18.37f, 16,
LINE_TO, 27.51f, 25.14f,
CUBIC_TO, 28.16f, 25.79f, 28.16f, 26.85f, 27.51f, 27.51f,
CUBIC_TO, 26.85f, 28.16f, 25.79f, 28.16f, 25.14f, 27.51f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kNavigateStop16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8, 9.19f,
LINE_TO, 3.43f, 13.75f,
CUBIC_TO, 3.1f, 14.08f, 2.57f, 14.08f, 2.25f, 13.75f,
CUBIC_TO, 1.92f, 13.43f, 1.92f, 12.9f, 2.25f, 12.57f,
LINE_TO, 6.81f, 8,
LINE_TO, 2.25f, 3.43f,
CUBIC_TO, 1.92f, 3.1f, 1.92f, 2.57f, 2.25f, 2.25f,
CUBIC_TO, 2.57f, 1.92f, 3.1f, 1.92f, 3.43f, 2.25f,
LINE_TO, 8, 6.81f,
LINE_TO, 12.57f, 2.25f,
CUBIC_TO, 12.9f, 1.92f, 13.43f, 1.92f, 13.75f, 2.25f,
CUBIC_TO, 14.08f, 2.57f, 14.08f, 3.1f, 13.75f, 3.43f,
LINE_TO, 9.19f, 8,
LINE_TO, 13.75f, 12.57f,
CUBIC_TO, 14.08f, 12.9f, 14.08f, 13.43f, 13.75f, 13.75f,
CUBIC_TO, 13.43f, 14.08f, 12.9f, 14.08f, 12.57f, 13.75f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateStopRepList, kNavigateStopIcon, {kNavigateStopPath, std::size(kNavigateStopPath)}, {kNavigateStop16Path, std::size(kNavigateStop16Path)})
VECTOR_ICON_REP_TEMPLATE(kNavigateStopChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 6.06f, 15,
LINE_TO, 5, 13.94f,
LINE_TO, 8.94f, 10,
LINE_TO, 5, 6.06f,
LINE_TO, 6.06f, 5,
LINE_TO, 10, 8.94f,
LINE_TO, 13.94f, 5,
LINE_TO, 15, 6.06f,
LINE_TO, 11.06f, 10,
LINE_TO, 15, 13.94f,
LINE_TO, 13.94f, 15,
LINE_TO, 10, 11.06f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateStopChromeRefreshRepList, kNavigateStopChromeRefreshIcon, {kNavigateStopChromeRefreshPath, std::size(kNavigateStopChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kNavigateStopTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17.51f, 5,
R_LINE_TO, 0.09f, 0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, 1.32f, 1.32f,
R_LINE_TO, 0.09f, 0.09f,
LINE_TO, 14.49f, 11,
R_LINE_TO, -0.99f, 1,
R_LINE_TO, 0.99f, 1,
LINE_TO, 19, 17.51f,
R_LINE_TO, -0.09f, 0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, -1.32f, 1.32f,
R_LINE_TO, -0.09f, 0.09f,
LINE_TO, 12, 13.5f,
LINE_TO, 6.49f, 19,
R_LINE_TO, -0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, -1.32f, -1.32f,
LINE_TO, 5, 17.51f,
LINE_TO, 9.51f, 13,
R_LINE_TO, 0.99f, -1,
R_LINE_TO, -0.99f, -1,
LINE_TO, 5, 6.49f,
R_LINE_TO, 0.09f, -0.09f,
R_ARC_TO, 1, 1, 0, 0, 1, 1.32f, -1.32f,
LINE_TO, 6.5f, 5,
LINE_TO, 12, 10.51f,
LINE_TO, 17.51f, 5,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNavigateStopTouchRepList, kNavigateStopTouchIcon, {kNavigateStopTouchPath, std::size(kNavigateStopTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kNewTabInGroupRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 10.75f, 12,
R_H_LINE_TO, 1.5f,
V_LINE_TO, 9.25f,
H_LINE_TO, 15,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, -2.75f,
V_LINE_TO, 5,
R_H_LINE_TO, -1.5f,
R_V_LINE_TO, 2.75f,
H_LINE_TO, 8,
R_V_LINE_TO, 1.5f,
R_H_LINE_TO, 2.75f,
CLOSE,
MOVE_TO, 6.5f, 15,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 5, 13.5f,
R_V_LINE_TO, -10,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.77f, 0.44f, -1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 6.5f, 2,
R_H_LINE_TO, 10,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.44f,
R_CUBIC_TO, 0.29f, 0.29f, 0.44f, 0.65f, 0.44f, 1.06f,
R_V_LINE_TO, 10,
R_CUBIC_TO, 0, 0.41f, -0.15f, 0.77f, -0.44f, 1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 16.5f, 15,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_H_LINE_TO, 10,
R_V_LINE_TO, -10,
R_H_LINE_TO, -10,
CLOSE,
R_MOVE_TO, -3, 4.5f,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 2, 16.5f,
V_LINE_TO, 5,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 11.5f,
H_LINE_TO, 15,
V_LINE_TO, 18,
CLOSE,
R_MOVE_TO, 3, -14.5f,
R_V_LINE_TO, 10,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kNewTabInGroupRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 8.69f, 9.51f,
R_H_LINE_TO, 1.2f,
R_V_LINE_TO, -2.2f,
R_H_LINE_TO, 2.2f,
V_LINE_TO, 6.11f,
R_H_LINE_TO, -2.2f,
V_LINE_TO, 3.91f,
H_LINE_TO, 8.69f,
R_V_LINE_TO, 2.2f,
H_LINE_TO, 6.49f,
R_V_LINE_TO, 1.2f,
R_H_LINE_TO, 2.2f,
CLOSE,
MOVE_TO, 5.34f, 12.04f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 2.76f,
R_CUBIC_TO, 0, -0.39f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 7.9f,
R_CUBIC_TO, 0.39f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.59f, 0.41f, 0.98f,
R_V_LINE_TO, 7.9f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.71f, -0.41f, 0.98f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_H_LINE_TO, 7.9f,
V_LINE_TO, 2.76f,
H_LINE_TO, 5.34f,
CLOSE,
MOVE_TO, 2.76f, 14.63f,
R_CUBIC_TO, -0.39f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 3.96f,
R_H_LINE_TO, 1.39f,
R_V_LINE_TO, 9.29f,
R_H_LINE_TO, 9.29f,
R_V_LINE_TO, 1.39f,
CLOSE,
MOVE_TO, 5.34f, 2.76f,
R_V_LINE_TO, 7.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNewTabInGroupRefreshRepList, kNewTabInGroupRefreshIcon, {kNewTabInGroupRefreshPath, std::size(kNewTabInGroupRefreshPath)}, {kNewTabInGroupRefresh16Path, std::size(kNewTabInGroupRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kNewTabRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 2.85f, 11.55f,
R_H_LINE_TO, 10.3f,
V_LINE_TO, 7.2f,
H_LINE_TO, 8.8f,
V_LINE_TO, 4.45f,
H_LINE_TO, 2.85f,
CLOSE,
R_MOVE_TO, 0, 1.38f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 4.44f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 10.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 7.11f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
V_LINE_TO, 4.45f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNewTabRefreshRepList, kNewTabRefreshIcon, {kNewTabRefreshPath, std::size(kNewTabRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kNewWindowPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.65f, 13.73f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 3.99f,
R_V_LINE_TO, 1.38f,
H_LINE_TO, 3.65f,
R_V_LINE_TO, 8.7f,
R_H_LINE_TO, 8.7f,
V_LINE_TO, 8.37f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 3.99f,
R_CUBIC_TO, 0, 0.39f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 6.99f, -6.57f,
R_V_LINE_TO, -1.8f,
H_LINE_TO, 8.83f,
V_LINE_TO, 4.07f,
R_H_LINE_TO, 1.8f,
R_V_LINE_TO, -1.8f,
R_H_LINE_TO, 1.3f,
R_V_LINE_TO, 1.8f,
R_H_LINE_TO, 1.8f,
R_V_LINE_TO, 1.3f,
R_H_LINE_TO, -1.8f,
R_V_LINE_TO, 1.8f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNewWindowRepList, kNewWindowIcon, {kNewWindowPath, std::size(kNewWindowPath)})
VECTOR_ICON_REP_TEMPLATE(kNotesPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 3, 18,
R_H_LINE_TO, 12,
R_V_LINE_TO, -2,
H_LINE_TO, 3,
R_V_LINE_TO, 2,
CLOSE,
MOVE_TO, 3, 6,
R_V_LINE_TO, 2,
R_H_LINE_TO, 18,
V_LINE_TO, 6,
H_LINE_TO, 3,
CLOSE,
R_MOVE_TO, 0, 7,
R_H_LINE_TO, 18,
R_V_LINE_TO, -2,
H_LINE_TO, 3,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNotesRepList, kNotesIcon, {kNotesPath, std::size(kNotesPath)})
VECTOR_ICON_REP_TEMPLATE(kOpenInBrowserPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 19, 4,
H_LINE_TO, 5,
R_ARC_TO, 2, 2, 0, 0, 0, -2, 2,
R_V_LINE_TO, 12,
R_ARC_TO, 2, 2, 0, 0, 0, 2, 2,
R_H_LINE_TO, 4,
R_V_LINE_TO, -2,
H_LINE_TO, 5,
V_LINE_TO, 8,
R_H_LINE_TO, 14,
R_V_LINE_TO, 10,
R_H_LINE_TO, -4,
R_V_LINE_TO, 2,
R_H_LINE_TO, 4,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 6,
R_ARC_TO, 2, 2, 0, 0, 0, -2, -2,
CLOSE,
R_MOVE_TO, -7, 6,
R_LINE_TO, -4, 4,
R_H_LINE_TO, 3,
R_V_LINE_TO, 6,
R_H_LINE_TO, 2,
R_V_LINE_TO, -6,
R_H_LINE_TO, 3,
R_LINE_TO, -4, -4,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kOpenInBrowser16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.65f, 13.73f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 8.7f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 8.7f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
R_H_LINE_TO, -2.66f,
R_V_LINE_TO, -1.38f,
R_H_LINE_TO, 2.66f,
V_LINE_TO, 4.91f,
H_LINE_TO, 3.65f,
R_V_LINE_TO, 7.44f,
R_H_LINE_TO, 2.66f,
R_V_LINE_TO, 1.38f,
CLOSE,
R_MOVE_TO, 3.66f, 0,
V_LINE_TO, 9.71f,
R_LINE_TO, -0.89f, 0.89f,
R_LINE_TO, -0.98f, -0.97f,
LINE_TO, 8, 7.07f,
R_LINE_TO, 2.56f, 2.56f,
R_LINE_TO, -0.98f, 0.97f,
R_LINE_TO, -0.89f, -0.89f,
R_V_LINE_TO, 4.03f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOpenInBrowserRepList, kOpenInBrowserIcon, {kOpenInBrowserPath, std::size(kOpenInBrowserPath)}, {kOpenInBrowser16Path, std::size(kOpenInBrowser16Path)})
VECTOR_ICON_REP_TEMPLATE(kOpenInNewPath, MOVE_TO, 38, 38,
H_LINE_TO, 10,
V_LINE_TO, 10,
R_H_LINE_TO, 14,
V_LINE_TO, 6,
H_LINE_TO, 10,
R_CUBIC_TO, -2.21f, 0, -4, 1.79f, -4, 4,
R_V_LINE_TO, 28,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 28,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
V_LINE_TO, 24,
R_H_LINE_TO, -4,
R_V_LINE_TO, 14,
CLOSE,
MOVE_TO, 28, 6,
R_V_LINE_TO, 4,
R_H_LINE_TO, 7.17f,
LINE_TO, 15.51f, 29.66f,
R_LINE_TO, 2.83f, 2.83f,
LINE_TO, 38, 12.83f,
V_LINE_TO, 20,
R_H_LINE_TO, 4,
V_LINE_TO, 6,
H_LINE_TO, 28,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOpenInNewRepList, kOpenInNewIcon, {kOpenInNewPath, std::size(kOpenInNewPath)})
VECTOR_ICON_REP_TEMPLATE(kOpenInNewChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 4.56f, 17.17f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.5f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -0.5f, -1.22f,
V_LINE_TO, 4.56f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.5f, -1.22f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, 1.22f, -0.5f,
H_LINE_TO, 10,
R_V_LINE_TO, 1.73f,
H_LINE_TO, 4.56f,
V_LINE_TO, 15.44f,
H_LINE_TO, 15.44f,
V_LINE_TO, 10,
R_H_LINE_TO, 1.73f,
R_V_LINE_TO, 5.44f,
R_CUBIC_TO, 0, 0.48f, -0.17f, 0.89f, -0.5f, 1.22f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -1.22f, 0.5f,
CLOSE,
MOVE_TO, 8.22f, 13,
LINE_TO, 7, 11.78f,
R_LINE_TO, 7.22f, -7.22f,
H_LINE_TO, 12,
V_LINE_TO, 2.83f,
R_H_LINE_TO, 5.17f,
V_LINE_TO, 8,
R_H_LINE_TO, -1.73f,
V_LINE_TO, 5.78f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kOpenInNewChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 3.72f, 13.92f,
R_ARC_TO, 1.58f, 1.58f, 0, 0, 1, -1.16f, -0.48f,
R_ARC_TO, 1.58f, 1.58f, 0, 0, 1, -0.48f, -1.16f,
R_V_LINE_TO, -8.57f,
R_CUBIC_TO, 0, -0.46f, 0.16f, -0.84f, 0.48f, -1.16f,
R_ARC_TO, 1.58f, 1.58f, 0, 0, 1, 1.16f, -0.48f,
H_LINE_TO, 8,
R_V_LINE_TO, 1.64f,
H_LINE_TO, 3.72f,
R_V_LINE_TO, 8.57f,
R_H_LINE_TO, 8.57f,
V_LINE_TO, 8,
R_H_LINE_TO, 1.64f,
R_V_LINE_TO, 4.29f,
R_CUBIC_TO, 0, 0.46f, -0.16f, 0.84f, -0.48f, 1.16f,
R_ARC_TO, 1.58f, 1.58f, 0, 0, 1, -1.16f, 0.48f,
CLOSE,
R_MOVE_TO, 3.03f, -3.52f,
R_LINE_TO, -1.14f, -1.14f,
R_LINE_TO, 5.54f, -5.54f,
H_LINE_TO, 9.6f,
V_LINE_TO, 2.08f,
R_H_LINE_TO, 4.32f,
R_V_LINE_TO, 4.32f,
R_H_LINE_TO, -1.64f,
V_LINE_TO, 4.86f,
CLOSE,
R_MOVE_TO, 0, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOpenInNewChromeRefreshRepList, kOpenInNewChromeRefreshIcon, {kOpenInNewChromeRefreshPath, std::size(kOpenInNewChromeRefreshPath)}, {kOpenInNewChromeRefresh16Path, std::size(kOpenInNewChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kOpenInNewOffChromeRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 16.19f, 18.34f,
R_LINE_TO, -1.17f, -1.17f,
R_CUBIC_TO, -0.02f, 0, -0.03f, 0, -0.04f, 0,
H_LINE_TO, 4.56f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.5f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -0.5f, -1.22f,
V_LINE_TO, 5.02f,
R_CUBIC_TO, 0, -0.01f, 0, -0.02f, 0, -0.04f,
LINE_TO, 1.66f, 3.81f,
R_LINE_TO, 1.11f, -1.11f,
R_LINE_TO, 14.52f, 14.52f,
CLOSE,
MOVE_TO, 4.56f, 15.44f,
R_H_LINE_TO, 8.73f,
R_LINE_TO, -3.75f, -3.77f,
LINE_TO, 8.22f, 13,
LINE_TO, 7, 11.78f,
R_LINE_TO, 1.33f, -1.32f,
R_LINE_TO, -3.77f, -3.75f,
CLOSE,
MOVE_TO, 6.86f, 4.56f,
LINE_TO, 5.14f, 2.83f,
H_LINE_TO, 10,
R_V_LINE_TO, 1.73f,
CLOSE,
R_MOVE_TO, 4.88f, 4.91f,
R_LINE_TO, -1.22f, -1.22f,
R_LINE_TO, 3.69f, -3.69f,
H_LINE_TO, 12,
V_LINE_TO, 2.83f,
R_H_LINE_TO, 5.17f,
V_LINE_TO, 8,
R_H_LINE_TO, -1.73f,
V_LINE_TO, 5.78f,
CLOSE,
R_MOVE_TO, 5.42f, 5.39f,
R_LINE_TO, -1.73f, -1.73f,
V_LINE_TO, 10,
R_H_LINE_TO, 1.73f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOpenInNewOffChromeRefreshRepList, kOpenInNewOffChromeRefreshIcon, {kOpenInNewOffChromeRefreshPath, std::size(kOpenInNewOffChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kOverflowButtonPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 7.94f, 10,
LINE_TO, 4, 6.06f,
LINE_TO, 5.06f, 5,
R_LINE_TO, 5, 5,
R_LINE_TO, -5, 5,
LINE_TO, 4, 13.94f,
CLOSE,
R_MOVE_TO, 5.94f, 0,
LINE_TO, 9.94f, 6.06f,
LINE_TO, 11, 5,
R_LINE_TO, 5, 5,
R_LINE_TO, -5, 5,
R_LINE_TO, -1.06f, -1.06f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOverflowButtonRepList, kOverflowButtonIcon, {kOverflowButtonPath, std::size(kOverflowButtonPath)})
VECTOR_ICON_REP_TEMPLATE(kOverflowButtonTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 9.35f, 12,
LINE_TO, 4.77f, 7.4f,
LINE_TO, 6.36f, 5.81f,
LINE_TO, 12.56f, 12,
LINE_TO, 6.36f, 18.19f,
LINE_TO, 4.77f, 16.6f,
LINE_TO, 9.35f, 12,
CLOSE,
MOVE_TO, 16.03f, 12,
LINE_TO, 11.44f, 7.4f,
LINE_TO, 13.04f, 5.81f,
LINE_TO, 19.23f, 12,
LINE_TO, 13.04f, 18.19f,
LINE_TO, 11.44f, 16.6f,
LINE_TO, 16.03f, 12,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kOverflowButtonTouchRepList, kOverflowButtonTouchIcon, {kOverflowButtonTouchPath, std::size(kOverflowButtonTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kPaintbrushPath, MOVE_TO, 19.08f, 26.09f,
LINE_TO, 27.77f, 34.8f,
LINE_TO, 30.66f, 31.9f,
CUBIC_TO, 31.46f, 31.09f, 30.66f, 28.99f, 30.66f, 28.99f,
LINE_TO, 29.22f, 24.63f,
CUBIC_TO, 28.85f, 23.54f, 28.78f, 22.17f, 29.22f, 21.73f,
CUBIC_TO, 33.56f, 17.37f, 39.35f, 14.47f, 40.8f, 13.01f,
CUBIC_TO, 42.4f, 11.41f, 42.4f, 8.81f, 40.8f, 7.2f,
CUBIC_TO, 39.2f, 5.6f, 36.61f, 5.6f, 35.01f, 7.2f,
CUBIC_TO, 33.56f, 8.66f, 30.66f, 14.47f, 26.32f, 18.82f,
CUBIC_TO, 25.89f, 19.26f, 24.51f, 19.19f, 23.42f, 18.82f,
LINE_TO, 19.08f, 17.37f,
CUBIC_TO, 19.08f, 17.37f, 16.98f, 16.57f, 16.19f, 17.37f,
LINE_TO, 13.29f, 20.28f,
LINE_TO, 19.08f, 26.09f,
LINE_TO, 19.08f, 26.09f,
CLOSE,
MOVE_TO, 39.35f, 8.66f,
CUBIC_TO, 40.15f, 9.46f, 40.15f, 10.77f, 39.35f, 11.57f,
CUBIC_TO, 38.55f, 12.37f, 37.25f, 12.37f, 36.45f, 11.57f,
CUBIC_TO, 35.65f, 10.77f, 35.65f, 9.46f, 36.45f, 8.66f,
CUBIC_TO, 37.25f, 7.86f, 38.55f, 7.86f, 39.35f, 8.66f,
LINE_TO, 39.35f, 8.66f,
CLOSE,
MOVE_TO, 13.8f, 35.83f,
LINE_TO, 17.9f, 35.83f,
LINE_TO, 17.9f, 39.94f,
LINE_TO, 19.95f, 42,
LINE_TO, 26.11f, 35.83f,
LINE_TO, 11.75f, 21.43f,
LINE_TO, 5.6f, 27.6f,
LINE_TO, 13.8f, 35.83f,
LINE_TO, 13.8f, 35.83f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPaintbrushRepList, kPaintbrushIcon, {kPaintbrushPath, std::size(kPaintbrushPath)})
VECTOR_ICON_REP_TEMPLATE(kPasteMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.65f, 13.73f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.41f,
R_ARC_TO, 1.32f, 1.32f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.41f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.59f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 2.7f,
R_CUBIC_TO, 0.11f, -0.36f, 0.3f, -0.64f, 0.61f, -0.87f,
R_CUBIC_TO, 0.3f, -0.23f, 0.65f, -0.35f, 1.05f, -0.35f,
R_CUBIC_TO, 0.39f, 0, 0.74f, 0.12f, 1.04f, 0.35f,
R_CUBIC_TO, 0.3f, 0.23f, 0.51f, 0.52f, 0.61f, 0.88f,
R_H_LINE_TO, 2.7f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.59f, 0.41f, 0.98f,
R_V_LINE_TO, 8.7f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.41f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.59f, 0.41f, -0.98f, 0.41f,
CLOSE,
R_MOVE_TO, 0, -1.38f,
R_H_LINE_TO, 8.7f,
V_LINE_TO, 3.65f,
H_LINE_TO, 11.18f,
R_V_LINE_TO, 2.06f,
H_LINE_TO, 4.82f,
V_LINE_TO, 3.65f,
H_LINE_TO, 3.65f,
CLOSE,
MOVE_TO, 8, 3.59f,
R_ARC_TO, 0.6f, 0.6f, 0, 0, 0, 0.43f, -0.17f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, 0.18f, -0.43f,
R_ARC_TO, 0.6f, 0.6f, 0, 0, 0, -0.17f, -0.43f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.43f, -0.17f,
R_ARC_TO, 0.6f, 0.6f, 0, 0, 0, -0.43f, 0.17f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.18f, 0.43f,
R_CUBIC_TO, 0, 0.17f, 0.06f, 0.31f, 0.17f, 0.43f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, 0.43f, 0.18f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPasteMenuRepList, kPasteMenuIcon, {kPasteMenuPath, std::size(kPasteMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kSaveCardAndVcnSuccessConfirmationPath, CANVAS_DIMENSIONS, 103,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
MOVE_TO, 48.29f, 35.38f,
CUBIC_TO, 36.13f, 35.38f, 26.27f, 45.24f, 26.27f, 57.4f,
CUBIC_TO, 26.27f, 69.55f, 36.13f, 79.42f, 48.29f, 79.42f,
CUBIC_TO, 60.44f, 79.42f, 70.3f, 69.55f, 70.3f, 57.4f,
CUBIC_TO, 70.3f, 45.24f, 60.44f, 35.38f, 48.29f, 35.38f,
CLOSE,
MOVE_TO, 43.88f, 68.41f,
LINE_TO, 35.08f, 59.6f,
LINE_TO, 38.16f, 56.52f,
LINE_TO, 43.88f, 62.24f,
LINE_TO, 58.41f, 47.71f,
LINE_TO, 61.5f, 50.79f,
LINE_TO, 43.88f, 68.41f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x39, 0x82, 0xF8,
MOVE_TO, 76.53f, 34.11f,
CUBIC_TO, 78.54f, 34.11f, 80.18f, 32.42f, 80.18f, 30.33f,
CUBIC_TO, 80.18f, 28.25f, 78.54f, 26.56f, 76.53f, 26.56f,
CUBIC_TO, 74.52f, 26.56f, 72.89f, 28.25f, 72.89f, 30.33f,
CUBIC_TO, 72.89f, 32.42f, 74.52f, 34.11f, 76.53f, 34.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xEA, 0xED,
MOVE_TO, 3.64f, 70.92f,
CUBIC_TO, 5.66f, 70.92f, 7.29f, 69.23f, 7.29f, 67.15f,
CUBIC_TO, 7.29f, 65.06f, 5.66f, 63.37f, 3.64f, 63.37f,
CUBIC_TO, 1.63f, 63.37f, 0, 65.06f, 0, 67.15f,
CUBIC_TO, 0, 69.23f, 1.63f, 70.92f, 3.64f, 70.92f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
MOVE_TO, 92.52f, 83.11f,
CUBIC_TO, 93.95f, 83.58f, 94.74f, 85.15f, 94.29f, 86.63f,
CUBIC_TO, 93.85f, 88.11f, 92.32f, 88.94f, 90.9f, 88.47f,
LINE_TO, 85.72f, 86.79f,
CUBIC_TO, 84.29f, 86.33f, 83.5f, 84.75f, 83.94f, 83.27f,
CUBIC_TO, 84.39f, 81.79f, 85.91f, 80.97f, 87.34f, 81.43f,
LINE_TO, 92.52f, 83.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
MOVE_TO, 10.4f, 27.22f,
CUBIC_TO, 8.67f, 25.5f, 8.62f, 22.66f, 10.28f, 20.87f,
CUBIC_TO, 11.94f, 19.08f, 14.68f, 19.03f, 16.41f, 20.75f,
LINE_TO, 22.66f, 26.98f,
CUBIC_TO, 24.38f, 28.7f, 24.44f, 31.54f, 22.78f, 33.33f,
CUBIC_TO, 21.12f, 35.12f, 18.37f, 35.17f, 16.65f, 33.45f,
LINE_TO, 10.4f, 27.22f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xEA, 0x43, 0x35,
MOVE_TO, 24.84f, 87.96f,
CUBIC_TO, 25.99f, 86.79f, 26.02f, 84.88f, 24.89f, 83.68f,
CUBIC_TO, 23.76f, 82.48f, 21.91f, 82.45f, 20.75f, 83.62f,
LINE_TO, 16.56f, 87.86f,
CUBIC_TO, 15.4f, 89.02f, 15.37f, 90.94f, 16.5f, 92.14f,
CUBIC_TO, 17.63f, 93.34f, 19.48f, 93.37f, 20.64f, 92.2f,
LINE_TO, 24.84f, 87.96f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
MOVE_TO, 93.08f, 55.06f,
CUBIC_TO, 91.31f, 55.99f, 89.15f, 55.26f, 88.25f, 53.43f,
CUBIC_TO, 87.36f, 51.6f, 88.07f, 49.36f, 89.83f, 48.43f,
LINE_TO, 96.24f, 45.07f,
CUBIC_TO, 98.01f, 44.14f, 100.17f, 44.87f, 101.07f, 46.7f,
CUBIC_TO, 101.97f, 48.53f, 101.26f, 50.77f, 99.49f, 51.7f,
LINE_TO, 93.08f, 55.06f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xEA, 0xED,
MOVE_TO, 49.06f, 6.42f,
CUBIC_TO, 49.05f, 5.65f, 49.89f, 5.17f, 50.55f, 5.55f,
LINE_TO, 61.56f, 11.85f,
CUBIC_TO, 62.22f, 12.23f, 62.23f, 13.19f, 61.56f, 13.58f,
LINE_TO, 50.61f, 19.96f,
CUBIC_TO, 49.94f, 20.35f, 49.11f, 19.87f, 49.11f, 19.1f,
LINE_TO, 49.06f, 6.42f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xEA, 0xED,
MOVE_TO, 64.23f, 94.97f,
CUBIC_TO, 64.23f, 95.74f, 63.39f, 96.22f, 62.73f, 95.84f,
LINE_TO, 55.84f, 91.86f,
CUBIC_TO, 55.18f, 91.48f, 55.18f, 90.52f, 55.84f, 90.13f,
LINE_TO, 62.73f, 86.16f,
CUBIC_TO, 63.39f, 85.77f, 64.23f, 86.25f, 64.23f, 87.02f,
LINE_TO, 64.23f, 94.97f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSaveCardAndVcnSuccessConfirmationRepList, kSaveCardAndVcnSuccessConfirmationIcon, {kSaveCardAndVcnSuccessConfirmationPath, std::size(kSaveCardAndVcnSuccessConfirmationPath)})
VECTOR_ICON_REP_TEMPLATE(kSaveCardAndVcnSuccessConfirmationDarkPath, CANVAS_DIMENSIONS, 103,
PATH_COLOR_ARGB, 0xFF, 0x81, 0xC9, 0x95,
MOVE_TO, 48.29f, 35.38f,
CUBIC_TO, 36.13f, 35.38f, 26.27f, 45.24f, 26.27f, 57.4f,
CUBIC_TO, 26.27f, 69.55f, 36.13f, 79.42f, 48.29f, 79.42f,
CUBIC_TO, 60.44f, 79.42f, 70.3f, 69.55f, 70.3f, 57.4f,
CUBIC_TO, 70.3f, 45.24f, 60.44f, 35.38f, 48.29f, 35.38f,
CLOSE,
MOVE_TO, 43.88f, 68.41f,
LINE_TO, 35.08f, 59.6f,
LINE_TO, 38.16f, 56.52f,
LINE_TO, 43.88f, 62.24f,
LINE_TO, 58.41f, 47.71f,
LINE_TO, 61.5f, 50.79f,
LINE_TO, 43.88f, 68.41f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x8A, 0xB4, 0xF8,
MOVE_TO, 76.53f, 34.11f,
CUBIC_TO, 78.54f, 34.11f, 80.18f, 32.42f, 80.18f, 30.33f,
CUBIC_TO, 80.18f, 28.25f, 78.54f, 26.56f, 76.53f, 26.56f,
CUBIC_TO, 74.52f, 26.56f, 72.89f, 28.25f, 72.89f, 30.33f,
CUBIC_TO, 72.89f, 32.42f, 74.52f, 34.11f, 76.53f, 34.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 3.64f, 70.92f,
CUBIC_TO, 5.66f, 70.92f, 7.29f, 69.23f, 7.29f, 67.15f,
CUBIC_TO, 7.29f, 65.06f, 5.66f, 63.37f, 3.64f, 63.37f,
CUBIC_TO, 1.63f, 63.37f, 0, 65.06f, 0, 67.15f,
CUBIC_TO, 0, 69.23f, 1.63f, 70.92f, 3.64f, 70.92f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x81, 0xC9, 0x95,
MOVE_TO, 92.52f, 83.11f,
CUBIC_TO, 93.95f, 83.58f, 94.74f, 85.15f, 94.29f, 86.63f,
CUBIC_TO, 93.85f, 88.11f, 92.32f, 88.94f, 90.9f, 88.47f,
LINE_TO, 85.72f, 86.79f,
CUBIC_TO, 84.29f, 86.33f, 83.5f, 84.75f, 83.94f, 83.27f,
CUBIC_TO, 84.39f, 81.79f, 85.91f, 80.97f, 87.34f, 81.43f,
LINE_TO, 92.52f, 83.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFD, 0xD6, 0x63,
MOVE_TO, 10.4f, 27.22f,
CUBIC_TO, 8.67f, 25.5f, 8.62f, 22.66f, 10.28f, 20.87f,
CUBIC_TO, 11.94f, 19.08f, 14.68f, 19.03f, 16.41f, 20.75f,
LINE_TO, 22.66f, 26.98f,
CUBIC_TO, 24.38f, 28.7f, 24.44f, 31.54f, 22.78f, 33.33f,
CUBIC_TO, 21.12f, 35.12f, 18.37f, 35.17f, 16.65f, 33.45f,
LINE_TO, 10.4f, 27.22f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xF2, 0x8B, 0x82,
MOVE_TO, 24.84f, 87.96f,
CUBIC_TO, 25.99f, 86.79f, 26.02f, 84.88f, 24.89f, 83.68f,
CUBIC_TO, 23.76f, 82.48f, 21.91f, 82.45f, 20.75f, 83.62f,
LINE_TO, 16.56f, 87.86f,
CUBIC_TO, 15.4f, 89.02f, 15.37f, 90.94f, 16.5f, 92.14f,
CUBIC_TO, 17.63f, 93.34f, 19.48f, 93.37f, 20.64f, 92.2f,
LINE_TO, 24.84f, 87.96f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFD, 0xD6, 0x63,
MOVE_TO, 93.08f, 55.06f,
CUBIC_TO, 91.31f, 55.99f, 89.15f, 55.26f, 88.25f, 53.43f,
CUBIC_TO, 87.36f, 51.6f, 88.07f, 49.36f, 89.83f, 48.43f,
LINE_TO, 96.24f, 45.07f,
CUBIC_TO, 98.01f, 44.14f, 100.17f, 44.87f, 101.07f, 46.7f,
CUBIC_TO, 101.97f, 48.53f, 101.26f, 50.77f, 99.49f, 51.7f,
LINE_TO, 93.08f, 55.06f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 49.06f, 6.42f,
CUBIC_TO, 49.05f, 5.65f, 49.89f, 5.17f, 50.55f, 5.55f,
LINE_TO, 61.56f, 11.85f,
CUBIC_TO, 62.22f, 12.23f, 62.23f, 13.19f, 61.56f, 13.58f,
LINE_TO, 50.61f, 19.96f,
CUBIC_TO, 49.94f, 20.35f, 49.11f, 19.87f, 49.11f, 19.1f,
LINE_TO, 49.06f, 6.42f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 64.23f, 94.97f,
CUBIC_TO, 64.23f, 95.74f, 63.39f, 96.22f, 62.73f, 95.84f,
LINE_TO, 55.84f, 91.86f,
CUBIC_TO, 55.18f, 91.48f, 55.18f, 90.52f, 55.84f, 90.13f,
LINE_TO, 62.73f, 86.16f,
CUBIC_TO, 63.39f, 85.77f, 64.23f, 86.25f, 64.23f, 87.02f,
LINE_TO, 64.23f, 94.97f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSaveCardAndVcnSuccessConfirmationDarkRepList, kSaveCardAndVcnSuccessConfirmationDarkIcon, {kSaveCardAndVcnSuccessConfirmationDarkPath, std::size(kSaveCardAndVcnSuccessConfirmationDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPerformancePath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 7.15f, 10.04f,
R_CUBIC_TO, 0.25f, 0.24f, 0.57f, 0.36f, 0.95f, 0.34f,
R_CUBIC_TO, 0.38f, -0.02f, 0.66f, -0.16f, 0.84f, -0.43f,
R_LINE_TO, 3.57f, -5.25f,
LINE_TO, 7.25f, 8.27f,
R_CUBIC_TO, -0.27f, 0.19f, -0.43f, 0.47f, -0.44f, 0.84f,
R_CUBIC_TO, -0.02f, 0.37f, 0.09f, 0.68f, 0.35f, 0.92f,
CLOSE,
R_MOVE_TO, 0.87f, -6.97f,
R_CUBIC_TO, 0.69f, 0, 1.31f, 0.09f, 1.87f, 0.27f,
R_CUBIC_TO, 0.56f, 0.18f, 1.06f, 0.4f, 1.5f, 0.66f,
R_LINE_TO, -1.32f, 0.88f,
R_ARC_TO, 5.13f, 5.13f, 0, 0, 0, -0.95f, -0.32f,
R_ARC_TO, 5.18f, 5.18f, 0, 0, 0, -1.1f, -0.11f,
R_CUBIC_TO, -1.43f, 0, -2.64f, 0.51f, -3.65f, 1.54f,
R_CUBIC_TO, -1.01f, 1.03f, -1.51f, 2.23f, -1.51f, 3.61f,
R_CUBIC_TO, 0, 0.42f, 0.03f, 0.76f, 0.09f, 1.02f,
R_CUBIC_TO, 0.06f, 0.26f, 0.15f, 0.57f, 0.28f, 0.93f,
R_H_LINE_TO, 9.56f,
R_ARC_TO, 5.93f, 5.93f, 0, 0, 0, 0.3f, -0.98f,
R_ARC_TO, 5.35f, 5.35f, 0, 0, 0, -0.1f, -2.33f,
R_ARC_TO, 4.71f, 4.71f, 0, 0, 0, -0.54f, -1.27f,
R_LINE_TO, 0.84f, -1.24f,
R_ARC_TO, 6.26f, 6.26f, 0, 0, 1, 0.94f, 1.82f,
R_CUBIC_TO, 0.22f, 0.67f, 0.33f, 1.34f, 0.33f, 2.03f,
R_ARC_TO, 7.24f, 7.24f, 0, 0, 1, -0.15f, 1.52f,
R_ARC_TO, 4.09f, 4.09f, 0, 0, 1, -0.47f, 1.2f,
R_ARC_TO, 1.32f, 1.32f, 0, 0, 1, -0.48f, 0.49f,
R_CUBIC_TO, -0.19f, 0.11f, -0.41f, 0.16f, -0.64f, 0.16f,
H_LINE_TO, 3.2f,
R_ARC_TO, 1.26f, 1.26f, 0, 0, 1, -1.11f, -0.65f,
R_ARC_TO, 4.25f, 4.25f, 0, 0, 1, -0.47f, -1.2f,
R_ARC_TO, 7, 7, 0, 0, 1, -0.15f, -1.52f,
R_CUBIC_TO, 0, -0.9f, 0.17f, -1.75f, 0.52f, -2.54f,
R_ARC_TO, 6.59f, 6.59f, 0, 0, 1, 1.4f, -2.06f,
R_ARC_TO, 6.62f, 6.62f, 0, 0, 1, 2.08f, -1.39f,
R_ARC_TO, 6.43f, 6.43f, 0, 0, 1, 2.55f, -0.51f,
CLOSE,
R_MOVE_TO, 0.06f, 4.71f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPerformanceRepList, kPerformanceIcon, {kPerformancePath, std::size(kPerformancePath)})
VECTOR_ICON_REP_TEMPLATE(kPerformanceSpeedometerPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 8.94f, 12.54f,
R_CUBIC_TO, 0.32f, 0.31f, 0.71f, 0.45f, 1.19f, 0.43f,
R_CUBIC_TO, 0.47f, -0.02f, 0.82f, -0.2f, 1.04f, -0.53f,
LINE_TO, 15.5f, 6,
R_LINE_TO, -6.44f, 4.34f,
R_CUBIC_TO, -0.34f, 0.23f, -0.53f, 0.58f, -0.56f, 1.05f,
R_CUBIC_TO, -0.03f, 0.47f, 0.12f, 0.85f, 0.44f, 1.16f,
CLOSE,
MOVE_TO, 10.02f, 4,
R_CUBIC_TO, 0.84f, 0, 1.59f, 0.11f, 2.25f, 0.32f,
R_ARC_TO, 8.83f, 8.83f, 0, 0, 1, 1.81f, 0.8f,
R_LINE_TO, -1.44f, 0.94f,
R_ARC_TO, 6.15f, 6.15f, 0, 0, 0, -1.22f, -0.42f,
ARC_TO, 6.25f, 6.25f, 0, 0, 0, 10.02f, 5.5f,
R_CUBIC_TO, -1.8f, 0, -3.34f, 0.65f, -4.61f, 1.95f,
CUBIC_TO, 4.14f, 8.74f, 3.5f, 10.27f, 3.5f, 12,
R_CUBIC_TO, 0, 0.53f, 0.04f, 0.96f, 0.11f, 1.3f,
R_CUBIC_TO, 0.08f, 0.34f, 0.2f, 0.74f, 0.37f, 1.2f,
H_LINE_TO, 16,
R_CUBIC_TO, 0.18f, -0.46f, 0.31f, -0.88f, 0.39f, -1.26f,
R_CUBIC_TO, 0.07f, -0.38f, 0.11f, -0.79f, 0.11f, -1.24f,
R_CUBIC_TO, 0, -0.58f, -0.08f, -1.17f, -0.24f, -1.75f,
R_ARC_TO, 6.05f, 6.05f, 0, 0, 0, -0.7f, -1.6f,
R_LINE_TO, 0.9f, -1.35f,
ARC_TO, 7.98f, 7.98f, 0, 0, 1, 17.59f, 9.5f,
R_CUBIC_TO, 0.27f, 0.81f, 0.41f, 1.63f, 0.41f, 2.46f,
R_CUBIC_TO, 0, 0.71f, -0.06f, 1.33f, -0.18f, 1.86f,
R_ARC_TO, 4.96f, 4.96f, 0, 0, 1, -0.57f, 1.46f,
ARC_TO, 1.4f, 1.4f, 0, 0, 1, 16, 16,
H_LINE_TO, 4,
R_ARC_TO, 1.4f, 1.4f, 0, 0, 1, -1.25f, -0.73f,
R_ARC_TO, 4.96f, 4.96f, 0, 0, 1, -0.57f, -1.46f,
CUBIC_TO, 2.06f, 13.28f, 2, 12.67f, 2, 11.96f,
R_CUBIC_TO, 0, -1.1f, 0.21f, -2.14f, 0.64f, -3.11f,
R_ARC_TO, 8.01f, 8.01f, 0, 0, 1, 1.72f, -2.52f,
R_ARC_TO, 8.1f, 8.1f, 0, 0, 1, 2.55f, -1.7f,
ARC_TO, 7.83f, 7.83f, 0, 0, 1, 10.02f, 4,
CLOSE,
MOVE_TO, 10, 9.81f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPerformanceSpeedometerRepList, kPerformanceSpeedometerIcon, {kPerformanceSpeedometerPath, std::size(kPerformanceSpeedometerPath)})
VECTOR_ICON_REP_TEMPLATE(kPersonPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 10, 10,
R_ARC_TO, 2.9f, 2.9f, 0, 0, 1, -2.12f, -0.87f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 7, 7,
R_CUBIC_TO, 0, -0.83f, 0.29f, -1.54f, 0.88f, -2.12f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 10, 4,
R_CUBIC_TO, 0.83f, 0, 1.54f, 0.29f, 2.13f, 0.88f,
CUBIC_TO, 12.71f, 5.46f, 13, 6.17f, 13, 7,
R_CUBIC_TO, 0, 0.83f, -0.29f, 1.54f, -0.87f, 2.13f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 10, 10,
CLOSE,
R_MOVE_TO, -6, 6,
R_V_LINE_TO, -2,
R_CUBIC_TO, 0, -0.32f, 0.09f, -0.62f, 0.26f, -0.91f,
R_CUBIC_TO, 0.17f, -0.28f, 0.41f, -0.52f, 0.72f, -0.72f,
R_CUBIC_TO, 0.76f, -0.44f, 1.57f, -0.78f, 2.42f, -1.02f,
R_ARC_TO, 9.62f, 9.62f, 0, 0, 1, 5.19f, 0,
R_CUBIC_TO, 0.86f, 0.24f, 1.67f, 0.57f, 2.43f, 1.02f,
R_CUBIC_TO, 0.31f, 0.18f, 0.55f, 0.42f, 0.72f, 0.71f,
R_CUBIC_TO, 0.18f, 0.29f, 0.26f, 0.6f, 0.26f, 0.92f,
R_V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 1.5f, -1.5f,
R_H_LINE_TO, 9,
V_LINE_TO, 14,
R_CUBIC_TO, 0, -0.07f, -0.02f, -0.14f, -0.06f, -0.19f,
R_ARC_TO, 0.41f, 0.41f, 0, 0, 0, -0.17f, -0.14f,
R_ARC_TO, 7.61f, 7.61f, 0, 0, 0, -2.04f, -0.87f,
ARC_TO, 8.5f, 8.5f, 0, 0, 0, 10, 12.5f,
R_CUBIC_TO, -0.77f, 0, -1.51f, 0.1f, -2.23f, 0.29f,
R_ARC_TO, 7.61f, 7.61f, 0, 0, 0, -2.04f, 0.88f,
R_CUBIC_TO, -0.07f, 0.06f, -0.12f, 0.11f, -0.17f, 0.16f,
ARC_TO, 0.26f, 0.26f, 0, 0, 0, 5.5f, 14,
CLOSE,
R_MOVE_TO, 4.5f, -6,
R_CUBIC_TO, 0.41f, 0, 0.77f, -0.15f, 1.06f, -0.44f,
R_CUBIC_TO, 0.29f, -0.3f, 0.44f, -0.65f, 0.44f, -1.06f,
R_CUBIC_TO, 0, -0.41f, -0.15f, -0.77f, -0.44f, -1.06f,
ARC_TO, 1.45f, 1.45f, 0, 0, 0, 10, 5.5f,
R_CUBIC_TO, -0.41f, 0, -0.77f, 0.15f, -1.06f, 0.44f,
ARC_TO, 1.45f, 1.45f, 0, 0, 0, 8.5f, 7,
R_CUBIC_TO, 0, 0.41f, 0.15f, 0.77f, 0.44f, 1.06f,
R_CUBIC_TO, 0.3f, 0.29f, 0.65f, 0.44f, 1.06f, 0.44f,
CLOSE,
MOVE_TO, 10, 7,
CLOSE,
R_MOVE_TO, 0, 7.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPersonRepList, kPersonIcon, {kPersonPath, std::size(kPersonPath)})
VECTOR_ICON_REP_TEMPLATE(kPersonFilledPaddedLargePath, CANVAS_DIMENSIONS, 60,
MOVE_TO, 30, 20,
R_CUBIC_TO, -2.76f, 0, -5, 2.24f, -5, 5,
R_CUBIC_TO, 0, 2.76f, 2.24f, 5, 5, 5,
R_CUBIC_TO, 2.76f, 0, 5, -2.24f, 5, -5,
R_CUBIC_TO, 0, -2.76f, -2.24f, -5, -5, -5,
CLOSE,
R_MOVE_TO, 0, 11.25f,
R_CUBIC_TO, -3.34f, 0, -10, 1.68f, -10, 5,
V_LINE_TO, 40,
R_H_LINE_TO, 20,
R_V_LINE_TO, -3.75f,
R_CUBIC_TO, 0, -3.32f, -6.66f, -5, -10, -5,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPersonFilledPaddedLargeRepList, kPersonFilledPaddedLargeIcon, {kPersonFilledPaddedLargePath, std::size(kPersonFilledPaddedLargePath)})
VECTOR_ICON_REP_TEMPLATE(kPersonFilledPaddedSmallPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 12, 5,
R_CUBIC_TO, -1.93f, 0, -3.5f, 1.57f, -3.5f, 3.5f,
R_CUBIC_TO, 0, 1.93f, 1.57f, 3.5f, 3.5f, 3.5f,
R_CUBIC_TO, 1.93f, 0, 3.5f, -1.57f, 3.5f, -3.5f,
CUBIC_TO, 15.5f, 6.57f, 13.93f, 5, 12, 5,
CLOSE,
R_MOVE_TO, 0, 7.88f,
R_CUBIC_TO, -2.34f, 0, -7, 1.17f, -7, 3.5f,
V_LINE_TO, 19,
R_H_LINE_TO, 14,
R_V_LINE_TO, -2.62f,
R_CUBIC_TO, 0, -2.33f, -4.66f, -3.5f, -7, -3.5f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kPersonFilledPaddedSmall22Path, CANVAS_DIMENSIONS, 22,
MOVE_TO, 11, 5,
CUBIC_TO, 9.34f, 5, 8, 6.34f, 8, 8,
R_CUBIC_TO, 0, 1.66f, 1.34f, 3, 3, 3,
R_CUBIC_TO, 1.66f, 0, 3, -1.34f, 3, -3,
R_CUBIC_TO, 0, -1.66f, -1.34f, -3, -3, -3,
CLOSE,
R_MOVE_TO, 0, 6.75f,
R_CUBIC_TO, -2, 0, -6, 1, -6, 3,
V_LINE_TO, 17,
R_H_LINE_TO, 12,
R_V_LINE_TO, -2.25f,
R_CUBIC_TO, 0, -1.99f, -4, -3, -6, -3,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kPersonFilledPaddedSmall20Path, CANVAS_DIMENSIONS, 20,
MOVE_TO, 10, 4,
CUBIC_TO, 8.34f, 4, 7, 5.34f, 7, 7,
R_CUBIC_TO, 0, 1.66f, 1.34f, 3, 3, 3,
R_CUBIC_TO, 1.66f, 0, 3, -1.34f, 3, -3,
R_CUBIC_TO, 0, -1.66f, -1.34f, -3, -3, -3,
CLOSE,
R_MOVE_TO, 0, 6.75f,
R_CUBIC_TO, -2, 0, -6, 1, -6, 3,
V_LINE_TO, 16,
R_H_LINE_TO, 12,
R_V_LINE_TO, -2.25f,
R_CUBIC_TO, 0, -1.99f, -4, -3, -6, -3,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPersonFilledPaddedSmallRepList, kPersonFilledPaddedSmallIcon, {kPersonFilledPaddedSmallPath, std::size(kPersonFilledPaddedSmallPath)}, {kPersonFilledPaddedSmall22Path, std::size(kPersonFilledPaddedSmall22Path)}, {kPersonFilledPaddedSmall20Path, std::size(kPersonFilledPaddedSmall20Path)})
VECTOR_ICON_REP_TEMPLATE(kPictureInPictureAltPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 16, 14,
H_LINE_TO, 0,
V_LINE_TO, 2,
R_H_LINE_TO, 16,
R_V_LINE_TO, 12,
CLOSE,
MOVE_TO, 1, 13,
R_H_LINE_TO, 14,
V_LINE_TO, 3,
H_LINE_TO, 1,
R_V_LINE_TO, 10,
CLOSE,
MOVE_TO, 8, 8,
R_H_LINE_TO, 6,
R_V_LINE_TO, 4,
H_LINE_TO, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPictureInPictureAltRepList, kPictureInPictureAltIcon, {kPictureInPictureAltPath, std::size(kPictureInPictureAltPath)})
VECTOR_ICON_REP_TEMPLATE(kPrintMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 10.75f, 5.47f,
V_LINE_TO, 3.65f,
H_LINE_TO, 5.25f,
R_V_LINE_TO, 1.82f,
H_LINE_TO, 3.86f,
R_V_LINE_TO, -3.2f,
R_H_LINE_TO, 8.27f,
R_V_LINE_TO, 3.2f,
CLOSE,
MOVE_TO, 2.85f, 6.85f,
R_H_LINE_TO, 10.32f,
CLOSE,
R_MOVE_TO, 8.86f, 1.72f,
R_ARC_TO, 0.65f, 0.65f, 0, 0, 0, 0.66f, -0.66f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 0, -0.19f, -0.47f,
R_ARC_TO, 0.63f, 0.63f, 0, 0, 0, -0.47f, -0.19f,
R_ARC_TO, 0.67f, 0.67f, 0, 0, 0, -0.47f, 0.19f,
R_ARC_TO, 0.64f, 0.64f, 0, 0, 0, -0.19f, 0.47f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 0, 0.66f, 0.66f,
CLOSE,
R_MOVE_TO, -0.95f, 3.88f,
R_V_LINE_TO, -2.1f,
H_LINE_TO, 5.25f,
R_V_LINE_TO, 2.09f,
CLOSE,
R_MOVE_TO, 1.38f, 1.36f,
H_LINE_TO, 3.86f,
R_V_LINE_TO, -2.49f,
H_LINE_TO, 1.47f,
V_LINE_TO, 7.2f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.5f, -1.23f,
R_CUBIC_TO, 0.33f, -0.34f, 0.74f, -0.51f, 1.22f, -0.51f,
R_H_LINE_TO, 9.63f,
R_CUBIC_TO, 0.48f, 0, 0.89f, 0.17f, 1.22f, 0.51f,
R_CUBIC_TO, 0.33f, 0.34f, 0.5f, 0.75f, 0.5f, 1.23f,
R_V_LINE_TO, 4.11f,
R_H_LINE_TO, -2.4f,
CLOSE,
R_MOVE_TO, 1.03f, -3.87f,
R_V_LINE_TO, -2.5f,
R_ARC_TO, 0.53f, 0.53f, 0, 0, 0, -0.18f, -0.41f,
R_ARC_TO, 0.61f, 0.61f, 0, 0, 0, -0.42f, -0.17f,
H_LINE_TO, 3.42f,
R_CUBIC_TO, -0.16f, 0, -0.3f, 0.06f, -0.41f, 0.17f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.16f, 0.42f,
V_LINE_TO, 9.93f,
R_H_LINE_TO, 1.01f,
R_V_LINE_TO, -0.96f,
R_H_LINE_TO, 8.27f,
R_V_LINE_TO, 0.96f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPrintMenuRepList, kPrintMenuIcon, {kPrintMenuPath, std::size(kPrintMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kPrivacyTipPath, CANVAS_DIMENSIONS, 960,
MOVE_TO, 440, 680,
R_H_LINE_TO, 80,
R_V_LINE_TO, -240,
R_H_LINE_TO, -80,
R_V_LINE_TO, 240,
CLOSE,
R_MOVE_TO, 40, -320,
R_QUADRATIC_TO, 17, 0, 28.5f, -11.5f,
QUADRATIC_TO_SHORTHAND, 520, 320,
R_QUADRATIC_TO, 0, -17, -11.5f, -28.5f,
QUADRATIC_TO_SHORTHAND, 480, 280,
R_QUADRATIC_TO, -17, 0, -28.5f, 11.5f,
QUADRATIC_TO_SHORTHAND, 440, 320,
R_QUADRATIC_TO, 0, 17, 11.5f, 28.5f,
QUADRATIC_TO_SHORTHAND, 480, 360,
CLOSE,
R_MOVE_TO, 0, 520,
R_QUADRATIC_TO, -139, -35, -229.5f, -159.5f,
QUADRATIC_TO_SHORTHAND, 160, 444,
R_V_LINE_TO, -244,
R_LINE_TO, 320, -120,
R_LINE_TO, 320, 120,
R_V_LINE_TO, 244,
R_QUADRATIC_TO, 0, 152, -90.5f, 276.5f,
QUADRATIC_TO_SHORTHAND, 480, 880,
CLOSE,
R_MOVE_TO, 0, -84,
R_QUADRATIC_TO, 104, -33, 172, -132,
R_QUADRATIC_TO, 68, -99, 68, -220,
R_V_LINE_TO, -189,
R_LINE_TO, -240, -90,
R_LINE_TO, -240, 90,
R_V_LINE_TO, 189,
R_QUADRATIC_TO, 0, 121, 68, 220,
R_QUADRATIC_TO, 68, 99, 172, 132,
CLOSE,
R_MOVE_TO, 0, -316,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPrivacyTipRepList, kPrivacyTipIcon, {kPrivacyTipPath, std::size(kPrivacyTipPath)})
VECTOR_ICON_REP_TEMPLATE(kQrCodeChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 3, 9,
V_LINE_TO, 3,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 1.5f, -1.5f,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
MOVE_TO, 3, 17,
R_V_LINE_TO, -6,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 1.5f, -1.5f,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
MOVE_TO, 11, 9,
V_LINE_TO, 3,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 1.5f, -1.5f,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, 3, 9.5f,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 17,
V_LINE_TO, 17,
CLOSE,
MOVE_TO, 11, 12.5f,
V_LINE_TO, 11,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 1.5f, 1.5f,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 14,
V_LINE_TO, 14,
CLOSE,
MOVE_TO, 11, 15.5f,
V_LINE_TO, 14,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 1.5f, 1.5f,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 14,
V_LINE_TO, 17,
CLOSE,
R_MOVE_TO, 1.5f, -1.5f,
V_LINE_TO, 14,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 0, -3,
V_LINE_TO, 11,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
CLOSE,
R_MOVE_TO, 1.5f, 1.5f,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 17,
V_LINE_TO, 14,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kQrCodeChromeRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 2.22f, 7.2f,
V_LINE_TO, 2.21f,
R_H_LINE_TO, 4.98f,
R_V_LINE_TO, 4.98f,
CLOSE,
R_MOVE_TO, 1.39f, -1.38f,
R_H_LINE_TO, 2.21f,
V_LINE_TO, 3.6f,
H_LINE_TO, 3.6f,
CLOSE,
R_MOVE_TO, -1.39f, 7.97f,
V_LINE_TO, 8.8f,
R_H_LINE_TO, 4.98f,
R_V_LINE_TO, 4.98f,
CLOSE,
R_MOVE_TO, 1.39f, -1.39f,
R_H_LINE_TO, 2.21f,
R_V_LINE_TO, -2.21f,
H_LINE_TO, 3.6f,
CLOSE,
MOVE_TO, 8.8f, 7.2f,
V_LINE_TO, 2.22f,
R_H_LINE_TO, 4.98f,
R_V_LINE_TO, 4.98f,
CLOSE,
R_MOVE_TO, 1.38f, -1.38f,
R_H_LINE_TO, 2.21f,
V_LINE_TO, 3.6f,
R_H_LINE_TO, -2.21f,
CLOSE,
R_MOVE_TO, 2.36f, 7.97f,
R_V_LINE_TO, -1.25f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.25f,
CLOSE,
R_MOVE_TO, -3.74f, -3.74f,
V_LINE_TO, 8.8f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.24f,
CLOSE,
R_MOVE_TO, 1.25f, 1.25f,
R_V_LINE_TO, -1.25f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.25f,
CLOSE,
MOVE_TO, 8.8f, 12.54f,
R_V_LINE_TO, -1.24f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.24f,
CLOSE,
R_MOVE_TO, 1.25f, 1.25f,
R_V_LINE_TO, -1.25f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.25f,
CLOSE,
R_MOVE_TO, 1.25f, -1.25f,
R_V_LINE_TO, -1.24f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.24f,
CLOSE,
R_MOVE_TO, 0, -2.49f,
V_LINE_TO, 8.8f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.24f,
CLOSE,
R_MOVE_TO, 1.25f, 1.25f,
R_V_LINE_TO, -1.25f,
R_H_LINE_TO, 1.25f,
R_V_LINE_TO, 1.25f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kQrCodeChromeRefreshRepList, kQrCodeChromeRefreshIcon, {kQrCodeChromeRefreshPath, std::size(kQrCodeChromeRefreshPath)}, {kQrCodeChromeRefresh16Path, std::size(kQrCodeChromeRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kQrcodeGeneratorPath, CANVAS_DIMENSIONS, 18,
ROUND_RECT, 1, 1, 16, 16, 2,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
ROUND_RECT, 2, 2, 14, 14, 1.72f,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
ROUND_RECT, 7.4f, 0, 3.2f, 18, 0,
ROUND_RECT, 0, 7.4f, 18, 3.2f, 0,
CLOSE,
NEW_PATH,
ROUND_RECT, 3.8f, 3.8f, 4.5f, 4.5f, 0.9f,
ROUND_RECT, 9.7f, 3.8f, 4.5f, 4.5f, 0.9f,
ROUND_RECT, 3.8f, 9.7f, 4.5f, 4.5f, 0.9f,
ROUND_RECT, 9.7f, 9.7f, 4.5f, 4.5f, 0.9f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kQrcodeGeneratorRepList, kQrcodeGeneratorIcon, {kQrcodeGeneratorPath, std::size(kQrcodeGeneratorPath)})
VECTOR_ICON_REP_TEMPLATE(kReadLaterAddPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 11.52f, 15.41f,
R_V_LINE_TO, -1.8f,
H_LINE_TO, 9.72f,
R_V_LINE_TO, -1.3f,
R_H_LINE_TO, 1.8f,
R_V_LINE_TO, -1.8f,
R_H_LINE_TO, 1.31f,
R_V_LINE_TO, 1.8f,
R_H_LINE_TO, 1.8f,
R_V_LINE_TO, 1.3f,
R_H_LINE_TO, -1.8f,
R_V_LINE_TO, 1.8f,
CLOSE,
R_MOVE_TO, -7.87f, -3.06f,
V_LINE_TO, 3.65f,
CLOSE,
R_MOVE_TO, 0, 1.38f,
R_CUBIC_TO, -0.39f, 0, -0.72f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 8.7f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 5.59f,
R_ARC_TO, 3.43f, 3.43f, 0, 0, 0, -0.67f, -0.22f,
R_ARC_TO, 3.92f, 3.92f, 0, 0, 0, -0.71f, -0.08f,
V_LINE_TO, 3.65f,
H_LINE_TO, 3.65f,
R_V_LINE_TO, 8.7f,
R_H_LINE_TO, 4.52f,
R_CUBIC_TO, -0.04f, 0.23f, -0.06f, 0.46f, -0.05f, 0.69f,
R_CUBIC_TO, 0, 0.23f, 0.03f, 0.46f, 0.08f, 0.69f,
CLOSE,
R_MOVE_TO, 1.74f, -2.56f,
R_ARC_TO, 0.59f, 0.59f, 0, 0, 0, 0.43f, -0.17f,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, 0.18f, -0.43f,
R_ARC_TO, 0.59f, 0.59f, 0, 0, 0, -0.17f, -0.43f,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, -0.43f, -0.17f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.42f, 0.17f,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, -0.18f, 0.43f,
R_CUBIC_TO, 0, 0.17f, 0.05f, 0.31f, 0.17f, 0.43f,
R_ARC_TO, 0.56f, 0.56f, 0, 0, 0, 0.42f, 0.18f,
CLOSE,
R_MOVE_TO, 0, -2.57f,
ARC_TO, 0.59f, 0.59f, 0, 0, 0, 6, 8,
R_ARC_TO, 0.6f, 0.6f, 0, 0, 0, -0.17f, -0.43f,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, -0.43f, -0.18f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.42f, 0.17f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.18f, 0.43f,
R_CUBIC_TO, 0, 0.17f, 0.05f, 0.31f, 0.17f, 0.43f,
R_ARC_TO, 0.56f, 0.56f, 0, 0, 0, 0.42f, 0.18f,
CLOSE,
R_MOVE_TO, 0, -2.58f,
R_ARC_TO, 0.59f, 0.59f, 0, 0, 0, 0.43f, -0.17f,
ARC_TO, 0.57f, 0.57f, 0, 0, 0, 6, 5.43f,
ARC_TO, 0.59f, 0.59f, 0, 0, 0, 5.83f, 5,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, -0.43f, -0.18f,
R_ARC_TO, 0.58f, 0.58f, 0, 0, 0, -0.42f, 0.17f,
R_ARC_TO, 0.57f, 0.57f, 0, 0, 0, -0.18f, 0.43f,
R_CUBIC_TO, 0, 0.17f, 0.05f, 0.31f, 0.17f, 0.43f,
R_ARC_TO, 0.56f, 0.56f, 0, 0, 0, 0.42f, 0.18f,
CLOSE,
R_MOVE_TO, 1.8f, 2.58f,
R_H_LINE_TO, 4,
V_LINE_TO, 7.4f,
R_H_LINE_TO, -4,
CLOSE,
R_MOVE_TO, 0, -2.58f,
R_H_LINE_TO, 4,
V_LINE_TO, 4.82f,
R_H_LINE_TO, -4,
CLOSE,
R_MOVE_TO, 0, 5.15f,
R_H_LINE_TO, 1.33f,
R_CUBIC_TO, 0.12f, -0.22f, 0.25f, -0.44f, 0.41f, -0.64f,
R_CUBIC_TO, 0.16f, -0.2f, 0.33f, -0.39f, 0.51f, -0.56f,
H_LINE_TO, 7.2f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kReadLaterAddRepList, kReadLaterAddIcon, {kReadLaterAddPath, std::size(kReadLaterAddPath)})
VECTOR_ICON_REP_TEMPLATE(kReadingListPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 6.75f, 14,
R_CUBIC_TO, 0.21f, 0, 0.39f, -0.07f, 0.54f, -0.21f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, 0.22f, -0.53f,
R_CUBIC_TO, 0, -0.21f, -0.07f, -0.39f, -0.21f, -0.53f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, -0.53f, -0.22f,
R_CUBIC_TO, -0.21f, 0, -0.39f, 0.07f, -0.53f, 0.22f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, -0.22f, 0.53f,
R_CUBIC_TO, 0, 0.21f, 0.07f, 0.39f, 0.22f, 0.54f,
R_CUBIC_TO, 0.14f, 0.14f, 0.32f, 0.22f, 0.53f, 0.22f,
CLOSE,
R_MOVE_TO, 0, -3.25f,
R_CUBIC_TO, 0.21f, 0, 0.39f, -0.07f, 0.54f, -0.21f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, 0.22f, -0.53f,
R_CUBIC_TO, 0, -0.21f, -0.07f, -0.39f, -0.21f, -0.53f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, -0.53f, -0.22f,
R_CUBIC_TO, -0.21f, 0, -0.39f, 0.07f, -0.53f, 0.22f,
ARC_TO, 0.72f, 0.72f, 0, 0, 0, 6, 10,
R_CUBIC_TO, 0, 0.21f, 0.07f, 0.39f, 0.22f, 0.54f,
R_CUBIC_TO, 0.14f, 0.14f, 0.32f, 0.22f, 0.53f, 0.22f,
CLOSE,
R_MOVE_TO, 0, -3.25f,
R_CUBIC_TO, 0.21f, 0, 0.39f, -0.07f, 0.54f, -0.21f,
R_ARC_TO, 0.72f, 0.72f, 0, 0, 0, 0.22f, -0.53f,
R_CUBIC_TO, 0, -0.21f, -0.07f, -0.39f, -0.21f, -0.53f,
ARC_TO, 0.72f, 0.72f, 0, 0, 0, 6.75f, 6,
R_CUBIC_TO, -0.21f, 0, -0.39f, 0.07f, -0.53f, 0.22f,
ARC_TO, 0.72f, 0.72f, 0, 0, 0, 6, 6.75f,
R_CUBIC_TO, 0, 0.21f, 0.07f, 0.39f, 0.22f, 0.54f,
R_CUBIC_TO, 0.14f, 0.14f, 0.32f, 0.22f, 0.53f, 0.22f,
CLOSE,
MOVE_TO, 9, 14,
R_H_LINE_TO, 5,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 9,
CLOSE,
R_MOVE_TO, 0, -3.25f,
R_H_LINE_TO, 5,
R_V_LINE_TO, -1.5f,
H_LINE_TO, 9,
CLOSE,
MOVE_TO, 9, 7.5f,
R_H_LINE_TO, 5,
V_LINE_TO, 6,
H_LINE_TO, 9,
CLOSE,
MOVE_TO, 4.5f, 17,
R_CUBIC_TO, -0.41f, 0, -0.77f, -0.15f, -1.06f, -0.44f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 3, 15.5f,
R_V_LINE_TO, -11,
R_CUBIC_TO, 0, -0.41f, 0.15f, -0.77f, 0.44f, -1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 4.5f, 3,
R_H_LINE_TO, 11,
R_CUBIC_TO, 0.41f, 0, 0.77f, 0.15f, 1.06f, 0.44f,
R_CUBIC_TO, 0.29f, 0.29f, 0.44f, 0.65f, 0.44f, 1.06f,
R_V_LINE_TO, 11,
R_CUBIC_TO, 0, 0.41f, -0.15f, 0.77f, -0.44f, 1.06f,
ARC_TO, 1.44f, 1.44f, 0, 0, 1, 15.5f, 17,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_H_LINE_TO, 11,
R_V_LINE_TO, -11,
R_H_LINE_TO, -11,
CLOSE,
R_MOVE_TO, 0, -11,
R_V_LINE_TO, 11,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kReadingListRepList, kReadingListIcon, {kReadingListPath, std::size(kReadingListPath)})
VECTOR_ICON_REP_TEMPLATE(kReleaseAlertPath, CANVAS_DIMENSIONS, 16,
R_MOVE_TO, 5.73f, 14.97f,
R_LINE_TO, -1.28f, -2.12f,
R_LINE_TO, -2.41f, -0.54f,
R_LINE_TO, 0.23f, -2.46f,
LINE_TO, 0.63f, 8,
R_LINE_TO, 1.64f, -1.84f,
R_LINE_TO, -0.23f, -2.46f,
R_LINE_TO, 2.41f, -0.54f,
R_LINE_TO, 1.28f, -2.12f,
LINE_TO, 8, 2,
R_LINE_TO, 2.27f, -0.98f,
R_LINE_TO, 1.28f, 2.13f,
R_LINE_TO, 2.41f, 0.54f,
R_LINE_TO, -0.23f, 2.46f,
LINE_TO, 15.38f, 8,
R_LINE_TO, -1.64f, 1.84f,
R_LINE_TO, 0.23f, 2.46f,
R_LINE_TO, -2.41f, 0.54f,
R_LINE_TO, -1.28f, 2.13f,
LINE_TO, 8, 14,
CLOSE,
R_MOVE_TO, 0.56f, -1.75f,
LINE_TO, 8, 12.5f,
R_LINE_TO, 1.72f, 0.73f,
R_LINE_TO, 0.95f, -1.6f,
R_LINE_TO, 1.81f, -0.41f,
R_LINE_TO, -0.16f, -1.84f,
LINE_TO, 13.53f, 8,
R_LINE_TO, -1.21f, -1.38f,
R_LINE_TO, 0.16f, -1.84f,
R_LINE_TO, -1.81f, -0.41f,
R_LINE_TO, -0.95f, -1.6f,
LINE_TO, 8, 3.5f,
R_LINE_TO, -1.71f, -0.73f,
R_LINE_TO, -0.95f, 1.6f,
R_LINE_TO, -1.81f, 0.39f,
R_LINE_TO, 0.16f, 1.85f,
LINE_TO, 2.47f, 8,
R_LINE_TO, 1.23f, 1.38f,
R_LINE_TO, -0.18f, 1.85f,
R_LINE_TO, 1.81f, 0.41f,
CLOSE,
MOVE_TO, 8, 8,
CLOSE,
R_MOVE_TO, -0.85f, 2.44f,
LINE_TO, 11.16f, 6.45f,
R_LINE_TO, -0.9f, -0.89f,
R_LINE_TO, -3.11f, 3.1f,
R_LINE_TO, -1.41f, -1.4f,
R_LINE_TO, -0.9f, 0.89f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kReleaseAlertRepList, kReleaseAlertIcon, {kReleaseAlertPath, std::size(kReleaseAlertPath)})
VECTOR_ICON_REP_TEMPLATE(kReloadTouchPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 17.68f, 6.32f,
LINE_TO, 20, 4,
R_V_LINE_TO, 7,
R_H_LINE_TO, -7,
R_LINE_TO, 3.27f, -3.27f,
ARC_TO, 6.01f, 6.01f, 0, 0, 0, 12.04f, 6,
R_CUBIC_TO, -3.33f, 0, -6.03f, 2.69f, -6.03f, 6,
R_CUBIC_TO, 0, 3.31f, 2.7f, 6, 6.03f, 6,
R_CUBIC_TO, 2.62f, 0, 4.71f, -1.62f, 5.47f, -3.89f,
R_LINE_TO, 0.11f, -0.01f,
R_ARC_TO, 1, 1, 0, 0, 1, 0.89f, -0.6f,
R_CUBIC_TO, 0.44f, 0, 1, 0.32f, 1, 1,
R_ARC_TO, 1, 1, 0, 0, 1, -0.09f, 0.41f,
R_LINE_TO, 0.05f, 0,
CUBIC_TO, 18.26f, 17.84f, 15.41f, 20, 12.04f, 20,
CUBIC_TO, 7.6f, 20, 4, 16.42f, 4, 12,
R_CUBIC_TO, 0, -4.42f, 3.6f, -8, 8.04f, -8,
R_CUBIC_TO, 2.2f, 0, 4.2f, 0.88f, 5.65f, 2.32f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kReloadTouchRepList, kReloadTouchIcon, {kReloadTouchPath, std::size(kReloadTouchPath)})
VECTOR_ICON_REP_TEMPLATE(kRemovePath, CANVAS_DIMENSIONS, 16,
STROKE, 2.f,
CAP_SQUARE,
MOVE_TO, 3, 8,
R_H_LINE_TO, 10)
VECTOR_ICON_TEMPLATE_CC(kRemoveRepList, kRemoveIcon, {kRemovePath, std::size(kRemovePath)})
VECTOR_ICON_REP_TEMPLATE(kReportPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8, 10.01f,
R_ARC_TO, 0.64f, 0.64f, 0, 0, 0, 0.47f, -0.19f,
R_ARC_TO, 0.64f, 0.64f, 0, 0, 0, 0.19f, -0.47f,
R_ARC_TO, 0.65f, 0.65f, 0, 0, 0, -0.19f, -0.47f,
R_ARC_TO, 0.63f, 0.63f, 0, 0, 0, -0.47f, -0.19f,
R_ARC_TO, 0.64f, 0.64f, 0, 0, 0, -0.47f, 0.19f,
R_ARC_TO, 0.62f, 0.62f, 0, 0, 0, -0.19f, 0.47f,
R_ARC_TO, 0.65f, 0.65f, 0, 0, 0, 0.66f, 0.67f,
CLOSE,
MOVE_TO, 7.37f, 8.03f,
R_H_LINE_TO, 1.27f,
R_V_LINE_TO, -4.43f,
H_LINE_TO, 7.37f,
CLOSE,
R_MOVE_TO, -5.9f, 6.5f,
V_LINE_TO, 2.85f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 10.31f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.13f, 0.98f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.6f, 0.4f, 0.98f,
R_V_LINE_TO, 7.91f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.41f, -0.98f, 0.41f,
R_H_LINE_TO, -9.29f,
CLOSE,
R_MOVE_TO, 1.9f, -3.78f,
R_H_LINE_TO, 9.79f,
V_LINE_TO, 2.85f,
H_LINE_TO, 2.85f,
R_V_LINE_TO, 8.43f,
CLOSE,
R_MOVE_TO, -0.51f, 0,
V_LINE_TO, 2.85f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kReportRepList, kReportIcon, {kReportPath, std::size(kReportPath)})
VECTOR_ICON_REP_TEMPLATE(kRequestMobileSiteCheckedPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.66f, 15.32f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.98f,
V_LINE_TO, 2.06f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.7f, 0.41f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 7.08f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.6f, 0.41f, 0.98f,
V_LINE_TO, 4.8f,
R_H_LINE_TO, -1.39f,
R_V_LINE_TO, -0.74f,
H_LINE_TO, 3.66f,
R_V_LINE_TO, 7.88f,
R_H_LINE_TO, 7.08f,
V_LINE_TO, 11.2f,
R_H_LINE_TO, 1.39f,
R_V_LINE_TO, 2.74f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.7f, -0.41f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.41f, -0.98f, 0.41f,
CLOSE,
R_MOVE_TO, 0, -2.18f,
R_V_LINE_TO, 0.8f,
R_H_LINE_TO, 7.08f,
R_V_LINE_TO, -0.8f,
CLOSE,
R_MOVE_TO, 6.37f, -2.72f,
LINE_TO, 7.15f, 7.52f,
R_LINE_TO, 0.96f, -0.96f,
R_LINE_TO, 1.92f, 1.93f,
R_LINE_TO, 3.63f, -3.63f,
R_LINE_TO, 0.96f, 0.96f,
CLOSE,
MOVE_TO, 3.66f, 2.86f,
R_H_LINE_TO, 7.08f,
R_V_LINE_TO, -0.8f,
H_LINE_TO, 3.66f,
CLOSE,
R_MOVE_TO, 0, 0,
R_V_LINE_TO, -0.8f,
CLOSE,
R_MOVE_TO, 0, 10.28f,
R_V_LINE_TO, 0.8f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kRequestMobileSiteCheckedRepList, kRequestMobileSiteCheckedIcon, {kRequestMobileSiteCheckedPath, std::size(kRequestMobileSiteCheckedPath)})
VECTOR_ICON_REP_TEMPLATE(kRequestMobileSiteUncheckedPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 4.46f, 15.32f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.14f, -0.98f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 2.06f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.41f, 0.98f, -0.41f,
R_H_LINE_TO, 7.08f,
R_CUBIC_TO, 0.38f, 0, 0.71f, 0.14f, 0.98f, 0.41f,
R_CUBIC_TO, 0.27f, 0.27f, 0.4f, 0.59f, 0.4f, 0.98f,
V_LINE_TO, 13.94f,
R_CUBIC_TO, 0, 0.38f, -0.13f, 0.71f, -0.4f, 0.98f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.41f, -0.98f, 0.41f,
CLOSE,
R_MOVE_TO, 0, -2.18f,
R_V_LINE_TO, 0.8f,
R_H_LINE_TO, 7.08f,
R_V_LINE_TO, -0.8f,
CLOSE,
R_MOVE_TO, 0, -1.2f,
R_H_LINE_TO, 7.08f,
V_LINE_TO, 4.06f,
H_LINE_TO, 4.46f,
CLOSE,
R_MOVE_TO, 0, -9.08f,
R_H_LINE_TO, 7.08f,
R_V_LINE_TO, -0.8f,
H_LINE_TO, 4.46f,
CLOSE,
R_MOVE_TO, 0, 0,
R_V_LINE_TO, -0.8f,
CLOSE,
R_MOVE_TO, 0, 10.28f,
R_V_LINE_TO, 0.8f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kRequestMobileSiteUncheckedRepList, kRequestMobileSiteUncheckedIcon, {kRequestMobileSiteUncheckedPath, std::size(kRequestMobileSiteUncheckedPath)})
VECTOR_ICON_REP_TEMPLATE(kResizeHandlePath, CANVAS_DIMENSIONS, 21,
STROKE, 2.f,
MOVE_TO, 5, 5,
R_LINE_TO, 11.31f, 11.31f,
MOVE_TO, 12, 5,
R_LINE_TO, 4.24f, 4.24f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kResizeHandleRepList, kResizeHandleIcon, {kResizeHandlePath, std::size(kResizeHandlePath)})
VECTOR_ICON_REP_TEMPLATE(kRightPanelClosePath, CANVAS_DIMENSIONS, 20,
FLIPS_IN_RTL,
MOVE_TO, 6.5f, 6.75f,
V_LINE_TO, 13.25f,
LINE_TO, 10, 10,
LINE_TO, 6.5f, 6.75f,
CLOSE,
MOVE_TO, 4.4f, 17,
CUBIC_TO, 4.01f, 17, 3.68f, 16.86f, 3.41f, 16.59f,
CUBIC_TO, 3.14f, 16.32f, 3, 15.99f, 3, 15.6f,
V_LINE_TO, 4.4f,
CUBIC_TO, 3, 4.01f, 3.14f, 3.68f, 3.41f, 3.41f,
CUBIC_TO, 3.68f, 3.14f, 4.01f, 3, 4.4f, 3,
H_LINE_TO, 15.6f,
CUBIC_TO, 15.99f, 3, 16.32f, 3.14f, 16.59f, 3.41f,
CUBIC_TO, 16.86f, 3.68f, 17, 4.01f, 17, 4.4f,
V_LINE_TO, 15.6f,
CUBIC_TO, 17, 15.99f, 16.86f, 16.32f, 16.59f, 16.59f,
CUBIC_TO, 16.32f, 16.86f, 15.99f, 17, 15.6f, 17,
H_LINE_TO, 4.4f,
CLOSE,
MOVE_TO, 13, 15.5f,
H_LINE_TO, 15.5f,
V_LINE_TO, 4.5f,
H_LINE_TO, 13,
V_LINE_TO, 15.5f,
CLOSE,
MOVE_TO, 11.5f, 15.5f,
V_LINE_TO, 4.5f,
H_LINE_TO, 4.5f,
V_LINE_TO, 15.5f,
H_LINE_TO, 11.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kRightPanelCloseRepList, kRightPanelCloseIcon, {kRightPanelClosePath, std::size(kRightPanelClosePath)})
VECTOR_ICON_REP_TEMPLATE(kSadTabPath, MOVE_TO, 38, 13,
LINE_TO, 21, 13,
LINE_TO, 17, 7,
LINE_TO, 10, 7,
CUBIC_TO, 7.7f, 7, 6.02f, 8.69f, 6, 11,
LINE_TO, 6, 37,
CUBIC_TO, 6, 39.31f, 7.7f, 41, 10, 41,
LINE_TO, 38, 41,
CUBIC_TO, 40.3f, 41, 42, 39.31f, 42, 37,
LINE_TO, 42, 17,
CUBIC_TO, 42, 14.36f, 40.3f, 12.67f, 38, 13,
LINE_TO, 38, 13,
LINE_TO, 38, 13,
CLOSE,
MOVE_TO, 16, 18,
LINE_TO, 20, 18,
LINE_TO, 20, 23,
LINE_TO, 16, 23,
LINE_TO, 16, 18,
LINE_TO, 16, 18,
CLOSE,
MOVE_TO, 33, 35,
LINE_TO, 29, 35,
LINE_TO, 29, 31,
LINE_TO, 20, 31,
LINE_TO, 20, 35,
LINE_TO, 16, 35,
LINE_TO, 16, 31,
LINE_TO, 20, 31,
LINE_TO, 20, 27,
LINE_TO, 29, 27,
LINE_TO, 29, 31,
LINE_TO, 33, 31,
LINE_TO, 33, 35,
LINE_TO, 33, 35,
CLOSE,
MOVE_TO, 28, 18,
LINE_TO, 32, 18,
LINE_TO, 32, 23,
LINE_TO, 28, 23,
LINE_TO, 28, 18,
LINE_TO, 28, 18,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSadTabRepList, kSadTabIcon, {kSadTabPath, std::size(kSadTabPath)})
VECTOR_ICON_REP_TEMPLATE(kSafetyCheckPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 10, 1,
LINE_TO, 18, 4.27f,
LINE_TO, 18, 9.18f,
CUBIC_TO, 18, 13.72f, 14.59f, 17.97f, 10, 19,
CUBIC_TO, 5.41f, 17.97f, 2, 13.72f, 2, 9.18f,
LINE_TO, 2, 9.18f,
LINE_TO, 2, 4.27f,
LINE_TO, 10, 1,
CLOSE,
MOVE_TO, 13.57f, 6.38f,
LINE_TO, 8.44f, 11.78f,
LINE_TO, 6.43f, 9.66f,
LINE_TO, 5.33f, 10.82f,
LINE_TO, 8.44f, 14.09f,
LINE_TO, 14.67f, 7.55f,
LINE_TO, 13.57f, 6.38f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSafetyCheckRepList, kSafetyCheckIcon, {kSafetyCheckPath, std::size(kSafetyCheckPath)})
VECTOR_ICON_REP_TEMPLATE(kSaveGroupRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 4.51f, 16.1f,
R_V_LINE_TO, -1.58f,
R_H_LINE_TO, 1.38f,
R_LINE_TO, 0.07f, 0.05f,
R_ARC_TO, 6.67f, 6.67f, 0, 0, 1, -1.51f, -2.03f,
ARC_TO, 5.79f, 5.79f, 0, 0, 1, 3.88f, 10,
R_CUBIC_TO, 0, -1.46f, 0.45f, -2.73f, 1.35f, -3.82f,
R_ARC_TO, 6.1f, 6.1f, 0, 0, 1, 3.4f, -2.12f,
R_V_LINE_TO, 1.79f,
R_ARC_TO, 4.41f, 4.41f, 0, 0, 0, -2.17f, 1.56f,
R_CUBIC_TO, -0.57f, 0.75f, -0.85f, 1.61f, -0.85f, 2.59f,
R_CUBIC_TO, 0, 0.65f, 0.13f, 1.25f, 0.4f, 1.8f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, 1.07f, 1.43f,
R_LINE_TO, -0.04f, -0.04f,
V_LINE_TO, 12,
R_H_LINE_TO, 1.57f,
R_V_LINE_TO, 4.1f,
CLOSE,
R_MOVE_TO, 6.86f, -0.16f,
R_V_LINE_TO, -1.79f,
R_ARC_TO, 4.41f, 4.41f, 0, 0, 0, 2.17f, -1.56f,
R_CUBIC_TO, 0.57f, -0.75f, 0.85f, -1.61f, 0.85f, -2.59f,
R_CUBIC_TO, 0, -0.65f, -0.13f, -1.25f, -0.4f, -1.8f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, -1.07f, -1.43f,
R_LINE_TO, 0.04f, 0.04f,
V_LINE_TO, 8,
R_H_LINE_TO, -1.57f,
V_LINE_TO, 3.9f,
R_H_LINE_TO, 4.09f,
R_V_LINE_TO, 1.58f,
R_H_LINE_TO, -1.37f,
R_LINE_TO, -0.07f, -0.05f,
R_ARC_TO, 6.38f, 6.38f, 0, 0, 1, 1.52f, 2.03f,
R_CUBIC_TO, 0.38f, 0.79f, 0.56f, 1.63f, 0.56f, 2.55f,
R_CUBIC_TO, 0, 1.46f, -0.45f, 2.73f, -1.35f, 3.82f,
R_ARC_TO, 6.1f, 6.1f, 0, 0, 1, -3.4f, 2.12f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSaveGroupRefreshRepList, kSaveGroupRefreshIcon, {kSaveGroupRefreshPath, std::size(kSaveGroupRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kSavePagePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 8.41f, 9.09f,
LINE_TO, 7, 10.5f,
R_LINE_TO, 5, 5,
R_LINE_TO, 5, -5,
R_LINE_TO, -1.41f, -1.41f,
LINE_TO, 13, 11.67f,
V_LINE_TO, 3,
R_H_LINE_TO, -2,
R_V_LINE_TO, 8.67f,
LINE_TO, 8.41f, 9.09f,
CLOSE,
MOVE_TO, 21, 19,
V_LINE_TO, 5,
R_CUBIC_TO, 0, -1.11f, -0.9f, -2, -2, -2,
R_H_LINE_TO, -4,
R_V_LINE_TO, 2,
R_H_LINE_TO, 4,
R_V_LINE_TO, 14,
H_LINE_TO, 5,
V_LINE_TO, 5,
R_H_LINE_TO, 4,
V_LINE_TO, 3,
H_LINE_TO, 5,
R_CUBIC_TO, -1.1f, 0, -2, 0.89f, -2, 2,
R_V_LINE_TO, 14,
R_CUBIC_TO, 0, 1.1f, 0.9f, 2, 2, 2,
R_H_LINE_TO, 14,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSavePageRepList, kSavePageIcon, {kSavePagePath, std::size(kSavePagePath)})
VECTOR_ICON_REP_TEMPLATE(kSavedTabGroupBarEverythingPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 3, 9,
V_LINE_TO, 3,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 0, 8,
R_V_LINE_TO, -6,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 8, -8,
V_LINE_TO, 3,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
R_MOVE_TO, 0, 8,
R_V_LINE_TO, -6,
R_H_LINE_TO, 6,
R_V_LINE_TO, 6,
CLOSE,
MOVE_TO, 4.5f, 7.5f,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, 8, 0,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, 0, 8,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, -8, 0,
R_H_LINE_TO, 3,
R_V_LINE_TO, -3,
R_H_LINE_TO, -3,
CLOSE,
R_MOVE_TO, 8, -8,
CLOSE,
R_MOVE_TO, 0, 5,
CLOSE,
R_MOVE_TO, -5, 0,
CLOSE,
R_MOVE_TO, 0, -5,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kSavedTabGroupBarEverything16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 2.27f, 7.2f,
V_LINE_TO, 2.27f,
R_H_LINE_TO, 4.93f,
R_V_LINE_TO, 4.93f,
CLOSE,
R_MOVE_TO, 0, 6.53f,
V_LINE_TO, 8.8f,
R_H_LINE_TO, 4.93f,
R_V_LINE_TO, 4.93f,
CLOSE,
MOVE_TO, 8.8f, 7.2f,
V_LINE_TO, 2.27f,
R_H_LINE_TO, 4.93f,
R_V_LINE_TO, 4.93f,
CLOSE,
R_MOVE_TO, 0, 6.54f,
V_LINE_TO, 8.8f,
R_H_LINE_TO, 4.93f,
R_V_LINE_TO, 4.93f,
CLOSE,
MOVE_TO, 3.65f, 5.82f,
R_H_LINE_TO, 2.17f,
V_LINE_TO, 3.65f,
H_LINE_TO, 3.65f,
CLOSE,
R_MOVE_TO, 6.54f, 0,
R_H_LINE_TO, 2.17f,
V_LINE_TO, 3.65f,
R_H_LINE_TO, -2.17f,
CLOSE,
R_MOVE_TO, 0, 6.54f,
R_H_LINE_TO, 2.17f,
R_V_LINE_TO, -2.17f,
R_H_LINE_TO, -2.17f,
CLOSE,
R_MOVE_TO, -6.54f, 0,
R_H_LINE_TO, 2.17f,
R_V_LINE_TO, -2.17f,
H_LINE_TO, 3.65f,
CLOSE,
R_MOVE_TO, 6.54f, -6.54f,
CLOSE,
R_MOVE_TO, 0, 4.37f,
CLOSE,
R_MOVE_TO, -4.37f, 0,
CLOSE,
R_MOVE_TO, 0, -4.37f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSavedTabGroupBarEverythingRepList, kSavedTabGroupBarEverythingIcon, {kSavedTabGroupBarEverythingPath, std::size(kSavedTabGroupBarEverythingPath)}, {kSavedTabGroupBarEverything16Path, std::size(kSavedTabGroupBarEverything16Path)})
VECTOR_ICON_REP_TEMPLATE(kSciencePath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 5, 21,
R_CUBIC_TO, -0.85f, 0, -1.45f, -0.38f, -1.81f, -1.14f,
R_CUBIC_TO, -0.36f, -0.76f, -0.27f, -1.46f, 0.26f, -2.11f,
LINE_TO, 9, 11,
V_LINE_TO, 5,
H_LINE_TO, 8,
R_ARC_TO, 0.96f, 0.96f, 0, 0, 1, -0.71f, -0.29f,
ARC_TO, 0.96f, 0.96f, 0, 0, 1, 7, 4,
R_CUBIC_TO, 0, -0.28f, 0.1f, -0.52f, 0.29f, -0.71f,
ARC_TO, 0.96f, 0.96f, 0, 0, 1, 8, 3,
R_H_LINE_TO, 8,
R_CUBIC_TO, 0.28f, 0, 0.52f, 0.1f, 0.71f, 0.29f,
R_CUBIC_TO, 0.19f, 0.19f, 0.29f, 0.43f, 0.29f, 0.71f,
R_CUBIC_TO, 0, 0.28f, -0.1f, 0.52f, -0.29f, 0.71f,
ARC_TO, 0.96f, 0.96f, 0, 0, 1, 16, 5,
R_H_LINE_TO, -1,
R_V_LINE_TO, 6,
R_LINE_TO, 5.55f, 6.75f,
R_CUBIC_TO, 0.53f, 0.65f, 0.62f, 1.36f, 0.26f, 2.11f,
CUBIC_TO, 20.45f, 20.62f, 19.85f, 21, 19, 21,
CLOSE,
R_MOVE_TO, 0, -2,
R_H_LINE_TO, 14,
R_LINE_TO, -6, -7.3f,
V_LINE_TO, 5,
R_H_LINE_TO, -2,
R_V_LINE_TO, 6.7f,
CLOSE,
R_MOVE_TO, 7, -7,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kScience20Path, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 4.5f, 17,
R_CUBIC_TO, -0.64f, 0, -1.1f, -0.28f, -1.37f, -0.84f,
R_CUBIC_TO, -0.28f, -0.56f, -0.21f, -1.09f, 0.19f, -1.58f,
LINE_TO, 7.5f, 9.5f,
R_V_LINE_TO, -5,
R_H_LINE_TO, -0.75f,
R_CUBIC_TO, -0.21f, 0, -0.39f, -0.07f, -0.53f, -0.21f,
ARC_TO, 0.72f, 0.72f, 0, 0, 1, 6, 3.75f,
R_CUBIC_TO, 0, -0.21f, 0.07f, -0.39f, 0.22f, -0.53f,
ARC_TO, 0.73f, 0.73f, 0, 0, 1, 6.75f, 3,
R_H_LINE_TO, 6.5f,
R_CUBIC_TO, 0.21f, 0, 0.39f, 0.07f, 0.54f, 0.22f,
R_CUBIC_TO, 0.14f, 0.14f, 0.22f, 0.32f, 0.22f, 0.53f,
R_CUBIC_TO, 0, 0.21f, -0.07f, 0.39f, -0.21f, 0.54f,
R_ARC_TO, 0.73f, 0.73f, 0, 0, 1, -0.53f, 0.22f,
R_H_LINE_TO, -0.75f,
R_V_LINE_TO, 5,
R_LINE_TO, 4.19f, 5.08f,
R_CUBIC_TO, 0.39f, 0.49f, 0.45f, 1.01f, 0.17f, 1.57f,
R_CUBIC_TO, -0.28f, 0.56f, -0.73f, 0.84f, -1.35f, 0.84f,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_H_LINE_TO, 11,
LINE_TO, 11, 10,
V_LINE_TO, 4.5f,
H_LINE_TO, 9,
V_LINE_TO, 10,
CLOSE,
R_MOVE_TO, 5.52f, -5.5f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kScience16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.6f, 13.73f,
R_CUBIC_TO, -0.57f, 0, -0.98f, -0.25f, -1.22f, -0.75f,
R_CUBIC_TO, -0.25f, -0.5f, -0.19f, -0.97f, 0.17f, -1.41f,
R_LINE_TO, 3.32f, -4.03f,
V_LINE_TO, 3.69f,
R_H_LINE_TO, -0.47f,
R_ARC_TO, 0.68f, 0.68f, 0, 0, 1, -0.49f, -0.2f,
R_ARC_TO, 0.67f, 0.67f, 0, 0, 1, -0.2f, -0.49f,
R_ARC_TO, 0.67f, 0.67f, 0, 0, 1, 0.2f, -0.49f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 1, 0.49f, -0.2f,
R_H_LINE_TO, 5.23f,
R_CUBIC_TO, 0.19f, 0, 0.36f, 0.07f, 0.49f, 0.2f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 1, 0.2f, 0.49f,
R_ARC_TO, 0.67f, 0.67f, 0, 0, 1, -0.2f, 0.49f,
R_ARC_TO, 0.66f, 0.66f, 0, 0, 1, -0.49f, 0.2f,
R_H_LINE_TO, -0.48f,
R_V_LINE_TO, 3.86f,
R_LINE_TO, 3.32f, 4.03f,
R_CUBIC_TO, 0.35f, 0.43f, 0.4f, 0.9f, 0.15f, 1.4f,
R_CUBIC_TO, -0.25f, 0.5f, -0.65f, 0.75f, -1.21f, 0.75f,
CLOSE,
R_MOVE_TO, 0.1f, -1.38f,
R_H_LINE_TO, 8.59f,
LINE_TO, 8.75f, 8.02f,
V_LINE_TO, 3.69f,
H_LINE_TO, 7.25f,
V_LINE_TO, 8.02f,
CLOSE,
MOVE_TO, 8.02f, 8.02f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kScienceRepList, kScienceIcon, {kSciencePath, std::size(kSciencePath)}, {kScience20Path, std::size(kScience20Path)}, {kScience16Path, std::size(kScience16Path)})
VECTOR_ICON_REP_TEMPLATE(kSearchMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 9.69f, 13.15f,
R_LINE_TO, 1.38f, 1.38f,
H_LINE_TO, 4.45f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 2.85f,
R_CUBIC_TO, 0, -0.38f, 0.13f, -0.71f, 0.4f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.98f, -0.4f,
R_H_LINE_TO, 5.21f,
R_LINE_TO, 3.28f, 3.28f,
R_V_LINE_TO, 7.99f,
R_CUBIC_TO, 0, 0.41f, -0.04f, 0.73f, -0.11f, 0.95f,
R_CUBIC_TO, -0.07f, 0.22f, -0.18f, 0.38f, -0.33f, 0.49f,
LINE_TO, 9.25f, 10.92f,
R_CUBIC_TO, -0.17f, 0.11f, -0.36f, 0.2f, -0.57f, 0.25f,
R_CUBIC_TO, -0.21f, 0.05f, -0.44f, 0.07f, -0.68f, 0.07f,
R_ARC_TO, 2.38f, 2.38f, 0, 0, 1, -1.75f, -0.72f,
R_CUBIC_TO, -0.48f, -0.48f, -0.72f, -1.06f, -0.72f, -1.75f,
R_CUBIC_TO, 0, -0.69f, 0.24f, -1.27f, 0.72f, -1.75f,
ARC_TO, 2.39f, 2.39f, 0, 0, 1, 8, 6.3f,
R_CUBIC_TO, 0.69f, 0, 1.27f, 0.24f, 1.75f, 0.72f,
R_CUBIC_TO, 0.48f, 0.48f, 0.72f, 1.06f, 0.72f, 1.75f,
R_CUBIC_TO, 0, 0.25f, -0.03f, 0.47f, -0.09f, 0.69f,
R_CUBIC_TO, -0.06f, 0.22f, -0.14f, 0.41f, -0.25f, 0.57f,
R_LINE_TO, 1.41f, 1.4f,
V_LINE_TO, 5.3f,
LINE_TO, 9.1f, 2.85f,
H_LINE_TO, 4.45f,
R_V_LINE_TO, 10.3f,
CLOSE,
MOVE_TO, 8, 9.92f,
R_CUBIC_TO, 0.32f, 0, 0.59f, -0.11f, 0.81f, -0.34f,
R_CUBIC_TO, 0.22f, -0.23f, 0.34f, -0.5f, 0.34f, -0.82f,
R_CUBIC_TO, 0, -0.32f, -0.11f, -0.59f, -0.34f, -0.81f,
R_ARC_TO, 1.13f, 1.13f, 0, 0, 0, -0.82f, -0.34f,
R_CUBIC_TO, -0.32f, 0, -0.59f, 0.11f, -0.81f, 0.34f,
R_ARC_TO, 1.11f, 1.11f, 0, 0, 0, -0.34f, 0.82f,
R_CUBIC_TO, 0, 0.32f, 0.11f, 0.59f, 0.34f, 0.82f,
R_CUBIC_TO, 0.23f, 0.22f, 0.5f, 0.33f, 0.82f, 0.33f,
CLOSE,
MOVE_TO, 8, 8.6f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSearchMenuRepList, kSearchMenuIcon, {kSearchMenuPath, std::size(kSearchMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kSecurityPath, CANVAS_DIMENSIONS, 48,
MOVE_TO, 24, 2,
LINE_TO, 6, 10,
R_V_LINE_TO, 12,
R_CUBIC_TO, 0, 11.1f, 7.67f, 21.47f, 18, 24,
R_CUBIC_TO, 10.33f, -2.53f, 18, -12.9f, 18, -24,
V_LINE_TO, 10,
LINE_TO, 24, 2,
CLOSE,
R_MOVE_TO, 0, 21.98f,
R_H_LINE_TO, 14,
R_CUBIC_TO, -1.06f, 8.24f, -6.55f, 15.58f, -14, 17.87f,
V_LINE_TO, 24,
H_LINE_TO, 10,
V_LINE_TO, 12.6f,
R_LINE_TO, 14, -6.22f,
R_V_LINE_TO, 17.6f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSecurityRepList, kSecurityIcon, {kSecurityPath, std::size(kSecurityPath)})
VECTOR_ICON_REP_TEMPLATE(kSettingsMenuPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 6.6f, 14.54f,
R_LINE_TO, -0.37f, -1.94f,
R_CUBIC_TO, -0.23f, -0.09f, -0.46f, -0.2f, -0.68f, -0.32f,
R_ARC_TO, 3.78f, 3.78f, 0, 0, 1, -0.62f, -0.43f,
R_LINE_TO, -1.87f, 0.64f,
R_LINE_TO, -1.4f, -2.43f,
R_LINE_TO, 1.48f, -1.29f,
R_ARC_TO, 6.3f, 6.3f, 0, 0, 1, -0.05f, -0.37f,
ARC_TO, 4.13f, 4.13f, 0, 0, 1, 3.06f, 8,
R_CUBIC_TO, 0, -0.13f, 0.01f, -0.26f, 0.02f, -0.39f,
R_CUBIC_TO, 0.01f, -0.12f, 0.03f, -0.24f, 0.05f, -0.37f,
LINE_TO, 1.64f, 5.96f,
R_LINE_TO, 1.4f, -2.43f,
R_LINE_TO, 1.88f, 0.63f,
R_CUBIC_TO, 0.2f, -0.16f, 0.4f, -0.3f, 0.62f, -0.43f,
R_CUBIC_TO, 0.22f, -0.12f, 0.45f, -0.23f, 0.68f, -0.32f,
R_LINE_TO, 0.38f, -1.95f,
R_H_LINE_TO, 2.8f,
R_LINE_TO, 0.38f, 1.95f,
R_CUBIC_TO, 0.24f, 0.09f, 0.46f, 0.2f, 0.68f, 0.32f,
R_CUBIC_TO, 0.22f, 0.12f, 0.43f, 0.26f, 0.63f, 0.43f,
R_LINE_TO, 1.88f, -0.63f,
R_LINE_TO, 1.4f, 2.43f,
R_LINE_TO, -1.49f, 1.29f,
R_ARC_TO, 4.15f, 4.15f, 0, 0, 1, 0.06f, 0.75f,
R_ARC_TO, 4.15f, 4.15f, 0, 0, 1, -0.06f, 0.75f,
R_LINE_TO, 1.49f, 1.29f,
R_LINE_TO, -1.41f, 2.43f,
R_LINE_TO, -1.88f, -0.64f,
R_ARC_TO, 4.07f, 4.07f, 0, 0, 1, -0.62f, 0.43f,
R_ARC_TO, 5.2f, 5.2f, 0, 0, 1, -0.68f, 0.32f,
R_LINE_TO, -0.37f, 1.95f,
CLOSE,
R_MOVE_TO, 1.14f, -1.38f,
R_H_LINE_TO, 0.51f,
R_LINE_TO, 0.32f, -1.64f,
R_ARC_TO, 3.73f, 3.73f, 0, 0, 0, 1.2f, -0.44f,
R_ARC_TO, 3.28f, 3.28f, 0, 0, 0, 0.96f, -0.82f,
R_LINE_TO, 1.59f, 0.53f,
R_LINE_TO, 0.25f, -0.43f,
R_LINE_TO, -1.25f, -1.11f,
R_CUBIC_TO, 0.07f, -0.19f, 0.12f, -0.39f, 0.16f, -0.6f,
R_CUBIC_TO, 0.04f, -0.21f, 0.06f, -0.42f, 0.06f, -0.65f,
R_CUBIC_TO, 0, -0.23f, -0.02f, -0.44f, -0.06f, -0.65f,
R_CUBIC_TO, -0.04f, -0.21f, -0.09f, -0.41f, -0.16f, -0.61f,
R_LINE_TO, 1.26f, -1.11f,
R_LINE_TO, -0.26f, -0.42f,
R_LINE_TO, -1.59f, 0.53f,
R_ARC_TO, 3.4f, 3.4f, 0, 0, 0, -0.96f, -0.82f,
R_ARC_TO, 3.72f, 3.72f, 0, 0, 0, -1.2f, -0.44f,
R_LINE_TO, -0.32f, -1.64f,
R_H_LINE_TO, -0.52f,
LINE_TO, 7.43f, 4.48f,
R_ARC_TO, 3.76f, 3.76f, 0, 0, 0, -1.21f, 0.44f,
R_ARC_TO, 3.42f, 3.42f, 0, 0, 0, -0.96f, 0.82f,
R_LINE_TO, -1.59f, -0.53f,
R_LINE_TO, -0.26f, 0.43f,
R_LINE_TO, 1.25f, 1.11f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, -0.16f, 0.61f,
R_CUBIC_TO, -0.04f, 0.21f, -0.06f, 0.43f, -0.06f, 0.65f,
R_CUBIC_TO, 0, 0.23f, 0.02f, 0.44f, 0.06f, 0.65f,
R_CUBIC_TO, 0.04f, 0.21f, 0.09f, 0.41f, 0.16f, 0.61f,
R_LINE_TO, -1.25f, 1.1f,
R_LINE_TO, 0.26f, 0.43f,
R_LINE_TO, 1.59f, -0.53f,
R_CUBIC_TO, 0.27f, 0.33f, 0.59f, 0.6f, 0.97f, 0.81f,
R_CUBIC_TO, 0.38f, 0.22f, 0.78f, 0.36f, 1.2f, 0.44f,
CLOSE,
R_MOVE_TO, 0.25f, -2.75f,
R_ARC_TO, 2.3f, 2.3f, 0, 0, 0, 1.7f, -0.7f,
ARC_TO, 2.3f, 2.3f, 0, 0, 0, 10.39f, 8,
R_ARC_TO, 2.3f, 2.3f, 0, 0, 0, -0.7f, -1.7f,
R_ARC_TO, 2.3f, 2.3f, 0, 0, 0, -1.7f, -0.7f,
R_CUBIC_TO, -0.66f, 0, -1.23f, 0.23f, -1.7f, 0.7f,
ARC_TO, 2.3f, 2.3f, 0, 0, 0, 5.59f, 8,
R_ARC_TO, 2.3f, 2.3f, 0, 0, 0, 0.7f, 1.7f,
R_CUBIC_TO, 0.47f, 0.47f, 1.03f, 0.7f, 1.7f, 0.7f,
CLOSE,
MOVE_TO, 8, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSettingsMenuRepList, kSettingsMenuIcon, {kSettingsMenuPath, std::size(kSettingsMenuPath)})
VECTOR_ICON_REP_TEMPLATE(kSharingHubScreenshotPath, CANVAS_DIMENSIONS, 18,
ROUND_RECT, 2, 2, 15, 15, 2,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
ROUND_RECT, 3, 3, 13, 13, 1.72f,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
ROUND_RECT, 7.4f, 0, 3.2f, 18, 0,
ROUND_RECT, 0, 7.4f, 18, 3.2f, 0,
CLOSE,
NEW_PATH,
STROKE, 1,
MOVE_TO, 16.5, 12,
R_V_LINE_TO, 7,
MOVE_TO, 12, 16.5,
R_H_LINE_TO, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSharingHubScreenshotRepList, kSharingHubScreenshotIcon, {kSharingHubScreenshotPath, std::size(kSharingHubScreenshotPath)})
VECTOR_ICON_REP_TEMPLATE(kSidePanelPath, CANVAS_DIMENSIONS, 16,
FLIPS_IN_RTL,
MOVE_TO, 15, 1,
H_LINE_TO, 1,
V_LINE_TO, 15,
H_LINE_TO, 15,
V_LINE_TO, 1,
CLOSE,
MOVE_TO, 10, 3,
H_LINE_TO, 3,
V_LINE_TO, 13,
H_LINE_TO, 10,
V_LINE_TO, 3,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSidePanelRepList, kSidePanelIcon, {kSidePanelPath, std::size(kSidePanelPath)})
VECTOR_ICON_REP_TEMPLATE(kSignOutPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 10.09f, 15.59f,
LINE_TO, 11.5f, 17,
R_LINE_TO, 5, -5,
R_LINE_TO, -5, -5,
R_LINE_TO, -1.41f, 1.41f,
LINE_TO, 12.67f, 11,
H_LINE_TO, 3,
R_V_LINE_TO, 2,
R_H_LINE_TO, 9.67f,
R_LINE_TO, -2.58f, 2.59f,
CLOSE,
MOVE_TO, 19, 3,
H_LINE_TO, 5,
R_CUBIC_TO, -1.11f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 4,
R_H_LINE_TO, 2,
V_LINE_TO, 5,
R_H_LINE_TO, 14,
R_V_LINE_TO, 14,
H_LINE_TO, 5,
R_V_LINE_TO, -4,
H_LINE_TO, 3,
R_V_LINE_TO, 4,
R_CUBIC_TO, 0, 1.1f, 0.89f, 2, 2, 2,
R_H_LINE_TO, 14,
R_CUBIC_TO, 1.1f, 0, 2, -0.9f, 2, -2,
V_LINE_TO, 5,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSignOutRepList, kSignOutIcon, {kSignOutPath, std::size(kSignOutPath)})
VECTOR_ICON_REP_TEMPLATE(kSmartphonePath, MOVE_TO, 34, 2.02f,
LINE_TO, 14, 2,
R_CUBIC_TO, -2.21f, 0, -4, 1.79f, -4, 4,
R_V_LINE_TO, 36,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 20,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
V_LINE_TO, 6,
R_CUBIC_TO, 0, -2.21f, -1.79f, -3.98f, -4, -3.98f,
CLOSE,
MOVE_TO, 34, 38,
H_LINE_TO, 14,
V_LINE_TO, 10,
R_H_LINE_TO, 20,
R_V_LINE_TO, 28,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSmartphoneRepList, kSmartphoneIcon, {kSmartphonePath, std::size(kSmartphonePath)})
VECTOR_ICON_REP_TEMPLATE(kSmartphoneRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 5.57f, 19.16f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.51f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -0.51f, -1.22f,
V_LINE_TO, 2.57f,
R_CUBIC_TO, 0, -0.48f, 0.17f, -0.89f, 0.51f, -1.22f,
ARC_TO, 1.67f, 1.67f, 0, 0, 1, 5.57f, 0.84f,
R_H_LINE_TO, 8.85f,
R_CUBIC_TO, 0.48f, 0, 0.89f, 0.17f, 1.22f, 0.51f,
R_CUBIC_TO, 0.34f, 0.34f, 0.51f, 0.75f, 0.51f, 1.22f,
R_V_LINE_TO, 14.85f,
R_CUBIC_TO, 0, 0.48f, -0.17f, 0.89f, -0.51f, 1.22f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -1.22f, 0.51f,
CLOSE,
R_MOVE_TO, 0, -2.73f,
R_V_LINE_TO, 1,
R_H_LINE_TO, 8.85f,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, 0, -1.5f,
R_H_LINE_TO, 8.85f,
V_LINE_TO, 5.07f,
H_LINE_TO, 5.57f,
CLOSE,
R_MOVE_TO, 0, -11.35f,
R_H_LINE_TO, 8.85f,
R_V_LINE_TO, -1,
H_LINE_TO, 5.57f,
CLOSE,
R_MOVE_TO, 0, 0,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, 0, 12.85f,
R_V_LINE_TO, 1,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSmartphoneRefreshRepList, kSmartphoneRefreshIcon, {kSmartphoneRefreshPath, std::size(kSmartphoneRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kSpeakerPath, CANVAS_DIMENSIONS, 960,
FILL_RULE_NONZERO,
MOVE_TO, 680, 880,
H_LINE_TO, 280,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 200, 800,
R_V_LINE_TO, -640,
R_QUADRATIC_TO, 0, -33, 23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 280, 80,
R_H_LINE_TO, 400,
R_QUADRATIC_TO, 33, 0, 56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 760, 160,
R_V_LINE_TO, 640,
R_QUADRATIC_TO, 0, 33, -23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 680, 880,
CLOSE,
R_MOVE_TO, 0, -80,
R_V_LINE_TO, -640,
H_LINE_TO, 280,
R_V_LINE_TO, 640,
R_H_LINE_TO, 400,
CLOSE,
MOVE_TO, 480, 360,
R_QUADRATIC_TO, 33, 0, 56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 560, 280,
R_QUADRATIC_TO, 0, -33, -23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 480, 200,
R_QUADRATIC_TO, -33, 0, -56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 400, 280,
R_QUADRATIC_TO, 0, 33, 23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 480, 360,
CLOSE,
R_MOVE_TO, 0, 400,
R_QUADRATIC_TO, 66, 0, 113, -47,
R_QUADRATIC_TO, 47, -47, 47, -113,
R_QUADRATIC_TO, 0, -66, -47, -113,
R_QUADRATIC_TO, -47, -47, -113, -47,
R_QUADRATIC_TO, -66, 0, -113, 47,
R_QUADRATIC_TO, -47, 47, -47, 113,
R_QUADRATIC_TO, 0, 66, 47, 113,
R_QUADRATIC_TO, 47, 47, 113, 47,
CLOSE,
R_MOVE_TO, 0, -80,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 400, 600,
R_QUADRATIC_TO, 0, -33, 23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 480, 520,
R_QUADRATIC_TO, 33, 0, 56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 560, 600,
R_QUADRATIC_TO, 0, 33, -23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 480, 680,
CLOSE,
MOVE_TO, 280, 160,
R_V_LINE_TO, 640,
R_V_LINE_TO, -640,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSpeakerRepList, kSpeakerIcon, {kSpeakerPath, std::size(kSpeakerPath)})
VECTOR_ICON_REP_TEMPLATE(kSpeakerGroupPath, CANVAS_DIMENSIONS, 960,
FILL_RULE_NONZERO,
MOVE_TO, 400, 680,
R_H_LINE_TO, 360,
R_V_LINE_TO, -560,
H_LINE_TO, 400,
R_V_LINE_TO, 560,
CLOSE,
R_MOVE_TO, 0, 80,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 320, 680,
R_V_LINE_TO, -560,
R_QUADRATIC_TO, 0, -33, 23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 400, 40,
R_H_LINE_TO, 360,
R_QUADRATIC_TO, 33, 0, 56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 840, 120,
R_V_LINE_TO, 560,
R_QUADRATIC_TO, 0, 33, -23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 760, 760,
H_LINE_TO, 400,
CLOSE,
R_MOVE_TO, 180, -460,
R_QUADRATIC_TO, 25, 0, 42.5f, -17.5f,
QUADRATIC_TO_SHORTHAND, 640, 240,
R_QUADRATIC_TO, 0, -25, -17.5f, -42.5f,
QUADRATIC_TO_SHORTHAND, 580, 180,
R_QUADRATIC_TO, -25, 0, -42.5f, 17.5f,
QUADRATIC_TO_SHORTHAND, 520, 240,
R_QUADRATIC_TO, 0, 25, 17.5f, 42.5f,
QUADRATIC_TO_SHORTHAND, 580, 300,
CLOSE,
R_MOVE_TO, 0, 340,
R_QUADRATIC_TO, 58, 0, 99, -41,
R_QUADRATIC_TO, 41, -41, 41, -99,
R_QUADRATIC_TO, 0, -58, -41, -99,
R_QUADRATIC_TO, -41, -41, -99, -41,
R_QUADRATIC_TO, -58, 0, -99, 41,
R_QUADRATIC_TO, -41, 41, -41, 99,
R_QUADRATIC_TO, 0, 58, 41, 99,
R_QUADRATIC_TO, 41, 41, 99, 41,
CLOSE,
R_MOVE_TO, 0, -80,
R_QUADRATIC_TO, -25, 0, -42.5f, -17.5f,
QUADRATIC_TO_SHORTHAND, 520, 500,
R_QUADRATIC_TO, 0, -25, 17.5f, -42.5f,
QUADRATIC_TO_SHORTHAND, 580, 440,
R_QUADRATIC_TO, 25, 0, 42.5f, 17.5f,
QUADRATIC_TO_SHORTHAND, 640, 500,
R_QUADRATIC_TO, 0, 25, -17.5f, 42.5f,
QUADRATIC_TO_SHORTHAND, 580, 560,
CLOSE,
R_MOVE_TO, 60, 360,
H_LINE_TO, 240,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 160, 840,
R_V_LINE_TO, -640,
R_H_LINE_TO, 80,
R_V_LINE_TO, 640,
R_H_LINE_TO, 400,
R_V_LINE_TO, 80,
CLOSE,
MOVE_TO, 400, 120,
R_V_LINE_TO, 560,
R_V_LINE_TO, -560,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSpeakerGroupRepList, kSpeakerGroupIcon, {kSpeakerGroupPath, std::size(kSpeakerGroupPath)})
VECTOR_ICON_REP_TEMPLATE(kSubmitFeedbackPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 2.4f, 1,
H_LINE_TO, 12.6f,
CUBIC_TO, 13.37f, 1, 14, 1.63f, 14, 2.4f,
V_LINE_TO, 10.62f,
CUBIC_TO, 14, 11.39f, 13.37f, 12.02f, 12.6f, 12.02f,
H_LINE_TO, 3.8f,
LINE_TO, 1, 15,
LINE_TO, 1.01f, 2.4f,
CUBIC_TO, 1.01f, 1.63f, 1.63f, 1, 2.4f, 1,
CLOSE,
MOVE_TO, 13, 10.99f,
V_LINE_TO, 2,
H_LINE_TO, 2,
V_LINE_TO, 12.02f,
LINE_TO, 3, 10.99f,
H_LINE_TO, 13,
CLOSE,
NEW_PATH,
MOVE_TO, 8, 8,
H_LINE_TO, 7,
V_LINE_TO, 9,
H_LINE_TO, 8,
V_LINE_TO, 8,
CLOSE,
NEW_PATH,
MOVE_TO, 7, 4,
H_LINE_TO, 8,
V_LINE_TO, 7,
H_LINE_TO, 7,
V_LINE_TO, 4,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSubmitFeedbackRepList, kSubmitFeedbackIcon, {kSubmitFeedbackPath, std::size(kSubmitFeedbackPath)})
VECTOR_ICON_REP_TEMPLATE(kSupervisorAccountCirclePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 11.99f, 2,
R_CUBIC_TO, -5.52f, 0, -10, 4.48f, -10, 10,
R_CUBIC_TO, 0, 5.52f, 4.48f, 10, 10, 10,
R_CUBIC_TO, 5.52f, 0, 10, -4.48f, 10, -10,
R_CUBIC_TO, 0, -5.52f, -4.48f, -10, -10, -10,
CLOSE,
R_MOVE_TO, 3.61f, 6.34f,
R_CUBIC_TO, 1.07f, 0, 1.93f, 0.86f, 1.93f, 1.93f,
R_CUBIC_TO, 0, 1.07f, -0.86f, 1.93f, -1.93f, 1.93f,
R_CUBIC_TO, -1.07f, 0, -1.93f, -0.86f, -1.93f, -1.93f,
R_CUBIC_TO, -0.01f, -1.07f, 0.86f, -1.93f, 1.93f, -1.93f,
CLOSE,
R_MOVE_TO, -6, -1.58f,
R_CUBIC_TO, 1.3f, 0, 2.36f, 1.06f, 2.36f, 2.36f,
R_CUBIC_TO, 0, 1.3f, -1.06f, 2.36f, -2.36f, 2.36f,
R_CUBIC_TO, -1.3f, 0, -2.36f, -1.06f, -2.36f, -2.36f,
R_CUBIC_TO, 0, -1.31f, 1.05f, -2.36f, 2.36f, -2.36f,
CLOSE,
R_MOVE_TO, 0, 9.13f,
R_V_LINE_TO, 3.75f,
R_CUBIC_TO, -2.4f, -0.75f, -4.3f, -2.6f, -5.14f, -4.96f,
R_CUBIC_TO, 1.05f, -1.12f, 3.67f, -1.69f, 5.14f, -1.69f,
R_CUBIC_TO, 0.53f, 0, 1.2f, 0.08f, 1.9f, 0.22f,
R_CUBIC_TO, -1.64f, 0.87f, -1.9f, 2.02f, -1.9f, 2.68f,
CLOSE,
MOVE_TO, 11.99f, 20,
R_CUBIC_TO, -0.27f, 0, -0.53f, -0.01f, -0.79f, -0.04f,
R_V_LINE_TO, -4.07f,
R_CUBIC_TO, 0, -1.42f, 2.94f, -2.13f, 4.4f, -2.13f,
R_CUBIC_TO, 1.07f, 0, 2.92f, 0.39f, 3.84f, 1.15f,
R_CUBIC_TO, -1.17f, 2.97f, -4.06f, 5.09f, -7.45f, 5.09f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSupervisorAccountCircleRepList, kSupervisorAccountCircleIcon, {kSupervisorAccountCirclePath, std::size(kSupervisorAccountCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kSyncChromeRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 4.51f, 16.1f,
R_V_LINE_TO, -1.58f,
R_H_LINE_TO, 1.38f,
R_LINE_TO, 0.07f, 0.05f,
R_ARC_TO, 6.67f, 6.67f, 0, 0, 1, -1.51f, -2.03f,
ARC_TO, 5.79f, 5.79f, 0, 0, 1, 3.88f, 10,
R_CUBIC_TO, 0, -1.46f, 0.45f, -2.73f, 1.35f, -3.82f,
R_ARC_TO, 6.1f, 6.1f, 0, 0, 1, 3.4f, -2.12f,
R_V_LINE_TO, 1.79f,
R_ARC_TO, 4.41f, 4.41f, 0, 0, 0, -2.17f, 1.56f,
R_CUBIC_TO, -0.57f, 0.75f, -0.85f, 1.61f, -0.85f, 2.59f,
R_CUBIC_TO, 0, 0.65f, 0.13f, 1.25f, 0.4f, 1.8f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, 1.07f, 1.43f,
R_LINE_TO, -0.04f, -0.04f,
V_LINE_TO, 12,
R_H_LINE_TO, 1.57f,
R_V_LINE_TO, 4.1f,
CLOSE,
R_MOVE_TO, 6.86f, -0.16f,
R_V_LINE_TO, -1.79f,
R_ARC_TO, 4.41f, 4.41f, 0, 0, 0, 2.17f, -1.56f,
R_CUBIC_TO, 0.57f, -0.75f, 0.85f, -1.61f, 0.85f, -2.59f,
R_CUBIC_TO, 0, -0.65f, -0.13f, -1.25f, -0.4f, -1.8f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, -1.07f, -1.43f,
R_LINE_TO, 0.04f, 0.04f,
V_LINE_TO, 8,
R_H_LINE_TO, -1.57f,
V_LINE_TO, 3.9f,
R_H_LINE_TO, 4.09f,
R_V_LINE_TO, 1.58f,
R_H_LINE_TO, -1.37f,
R_LINE_TO, -0.07f, -0.05f,
R_ARC_TO, 6.38f, 6.38f, 0, 0, 1, 1.52f, 2.03f,
R_CUBIC_TO, 0.38f, 0.79f, 0.56f, 1.63f, 0.56f, 2.55f,
R_CUBIC_TO, 0, 1.46f, -0.45f, 2.73f, -1.35f, 3.82f,
R_ARC_TO, 6.1f, 6.1f, 0, 0, 1, -3.4f, 2.12f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncChromeRefreshRepList, kSyncChromeRefreshIcon, {kSyncChromeRefreshPath, std::size(kSyncChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kSyncCirclePath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 8,
MOVE_TO, 8, 11,
R_CUBIC_TO, -1.65f, 0, -3, -1.34f, -3, -3,
R_CUBIC_TO, 0, -0.5f, 0.13f, -0.98f, 0.35f, -1.4f,
R_LINE_TO, -0.73f, -0.73f,
CUBIC_TO, 4.23f, 6.49f, 4, 7.22f, 4, 8,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_V_LINE_TO, 1.5f,
R_LINE_TO, 2, -2,
R_LINE_TO, -2, -2,
V_LINE_TO, 11,
CLOSE,
R_MOVE_TO, 0, -7,
V_LINE_TO, 2.5f,
R_LINE_TO, -2, 2,
R_LINE_TO, 2, 2,
V_LINE_TO, 5,
R_CUBIC_TO, 1.66f, 0, 3, 1.35f, 3, 3,
R_CUBIC_TO, 0, 0.51f, -0.12f, 0.99f, -0.35f, 1.4f,
R_LINE_TO, 0.73f, 0.73f,
R_CUBIC_TO, 0.39f, -0.61f, 0.62f, -1.34f, 0.62f, -2.13f,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncCircleRepList, kSyncCircleIcon, {kSyncCirclePath, std::size(kSyncCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kSyncDisabledChromeRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 15.97f, 17.11f,
R_LINE_TO, -2.93f, -2.92f,
R_ARC_TO, 7.31f, 7.31f, 0, 0, 1, -0.82f, 0.42f,
R_ARC_TO, 4.51f, 4.51f, 0, 0, 1, -0.88f, 0.3f,
R_V_LINE_TO, -1.81f,
R_CUBIC_TO, 0.07f, -0.03f, 0.14f, -0.06f, 0.22f, -0.09f,
R_CUBIC_TO, 0.07f, -0.03f, 0.14f, -0.06f, 0.21f, -0.09f,
R_LINE_TO, -5.78f, -5.8f,
R_CUBIC_TO, -0.13f, 0.28f, -0.23f, 0.57f, -0.3f, 0.87f,
R_ARC_TO, 3.81f, 3.81f, 0, 0, 0, -0.11f, 0.95f,
R_CUBIC_TO, 0, 0.64f, 0.13f, 1.24f, 0.4f, 1.8f,
R_ARC_TO, 4.45f, 4.45f, 0, 0, 0, 1.07f, 1.43f,
R_LINE_TO, -0.04f, -0.04f,
R_V_LINE_TO, -1.19f,
H_LINE_TO, 8.57f,
R_V_LINE_TO, 4.1f,
H_LINE_TO, 4.47f,
R_V_LINE_TO, -1.58f,
R_H_LINE_TO, 1.38f,
R_LINE_TO, 0.06f, 0.05f,
R_ARC_TO, 6.38f, 6.38f, 0, 0, 1, -1.52f, -2.03f,
R_ARC_TO, 5.88f, 5.88f, 0, 0, 1, -0.56f, -2.55f,
R_CUBIC_TO, 0, -0.57f, 0.07f, -1.11f, 0.23f, -1.64f,
R_ARC_TO, 6.28f, 6.28f, 0, 0, 1, 0.63f, -1.46f,
LINE_TO, 1.84f, 2.97f,
R_LINE_TO, 1.12f, -1.12f,
R_LINE_TO, 14.13f, 14.15f,
CLOSE,
R_MOVE_TO, -0.73f, -5.09f,
R_LINE_TO, -1.3f, -1.3f,
R_ARC_TO, 4.64f, 4.64f, 0, 0, 0, 0.3f, -0.85f,
R_ARC_TO, 4.24f, 4.24f, 0, 0, 0, -0.28f, -2.75f,
R_ARC_TO, 4.46f, 4.46f, 0, 0, 0, -1.07f, -1.44f,
R_LINE_TO, 0.04f, 0.04f,
R_V_LINE_TO, 1.21f,
R_H_LINE_TO, -1.57f,
V_LINE_TO, 2.84f,
R_H_LINE_TO, 4.1f,
R_V_LINE_TO, 1.58f,
R_H_LINE_TO, -1.38f,
R_LINE_TO, -0.06f, -0.05f,
R_ARC_TO, 6.31f, 6.31f, 0, 0, 1, 1.51f, 2.03f,
R_CUBIC_TO, 0.38f, 0.79f, 0.56f, 1.63f, 0.56f, 2.55f,
R_CUBIC_TO, 0, 0.57f, -0.07f, 1.11f, -0.22f, 1.63f,
R_ARC_TO, 5.92f, 5.92f, 0, 0, 1, -0.62f, 1.46f,
CLOSE,
R_MOVE_TO, -7.07f, -7.07f,
LINE_TO, 6.9f, 3.67f,
R_ARC_TO, 7.13f, 7.13f, 0, 0, 1, 0.81f, -0.41f,
R_CUBIC_TO, 0.28f, -0.12f, 0.57f, -0.22f, 0.88f, -0.29f,
R_V_LINE_TO, 1.81f,
R_CUBIC_TO, -0.07f, 0.03f, -0.14f, 0.06f, -0.21f, 0.09f,
R_ARC_TO, 1.92f, 1.92f, 0, 0, 1, -0.21f, 0.09f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncDisabledChromeRefreshRepList, kSyncDisabledChromeRefreshIcon, {kSyncDisabledChromeRefreshPath, std::size(kSyncDisabledChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kSyncErrorCirclePath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 8,
MOVE_TO, 3.5f, 8,
R_CUBIC_TO, 0, 1.11f, 0.46f, 2.1f, 1.18f, 2.82f,
LINE_TO, 3.5f, 12,
R_H_LINE_TO, 3,
V_LINE_TO, 9,
R_LINE_TO, -1.12f, 1.12f,
CUBIC_TO, 4.84f, 9.57f, 4.5f, 8.83f, 4.5f, 8,
R_CUBIC_TO, 0, -1.3f, 0.84f, -2.41f, 2, -2.82f,
V_LINE_TO, 4.13f,
R_CUBIC_TO, -1.72f, 0.45f, -3, 2.01f, -3, 3.87f,
CLOSE,
R_MOVE_TO, 4, 2.5f,
R_H_LINE_TO, 1,
R_V_LINE_TO, -1,
R_H_LINE_TO, -1,
R_V_LINE_TO, 1,
CLOSE,
R_MOVE_TO, 5, -6.5f,
R_H_LINE_TO, -3,
R_V_LINE_TO, 3,
R_LINE_TO, 1.12f, -1.12f,
R_CUBIC_TO, 0.54f, 0.55f, 0.88f, 1.29f, 0.88f, 2.12f,
R_CUBIC_TO, 0, 1.31f, -0.83f, 2.42f, -2, 2.83f,
R_V_LINE_TO, 1.05f,
R_CUBIC_TO, 1.73f, -0.44f, 3, -2, 3, -3.87f,
R_CUBIC_TO, 0, -1.1f, -0.45f, -2.1f, -1.18f, -2.82f,
LINE_TO, 12.5f, 4,
CLOSE,
R_MOVE_TO, -5, 4.5f,
R_H_LINE_TO, 1,
R_V_LINE_TO, -3,
R_H_LINE_TO, -1,
R_V_LINE_TO, 3,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncErrorCircleRepList, kSyncErrorCircleIcon, {kSyncErrorCirclePath, std::size(kSyncErrorCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kSyncPausedCirclePath, CANVAS_DIMENSIONS, 16,
CIRCLE, 8, 8, 8,
MOVE_TO, 2, 2,
R_H_LINE_TO, 12,
R_V_LINE_TO, 12,
H_LINE_TO, 2,
V_LINE_TO, 2,
CLOSE,
R_MOVE_TO, 0, 0,
R_H_LINE_TO, 12,
R_V_LINE_TO, 12,
H_LINE_TO, 2,
V_LINE_TO, 2,
CLOSE,
MOVE_TO, 7, 5.18f,
V_LINE_TO, 4.13f,
R_CUBIC_TO, -0.4f, 0.11f, -0.77f, 0.27f, -1.11f, 0.48f,
R_LINE_TO, 0.73f, 0.73f,
R_CUBIC_TO, 0.13f, -0.06f, 0.25f, -0.12f, 0.39f, -0.16f,
CLOSE,
R_MOVE_TO, -3.57f, -0.47f,
R_LINE_TO, 1.18f, 1.18f,
R_ARC_TO, 3.96f, 3.96f, 0, 0, 0, 0.57f, 4.93f,
LINE_TO, 4, 12,
R_H_LINE_TO, 3,
V_LINE_TO, 9,
R_LINE_TO, -1.12f, 1.12f,
ARC_TO, 3, 3, 0, 0, 1, 5, 8,
R_CUBIC_TO, 0, -0.5f, 0.13f, -0.97f, 0.34f, -1.38f,
R_LINE_TO, 4.04f, 4.04f,
R_CUBIC_TO, -0.12f, 0.07f, -0.25f, 0.13f, -0.38f, 0.17f,
R_V_LINE_TO, 1.05f,
R_CUBIC_TO, 0.4f, -0.1f, 0.78f, -0.27f, 1.12f, -0.48f,
R_LINE_TO, 1.18f, 1.18f,
R_LINE_TO, 0.64f, -0.63f,
LINE_TO, 4.07f, 4.07f,
R_LINE_TO, -0.64f, 0.64f,
CLOSE,
MOVE_TO, 12, 4,
H_LINE_TO, 9,
R_V_LINE_TO, 3,
R_LINE_TO, 1.12f, -1.12f,
R_CUBIC_TO, 0.54f, 0.55f, 0.88f, 1.29f, 0.88f, 2.12f,
R_CUBIC_TO, 0, 0.5f, -0.12f, 0.97f, -0.34f, 1.39f,
R_LINE_TO, 0.73f, 0.73f,
R_ARC_TO, 3.96f, 3.96f, 0, 0, 0, -0.57f, -4.93f,
LINE_TO, 12, 4,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncPausedCircleRepList, kSyncPausedCircleIcon, {kSyncPausedCirclePath, std::size(kSyncPausedCirclePath)})
VECTOR_ICON_REP_TEMPLATE(kSyncRefreshPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 3.61f, 12.88f,
V_LINE_TO, 11.62f,
R_H_LINE_TO, 1.1f,
R_LINE_TO, 0.05f, 0.04f,
R_ARC_TO, 5.3f, 5.3f, 0, 0, 1, -1.21f, -1.63f,
ARC_TO, 4.57f, 4.57f, 0, 0, 1, 3.1f, 8,
R_CUBIC_TO, 0, -1.16f, 0.36f, -2.18f, 1.08f, -3.06f,
R_ARC_TO, 4.88f, 4.88f, 0, 0, 1, 2.72f, -1.7f,
R_V_LINE_TO, 1.43f,
R_ARC_TO, 3.49f, 3.49f, 0, 0, 0, -1.73f, 1.25f,
R_CUBIC_TO, -0.45f, 0.6f, -0.68f, 1.29f, -0.68f, 2.07f,
R_CUBIC_TO, 0, 0.52f, 0.11f, 1, 0.32f, 1.44f,
R_CUBIC_TO, 0.22f, 0.45f, 0.5f, 0.83f, 0.86f, 1.14f,
R_LINE_TO, -0.03f, -0.03f,
V_LINE_TO, 9.6f,
R_H_LINE_TO, 1.26f,
R_V_LINE_TO, 3.28f,
CLOSE,
R_MOVE_TO, 5.49f, -0.12f,
R_V_LINE_TO, -1.43f,
R_ARC_TO, 3.49f, 3.49f, 0, 0, 0, 1.73f, -1.25f,
R_CUBIC_TO, 0.45f, -0.6f, 0.68f, -1.29f, 0.68f, -2.07f,
R_CUBIC_TO, 0, -0.52f, -0.11f, -1, -0.32f, -1.44f,
R_ARC_TO, 3.61f, 3.61f, 0, 0, 0, -0.85f, -1.14f,
R_LINE_TO, 0.03f, 0.03f,
R_V_LINE_TO, 0.95f,
H_LINE_TO, 9.12f,
V_LINE_TO, 3.12f,
R_H_LINE_TO, 3.27f,
V_LINE_TO, 4.38f,
R_H_LINE_TO, -1.1f,
R_LINE_TO, -0.05f, -0.04f,
R_CUBIC_TO, 0.51f, 0.46f, 0.91f, 1, 1.21f, 1.63f,
R_CUBIC_TO, 0.3f, 0.63f, 0.45f, 1.31f, 0.45f, 2.04f,
R_CUBIC_TO, 0, 1.16f, -0.36f, 2.18f, -1.08f, 3.06f,
R_ARC_TO, 4.88f, 4.88f, 0, 0, 1, -2.72f, 1.7f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSyncRefreshRepList, kSyncRefreshIcon, {kSyncRefreshPath, std::size(kSyncRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kTabPath, MOVE_TO, 42, 6,
H_LINE_TO, 6,
R_CUBIC_TO, -2.21f, 0, -4, 1.79f, -4, 4,
R_V_LINE_TO, 28,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 36,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
V_LINE_TO, 10,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CLOSE,
R_MOVE_TO, 0, 32,
H_LINE_TO, 6,
V_LINE_TO, 10,
R_H_LINE_TO, 20,
R_V_LINE_TO, 8,
R_H_LINE_TO, 16,
R_V_LINE_TO, 20,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabRepList, kTabIcon, {kTabPath, std::size(kTabPath)})
VECTOR_ICON_REP_TEMPLATE(kTabAudioPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 17, 7,
R_V_LINE_TO, 18,
R_LINE_TO, -6, -6,
R_H_LINE_TO, -4,
R_V_LINE_TO, -6,
R_H_LINE_TO, 4,
CLOSE,
MOVE_TO, 19, 13,
R_V_LINE_TO, 6,
R_H_LINE_TO, 2,
R_V_LINE_TO, -6,
CLOSE,
MOVE_TO, 23, 11,
R_V_LINE_TO, 10,
R_H_LINE_TO, 2,
R_V_LINE_TO, -10,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kTabAudio16Path, CANVAS_DIMENSIONS, 16,
DISABLE_AA,
MOVE_TO, 9, 2,
R_V_LINE_TO, 11,
R_LINE_TO, -4, -4,
R_H_LINE_TO, -1,
R_V_LINE_TO, -3,
R_H_LINE_TO, 1,
CLOSE,
NEW_PATH,
STROKE, 1,
MOVE_TO, 10.5f, 6,
R_V_LINE_TO, 3,
MOVE_TO, 12.5f, 5,
R_V_LINE_TO, 5)
VECTOR_ICON_TEMPLATE_CC(kTabAudioRepList, kTabAudioIcon, {kTabAudioPath, std::size(kTabAudioPath)}, {kTabAudio16Path, std::size(kTabAudio16Path)})
VECTOR_ICON_REP_TEMPLATE(kTabCloseNormalPath, CANVAS_DIMENSIONS, 32,
STROKE, 2.45f,
MOVE_TO, 10.25f, 10.25f,
R_LINE_TO, 11.5f, 11.5f,
MOVE_TO, 10.25f, 21.75f,
R_LINE_TO, 11.5f, -11.5f)
VECTOR_ICON_REP_TEMPLATE(kTabCloseNormal16Path, CANVAS_DIMENSIONS, 16,
STROKE, 1.415f,
MOVE_TO, 4.75f, 4.75f,
R_LINE_TO, 6.5f, 6.5f,
MOVE_TO, 4.75f, 11.25f,
R_LINE_TO, 6.5f, -6.5f)
VECTOR_ICON_TEMPLATE_CC(kTabCloseNormalRepList, kTabCloseNormalIcon, {kTabCloseNormalPath, std::size(kTabCloseNormalPath)}, {kTabCloseNormal16Path, std::size(kTabCloseNormal16Path)})
VECTOR_ICON_REP_TEMPLATE(kTabGroupPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 8, 14.4f,
CUBIC_TO, 7.13f, 14.4f, 6.29f, 14.23f, 5.52f, 13.9f,
CUBIC_TO, 4.74f, 13.57f, 4.06f, 13.11f, 3.48f, 12.52f,
CUBIC_TO, 2.89f, 11.94f, 2.43f, 11.26f, 2.1f, 10.48f,
CUBIC_TO, 1.77f, 9.71f, 1.6f, 8.88f, 1.6f, 7.99f,
CUBIC_TO, 1.6f, 7.11f, 1.77f, 6.28f, 2.1f, 5.51f,
CUBIC_TO, 2.43f, 4.73f, 2.89f, 4.06f, 3.48f, 3.48f,
CUBIC_TO, 4.06f, 2.89f, 4.74f, 2.43f, 5.52f, 2.1f,
CUBIC_TO, 6.29f, 1.77f, 7.13f, 1.6f, 8.01f, 1.6f,
CUBIC_TO, 8.89f, 1.6f, 9.72f, 1.77f, 10.49f, 2.1f,
CUBIC_TO, 11.27f, 2.43f, 11.94f, 2.89f, 12.52f, 3.48f,
CUBIC_TO, 13.11f, 4.06f, 13.57f, 4.74f, 13.9f, 5.51f,
CUBIC_TO, 14.23f, 6.29f, 14.4f, 7.11f, 14.4f, 8,
CUBIC_TO, 14.4f, 8.88f, 14.23f, 9.71f, 13.9f, 10.48f,
CUBIC_TO, 13.57f, 11.26f, 13.11f, 11.94f, 12.52f, 12.52f,
CUBIC_TO, 11.94f, 13.11f, 11.26f, 13.57f, 10.49f, 13.9f,
CUBIC_TO, 9.71f, 14.23f, 8.89f, 14.4f, 8, 14.4f,
CLOSE,
MOVE_TO, 8, 14.4f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabGroupRepList, kTabGroupIcon, {kTabGroupPath, std::size(kTabGroupPath)})
VECTOR_ICON_REP_TEMPLATE(kTabGroupSharingPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 2, 16,
R_V_LINE_TO, -1.92f,
R_CUBIC_TO, 0, -0.35f, 0.09f, -0.69f, 0.26f, -0.98f,
R_CUBIC_TO, 0.17f, -0.3f, 0.41f, -0.54f, 0.72f, -0.72f,
R_ARC_TO, 9.81f, 9.81f, 0, 0, 1, 2.38f, -1.02f,
ARC_TO, 9.61f, 9.61f, 0, 0, 1, 8, 11,
R_CUBIC_TO, 0.92f, 0, 1.8f, 0.12f, 2.64f, 0.36f,
R_CUBIC_TO, 0.84f, 0.24f, 1.63f, 0.57f, 2.38f, 1.02f,
R_CUBIC_TO, 0.31f, 0.18f, 0.55f, 0.42f, 0.72f, 0.72f,
R_CUBIC_TO, 0.18f, 0.3f, 0.26f, 0.63f, 0.26f, 0.98f,
V_LINE_TO, 16,
CLOSE,
R_MOVE_TO, 13.5f, 0,
R_V_LINE_TO, -1.92f,
R_CUBIC_TO, 0, -0.58f, -0.14f, -1.12f, -0.41f, -1.62f,
R_CUBIC_TO, -0.27f, -0.5f, -0.64f, -0.91f, -1.09f, -1.23f,
R_CUBIC_TO, 0.54f, 0.11f, 1.07f, 0.26f, 1.57f, 0.45f,
R_CUBIC_TO, 0.5f, 0.19f, 0.99f, 0.42f, 1.45f, 0.7f,
R_CUBIC_TO, 0.31f, 0.18f, 0.55f, 0.42f, 0.72f, 0.72f,
R_CUBIC_TO, 0.18f, 0.3f, 0.26f, 0.63f, 0.26f, 0.98f,
V_LINE_TO, 16,
CLOSE,
MOVE_TO, 8, 10,
R_ARC_TO, 2.9f, 2.9f, 0, 0, 1, -2.12f, -0.87f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 5, 7,
R_CUBIC_TO, 0, -0.83f, 0.29f, -1.54f, 0.88f, -2.12f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 8, 4,
R_CUBIC_TO, 0.83f, 0, 1.54f, 0.29f, 2.13f, 0.88f,
CUBIC_TO, 10.71f, 5.46f, 11, 6.17f, 11, 7,
R_CUBIC_TO, 0, 0.83f, -0.29f, 1.54f, -0.87f, 2.13f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 8, 10,
CLOSE,
R_MOVE_TO, 7, -3,
R_CUBIC_TO, 0, 0.83f, -0.29f, 1.54f, -0.87f, 2.13f,
ARC_TO, 2.9f, 2.9f, 0, 0, 1, 12, 10,
R_CUBIC_TO, -0.11f, 0, -0.21f, 0, -0.31f, -0.01f,
R_ARC_TO, 1.57f, 1.57f, 0, 0, 1, -0.31f, -0.05f,
R_ARC_TO, 4.9f, 4.9f, 0, 0, 0, 0.82f, -1.34f,
R_CUBIC_TO, 0.2f, -0.49f, 0.3f, -1.02f, 0.3f, -1.59f,
R_CUBIC_TO, 0, -0.57f, -0.1f, -1.1f, -0.3f, -1.59f,
R_ARC_TO, 4.9f, 4.9f, 0, 0, 0, -0.82f, -1.34f,
R_CUBIC_TO, 0.11f, -0.03f, 0.22f, -0.05f, 0.31f, -0.05f,
CUBIC_TO, 11.78f, 4, 11.89f, 4, 12, 4,
R_CUBIC_TO, 0.83f, 0, 1.54f, 0.29f, 2.13f, 0.88f,
CUBIC_TO, 14.71f, 5.46f, 15, 6.17f, 15, 7,
CLOSE,
MOVE_TO, 3.5f, 14.5f,
R_H_LINE_TO, 9,
R_V_LINE_TO, -0.42f,
R_CUBIC_TO, 0, -0.09f, -0.02f, -0.17f, -0.06f, -0.24f,
R_ARC_TO, 0.44f, 0.44f, 0, 0, 0, -0.17f, -0.17f,
R_ARC_TO, 9.02f, 9.02f, 0, 0, 0, -2.06f, -0.87f,
CUBIC_TO, 9.48f, 12.6f, 8.75f, 12.5f, 8, 12.5f,
R_CUBIC_TO, -0.75f, 0, -1.48f, 0.1f, -2.21f, 0.29f,
R_ARC_TO, 7.97f, 7.97f, 0, 0, 0, -2.06f, 0.88f,
R_ARC_TO, 0.45f, 0.45f, 0, 0, 0, -0.23f, 0.41f,
CLOSE,
R_MOVE_TO, 4.5f, -6,
R_CUBIC_TO, 0.41f, 0, 0.77f, -0.15f, 1.06f, -0.44f,
R_CUBIC_TO, 0.29f, -0.3f, 0.44f, -0.65f, 0.44f, -1.06f,
R_CUBIC_TO, 0, -0.41f, -0.15f, -0.77f, -0.44f, -1.06f,
ARC_TO, 1.45f, 1.45f, 0, 0, 0, 8, 5.5f,
R_CUBIC_TO, -0.41f, 0, -0.77f, 0.15f, -1.06f, 0.44f,
ARC_TO, 1.45f, 1.45f, 0, 0, 0, 6.5f, 7,
R_CUBIC_TO, 0, 0.41f, 0.15f, 0.77f, 0.44f, 1.06f,
R_CUBIC_TO, 0.3f, 0.29f, 0.65f, 0.44f, 1.06f, 0.44f,
CLOSE,
MOVE_TO, 8, 14.5f,
CLOSE,
MOVE_TO, 8, 7,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabGroupSharingRepList, kTabGroupSharingIcon, {kTabGroupSharingPath, std::size(kTabGroupSharingPath)})
VECTOR_ICON_REP_TEMPLATE(kTabGroupsSyncPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 3.62f, 12.99f,
R_V_LINE_TO, -1.34f,
R_H_LINE_TO, 0.93f,
R_LINE_TO, 0.12f, 0.09f,
R_ARC_TO, 5.92f, 5.92f, 0, 0, 1, -1.23f, -1.68f,
ARC_TO, 4.62f, 4.62f, 0, 0, 1, 2.96f, 8,
R_CUBIC_TO, 0, -1.22f, 0.39f, -2.29f, 1.17f, -3.2f,
R_ARC_TO, 5.07f, 5.07f, 0, 0, 1, 2.91f, -1.73f,
R_V_LINE_TO, 1.68f,
R_CUBIC_TO, -0.71f, 0.2f, -1.29f, 0.59f, -1.75f, 1.19f,
ARC_TO, 3.28f, 3.28f, 0, 0, 0, 4.6f, 8,
R_CUBIC_TO, 0, 0.5f, 0.11f, 0.96f, 0.31f, 1.38f,
R_CUBIC_TO, 0.21f, 0.43f, 0.48f, 0.8f, 0.82f, 1.11f,
R_LINE_TO, -0.07f, -0.07f,
R_V_LINE_TO, -0.81f,
R_H_LINE_TO, 1.34f,
R_V_LINE_TO, 3.39f,
CLOSE,
R_MOVE_TO, 5.35f, -0.06f,
R_V_LINE_TO, -1.68f,
R_CUBIC_TO, 0.71f, -0.2f, 1.29f, -0.59f, 1.75f, -1.19f,
ARC_TO, 3.28f, 3.28f, 0, 0, 0, 11.4f, 8,
R_CUBIC_TO, 0, -0.5f, -0.1f, -0.96f, -0.31f, -1.38f,
R_ARC_TO, 3.57f, 3.57f, 0, 0, 0, -0.82f, -1.1f,
R_LINE_TO, 0.07f, 0.07f,
R_V_LINE_TO, 0.81f,
R_H_LINE_TO, -1.34f,
V_LINE_TO, 3.01f,
R_H_LINE_TO, 3.38f,
R_V_LINE_TO, 1.34f,
R_H_LINE_TO, -0.93f,
R_LINE_TO, -0.12f, -0.09f,
R_CUBIC_TO, 0.52f, 0.47f, 0.93f, 1.03f, 1.24f, 1.67f,
R_CUBIC_TO, 0.31f, 0.64f, 0.46f, 1.33f, 0.46f, 2.07f,
R_CUBIC_TO, 0, 1.22f, -0.39f, 2.29f, -1.16f, 3.2f,
R_ARC_TO, 5.07f, 5.07f, 0, 0, 1, -2.91f, 1.73f,
CLOSE,
R_MOVE_TO, 0, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabGroupsSyncRepList, kTabGroupsSyncIcon, {kTabGroupsSyncPath, std::size(kTabGroupsSyncPath)})
VECTOR_ICON_REP_TEMPLATE(kTabUsbConnectedPath, CANVAS_DIMENSIONS, 32,
MOVE_TO, 19, 16,
R_H_LINE_TO, 1,
R_V_LINE_TO, 2,
R_H_LINE_TO, -3,
R_V_LINE_TO, -8,
R_H_LINE_TO, 2,
R_LINE_TO, -3, -4,
R_LINE_TO, -3, 4,
R_H_LINE_TO, 2,
R_V_LINE_TO, 8,
R_H_LINE_TO, -3,
R_V_LINE_TO, -2,
R_CUBIC_TO, 0.85f, -0.53f, 1.34f, -1.23f, 1, -2,
R_CUBIC_TO, 0.34f, -1.27f, -0.64f, -2.25f, -2, -2,
R_CUBIC_TO, -1.02f, -0.25f, -2, 0.73f, -2, 2,
R_CUBIC_TO, 0, 0.77f, 0.49f, 1.47f, 1, 2,
R_V_LINE_TO, 2,
R_CUBIC_TO, 0.18f, 0.99f, 1.06f, 1.87f, 2, 2,
R_H_LINE_TO, 3,
R_V_LINE_TO, 3,
R_CUBIC_TO, -0.59f, 0.26f, -1.07f, 0.98f, -1, 2,
R_CUBIC_TO, -0.07f, 1.03f, 0.91f, 2, 2, 2,
R_CUBIC_TO, 1.29f, 0, 2.27f, -0.97f, 2, -2,
R_CUBIC_TO, 0.27f, -1.02f, -0.21f, -1.74f, -1, -2,
R_V_LINE_TO, -3,
R_H_LINE_TO, 3,
R_CUBIC_TO, 1.14f, -0.13f, 2.01f, -1.01f, 2, -2,
R_V_LINE_TO, -2,
R_H_LINE_TO, 1,
R_V_LINE_TO, -4,
R_H_LINE_TO, -4,
R_V_LINE_TO, 4,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kTabUsbConnected16Path, CANVAS_DIMENSIONS, 16,
MOVE_TO, 10, 8,
R_H_LINE_TO, 1,
R_V_LINE_TO, 1,
H_LINE_TO, 9,
V_LINE_TO, 5,
R_H_LINE_TO, 1,
LINE_TO, 8.5f, 3,
LINE_TO, 7, 5,
R_H_LINE_TO, 1,
R_V_LINE_TO, 4,
H_LINE_TO, 7,
V_LINE_TO, 8,
R_CUBIC_TO, -0.08f, -0.04f, 0, -0.5f, 0, -1,
R_CUBIC_TO, 0, -0.5f, -0.5f, -1, -1, -1,
R_CUBIC_TO, -0.5f, 0, -1, 0.5f, -1, 1,
R_CUBIC_TO, 0, 0.5f, 0.25f, 0.96f, 1, 1,
R_V_LINE_TO, 1,
R_CUBIC_TO, 0, 0.5f, 0.5f, 1, 1, 1,
R_H_LINE_TO, 1,
R_V_LINE_TO, 2,
R_LINE_TO, -1, 1,
R_LINE_TO, 1, 1,
R_H_LINE_TO, 1,
R_LINE_TO, 1, -1,
R_LINE_TO, -1, -1,
R_V_LINE_TO, -2,
R_H_LINE_TO, 2,
R_CUBIC_TO, 0.5f, 0, 1, -0.5f, 1, -1,
V_LINE_TO, 6,
R_H_LINE_TO, -2,
R_V_LINE_TO, 2,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabUsbConnectedRepList, kTabUsbConnectedIcon, {kTabUsbConnectedPath, std::size(kTabUsbConnectedPath)}, {kTabUsbConnected16Path, std::size(kTabUsbConnected16Path)})
VECTOR_ICON_REP_TEMPLATE(kTabletPath, MOVE_TO, 42, 8,
H_LINE_TO, 6,
R_CUBIC_TO, -2.21f, 0, -4, 1.79f, -4, 4,
R_V_LINE_TO, 24,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 36,
R_CUBIC_TO, 2.21f, 0, 3.98f, -1.79f, 3.98f, -4,
LINE_TO, 46, 12,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CLOSE,
R_MOVE_TO, -4, 28,
H_LINE_TO, 10,
V_LINE_TO, 12,
R_H_LINE_TO, 28,
R_V_LINE_TO, 24,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTabletRepList, kTabletIcon, {kTabletPath, std::size(kTabletPath)})
VECTOR_ICON_REP_TEMPLATE(kTaskManagerPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 12.35f, 13.73f,
H_LINE_TO, 3.65f,
R_CUBIC_TO, -0.37f, 0, -0.7f, -0.14f, -0.97f, -0.41f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.41f, -0.97f,
V_LINE_TO, 3.65f,
R_CUBIC_TO, 0, -0.38f, 0.14f, -0.71f, 0.41f, -0.98f,
R_CUBIC_TO, 0.27f, -0.27f, 0.6f, -0.4f, 0.97f, -0.4f,
R_H_LINE_TO, 8.7f,
R_CUBIC_TO, 0.38f, 0, 0.7f, 0.13f, 0.97f, 0.4f,
R_CUBIC_TO, 0.27f, 0.27f, 0.41f, 0.6f, 0.41f, 0.98f,
R_V_LINE_TO, 8.7f,
R_CUBIC_TO, 0, 0.38f, -0.14f, 0.7f, -0.41f, 0.97f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 1, -0.97f, 0.41f,
CLOSE,
MOVE_TO, 3.65f, 5.41f,
R_H_LINE_TO, 8.7f,
V_LINE_TO, 3.65f,
H_LINE_TO, 3.65f,
CLOSE,
R_MOVE_TO, 1.57f, 1.2f,
R_H_LINE_TO, -1.57f,
R_V_LINE_TO, 5.74f,
R_H_LINE_TO, 1.57f,
CLOSE,
R_MOVE_TO, 5.56f, 0,
R_V_LINE_TO, 5.74f,
R_H_LINE_TO, 1.57f,
V_LINE_TO, 6.61f,
CLOSE,
R_MOVE_TO, -1.2f, 0,
H_LINE_TO, 6.42f,
R_V_LINE_TO, 5.74f,
R_H_LINE_TO, 3.16f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTaskManagerRepList, kTaskManagerIcon, {kTaskManagerPath, std::size(kTaskManagerPath)})
VECTOR_ICON_REP_TEMPLATE(kToolbarChromeRefreshPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 3.57f, 13.73f,
CUBIC_TO, 3.19f, 13.73f, 2.88f, 13.61f, 2.63f, 13.37f,
CUBIC_TO, 2.39f, 13.12f, 2.27f, 12.81f, 2.27f, 12.43f,
LINE_TO, 2.27f, 3.57f,
CUBIC_TO, 2.27f, 3.18f, 2.39f, 2.86f, 2.63f, 2.63f,
CUBIC_TO, 2.88f, 2.38f, 3.19f, 2.27f, 3.57f, 2.27f,
LINE_TO, 12.43f, 2.27f,
CUBIC_TO, 12.82f, 2.27f, 13.14f, 2.38f, 13.38f, 2.63f,
CUBIC_TO, 13.62f, 2.86f, 13.73f, 3.18f, 13.73f, 3.57f,
LINE_TO, 13.73f, 12.43f,
CUBIC_TO, 13.73f, 12.81f, 13.62f, 13.12f, 13.38f, 13.37f,
CUBIC_TO, 13.14f, 13.61f, 12.82f, 13.73f, 12.43f, 13.73f,
CLOSE,
MOVE_TO, 3.65f, 5.18f,
LINE_TO, 12.35f, 5.18f,
LINE_TO, 12.35f, 3.65f,
LINE_TO, 3.65f, 3.65f,
CLOSE,
MOVE_TO, 12.35f, 6.56f,
LINE_TO, 3.65f, 6.56f,
LINE_TO, 3.65f, 12.35f,
LINE_TO, 12.35f, 12.35f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kToolbarChromeRefreshRepList, kToolbarChromeRefreshIcon, {kToolbarChromeRefreshPath, std::size(kToolbarChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kTrailingScrollPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 8.97f, 8,
LINE_TO, 4, 3.4f,
LINE_TO, 5.51f, 2,
LINE_TO, 12, 8,
R_LINE_TO, -6.49f, 6,
LINE_TO, 4, 12.6f,
LINE_TO, 8.97f, 8,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTrailingScrollRepList, kTrailingScrollIcon, {kTrailingScrollPath, std::size(kTrailingScrollPath)})
VECTOR_ICON_REP_TEMPLATE(kTranslatePath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
R_MOVE_TO, 8, 14.52f,
R_LINE_TO, -0.81f, -2.44f,
H_LINE_TO, 2.7f,
R_ARC_TO, 1.25f, 1.25f, 0, 0, 1, -0.87f, -0.36f,
R_ARC_TO, 1.16f, 1.16f, 0, 0, 1, -0.35f, -0.86f,
V_LINE_TO, 2.7f,
R_CUBIC_TO, 0, -0.33f, 0.12f, -0.62f, 0.36f, -0.86f,
R_CUBIC_TO, 0.24f, -0.24f, 0.53f, -0.37f, 0.87f, -0.37f,
R_H_LINE_TO, 3.67f,
R_LINE_TO, 0.81f, 2.44f,
H_LINE_TO, 13.3f,
R_CUBIC_TO, 0.33f, 0, 0.62f, 0.12f, 0.86f, 0.36f,
R_CUBIC_TO, 0.24f, 0.24f, 0.36f, 0.53f, 0.36f, 0.86f,
R_V_LINE_TO, 8.16f,
R_CUBIC_TO, 0, 0.34f, -0.12f, 0.63f, -0.36f, 0.87f,
R_ARC_TO, 1.19f, 1.19f, 0, 0, 1, -0.86f, 0.36f,
CLOSE,
R_MOVE_TO, -3.21f, -4.89f,
R_CUBIC_TO, 0.65f, 0, 1.2f, -0.21f, 1.66f, -0.63f,
R_CUBIC_TO, 0.46f, -0.42f, 0.69f, -1.01f, 0.69f, -1.77f,
R_CUBIC_TO, 0, -0.08f, 0, -0.16f, -0.01f, -0.23f,
R_ARC_TO, 1.33f, 1.33f, 0, 0, 0, -0.04f, -0.25f,
H_LINE_TO, 4.79f,
R_V_LINE_TO, 0.83f,
R_H_LINE_TO, 1.43f,
R_CUBIC_TO, -0.04f, 0.38f, -0.19f, 0.66f, -0.46f, 0.88f,
R_ARC_TO, 1.56f, 1.56f, 0, 0, 1, -0.97f, 0.31f,
R_CUBIC_TO, -0.43f, 0, -0.79f, -0.15f, -1.09f, -0.46f,
R_ARC_TO, 1.55f, 1.55f, 0, 0, 1, -0.45f, -1.12f,
R_CUBIC_TO, 0, -0.43f, 0.15f, -0.8f, 0.45f, -1.11f,
R_CUBIC_TO, 0.3f, -0.31f, 0.67f, -0.46f, 1.1f, -0.46f,
R_CUBIC_TO, 0.18f, 0, 0.36f, 0.03f, 0.53f, 0.1f,
R_CUBIC_TO, 0.17f, 0.07f, 0.32f, 0.16f, 0.46f, 0.29f,
R_LINE_TO, 0.65f, -0.63f,
R_ARC_TO, 2.18f, 2.18f, 0, 0, 0, -0.77f, -0.47f,
R_ARC_TO, 2.58f, 2.58f, 0, 0, 0, -0.87f, -0.16f,
R_CUBIC_TO, -0.68f, 0, -1.26f, 0.24f, -1.74f, 0.72f,
R_ARC_TO, 2.37f, 2.37f, 0, 0, 0, -0.71f, 1.73f,
R_CUBIC_TO, 0, 0.68f, 0.24f, 1.25f, 0.71f, 1.73f,
R_ARC_TO, 2.37f, 2.37f, 0, 0, 0, 1.74f, 0.72f,
CLOSE,
R_MOVE_TO, 4.31f, 0.57f,
R_LINE_TO, 0.33f, -0.32f,
R_CUBIC_TO, -0.16f, -0.17f, -0.3f, -0.35f, -0.43f, -0.53f,
R_ARC_TO, 7.06f, 7.06f, 0, 0, 1, -0.36f, -0.59f,
CLOSE,
R_MOVE_TO, 0.8f, -0.83f,
R_CUBIC_TO, 0.23f, -0.27f, 0.44f, -0.56f, 0.63f, -0.86f,
R_CUBIC_TO, 0.2f, -0.3f, 0.34f, -0.63f, 0.43f, -0.99f,
H_LINE_TO, 8.26f,
R_LINE_TO, 0.24f, 0.71f,
R_LINE_TO, 0.66f, -0.02f,
R_CUBIC_TO, 0.1f, 0.22f, 0.21f, 0.42f, 0.33f, 0.61f,
R_CUBIC_TO, 0.12f, 0.19f, 0.26f, 0.37f, 0.42f, 0.54f,
CLOSE,
R_MOVE_TO, -1.09f, 4.34f,
R_H_LINE_TO, 4.49f,
R_ARC_TO, 0.41f, 0.41f, 0, 0, 0, 0.29f, -0.11f,
R_ARC_TO, 0.4f, 0.4f, 0, 0, 0, 0.12f, -0.3f,
V_LINE_TO, 5.14f,
R_ARC_TO, 0.4f, 0.4f, 0, 0, 0, -0.12f, -0.29f,
R_ARC_TO, 0.4f, 0.4f, 0, 0, 0, -0.29f, -0.12f,
H_LINE_TO, 7.39f,
R_LINE_TO, 0.65f, 2.1f,
R_H_LINE_TO, 1.36f,
V_LINE_TO, 6.13f,
R_H_LINE_TO, 0.68f,
R_V_LINE_TO, 0.7f,
R_H_LINE_TO, 2.46f,
R_V_LINE_TO, 0.7f,
R_H_LINE_TO, -0.86f,
R_ARC_TO, 4.86f, 4.86f, 0, 0, 1, -0.51f, 1.26f,
R_ARC_TO, 7.07f, 7.07f, 0, 0, 1, -0.78f, 1.1f,
R_LINE_TO, 1.84f, 1.8f,
R_LINE_TO, -0.49f, 0.49f,
R_LINE_TO, -1.8f, -1.8f,
R_LINE_TO, -0.59f, 0.58f,
R_LINE_TO, 0.61f, 1.93f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTranslateRepList, kTranslateIcon, {kTranslatePath, std::size(kTranslatePath)})
VECTOR_ICON_REP_TEMPLATE(kTrashCanPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 4, 12.67f,
CUBIC_TO, 4, 13.4f, 4.6f, 14, 5.33f, 14,
R_H_LINE_TO, 5.33f,
CUBIC_TO, 11.4f, 14, 12, 13.4f, 12, 12.67f,
R_V_LINE_TO, -8,
H_LINE_TO, 4,
CLOSE,
R_MOVE_TO, 8.67f, -10,
R_H_LINE_TO, -2.33f,
LINE_TO, 9.67f, 2,
H_LINE_TO, 6.33f,
R_LINE_TO, -0.67f, 0.67f,
H_LINE_TO, 3.33f,
V_LINE_TO, 4,
R_H_LINE_TO, 9.33f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTrashCanRepList, kTrashCanIcon, {kTrashCanPath, std::size(kTrashCanPath)})
VECTOR_ICON_REP_TEMPLATE(kTrashCanRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
MOVE_TO, 6.5f, 17,
CUBIC_TO, 6.09f, 17, 5.73f, 16.85f, 5.44f, 16.56f,
CUBIC_TO, 5.15f, 16.27f, 5, 15.91f, 5, 15.5f,
LINE_TO, 5, 5.5f,
LINE_TO, 4, 5.5f,
LINE_TO, 4, 4,
LINE_TO, 8, 4,
LINE_TO, 8, 3,
LINE_TO, 12, 3,
LINE_TO, 12, 4,
LINE_TO, 16, 4,
LINE_TO, 16, 5.5f,
LINE_TO, 15, 5.5f,
LINE_TO, 15, 15.49f,
CUBIC_TO, 15, 15.91f, 14.85f, 16.27f, 14.56f, 16.56f,
CUBIC_TO, 14.27f, 16.86f, 13.91f, 17, 13.5f, 17,
CLOSE,
MOVE_TO, 13.5f, 5.5f,
LINE_TO, 6.5f, 5.5f,
LINE_TO, 6.5f, 15.5f,
LINE_TO, 13.5f, 15.5f,
CLOSE,
MOVE_TO, 8, 14,
LINE_TO, 9.5f, 14,
LINE_TO, 9.5f, 7,
LINE_TO, 8, 7,
CLOSE,
MOVE_TO, 10.5f, 14,
LINE_TO, 12, 14,
LINE_TO, 12, 7,
LINE_TO, 10.5f, 7,
CLOSE,
MOVE_TO, 6.5f, 5.5f,
LINE_TO, 6.5f, 15.5f,
CLOSE,
MOVE_TO, 6.5f, 5.5f,
CLOSE)
VECTOR_ICON_REP_TEMPLATE(kTrashCanRefresh16Path, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 5.16f, 13.73f,
R_CUBIC_TO, -0.38f, 0, -0.71f, -0.13f, -0.98f, -0.4f,
R_ARC_TO, 1.34f, 1.34f, 0, 0, 1, -0.4f, -0.98f,
V_LINE_TO, 4.45f,
R_H_LINE_TO, -0.89f,
V_LINE_TO, 3.06f,
R_H_LINE_TO, 3.43f,
R_V_LINE_TO, -0.89f,
R_H_LINE_TO, 3.38f,
R_V_LINE_TO, 0.89f,
R_H_LINE_TO, 3.43f,
R_V_LINE_TO, 1.39f,
R_H_LINE_TO, -0.89f,
R_V_LINE_TO, 7.9f,
R_CUBIC_TO, 0, 0.39f, -0.13f, 0.72f, -0.4f, 0.99f,
R_CUBIC_TO, -0.27f, 0.27f, -0.6f, 0.4f, -0.98f, 0.4f,
CLOSE,
R_MOVE_TO, 5.69f, -9.29f,
H_LINE_TO, 5.16f,
R_V_LINE_TO, 7.91f,
R_H_LINE_TO, 5.69f,
CLOSE,
R_MOVE_TO, -4.52f, 6.75f,
R_H_LINE_TO, 1.29f,
V_LINE_TO, 5.6f,
H_LINE_TO, 6.33f,
CLOSE,
R_MOVE_TO, 2.05f, 0,
R_H_LINE_TO, 1.29f,
V_LINE_TO, 5.6f,
H_LINE_TO, 8.38f,
CLOSE,
R_MOVE_TO, -3.22f, -6.75f,
R_V_LINE_TO, 7.91f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTrashCanRefreshRepList, kTrashCanRefreshIcon, {kTrashCanRefreshPath, std::size(kTrashCanRefreshPath)}, {kTrashCanRefresh16Path, std::size(kTrashCanRefresh16Path)})
VECTOR_ICON_REP_TEMPLATE(kTvPath, CANVAS_DIMENSIONS, 960,
FILL_RULE_NONZERO,
MOVE_TO, 320, 840,
R_V_LINE_TO, -80,
H_LINE_TO, 160,
R_QUADRATIC_TO, -33, 0, -56.5f, -23.5f,
QUADRATIC_TO_SHORTHAND, 80, 680,
R_V_LINE_TO, -480,
R_QUADRATIC_TO, 0, -33, 23.5f, -56.5f,
QUADRATIC_TO_SHORTHAND, 160, 120,
R_H_LINE_TO, 640,
R_QUADRATIC_TO, 33, 0, 56.5f, 23.5f,
QUADRATIC_TO_SHORTHAND, 880, 200,
R_V_LINE_TO, 480,
R_QUADRATIC_TO, 0, 33, -23.5f, 56.5f,
QUADRATIC_TO_SHORTHAND, 800, 760,
H_LINE_TO, 640,
R_V_LINE_TO, 80,
H_LINE_TO, 320,
CLOSE,
MOVE_TO, 160, 680,
R_H_LINE_TO, 640,
R_V_LINE_TO, -480,
H_LINE_TO, 160,
R_V_LINE_TO, 480,
CLOSE,
R_MOVE_TO, 0, 0,
R_V_LINE_TO, -480,
R_V_LINE_TO, 480,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kTvRepList, kTvIcon, {kTvPath, std::size(kTvPath)})
VECTOR_ICON_REP_TEMPLATE(kUngroupRefreshPath, CANVAS_DIMENSIONS, 20,
R_MOVE_TO, 7.06f, 14.16f,
R_LINE_TO, -1.23f, -1.23f,
R_LINE_TO, 3.86f, -3.84f,
H_LINE_TO, 6.41f,
R_V_LINE_TO, -1.73f,
R_H_LINE_TO, 6.23f,
R_V_LINE_TO, 6.23f,
R_H_LINE_TO, -1.73f,
V_LINE_TO, 10.3f,
CLOSE,
R_MOVE_TO, 8.38f, -4.11f,
R_V_LINE_TO, -5.5f,
R_H_LINE_TO, -5.5f,
V_LINE_TO, 2.83f,
R_H_LINE_TO, 7.23f,
R_V_LINE_TO, 7.23f,
CLOSE,
MOVE_TO, 4.56f, 17.17f,
R_CUBIC_TO, -0.48f, 0, -0.89f, -0.17f, -1.22f, -0.5f,
R_ARC_TO, 1.67f, 1.67f, 0, 0, 1, -0.5f, -1.22f,
V_LINE_TO, 2.83f,
R_H_LINE_TO, 1.73f,
R_V_LINE_TO, 12.61f,
R_H_LINE_TO, 12.61f,
R_V_LINE_TO, 1.73f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kUngroupRefreshRepList, kUngroupRefreshIcon, {kUngroupRefreshPath, std::size(kUngroupRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kUsbCablePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 20, 5,
V_LINE_TO, 4,
R_CUBIC_TO, 0, -0.55f, -0.45f, -1, -1, -1,
R_H_LINE_TO, -2,
R_CUBIC_TO, -0.55f, 0, -1, 0.45f, -1, 1,
R_V_LINE_TO, 1,
R_H_LINE_TO, -1,
R_V_LINE_TO, 4,
R_CUBIC_TO, 0, 0.55f, 0.45f, 1, 1, 1,
R_H_LINE_TO, 1,
R_V_LINE_TO, 7,
R_CUBIC_TO, 0, 1.1f, -0.9f, 2, -2, 2,
R_CUBIC_TO, -1.1f, 0, -2, -0.9f, -2, -2,
V_LINE_TO, 7,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CUBIC_TO_SHORTHAND, 5, 4.79f, 5, 7,
R_V_LINE_TO, 7,
H_LINE_TO, 4,
R_CUBIC_TO, -0.55f, 0, -1, 0.45f, -1, 1,
R_V_LINE_TO, 4,
R_H_LINE_TO, 1,
R_V_LINE_TO, 1,
R_CUBIC_TO, 0, 0.55f, 0.45f, 1, 1, 1,
R_H_LINE_TO, 2,
R_CUBIC_TO, 0.55f, 0, 1, -0.45f, 1, -1,
R_V_LINE_TO, -1,
R_H_LINE_TO, 1,
R_V_LINE_TO, -4,
R_CUBIC_TO, 0, -0.55f, -0.45f, -1, -1, -1,
H_LINE_TO, 7,
V_LINE_TO, 7,
R_CUBIC_TO, 0, -1.1f, 0.9f, -2, 2, -2,
R_CUBIC_TO, 1.1f, 0, 2, 0.9f, 2, 2,
R_V_LINE_TO, 10,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
R_V_LINE_TO, -7,
R_H_LINE_TO, 1,
R_CUBIC_TO, 0.55f, 0, 1, -0.45f, 1, -1,
V_LINE_TO, 5,
R_H_LINE_TO, -1,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kUsbCableRepList, kUsbCableIcon, {kUsbCablePath, std::size(kUsbCablePath)})
VECTOR_ICON_REP_TEMPLATE(kUserAccountAvatarPath, CANVAS_DIMENSIONS, 64,
MOVE_TO, 32, 0,
CUBIC_TO, 14.34f, 0, 0, 14.34f, 0, 32,
CUBIC_TO, 0, 49.66f, 14.34f, 64, 32, 64,
CUBIC_TO, 49.66f, 64, 64, 49.66f, 64, 32,
CUBIC_TO, 64, 14.34f, 49.66f, 0, 32, 0,
CLOSE,
MOVE_TO, 32, 9.6f,
CUBIC_TO, 37.31f, 9.6f, 41.6f, 13.89f, 41.6f, 19.2f,
CUBIC_TO, 41.6f, 24.51f, 37.31f, 28.8f, 32, 28.8f,
CUBIC_TO, 26.69f, 28.8f, 22.4f, 24.51f, 22.4f, 19.2f,
CUBIC_TO, 22.4f, 13.89f, 26.69f, 9.6f, 32, 9.6f,
CLOSE,
MOVE_TO, 32, 55.04f,
CUBIC_TO, 24, 55.04f, 16.93f, 50.94f, 12.8f, 44.74f,
CUBIC_TO, 12.9f, 38.37f, 25.6f, 34.88f, 32, 34.88f,
CUBIC_TO, 38.37f, 34.88f, 51.1f, 38.37f, 51.2f, 44.74f,
CUBIC_TO, 47.07f, 50.94f, 40, 55.04f, 32, 55.04f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kUserAccountAvatarRepList, kUserAccountAvatarIcon, {kUserAccountAvatarPath, std::size(kUserAccountAvatarPath)})
VECTOR_ICON_REP_TEMPLATE(kUserAccountAvatarRefreshPath, CANVAS_DIMENSIONS, 20,
MOVE_TO, 4.96f, 13.99f,
R_ARC_TO, 8.24f, 8.24f, 0, 0, 1, 2.4f, -1.16f,
R_ARC_TO, 9.28f, 9.28f, 0, 0, 1, 2.64f, -0.37f,
R_CUBIC_TO, 0.91f, 0, 1.8f, 0.13f, 2.65f, 0.38f,
R_ARC_TO, 8.34f, 8.34f, 0, 0, 1, 2.39f, 1.15f,
R_ARC_TO, 5.84f, 5.84f, 0, 0, 0, 1.06f, -1.87f,
R_CUBIC_TO, 0.23f, -0.68f, 0.34f, -1.38f, 0.34f, -2.12f,
R_CUBIC_TO, 0, -1.78f, -0.63f, -3.3f, -1.89f, -4.55f,
R_CUBIC_TO, -1.26f, -1.26f, -2.78f, -1.89f, -4.56f, -1.89f,
R_CUBIC_TO, -1.78f, 0, -3.3f, 0.63f, -4.55f, 1.89f,
CUBIC_TO, 4.19f, 6.7f, 3.56f, 8.22f, 3.56f, 10,
R_CUBIC_TO, 0, 0.73f, 0.11f, 1.44f, 0.34f, 2.11f,
R_CUBIC_TO, 0.23f, 0.68f, 0.58f, 1.3f, 1.06f, 1.88f,
CLOSE,
MOVE_TO, 10, 11.54f,
R_CUBIC_TO, -0.85f, 0, -1.57f, -0.29f, -2.16f, -0.88f,
R_CUBIC_TO, -0.59f, -0.59f, -0.88f, -1.31f, -0.88f, -2.15f,
R_CUBIC_TO, 0, -0.85f, 0.29f, -1.57f, 0.88f, -2.16f,
ARC_TO, 2.94f, 2.94f, 0, 0, 1, 10, 5.47f,
R_CUBIC_TO, 0.85f, 0, 1.57f, 0.3f, 2.16f, 0.89f,
R_CUBIC_TO, 0.59f, 0.59f, 0.88f, 1.31f, 0.88f, 2.16f,
R_CUBIC_TO, 0, 0.84f, -0.29f, 1.56f, -0.88f, 2.15f,
R_CUBIC_TO, -0.59f, 0.59f, -1.31f, 0.88f, -2.16f, 0.88f,
CLOSE,
R_MOVE_TO, 0, 6.63f,
R_ARC_TO, 7.96f, 7.96f, 0, 0, 1, -3.17f, -0.64f,
R_ARC_TO, 8.25f, 8.25f, 0, 0, 1, -2.61f, -1.75f,
R_ARC_TO, 8.25f, 8.25f, 0, 0, 1, -1.75f, -2.61f,
R_ARC_TO, 7.97f, 7.97f, 0, 0, 1, -0.64f, -3.18f,
R_CUBIC_TO, 0, -1.12f, 0.21f, -2.18f, 0.64f, -3.17f,
R_ARC_TO, 8.29f, 8.29f, 0, 0, 1, 1.75f, -2.6f,
R_ARC_TO, 8.25f, 8.25f, 0, 0, 1, 2.61f, -1.75f,
R_ARC_TO, 7.97f, 7.97f, 0, 0, 1, 3.18f, -0.64f,
R_CUBIC_TO, 1.13f, 0, 2.18f, 0.21f, 3.17f, 0.64f,
R_ARC_TO, 8.29f, 8.29f, 0, 0, 1, 2.6f, 1.75f,
R_ARC_TO, 8.26f, 8.26f, 0, 0, 1, 1.75f, 2.6f,
R_CUBIC_TO, 0.43f, 0.99f, 0.64f, 2.05f, 0.64f, 3.17f,
R_ARC_TO, 7.96f, 7.96f, 0, 0, 1, -0.64f, 3.17f,
R_ARC_TO, 8.25f, 8.25f, 0, 0, 1, -1.75f, 2.61f,
R_ARC_TO, 8.26f, 8.26f, 0, 0, 1, -2.6f, 1.75f,
R_ARC_TO, 7.93f, 7.93f, 0, 0, 1, -3.17f, 0.64f,
CLOSE,
MOVE_TO, 10, 16.44f,
R_CUBIC_TO, 0.71f, 0, 1.38f, -0.11f, 2.03f, -0.33f,
R_ARC_TO, 6.9f, 6.9f, 0, 0, 0, 1.85f, -0.98f,
R_ARC_TO, 7.87f, 7.87f, 0, 0, 0, -1.86f, -0.83f,
ARC_TO, 7.25f, 7.25f, 0, 0, 0, 10, 14.03f,
R_CUBIC_TO, -0.7f, 0, -1.37f, 0.09f, -2.02f, 0.27f,
R_ARC_TO, 6.69f, 6.69f, 0, 0, 0, -1.85f, 0.84f,
R_CUBIC_TO, 0.58f, 0.43f, 1.2f, 0.75f, 1.84f, 0.98f,
R_CUBIC_TO, 0.64f, 0.22f, 1.32f, 0.33f, 2.03f, 0.33f,
CLOSE,
R_MOVE_TO, 0, -6.47f,
R_CUBIC_TO, 0.4f, 0, 0.75f, -0.14f, 1.03f, -0.43f,
R_CUBIC_TO, 0.28f, -0.28f, 0.43f, -0.63f, 0.43f, -1.03f,
R_CUBIC_TO, 0, -0.41f, -0.14f, -0.75f, -0.43f, -1.03f,
ARC_TO, 1.4f, 1.4f, 0, 0, 0, 10, 7.04f,
R_CUBIC_TO, -0.4f, 0, -0.75f, 0.14f, -1.03f, 0.43f,
R_ARC_TO, 1.41f, 1.41f, 0, 0, 0, -0.43f, 1.03f,
R_CUBIC_TO, 0, 0.4f, 0.14f, 0.75f, 0.43f, 1.03f,
R_CUBIC_TO, 0.28f, 0.28f, 0.63f, 0.43f, 1.03f, 0.43f,
CLOSE,
R_MOVE_TO, 0, -1.46f,
CLOSE,
R_MOVE_TO, 0, 6.62f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kUserAccountAvatarRefreshRepList, kUserAccountAvatarRefreshIcon, {kUserAccountAvatarRefreshPath, std::size(kUserAccountAvatarRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kWebPath, MOVE_TO, 40, 8,
H_LINE_TO, 8,
R_CUBIC_TO, -2.21f, 0, -3.98f, 1.79f, -3.98f, 4,
LINE_TO, 4, 36,
R_CUBIC_TO, 0, 2.21f, 1.79f, 4, 4, 4,
R_H_LINE_TO, 32,
R_CUBIC_TO, 2.21f, 0, 4, -1.79f, 4, -4,
V_LINE_TO, 12,
R_CUBIC_TO, 0, -2.21f, -1.79f, -4, -4, -4,
CLOSE,
MOVE_TO, 30, 36,
H_LINE_TO, 8,
R_V_LINE_TO, -8,
R_H_LINE_TO, 22,
R_V_LINE_TO, 8,
CLOSE,
R_MOVE_TO, 0, -10,
H_LINE_TO, 8,
R_V_LINE_TO, -8,
R_H_LINE_TO, 22,
R_V_LINE_TO, 8,
CLOSE,
R_MOVE_TO, 10, 10,
R_H_LINE_TO, -8,
V_LINE_TO, 18,
R_H_LINE_TO, 8,
R_V_LINE_TO, 18,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebRepList, kWebIcon, {kWebPath, std::size(kWebPath)})
VECTOR_ICON_REP_TEMPLATE(kIcloudKeychainPath, CANVAS_DIMENSIONS, 100,
MOVE_TO, 15.6f, 79,
R_CUBIC_TO, -17.4f, -4.7f, -21, -30, -5.2f, -37.1f,
R_CUBIC_TO, 1, -0.5f, 1.8f, -0.5f, 2.5f, -2.6f,
R_CUBIC_TO, 2.6f, -7.8f, 10.8f, -13.1f, 18.6f, -10.4f,
R_LINE_TO, 2, 0.7f,
R_LINE_TO, 1.3f, -1.8f,
CUBIC_TO, 49.3f, 7, 84.4f, 15.9f, 84.4f, 40.2f,
R_CUBIC_TO, 0, 1.5f, 0.4f, 1.9f, 2.4f, 2.5f,
R_CUBIC_TO, 16.2f, 4.8f, 17.8f, 27.6f, 2.5f, 35.2f,
R_LINE_TO, -3.8f, 2,
R_LINE_TO, -33.5f, -0.1f,
R_CUBIC_TO, -25, 0, -34.2f, -0.3f, -36.4f, -0.9f,
CLOSE,
R_MOVE_TO, 23.7f, -12.5f,
R_CUBIC_TO, 1.7f, -0.4f, 3.6f, -1.7f, 5.5f, -3.8f,
R_CUBIC_TO, 2.5f, -2.5f, 1.7f, -3, 3.5f, -3,
R_H_LINE_TO, 4.1f,
R_LINE_TO, 2, -2.5f,
R_LINE_TO, 2.3f, 2.7f,
R_LINE_TO, 1.4f, -1.4f,
R_LINE_TO, 1.3f, -1.3f,
R_LINE_TO, 1.5f, 1.3f,
R_LINE_TO, 1.4f, 1.4f,
R_LINE_TO, 1.3f, -1.2f,
R_LINE_TO, 1.3f, -1.1f,
R_LINE_TO, 1.2f, 1.1f,
R_LINE_TO, 1.3f, 1.2f,
R_LINE_TO, 1.5f, -1.4f,
R_LINE_TO, 1.4f, -1.4f,
R_LINE_TO, 1.2f, 1.3f,
CUBIC_TO, 73, 60, 74, 60, 75.9f, 58,
R_CUBIC_TO, 1.9f, -1.9f, 2, -4.3f, 0.3f, -6,
R_LINE_TO, -1.3f, -1.3f,
H_LINE_TO, 47.3f,
LINE_TO, 46, 48.7f,
CUBIC_TO, 41.3f, 41, 28.2f, 42, 24.5f, 50.4f,
R_CUBIC_TO, -4.2f, 9.2f, 4.8f, 18.9f, 14.8f, 16.1f,
CLOSE,
NEW_PATH,
NEW_PATH,
CIRCLE, 31.7, 55.1, 2.7)
VECTOR_ICON_TEMPLATE_CC(kIcloudKeychainRepList, kIcloudKeychainIcon, {kIcloudKeychainPath, std::size(kIcloudKeychainPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyAoaPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
STROKE, 2,
MOVE_TO, 216, 69.5f,
R_V_LINE_TO, 29.75f,
R_ARC_TO, 6.75f, 6.75f, 0, 1, 0, 13.5f, 0,
V_LINE_TO, 99,
R_ARC_TO, 7, 7, 0, 0, 0, -7, -7,
H_LINE_TO, 183,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
ROUND_RECT, 202, 8, 28, 58, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
ROUND_RECT, 202, 8, 28, 58, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 205, 11, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 210, 62,
R_ARC_TO, 1, 1, 0, 0, 0, 1, 1,
R_H_LINE_TO, 10,
R_ARC_TO, 1, 1, 0, 0, 0, 0, -2,
R_H_LINE_TO, -10,
R_ARC_TO, 1, 1, 0, 0, 0, -1, 1,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 218, 67,
R_H_LINE_TO, -4,
R_V_LINE_TO, 4,
R_H_LINE_TO, 4,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 95, 83,
R_ARC_TO, 4, 4, 0, 0, 0, 4, 4,
R_H_LINE_TO, 78,
R_ARC_TO, 4, 4, 0, 0, 0, 4, -4,
V_LINE_TO, 31,
R_ARC_TO, 4, 4, 0, 0, 0, -4, -4,
H_LINE_TO, 99,
R_ARC_TO, 4, 4, 0, 0, 0, -4, 4,
R_V_LINE_TO, 52,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 177, 85,
H_LINE_TO, 99,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 31,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 78,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
R_V_LINE_TO, 52,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
CLOSE,
R_MOVE_TO, -78, 2,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 31,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 78,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 52,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
H_LINE_TO, 99,
CLOSE,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 85, 90,
R_H_LINE_TO, 106,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
H_LINE_TO, 89,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 138, 31, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 277, 90, 1,
CIRCLE, 281, 90, 1,
CIRCLE, 273, 86, 1,
CIRCLE, 277, 86, 1,
CIRCLE, 285, 90, 1,
CIRCLE, 41, 75, 1,
CIRCLE, 281, 86, 1,
CIRCLE, 45, 75, 1,
CIRCLE, 37, 71, 1,
CIRCLE, 273, 90, 1,
CIRCLE, 269, 86, 1,
CIRCLE, 41, 71, 1,
CIRCLE, 49, 75, 1,
CIRCLE, 45, 71, 1,
CIRCLE, 21, 75, 1,
CIRCLE, 37, 75, 1,
CIRCLE, 25, 75, 1,
CIRCLE, 29, 75, 1,
CIRCLE, 33, 75, 1,
CIRCLE, 33, 71, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 64, 49, 3,
CIRCLE, 72, 49, 3,
CIRCLE, 48, 49, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 281, 66,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE,
R_MOVE_TO, -16, 0,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE,
R_MOVE_TO, -8, 0,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPasskeyAoaRepList, kPasskeyAoaIcon, {kPasskeyAoaPath, std::size(kPasskeyAoaPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyAoaDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
STROKE, 2,
MOVE_TO, 216, 69.5f,
R_V_LINE_TO, 29.75f,
R_ARC_TO, 6.75f, 6.75f, 0, 1, 0, 13.5f, 0,
V_LINE_TO, 99,
R_ARC_TO, 7, 7, 0, 0, 0, -7, -7,
H_LINE_TO, 183,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
ROUND_RECT, 202, 8, 28, 58, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
ROUND_RECT, 202, 8, 28, 58, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 205, 11, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 210, 62,
R_ARC_TO, 1, 1, 0, 0, 0, 1, 1,
R_H_LINE_TO, 10,
R_ARC_TO, 1, 1, 0, 0, 0, 0, -2,
R_H_LINE_TO, -10,
R_ARC_TO, 1, 1, 0, 0, 0, -1, 1,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 218, 67,
R_H_LINE_TO, -4,
R_V_LINE_TO, 4,
R_H_LINE_TO, 4,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 95, 83,
R_ARC_TO, 4, 4, 0, 0, 0, 4, 4,
R_H_LINE_TO, 78,
R_ARC_TO, 4, 4, 0, 0, 0, 4, -4,
V_LINE_TO, 31,
R_ARC_TO, 4, 4, 0, 0, 0, -4, -4,
H_LINE_TO, 99,
R_ARC_TO, 4, 4, 0, 0, 0, -4, 4,
R_V_LINE_TO, 52,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 177, 85,
H_LINE_TO, 99,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 31,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 78,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
R_V_LINE_TO, 52,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
CLOSE,
R_MOVE_TO, -78, 2,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 31,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 78,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 52,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
H_LINE_TO, 99,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 85, 90,
R_H_LINE_TO, 106,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
H_LINE_TO, 89,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 138, 31, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 277, 90, 1,
CIRCLE, 281, 90, 1,
CIRCLE, 273, 86, 1,
CIRCLE, 277, 86, 1,
CIRCLE, 285, 90, 1,
CIRCLE, 41, 75, 1,
CIRCLE, 281, 86, 1,
CIRCLE, 45, 75, 1,
CIRCLE, 37, 71, 1,
CIRCLE, 273, 90, 1,
CIRCLE, 269, 86, 1,
CIRCLE, 41, 71, 1,
CIRCLE, 49, 75, 1,
CIRCLE, 45, 71, 1,
CIRCLE, 21, 75, 1,
CIRCLE, 37, 75, 1,
CIRCLE, 25, 75, 1,
CIRCLE, 29, 75, 1,
CIRCLE, 33, 75, 1,
CIRCLE, 33, 71, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 64, 49, 3,
CIRCLE, 72, 49, 3,
CIRCLE, 48, 49, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 281, 66,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE,
R_MOVE_TO, -16, 0,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE,
R_MOVE_TO, -8, 0,
R_ARC_TO, 3, 3, 0, 1, 0, -6, 0,
R_ARC_TO, 3, 3, 0, 0, 0, 6, 0,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPasskeyAoaDarkRepList, kPasskeyAoaDarkIcon, {kPasskeyAoaDarkPath, std::size(kPasskeyAoaDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyErrorPath, CANVAS_DIMENSIONS, 316,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 206, 48, 10,
CIRCLE, 181, 48, 10,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
MOVE_TO, 152.87f, 70.67f,
R_CUBIC_TO, 0, -4.95f, 0.44f, -8.51f, 1.33f, -10.68f,
R_CUBIC_TO, 0.88f, -2.17f, 2.76f, -4.54f, 5.64f, -7.1f,
R_CUBIC_TO, 2.51f, -2.2f, 4.42f, -4.11f, 5.73f, -5.73f,
R_CUBIC_TO, 1.32f, -1.62f, 1.97f, -3.47f, 1.97f, -5.54f,
R_CUBIC_TO, 0, -2.5f, -0.84f, -4.58f, -2.52f, -6.23f,
R_CUBIC_TO, -1.68f, -1.65f, -4.02f, -2.47f, -7.01f, -2.47f,
R_CUBIC_TO, -3.12f, 0, -5.48f, 0.95f, -7.11f, 2.84f,
R_CUBIC_TO, -1.62f, 1.89f, -2.76f, 3.82f, -3.44f, 5.78f,
R_LINE_TO, -9.44f, -4.03f,
R_CUBIC_TO, 1.28f, -3.91f, 3.64f, -7.3f, 7.06f, -10.18f,
R_CUBIC_TO, 3.42f, -2.87f, 7.73f, -4.31f, 12.93f, -4.31f,
R_CUBIC_TO, 6.42f, 0, 11.35f, 1.79f, 14.81f, 5.36f,
R_CUBIC_TO, 3.45f, 3.57f, 5.18f, 7.87f, 5.18f, 12.88f,
R_CUBIC_TO, 0, 3.06f, -0.66f, 5.67f, -1.97f, 7.84f,
R_CUBIC_TO, -1.31f, 2.17f, -3.38f, 4.63f, -6.19f, 7.38f,
R_CUBIC_TO, -2.99f, 2.87f, -4.81f, 5.06f, -5.45f, 6.55f,
R_CUBIC_TO, -0.64f, 1.5f, -0.96f, 4.05f, -0.96f, 7.66f,
R_H_LINE_TO, -10.54f,
CLOSE,
R_MOVE_TO, 5.13f, 22,
R_CUBIC_TO, -2.02f, 0, -3.74f, -0.72f, -5.18f, -2.15f,
R_CUBIC_TO, -1.44f, -1.44f, -2.16f, -3.16f, -2.16f, -5.18f,
R_CUBIC_TO, 0, -2.02f, 0.72f, -3.74f, 2.16f, -5.18f,
R_CUBIC_TO, 1.43f, -1.43f, 3.16f, -2.15f, 5.18f, -2.15f,
R_CUBIC_TO, 2.02f, 0, 3.74f, 0.72f, 5.18f, 2.15f,
R_CUBIC_TO, 1.44f, 1.44f, 2.15f, 3.17f, 2.15f, 5.18f,
R_CUBIC_TO, 0, 2.02f, -0.72f, 3.74f, -2.15f, 5.18f,
R_CUBIC_TO, -1.44f, 1.44f, -3.16f, 2.15f, -5.18f, 2.15f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 172.65f, 53.51f,
ARC_TO, 9.96f, 9.96f, 0, 0, 1, 171, 48,
R_CUBIC_TO, 0, -4.4f, 2.85f, -8.15f, 6.81f, -9.48f,
R_CUBIC_TO, 0.12f, 0.88f, 0.18f, 1.79f, 0.18f, 2.72f,
R_CUBIC_TO, 0, 3.06f, -0.66f, 5.67f, -1.97f, 7.84f,
R_CUBIC_TO, -0.82f, 1.36f, -1.95f, 2.84f, -3.36f, 4.43f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 135, 65, 10,
CIRCLE, 110, 65, 10,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3)
VECTOR_ICON_TEMPLATE_CC(kPasskeyErrorRepList, kPasskeyErrorIcon, {kPasskeyErrorPath, std::size(kPasskeyErrorPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyErrorBluetoothPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 125, 35,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 25, 0,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 41, 42,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 25, 0,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
R_MOVE_TO, 160.58f, 40.06f,
R_LINE_TO, 4.86f, 4.86f,
R_LINE_TO, -4.13f, 4.13f,
R_LINE_TO, 3.64f, 3.64f,
R_LINE_TO, 7.8f, -7.8f,
LINE_TO, 158, 30.17f,
R_H_LINE_TO, -2.58f,
V_LINE_TO, 43.16f,
R_LINE_TO, 5.17f, 5.17f,
V_LINE_TO, 40.06f,
CLOSE,
R_MOVE_TO, -19.61f, -4.73f,
R_LINE_TO, -3.64f, 3.64f,
LINE_TO, 154.36f, 56,
R_LINE_TO, -14.44f, 14.44f,
R_LINE_TO, 3.64f, 3.64f,
R_LINE_TO, 11.86f, -11.86f,
R_V_LINE_TO, 19.61f,
H_LINE_TO, 158,
R_LINE_TO, 11.08f, -11.08f,
R_LINE_TO, 5.94f, 5.92f,
R_LINE_TO, 3.64f, -3.64f,
R_LINE_TO, -37.69f, -37.69f,
CLOSE,
R_MOVE_TO, 24.46f, 31.75f,
R_LINE_TO, -4.86f, -4.86f,
R_V_LINE_TO, 9.71f,
R_LINE_TO, 4.86f, -4.86f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPasskeyErrorBluetoothRepList, kPasskeyErrorBluetoothIcon, {kPasskeyErrorBluetoothPath, std::size(kPasskeyErrorBluetoothPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyErrorBluetoothDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 125, 35,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 25, 0,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 41, 42,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
R_MOVE_TO, 25, 0,
R_CUBIC_TO, 0, 5.52f, -4.48f, 10, -10, 10,
R_CUBIC_TO, -5.52f, 0, -10, -4.48f, -10, -10,
R_CUBIC_TO, 0, -5.52f, 4.48f, -10, 10, -10,
R_CUBIC_TO, 5.52f, 0, 10, 4.48f, 10, 10,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
R_MOVE_TO, 160.58f, 40.06f,
R_LINE_TO, 4.86f, 4.86f,
R_LINE_TO, -4.13f, 4.13f,
R_LINE_TO, 3.64f, 3.64f,
R_LINE_TO, 7.8f, -7.8f,
LINE_TO, 158, 30.17f,
R_H_LINE_TO, -2.58f,
V_LINE_TO, 43.16f,
R_LINE_TO, 5.17f, 5.17f,
V_LINE_TO, 40.06f,
CLOSE,
R_MOVE_TO, -19.61f, -4.73f,
R_LINE_TO, -3.64f, 3.64f,
LINE_TO, 154.36f, 56,
R_LINE_TO, -14.44f, 14.44f,
R_LINE_TO, 3.64f, 3.64f,
R_LINE_TO, 11.86f, -11.86f,
R_V_LINE_TO, 19.61f,
H_LINE_TO, 158,
R_LINE_TO, 11.08f, -11.08f,
R_LINE_TO, 5.94f, 5.92f,
R_LINE_TO, 3.64f, -3.64f,
R_LINE_TO, -37.69f, -37.69f,
CLOSE,
R_MOVE_TO, 24.46f, 31.75f,
R_LINE_TO, -4.86f, -4.86f,
R_V_LINE_TO, 9.71f,
R_LINE_TO, 4.86f, -4.86f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kPasskeyErrorBluetoothDarkRepList, kPasskeyErrorBluetoothDarkIcon, {kPasskeyErrorBluetoothDarkPath, std::size(kPasskeyErrorBluetoothDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyErrorDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
CIRCLE, 206, 48, 10,
CIRCLE, 181, 48, 10,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 152.87f, 70.67f,
R_CUBIC_TO, 0, -4.95f, 0.44f, -8.51f, 1.33f, -10.68f,
R_CUBIC_TO, 0.88f, -2.17f, 2.76f, -4.54f, 5.64f, -7.1f,
R_CUBIC_TO, 2.51f, -2.2f, 4.42f, -4.11f, 5.73f, -5.73f,
R_CUBIC_TO, 1.32f, -1.62f, 1.97f, -3.47f, 1.97f, -5.54f,
R_CUBIC_TO, 0, -2.5f, -0.84f, -4.58f, -2.52f, -6.23f,
R_CUBIC_TO, -1.68f, -1.65f, -4.02f, -2.47f, -7.01f, -2.47f,
R_CUBIC_TO, -3.12f, 0, -5.48f, 0.95f, -7.11f, 2.84f,
R_CUBIC_TO, -1.62f, 1.89f, -2.76f, 3.82f, -3.44f, 5.78f,
R_LINE_TO, -9.44f, -4.03f,
R_CUBIC_TO, 1.28f, -3.91f, 3.64f, -7.3f, 7.06f, -10.18f,
R_CUBIC_TO, 3.42f, -2.87f, 7.73f, -4.31f, 12.93f, -4.31f,
R_CUBIC_TO, 6.42f, 0, 11.35f, 1.79f, 14.81f, 5.36f,
R_CUBIC_TO, 3.45f, 3.57f, 5.18f, 7.87f, 5.18f, 12.88f,
R_CUBIC_TO, 0, 3.06f, -0.66f, 5.67f, -1.97f, 7.84f,
R_CUBIC_TO, -1.31f, 2.17f, -3.38f, 4.63f, -6.19f, 7.38f,
R_CUBIC_TO, -2.99f, 2.87f, -4.81f, 5.06f, -5.45f, 6.55f,
R_CUBIC_TO, -0.64f, 1.5f, -0.96f, 4.05f, -0.96f, 7.66f,
R_H_LINE_TO, -10.54f,
CLOSE,
R_MOVE_TO, 5.13f, 22,
R_CUBIC_TO, -2.02f, 0, -3.74f, -0.72f, -5.18f, -2.15f,
R_CUBIC_TO, -1.44f, -1.44f, -2.16f, -3.16f, -2.16f, -5.18f,
R_CUBIC_TO, 0, -2.02f, 0.72f, -3.74f, 2.16f, -5.18f,
R_CUBIC_TO, 1.43f, -1.43f, 3.16f, -2.15f, 5.18f, -2.15f,
R_CUBIC_TO, 2.02f, 0, 3.74f, 0.72f, 5.18f, 2.15f,
R_CUBIC_TO, 1.44f, 1.44f, 2.15f, 3.17f, 2.15f, 5.18f,
R_CUBIC_TO, 0, 2.02f, -0.72f, 3.74f, -2.15f, 5.18f,
R_CUBIC_TO, -1.44f, 1.44f, -3.16f, 2.15f, -5.18f, 2.15f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x20, 0x21, 0x24,
MOVE_TO, 172.65f, 53.51f,
ARC_TO, 9.96f, 9.96f, 0, 0, 1, 171, 48,
R_CUBIC_TO, 0, -4.4f, 2.85f, -8.15f, 6.81f, -9.48f,
R_CUBIC_TO, 0.12f, 0.88f, 0.18f, 1.79f, 0.18f, 2.72f,
R_CUBIC_TO, 0, 3.06f, -0.66f, 5.67f, -1.97f, 7.84f,
R_CUBIC_TO, -0.82f, 1.36f, -1.95f, 2.84f, -3.36f, 4.43f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
CIRCLE, 135, 65, 10,
CIRCLE, 110, 65, 10,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3)
VECTOR_ICON_TEMPLATE_CC(kPasskeyErrorDarkRepList, kPasskeyErrorDarkIcon, {kPasskeyErrorDarkPath, std::size(kPasskeyErrorDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyFingerprintPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 152.39f, 15.35f,
R_CUBIC_TO, 3.03f, -4.13f, 9.2f, -4.13f, 12.23f, 0,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 9.17f, 2.46f,
R_CUBIC_TO, 4.69f, -2.07f, 10.03f, 1.02f, 10.59f, 6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 6.71f, 6.71f,
R_CUBIC_TO, 5.09f, 0.56f, 8.18f, 5.9f, 6.11f, 10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 2.46f, 9.17f,
R_CUBIC_TO, 4.13f, 3.03f, 4.13f, 9.2f, 0, 12.23f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -2.46f, 9.17f,
R_CUBIC_TO, 2.07f, 4.69f, -1.02f, 10.03f, -6.11f, 10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -6.71f, 6.71f,
R_CUBIC_TO, -0.56f, 5.09f, -5.9f, 8.18f, -10.59f, 6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -9.17f, 2.46f,
R_CUBIC_TO, -3.03f, 4.13f, -9.2f, 4.13f, -12.23f, 0,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -9.17f, -2.46f,
R_CUBIC_TO, -4.69f, 2.07f, -10.03f, -1.02f, -10.59f, -6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -6.71f, -6.71f,
R_CUBIC_TO, -5.09f, -0.56f, -8.18f, -5.9f, -6.11f, -10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -2.46f, -9.17f,
R_CUBIC_TO, -4.13f, -3.03f, -4.13f, -9.2f, 0, -12.23f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 2.46f, -9.17f,
R_CUBIC_TO, -2.07f, -4.69f, 1.02f, -10.03f, 6.11f, -10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 6.71f, -6.71f,
R_CUBIC_TO, 0.56f, -5.09f, 5.9f, -8.18f, 10.59f, -6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 9.17f, -2.46f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
MOVE_TO, 173.04f, 36.8f,
R_ARC_TO, 1.12f, 1.12f, 0, 0, 1, -0.56f, -0.14f,
R_CUBIC_TO, -4.64f, -2.39f, -8.65f, -3.41f, -13.46f, -3.41f,
R_CUBIC_TO, -4.78f, 0, -9.33f, 1.14f, -13.46f, 3.41f,
R_CUBIC_TO, -0.58f, 0.31f, -1.3f, 0.1f, -1.64f, -0.48f,
R_ARC_TO, 1.22f, 1.22f, 0, 0, 1, 0.48f, -1.64f,
R_CUBIC_TO, 4.5f, -2.44f, 9.43f, -3.7f, 14.62f, -3.7f,
R_CUBIC_TO, 5.15f, 0, 9.64f, 1.14f, 14.57f, 3.67f,
R_CUBIC_TO, 0.6f, 0.31f, 0.82f, 1.04f, 0.51f, 1.62f,
R_ARC_TO, 1.19f, 1.19f, 0, 0, 1, -1.06f, 0.68f,
CLOSE,
R_MOVE_TO, -34.58f, 12.69f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -0.99f, -1.91f,
R_CUBIC_TO, 2.39f, -3.38f, 5.44f, -6.04f, 9.06f, -7.9f,
R_CUBIC_TO, 7.59f, -3.91f, 17.3f, -3.94f, 24.92f, -0.02f,
R_CUBIC_TO, 3.63f, 1.86f, 6.67f, 4.5f, 9.06f, 7.85f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -0.29f, 1.69f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -1.69f, -0.29f,
R_CUBIC_TO, -2.17f, -3.04f, -4.93f, -5.44f, -8.19f, -7.1f,
R_CUBIC_TO, -6.94f, -3.55f, -15.8f, -3.55f, -22.72f, 0.03f,
R_CUBIC_TO, -3.29f, 1.69f, -6.04f, 4.11f, -8.22f, 7.15f,
R_CUBIC_TO, -0.19f, 0.34f, -0.56f, 0.51f, -0.94f, 0.51f,
CLOSE,
R_MOVE_TO, 14.26f, 28.81f,
R_CUBIC_TO, 0.22f, 0.24f, 0.53f, 0.36f, 0.85f, 0.36f,
R_CUBIC_TO, 0.32f, 0, 0.63f, -0.12f, 0.9f, -0.36f,
R_ARC_TO, 1.24f, 1.24f, 0, 0, 0, 0, -1.72f,
R_CUBIC_TO, -1.86f, -1.88f, -2.92f, -3.07f, -4.47f, -5.85f,
R_CUBIC_TO, -1.47f, -2.61f, -2.25f, -5.82f, -2.25f, -9.3f,
R_CUBIC_TO, 0, -5.85f, 5.05f, -10.61f, 11.26f, -10.61f,
R_CUBIC_TO, 6.21f, 0, 11.26f, 4.76f, 11.26f, 10.61f,
R_CUBIC_TO, 0, 0.68f, 0.53f, 1.21f, 1.21f, 1.21f,
R_CUBIC_TO, 0.68f, 0, 1.21f, -0.53f, 1.21f, -1.21f,
R_CUBIC_TO, 0, -7.18f, -6.14f, -13.03f, -13.68f, -13.03f,
R_CUBIC_TO, -7.54f, 0, -13.68f, 5.85f, -13.68f, 13.03f,
R_CUBIC_TO, 0, 3.89f, 0.87f, 7.52f, 2.54f, 10.49f,
R_CUBIC_TO, 1.62f, 2.92f, 2.76f, 4.28f, 4.86f, 6.38f,
CLOSE,
R_MOVE_TO, 18.17f, -4.11f,
R_CUBIC_TO, -2.88f, 0, -5.41f, -0.72f, -7.49f, -2.15f,
R_CUBIC_TO, -3.6f, -2.44f, -5.75f, -6.4f, -5.75f, -10.61f,
R_CUBIC_TO, 0, -0.68f, 0.53f, -1.21f, 1.21f, -1.21f,
R_CUBIC_TO, 0.68f, 0, 1.21f, 0.53f, 1.21f, 1.21f,
R_CUBIC_TO, 0, 3.41f, 1.74f, 6.62f, 4.69f, 8.6f,
R_CUBIC_TO, 1.72f, 1.16f, 3.72f, 1.72f, 6.14f, 1.72f,
R_CUBIC_TO, 0.58f, 0, 1.55f, -0.07f, 2.51f, -0.24f,
R_CUBIC_TO, 0.65f, -0.12f, 1.28f, 0.31f, 1.4f, 0.99f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 1, -0.99f, 1.4f,
R_CUBIC_TO, -1.38f, 0.27f, -2.58f, 0.29f, -2.92f, 0.29f,
CLOSE,
R_MOVE_TO, -5.17f, 4.93f,
R_CUBIC_TO, 0.1f, 0.03f, 0.22f, 0.05f, 0.31f, 0.05f,
R_ARC_TO, 1.18f, 1.18f, 0, 0, 0, 1.14f, -0.92f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 0, -0.85f, -1.47f,
R_CUBIC_TO, -3.41f, -0.94f, -5.61f, -2.2f, -7.9f, -4.47f,
R_CUBIC_TO, -2.92f, -2.9f, -4.52f, -6.77f, -4.52f, -10.9f,
R_CUBIC_TO, 0, -2.58f, 2.25f, -4.69f, 5.03f, -4.69f,
R_CUBIC_TO, 2.78f, 0, 5.03f, 2.1f, 5.03f, 4.69f,
R_CUBIC_TO, 0, 3.92f, 3.34f, 7.11f, 7.44f, 7.11f,
R_CUBIC_TO, 4.1f, 0, 7.44f, -3.19f, 7.44f, -7.1f,
R_CUBIC_TO, 0, -10.44f, -8.94f, -18.92f, -19.94f, -18.92f,
R_CUBIC_TO, -7.81f, 0, -14.93f, 4.37f, -18.15f, 11.12f,
R_CUBIC_TO, -1.09f, 2.3f, -1.64f, 4.93f, -1.64f, 7.83f,
R_CUBIC_TO, 0, 3.26f, 0.58f, 6.4f, 1.76f, 9.57f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 0, 1.55f, 0.7f,
R_ARC_TO, 1.18f, 1.18f, 0, 0, 0, 0.7f, -1.55f,
R_CUBIC_TO, -1.45f, -3.87f, -1.62f, -6.84f, -1.62f, -8.72f,
R_CUBIC_TO, 0, -2.51f, 0.48f, -4.81f, 1.43f, -6.77f,
R_CUBIC_TO, 2.83f, -5.92f, 9.11f, -9.74f, 15.97f, -9.74f,
R_CUBIC_TO, 9.67f, 0, 17.52f, 7.4f, 17.52f, 16.51f,
R_CUBIC_TO, 0, 2.59f, -2.25f, 4.69f, -5.03f, 4.69f,
R_CUBIC_TO, -2.78f, 0, -5.03f, -2.1f, -5.03f, -4.69f,
R_CUBIC_TO, 0, -3.91f, -3.33f, -7.1f, -7.44f, -7.1f,
R_CUBIC_TO, -4.11f, 0, -7.44f, 3.19f, -7.44f, 7.11f,
R_CUBIC_TO, 0, 4.79f, 1.86f, 9.26f, 5.24f, 12.62f,
R_CUBIC_TO, 2.63f, 2.59f, 5.15f, 4.01f, 8.99f, 5.08f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3)
VECTOR_ICON_TEMPLATE_CC(kPasskeyFingerprintRepList, kPasskeyFingerprintIcon, {kPasskeyFingerprintPath, std::size(kPasskeyFingerprintPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyFingerprintDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 152.39f, 15.35f,
R_CUBIC_TO, 3.03f, -4.13f, 9.2f, -4.13f, 12.23f, 0,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 9.17f, 2.46f,
R_CUBIC_TO, 4.69f, -2.07f, 10.03f, 1.02f, 10.59f, 6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 6.71f, 6.71f,
R_CUBIC_TO, 5.09f, 0.56f, 8.18f, 5.9f, 6.11f, 10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 2.46f, 9.17f,
R_CUBIC_TO, 4.13f, 3.03f, 4.13f, 9.2f, 0, 12.23f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -2.46f, 9.17f,
R_CUBIC_TO, 2.07f, 4.69f, -1.02f, 10.03f, -6.11f, 10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -6.71f, 6.71f,
R_CUBIC_TO, -0.56f, 5.09f, -5.9f, 8.18f, -10.59f, 6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -9.17f, 2.46f,
R_CUBIC_TO, -3.03f, 4.13f, -9.2f, 4.13f, -12.23f, 0,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -9.17f, -2.46f,
R_CUBIC_TO, -4.69f, 2.07f, -10.03f, -1.02f, -10.59f, -6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -6.71f, -6.71f,
R_CUBIC_TO, -5.09f, -0.56f, -8.18f, -5.9f, -6.11f, -10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, -2.46f, -9.17f,
R_CUBIC_TO, -4.13f, -3.03f, -4.13f, -9.2f, 0, -12.23f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 2.46f, -9.17f,
R_CUBIC_TO, -2.07f, -4.69f, 1.02f, -10.03f, 6.11f, -10.59f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 6.71f, -6.71f,
R_CUBIC_TO, 0.56f, -5.09f, 5.9f, -8.18f, 10.59f, -6.11f,
R_ARC_TO, 7.58f, 7.58f, 0, 0, 0, 9.17f, -2.46f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 173.04f, 36.8f,
R_ARC_TO, 1.12f, 1.12f, 0, 0, 1, -0.56f, -0.14f,
R_CUBIC_TO, -4.64f, -2.39f, -8.65f, -3.41f, -13.46f, -3.41f,
R_CUBIC_TO, -4.78f, 0, -9.33f, 1.14f, -13.46f, 3.41f,
R_CUBIC_TO, -0.58f, 0.31f, -1.3f, 0.1f, -1.64f, -0.48f,
R_ARC_TO, 1.22f, 1.22f, 0, 0, 1, 0.48f, -1.64f,
R_CUBIC_TO, 4.5f, -2.44f, 9.43f, -3.7f, 14.62f, -3.7f,
R_CUBIC_TO, 5.15f, 0, 9.64f, 1.14f, 14.57f, 3.67f,
R_CUBIC_TO, 0.6f, 0.31f, 0.82f, 1.04f, 0.51f, 1.62f,
R_ARC_TO, 1.19f, 1.19f, 0, 0, 1, -1.06f, 0.68f,
CLOSE,
R_MOVE_TO, -34.58f, 12.69f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -0.99f, -1.91f,
R_CUBIC_TO, 2.39f, -3.38f, 5.44f, -6.04f, 9.06f, -7.9f,
R_CUBIC_TO, 7.59f, -3.91f, 17.3f, -3.94f, 24.92f, -0.02f,
R_CUBIC_TO, 3.63f, 1.86f, 6.67f, 4.5f, 9.06f, 7.85f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -0.29f, 1.69f,
R_ARC_TO, 1.21f, 1.21f, 0, 0, 1, -1.69f, -0.29f,
R_CUBIC_TO, -2.17f, -3.04f, -4.93f, -5.44f, -8.19f, -7.1f,
R_CUBIC_TO, -6.94f, -3.55f, -15.8f, -3.55f, -22.72f, 0.03f,
R_CUBIC_TO, -3.29f, 1.69f, -6.04f, 4.11f, -8.22f, 7.15f,
R_CUBIC_TO, -0.19f, 0.34f, -0.56f, 0.51f, -0.94f, 0.51f,
CLOSE,
R_MOVE_TO, 14.26f, 28.81f,
R_CUBIC_TO, 0.22f, 0.24f, 0.53f, 0.36f, 0.85f, 0.36f,
R_CUBIC_TO, 0.32f, 0, 0.63f, -0.12f, 0.9f, -0.36f,
R_ARC_TO, 1.24f, 1.24f, 0, 0, 0, 0, -1.72f,
R_CUBIC_TO, -1.86f, -1.88f, -2.92f, -3.07f, -4.47f, -5.85f,
R_CUBIC_TO, -1.47f, -2.61f, -2.25f, -5.82f, -2.25f, -9.3f,
R_CUBIC_TO, 0, -5.85f, 5.05f, -10.61f, 11.26f, -10.61f,
R_CUBIC_TO, 6.21f, 0, 11.26f, 4.76f, 11.26f, 10.61f,
R_CUBIC_TO, 0, 0.68f, 0.53f, 1.21f, 1.21f, 1.21f,
R_CUBIC_TO, 0.68f, 0, 1.21f, -0.53f, 1.21f, -1.21f,
R_CUBIC_TO, 0, -7.18f, -6.14f, -13.03f, -13.68f, -13.03f,
R_CUBIC_TO, -7.54f, 0, -13.68f, 5.85f, -13.68f, 13.03f,
R_CUBIC_TO, 0, 3.89f, 0.87f, 7.52f, 2.54f, 10.49f,
R_CUBIC_TO, 1.62f, 2.92f, 2.76f, 4.28f, 4.86f, 6.38f,
CLOSE,
R_MOVE_TO, 18.17f, -4.11f,
R_CUBIC_TO, -2.88f, 0, -5.41f, -0.72f, -7.49f, -2.15f,
R_CUBIC_TO, -3.6f, -2.44f, -5.75f, -6.4f, -5.75f, -10.61f,
R_CUBIC_TO, 0, -0.68f, 0.53f, -1.21f, 1.21f, -1.21f,
R_CUBIC_TO, 0.68f, 0, 1.21f, 0.53f, 1.21f, 1.21f,
R_CUBIC_TO, 0, 3.41f, 1.74f, 6.62f, 4.69f, 8.6f,
R_CUBIC_TO, 1.72f, 1.16f, 3.72f, 1.72f, 6.14f, 1.72f,
R_CUBIC_TO, 0.58f, 0, 1.55f, -0.07f, 2.51f, -0.24f,
R_CUBIC_TO, 0.65f, -0.12f, 1.28f, 0.31f, 1.4f, 0.99f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 1, -0.99f, 1.4f,
R_CUBIC_TO, -1.38f, 0.27f, -2.58f, 0.29f, -2.92f, 0.29f,
CLOSE,
R_MOVE_TO, -5.17f, 4.93f,
R_CUBIC_TO, 0.1f, 0.03f, 0.22f, 0.05f, 0.31f, 0.05f,
R_ARC_TO, 1.18f, 1.18f, 0, 0, 0, 1.14f, -0.92f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 0, -0.85f, -1.47f,
R_CUBIC_TO, -3.41f, -0.94f, -5.61f, -2.2f, -7.9f, -4.47f,
R_CUBIC_TO, -2.92f, -2.9f, -4.52f, -6.77f, -4.52f, -10.9f,
R_CUBIC_TO, 0, -2.58f, 2.25f, -4.69f, 5.03f, -4.69f,
R_CUBIC_TO, 2.78f, 0, 5.03f, 2.1f, 5.03f, 4.69f,
R_CUBIC_TO, 0, 3.92f, 3.34f, 7.11f, 7.44f, 7.11f,
R_CUBIC_TO, 4.1f, 0, 7.44f, -3.19f, 7.44f, -7.1f,
R_CUBIC_TO, 0, -10.44f, -8.94f, -18.92f, -19.94f, -18.92f,
R_CUBIC_TO, -7.81f, 0, -14.93f, 4.37f, -18.15f, 11.12f,
R_CUBIC_TO, -1.09f, 2.3f, -1.64f, 4.93f, -1.64f, 7.83f,
R_CUBIC_TO, 0, 3.26f, 0.58f, 6.4f, 1.76f, 9.57f,
R_ARC_TO, 1.2f, 1.2f, 0, 0, 0, 1.55f, 0.7f,
R_ARC_TO, 1.18f, 1.18f, 0, 0, 0, 0.7f, -1.55f,
R_CUBIC_TO, -1.45f, -3.87f, -1.62f, -6.84f, -1.62f, -8.72f,
R_CUBIC_TO, 0, -2.51f, 0.48f, -4.81f, 1.43f, -6.77f,
R_CUBIC_TO, 2.83f, -5.92f, 9.11f, -9.74f, 15.97f, -9.74f,
R_CUBIC_TO, 9.67f, 0, 17.52f, 7.4f, 17.52f, 16.51f,
R_CUBIC_TO, 0, 2.59f, -2.25f, 4.69f, -5.03f, 4.69f,
R_CUBIC_TO, -2.78f, 0, -5.03f, -2.1f, -5.03f, -4.69f,
R_CUBIC_TO, 0, -3.91f, -3.33f, -7.1f, -7.44f, -7.1f,
R_CUBIC_TO, -4.11f, 0, -7.44f, 3.19f, -7.44f, 7.11f,
R_CUBIC_TO, 0, 4.79f, 1.86f, 9.26f, 5.24f, 12.62f,
R_CUBIC_TO, 2.63f, 2.59f, 5.15f, 4.01f, 8.99f, 5.08f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3)
VECTOR_ICON_TEMPLATE_CC(kPasskeyFingerprintDarkRepList, kPasskeyFingerprintDarkIcon, {kPasskeyFingerprintDarkPath, std::size(kPasskeyFingerprintDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyPhonePath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 16, 1,
CIRCLE, 96, 16, 1,
CIRCLE, 84, 16, 1,
CIRCLE, 88, 16, 1,
CIRCLE, 100, 16, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 20, 1,
CIRCLE, 80, 20, 1,
CIRCLE, 64, 20, 1,
CIRCLE, 84, 20, 1,
CIRCLE, 88, 20, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 20, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
ROUND_RECT, 139, 14, 38, 82, 3,
NEW_PATH,
STROKE, 2,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
ROUND_RECT, 139, 14, 38, 82, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
MOVE_TO, 161.38f, 59.62f,
V_LINE_TO, 54.82f,
R_LINE_TO, 0.01f, -0.11f,
R_ARC_TO, 3.27f, 3.27f, 0, 0, 0, -0.78f, -2.41f,
R_ARC_TO, 3.16f, 3.16f, 0, 0, 0, -1.61f, -0.99f,
R_ARC_TO, 3.09f, 3.09f, 0, 0, 0, -1.49f, 0,
R_H_LINE_TO, 0,
R_ARC_TO, 3.14f, 3.14f, 0, 0, 0, -1.79f, 1.22f,
R_ARC_TO, 3.27f, 3.27f, 0, 0, 0, -0.6f, 2.18f,
R_LINE_TO, 0.01f, 0.11f,
R_V_LINE_TO, 4.8f,
R_H_LINE_TO, 6.25f,
CLOSE,
R_MOVE_TO, -7.81f, -4.8f,
R_ARC_TO, 4.98f, 4.98f, 0, 0, 1, 0.87f, -3.18f,
R_ARC_TO, 4.77f, 4.77f, 0, 0, 1, 2.65f, -1.88f,
R_V_LINE_TO, -0.55f,
R_ARC_TO, 1.23f, 1.23f, 0, 0, 1, 0.34f, -0.86f,
R_ARC_TO, 1.17f, 1.17f, 0, 0, 1, 0.83f, -0.36f,
R_ARC_TO, 1.15f, 1.15f, 0, 0, 1, 0.83f, 0.36f,
R_ARC_TO, 1.22f, 1.22f, 0, 0, 1, 0.34f, 0.86f,
R_V_LINE_TO, 0.54f,
R_ARC_TO, 4.76f, 4.76f, 0, 0, 1, 2.65f, 1.88f,
R_ARC_TO, 4.94f, 4.94f, 0, 0, 1, 0.87f, 3.18f,
R_V_LINE_TO, 4.8f,
R_H_LINE_TO, 1.56f,
R_V_LINE_TO, 1.6f,
H_LINE_TO, 152,
R_V_LINE_TO, -1.6f,
R_H_LINE_TO, 1.56f,
R_V_LINE_TO, -4.8f,
CLOSE,
R_MOVE_TO, 4.69f, 8.81f,
R_ARC_TO, 1.55f, 1.55f, 0, 0, 0, 1.1f, -0.47f,
R_CUBIC_TO, 0.29f, -0.3f, 0.46f, -0.71f, 0.46f, -1.13f,
R_H_LINE_TO, -3.12f,
R_ARC_TO, 1.62f, 1.62f, 0, 0, 0, 0.53f, 1.2f,
R_CUBIC_TO, 0.28f, 0.26f, 0.65f, 0.4f, 1.03f, 0.4f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
ROUND_RECT, 149, 92, 17.143, 2, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
STROKE, 2,
CIRCLE, 158, 56, 13,
CIRCLE, 158, 56, 27,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 144.055, 20, 2)
VECTOR_ICON_TEMPLATE_CC(kPasskeyPhoneRepList, kPasskeyPhoneIcon, {kPasskeyPhonePath, std::size(kPasskeyPhonePath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyPhoneDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 16, 1,
CIRCLE, 96, 16, 1,
CIRCLE, 84, 16, 1,
CIRCLE, 88, 16, 1,
CIRCLE, 100, 16, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 20, 1,
CIRCLE, 80, 20, 1,
CIRCLE, 64, 20, 1,
CIRCLE, 84, 20, 1,
CIRCLE, 88, 20, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 20, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
ROUND_RECT, 139, 14, 38, 82, 3,
NEW_PATH,
STROKE, 2,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
ROUND_RECT, 139, 14, 38, 82, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
MOVE_TO, 161.38f, 59.62f,
V_LINE_TO, 54.82f,
R_LINE_TO, 0.01f, -0.11f,
R_ARC_TO, 3.27f, 3.27f, 0, 0, 0, -0.78f, -2.41f,
R_ARC_TO, 3.16f, 3.16f, 0, 0, 0, -1.61f, -0.99f,
R_ARC_TO, 3.09f, 3.09f, 0, 0, 0, -1.49f, 0,
R_H_LINE_TO, 0,
R_ARC_TO, 3.14f, 3.14f, 0, 0, 0, -1.79f, 1.22f,
R_ARC_TO, 3.27f, 3.27f, 0, 0, 0, -0.6f, 2.18f,
R_LINE_TO, 0.01f, 0.11f,
R_V_LINE_TO, 4.8f,
R_H_LINE_TO, 6.25f,
CLOSE,
R_MOVE_TO, -7.81f, -4.8f,
R_ARC_TO, 4.98f, 4.98f, 0, 0, 1, 0.87f, -3.18f,
R_ARC_TO, 4.77f, 4.77f, 0, 0, 1, 2.65f, -1.88f,
R_V_LINE_TO, -0.55f,
R_ARC_TO, 1.23f, 1.23f, 0, 0, 1, 0.34f, -0.86f,
R_ARC_TO, 1.17f, 1.17f, 0, 0, 1, 0.83f, -0.36f,
R_ARC_TO, 1.15f, 1.15f, 0, 0, 1, 0.83f, 0.36f,
R_ARC_TO, 1.22f, 1.22f, 0, 0, 1, 0.34f, 0.86f,
R_V_LINE_TO, 0.54f,
R_ARC_TO, 4.76f, 4.76f, 0, 0, 1, 2.65f, 1.88f,
R_ARC_TO, 4.94f, 4.94f, 0, 0, 1, 0.87f, 3.18f,
R_V_LINE_TO, 4.8f,
R_H_LINE_TO, 1.56f,
R_V_LINE_TO, 1.6f,
H_LINE_TO, 152,
R_V_LINE_TO, -1.6f,
R_H_LINE_TO, 1.56f,
R_V_LINE_TO, -4.8f,
CLOSE,
R_MOVE_TO, 4.69f, 8.81f,
R_ARC_TO, 1.55f, 1.55f, 0, 0, 0, 1.1f, -0.47f,
R_CUBIC_TO, 0.29f, -0.3f, 0.46f, -0.71f, 0.46f, -1.13f,
R_H_LINE_TO, -3.12f,
R_ARC_TO, 1.62f, 1.62f, 0, 0, 0, 0.53f, 1.2f,
R_CUBIC_TO, 0.28f, 0.26f, 0.65f, 0.4f, 1.03f, 0.4f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
ROUND_RECT, 149, 92, 17.143, 2, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
CIRCLE, 158, 56, 13,
CIRCLE, 158, 56, 27,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 144.055, 20, 2)
VECTOR_ICON_TEMPLATE_CC(kPasskeyPhoneDarkRepList, kPasskeyPhoneDarkIcon, {kPasskeyPhoneDarkPath, std::size(kPasskeyPhoneDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyUsbPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 149, 74,
R_H_LINE_TO, 18,
R_V_LINE_TO, 16,
R_H_LINE_TO, -18,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
MOVE_TO, 149, 74,
R_H_LINE_TO, 18,
R_V_LINE_TO, 16,
R_H_LINE_TO, -18,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x7C, 0xAC, 0xF8,
ROUND_RECT, 161, 75, 2, 10, 1,
ROUND_RECT, 157, 75, 2, 10, 1,
ROUND_RECT, 153, 75, 2, 10, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x7C, 0xAC, 0xF8,
MOVE_TO, 150, 75,
R_H_LINE_TO, 16,
R_V_LINE_TO, 2,
R_H_LINE_TO, -16,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xDA, 0xDC, 0xE0,
MOVE_TO, 158, 20,
R_CUBIC_TO, 9.94f, 0, 18, 8.06f, 18, 18,
R_V_LINE_TO, 33,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
R_H_LINE_TO, -28,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 38,
R_CUBIC_TO, 0, -9.94f, 8.06f, -18, 18, -18,
CLOSE,
R_MOVE_TO, 0, 23,
R_ARC_TO, 5, 5, 0, 1, 0, 0, -10,
R_ARC_TO, 5, 5, 0, 0, 0, 0, 10,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
STROKE, 2,
MOVE_TO, 158, 20,
R_CUBIC_TO, 9.94f, 0, 18, 8.06f, 18, 18,
R_V_LINE_TO, 33,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
R_H_LINE_TO, -28,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 38,
R_CUBIC_TO, 0, -9.94f, 8.06f, -18, 18, -18,
CLOSE,
R_MOVE_TO, 0, 23,
R_ARC_TO, 5, 5, 0, 1, 0, 0, -10,
R_ARC_TO, 5, 5, 0, 0, 0, 0, 10,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
CIRCLE, 158, 38, 6,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
CIRCLE, 158, 38, 6)
VECTOR_ICON_TEMPLATE_CC(kPasskeyUsbRepList, kPasskeyUsbIcon, {kPasskeyUsbPath, std::size(kPasskeyUsbPath)})
VECTOR_ICON_REP_TEMPLATE(kPasskeyUsbDarkPath, CANVAS_DIMENSIONS, 316,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 149, 74,
R_H_LINE_TO, 18,
R_V_LINE_TO, 16,
R_H_LINE_TO, -18,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x1B, 0x6E, 0xF3,
STROKE, 2,
MOVE_TO, 149, 74,
R_H_LINE_TO, 18,
R_V_LINE_TO, 16,
R_H_LINE_TO, -18,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 92, 17, 1,
CIRCLE, 96, 17, 1,
CIRCLE, 84, 17, 1,
CIRCLE, 88, 17, 1,
CIRCLE, 100, 17, 1,
CIRCLE, 107, 86, 1,
CIRCLE, 99, 86, 1,
CIRCLE, 103, 86, 1,
CIRCLE, 111, 86, 1,
CIRCLE, 229, 20, 1,
CIRCLE, 233, 20, 1,
CIRCLE, 221, 20, 1,
CIRCLE, 225, 20, 1,
CIRCLE, 115, 86, 1,
CIRCLE, 281, 37, 1,
CIRCLE, 285, 37, 1,
CIRCLE, 76, 21, 1,
CIRCLE, 80, 21, 1,
CIRCLE, 64, 21, 1,
CIRCLE, 84, 21, 1,
CIRCLE, 88, 21, 1,
CIRCLE, 273, 37, 1,
CIRCLE, 72, 21, 1,
CIRCLE, 277, 37, 1,
CIRCLE, 289, 37, 1,
CIRCLE, 293, 37, 1,
CIRCLE, 297, 37, 1,
CIRCLE, 91, 90, 1,
CIRCLE, 95, 90, 1,
CIRCLE, 99, 90, 1,
CIRCLE, 103, 90, 1,
CIRCLE, 107, 90, 1,
CIRCLE, 87, 90, 1,
CIRCLE, 265, 41, 1,
CIRCLE, 269, 41, 1,
CIRCLE, 273, 41, 1,
CIRCLE, 253, 90, 1,
CIRCLE, 257, 90, 1,
CIRCLE, 249, 86, 1,
CIRCLE, 253, 86, 1,
CIRCLE, 261, 90, 1,
CIRCLE, 39, 69, 1,
CIRCLE, 257, 86, 1,
CIRCLE, 43, 69, 1,
CIRCLE, 35, 65, 1,
CIRCLE, 233, 90, 1,
CIRCLE, 249, 90, 1,
CIRCLE, 237, 90, 1,
CIRCLE, 241, 90, 1,
CIRCLE, 245, 90, 1,
CIRCLE, 245, 86, 1,
CIRCLE, 39, 65, 1,
CIRCLE, 47, 69, 1,
CIRCLE, 43, 65, 1,
CIRCLE, 19, 69, 1,
CIRCLE, 35, 69, 1,
CIRCLE, 23, 69, 1,
CIRCLE, 27, 69, 1,
CIRCLE, 31, 69, 1,
CIRCLE, 31, 65, 1,
CIRCLE, 261, 41, 1,
CIRCLE, 277, 41, 1,
CIRCLE, 281, 41, 1,
CIRCLE, 70, 44, 3,
CIRCLE, 78, 44, 3,
CIRCLE, 86, 44, 3,
CIRCLE, 54, 44, 3,
CIRCLE, 220, 69, 3,
CIRCLE, 228, 69, 3,
CIRCLE, 236, 69, 3,
CIRCLE, 252, 69, 3,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
ROUND_RECT, 161, 75, 2, 10, 1,
ROUND_RECT, 157, 75, 2, 10, 1,
ROUND_RECT, 153, 75, 2, 10, 1,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
MOVE_TO, 150, 75,
R_H_LINE_TO, 16,
R_V_LINE_TO, 2,
R_H_LINE_TO, -16,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x3C, 0x40, 0x43,
MOVE_TO, 158, 20,
R_CUBIC_TO, 9.94f, 0, 18, 8.06f, 18, 18,
R_V_LINE_TO, 33,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
R_H_LINE_TO, -28,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 38,
R_CUBIC_TO, 0, -9.94f, 8.06f, -18, 18, -18,
CLOSE,
R_MOVE_TO, 0, 23,
R_ARC_TO, 5, 5, 0, 1, 0, 0, -10,
R_ARC_TO, 5, 5, 0, 0, 0, 0, 10,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x4C, 0x8D, 0xF6,
STROKE, 2,
MOVE_TO, 158, 20,
R_CUBIC_TO, 9.94f, 0, 18, 8.06f, 18, 18,
R_V_LINE_TO, 33,
R_ARC_TO, 4, 4, 0, 0, 1, -4, 4,
R_H_LINE_TO, -28,
R_ARC_TO, 4, 4, 0, 0, 1, -4, -4,
V_LINE_TO, 38,
R_CUBIC_TO, 0, -9.94f, 8.06f, -18, 18, -18,
CLOSE,
R_MOVE_TO, 0, 23,
R_ARC_TO, 5, 5, 0, 1, 0, 0, -10,
R_ARC_TO, 5, 5, 0, 0, 0, 0, 10,
CLOSE,
NEW_PATH,
PATH_MODE_CLEAR,
CIRCLE, 158, 38, 6,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x0B, 0x57, 0xD0,
STROKE, 2,
CIRCLE, 158, 38, 6)
VECTOR_ICON_TEMPLATE_CC(kPasskeyUsbDarkRepList, kPasskeyUsbDarkIcon, {kPasskeyUsbDarkPath, std::size(kPasskeyUsbDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kUsbSecurityKeyPath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 9.5f, 9.5f,
R_ARC_TO, 2.5f, 2.5f, 0, 1, 1, 5, 0,
R_ARC_TO, 2.5f, 2.5f, 0, 0, 1, -5, 0,
CLOSE,
MOVE_TO, 16, 3,
R_V_LINE_TO, 12.5f,
H_LINE_TO, 8,
LINE_TO, 8, 3,
H_LINE_TO, 16,
CLOSE,
MOVE_TO, 7.5f, 17.5f,
H_LINE_TO, 8,
R_V_LINE_TO, 4,
R_CUBIC_TO, 0, 0.83f, 0.67f, 1.5f, 1.5f, 1.5f,
R_H_LINE_TO, 5,
R_CUBIC_TO, 0.83f, 0, 1.5f, -0.67f, 1.5f, -1.5f,
R_V_LINE_TO, -4,
R_H_LINE_TO, 0.5f,
R_CUBIC_TO, 0.83f, 0, 1.5f, -0.67f, 1.5f, -1.5f,
V_LINE_TO, 3,
R_CUBIC_TO, 0, -1.1f, -0.9f, -2, -2, -2,
H_LINE_TO, 8,
R_CUBIC_TO, -1.1f, 0, -2, 0.9f, -2, 2,
R_V_LINE_TO, 13,
R_CUBIC_TO, 0, 0.83f, 0.67f, 1.5f, 1.5f, 1.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kUsbSecurityKeyRepList, kUsbSecurityKeyIcon, {kUsbSecurityKeyPath, std::size(kUsbSecurityKeyPath)})
VECTOR_ICON_REP_TEMPLATE(kWebauthnErrorPath, CANVAS_DIMENSIONS, 448,
NEW_PATH,
PATH_COLOR_ARGB, 0x0A, 0x00, 0x00, 0x00,
MOVE_TO, 0, 0,
R_H_LINE_TO, 448,
R_V_LINE_TO, 119,
H_LINE_TO, 0,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
MOVE_TO, 461.49f, 67.44f,
H_LINE_TO, 418,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 445.36f, 52,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
R_MOVE_TO, -89, -34,
H_LINE_TO, 329,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 356.36f, 18,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
R_MOVE_TO, -312, 39,
H_LINE_TO, 17,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 44.36f, 57,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
MOVE_TO, 169.11f, 40.08f,
V_LINE_TO, 33.5f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 30.94f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 6.58f,
H_LINE_TO, 288,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
V_LINE_TO, 131.5f,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
H_LINE_TO, 163,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 42.08f,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 6.11f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0xE8, 0xE9, 0xEB,
MOVE_TO, 169.11f, 40.08f,
V_LINE_TO, 33.5f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 30.94f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 6.58f,
H_LINE_TO, 288,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
V_LINE_TO, 131.5f,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
H_LINE_TO, 163,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 42.08f,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 6.11f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 208.88f, 83.59f,
R_LINE_TO, 0.70f, -0.70f,
R_LINE_TO, 3.54f, 3.54f,
R_LINE_TO, -0.70f, 0.70f,
CLOSE,
MOVE_TO, 212.41f, 82.88f,
R_LINE_TO, 0.70f, 0.70f,
R_LINE_TO, -3.53f, 3.54f,
R_LINE_TO, -0.70f, -0.70f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 229.5f, 100.5f,
R_V_LINE_TO, 1,
R_H_LINE_TO, -8,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, -9, 2,
R_H_LINE_TO, -2,
R_V_LINE_TO, 1,
R_H_LINE_TO, 2,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, 12, 0,
R_H_LINE_TO, -2,
R_V_LINE_TO, 1,
R_H_LINE_TO, 2,
R_V_LINE_TO, -1,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 236.88f, 83.59f,
R_LINE_TO, 0.71f, -0.70f,
R_LINE_TO, 3.54f, 3.54f,
R_LINE_TO, -0.70f, 0.71f,
CLOSE,
MOVE_TO, 240.41f, 82.88f,
R_LINE_TO, 0.70f, 0.70f,
R_LINE_TO, -3.53f, 3.54f,
R_LINE_TO, -0.70f, -0.70f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xF8, 0xF9, 0xFA,
ROUND_RECT, 170, 48, 111, 7, 3.5,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xBD, 0xC1, 0xC6,
CIRCLE, 147, 92.5, 31,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xff, 0xff, 0xff,
MOVE_TO, 148.48f, 98.09f,
R_H_LINE_TO, -2.91f,
R_CUBIC_TO, -0.01f, -1.57f, 0.25f, -3.23f, 1.32f, -4.45f,
R_CUBIC_TO, 1.1f, -1.47f, 2.58f, -2.62f, 3.6f, -4.15f,
R_CUBIC_TO, 0.69f, -1.43f, 0.67f, -3.35f, -0.4f, -4.59f,
R_CUBIC_TO, -1.19f, -1.23f, -3.25f, -1.17f, -4.66f, -0.36f,
R_CUBIC_TO, -0.98f, 0.44f, -1.34f, 1.63f, -1.42f, 2.54f,
R_H_LINE_TO, -2.84f,
R_CUBIC_TO, -0.04f, -1.82f, 0.89f, -3.64f, 2.48f, -4.57f,
R_CUBIC_TO, 2.03f, -1.26f, 4.67f, -1.37f, 6.89f, -0.56f,
R_CUBIC_TO, 2, 0.75f, 3.27f, 2.83f, 3.31f, 4.93f,
R_CUBIC_TO, 0.16f, 1.78f, -0.51f, 3.56f, -1.69f, 4.88f,
R_CUBIC_TO, -0.78f, 1.03f, -1.75f, 1.91f, -2.64f, 2.84f,
R_CUBIC_TO, -0.78f, 0.98f, -1.04f, 2.27f, -1.03f, 3.5f,
CLOSE,
R_MOVE_TO, -3.03f, 4.95f,
R_CUBIC_TO, -0.09f, -1.19f, 1.21f, -1.97f, 2.28f, -1.6f,
R_CUBIC_TO, 1.43f, 0.32f, 1.54f, 2.64f, 0.17f, 3.12f,
R_CUBIC_TO, -1.01f, 0.43f, -2.4f, -0.16f, -2.44f, -1.35f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xAC, 0xB0, 0xB5,
MOVE_TO, 161, 64.83f,
CUBIC_TO, 171.09f, 69.94f, 178, 80.41f, 178, 92.49f,
R_CUBIC_TO, 0, 12.04f, -6.87f, 22.48f, -16.9f, 27.62f,
R_ARC_TO, 2, 2, 0, 0, 1, -0.1f, -0.62f,
V_LINE_TO, 64.83f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebauthnErrorRepList, kWebauthnErrorIcon, {kWebauthnErrorPath, std::size(kWebauthnErrorPath)})
VECTOR_ICON_REP_TEMPLATE(kWebauthnErrorDarkPath, CANVAS_DIMENSIONS, 448,
NEW_PATH,
PATH_COLOR_ARGB, 0x18, 0x00, 0x00, 0x00,
MOVE_TO, 0, 0,
R_H_LINE_TO, 448,
R_V_LINE_TO, 119,
H_LINE_TO, 0,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 461.49f, 67.44f,
H_LINE_TO, 418,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 445.36f, 52,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
R_MOVE_TO, -89, -34,
H_LINE_TO, 329,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 356.36f, 18,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
R_MOVE_TO, -312, 39,
H_LINE_TO, 17,
R_CUBIC_TO, 0.36f, -5.48f, 4.94f, -9.83f, 10.51f, -9.83f,
R_CUBIC_TO, 1.18f, 0, 2.36f, 0.21f, 3.51f, 0.61f,
R_LINE_TO, 0.47f, 0.17f,
R_LINE_TO, 0.31f, -0.39f,
ARC_TO, 16.09f, 16.09f, 0, 0, 1, 44.36f, 57,
R_CUBIC_TO, 8.67f, 0, 15.76f, 6.87f, 16.13f, 15.44f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xff, 0xff, 0xff,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x41, 0x44, 0x47,
MOVE_TO, 169.11f, 40.08f,
V_LINE_TO, 33.5f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 30.94f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 6.58f,
H_LINE_TO, 288,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
V_LINE_TO, 131.5f,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
H_LINE_TO, 163,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 42.08f,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 6.11f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
MOVE_TO, 169.11f, 40.08f,
V_LINE_TO, 33.5f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, -4,
R_H_LINE_TO, 30.94f,
R_ARC_TO, 4, 4, 0, 0, 1, 4, 4,
R_V_LINE_TO, 6.58f,
H_LINE_TO, 288,
R_ARC_TO, 2, 2, 0, 0, 1, 2, 2,
V_LINE_TO, 131.5f,
R_ARC_TO, 2, 2, 0, 0, 1, -2, 2,
H_LINE_TO, 163,
R_ARC_TO, 2, 2, 0, 0, 1, -2, -2,
V_LINE_TO, 42.08f,
R_ARC_TO, 2, 2, 0, 0, 1, 2, -2,
R_H_LINE_TO, 6.11f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 208.88f, 83.59f,
R_LINE_TO, 0.70f, -0.70f,
R_LINE_TO, 3.54f, 3.54f,
R_LINE_TO, -0.70f, 0.70f,
CLOSE,
MOVE_TO, 212.41f, 82.88f,
R_LINE_TO, 0.70f, 0.70f,
R_LINE_TO, -3.53f, 3.54f,
R_LINE_TO, -0.70f, -0.70f,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 229.5f, 100.5f,
R_V_LINE_TO, 1,
R_H_LINE_TO, -8,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, -9, 2,
R_H_LINE_TO, -2,
R_V_LINE_TO, 1,
R_H_LINE_TO, 2,
R_V_LINE_TO, -1,
CLOSE,
R_MOVE_TO, 12, 0,
R_H_LINE_TO, -2,
R_V_LINE_TO, 1,
R_H_LINE_TO, 2,
R_V_LINE_TO, -1,
CLOSE,
NEW_PATH,
STROKE, 1,
PATH_COLOR_ARGB, 0xFF, 0x97, 0x97, 0x97,
DISABLE_AA,
MOVE_TO, 236.88f, 83.59f,
R_LINE_TO, 0.70f, -0.70f,
R_LINE_TO, 3.54f, 3.54f,
R_LINE_TO, -0.70f, 0.70f,
CLOSE,
MOVE_TO, 240.41f, 82.88f,
R_LINE_TO, 0.70f, 0.70f,
R_LINE_TO, -3.53f, 3.54f,
R_LINE_TO, -0.70f, -0.70f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
ROUND_RECT, 170, 48, 111, 7, 3.5,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x5F, 0x63, 0x68,
CIRCLE, 147, 92.5, 31,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xff, 0xff, 0xff,
MOVE_TO, 148.48f, 98.09f,
R_H_LINE_TO, -2.91f,
R_CUBIC_TO, -0.01f, -1.57f, 0.25f, -3.23f, 1.32f, -4.45f,
R_CUBIC_TO, 1.1f, -1.47f, 2.58f, -2.62f, 3.6f, -4.15f,
R_CUBIC_TO, 0.69f, -1.43f, 0.67f, -3.35f, -0.4f, -4.59f,
R_CUBIC_TO, -1.19f, -1.23f, -3.25f, -1.17f, -4.66f, -0.36f,
R_CUBIC_TO, -0.98f, 0.44f, -1.34f, 1.63f, -1.42f, 2.54f,
R_H_LINE_TO, -2.84f,
R_CUBIC_TO, -0.04f, -1.82f, 0.89f, -3.64f, 2.48f, -4.57f,
R_CUBIC_TO, 2.03f, -1.26f, 4.67f, -1.37f, 6.89f, -0.56f,
R_CUBIC_TO, 2, 0.75f, 3.27f, 2.83f, 3.31f, 4.93f,
R_CUBIC_TO, 0.16f, 1.78f, -0.51f, 3.56f, -1.69f, 4.88f,
R_CUBIC_TO, -0.78f, 1.03f, -1.75f, 1.91f, -2.64f, 2.84f,
R_CUBIC_TO, -0.78f, 0.98f, -1.04f, 2.27f, -1.03f, 3.5f,
CLOSE,
R_MOVE_TO, -3.03f, 4.95f,
R_CUBIC_TO, -0.09f, -1.19f, 1.21f, -1.97f, 2.28f, -1.6f,
R_CUBIC_TO, 1.43f, 0.32f, 1.54f, 2.64f, 0.17f, 3.12f,
R_CUBIC_TO, -1.01f, 0.43f, -2.4f, -0.16f, -2.44f, -1.35f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x53, 0x57, 0x5C,
MOVE_TO, 161, 64.83f,
CUBIC_TO, 171.09f, 69.94f, 178, 80.41f, 178, 92.49f,
R_CUBIC_TO, 0, 12.04f, -6.87f, 22.48f, -16.9f, 27.62f,
R_ARC_TO, 2, 2, 0, 0, 1, -0.1f, -0.62f,
V_LINE_TO, 64.83f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebauthnErrorDarkRepList, kWebauthnErrorDarkIcon, {kWebauthnErrorDarkPath, std::size(kWebauthnErrorDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kWebidArrowPath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 7, 16,
LINE_TO, 3, 12,
LINE_TO, 7, 8,
LINE_TO, 8.43f, 9.4f,
LINE_TO, 6.82f, 11,
LINE_TO, 17.18f, 11,
LINE_TO, 15.6f, 9.4f,
LINE_TO, 17, 8,
LINE_TO, 21, 12,
LINE_TO, 17, 16,
LINE_TO, 15.6f, 14.6f,
LINE_TO, 17.18f, 13,
LINE_TO, 6.82f, 13,
LINE_TO, 8.4f, 14.6f,
CLOSE,
MOVE_TO, 7, 16,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebidArrowRepList, kWebidArrowIcon, {kWebidArrowPath, std::size(kWebidArrowPath)})
VECTOR_ICON_REP_TEMPLATE(kWebidGlobePath, CANVAS_DIMENSIONS, 24,
FILL_RULE_NONZERO,
MOVE_TO, 12, 22,
CUBIC_TO, 10.62f, 22, 9.32f, 21.74f, 8.1f, 21.21f,
CUBIC_TO, 6.88f, 20.69f, 5.82f, 19.98f, 4.93f, 19.07f,
CUBIC_TO, 4.02f, 18.18f, 3.31f, 17.12f, 2.79f, 15.9f,
CUBIC_TO, 2.26f, 14.68f, 2, 13.38f, 2, 12,
CUBIC_TO, 2, 10.62f, 2.26f, 9.32f, 2.79f, 8.1f,
CUBIC_TO, 3.31f, 6.88f, 4.02f, 5.82f, 4.93f, 4.93f,
CUBIC_TO, 5.82f, 4.02f, 6.88f, 3.31f, 8.1f, 2.79f,
CUBIC_TO, 9.32f, 2.26f, 10.62f, 2, 12, 2,
CUBIC_TO, 13.38f, 2, 14.68f, 2.26f, 15.9f, 2.79f,
CUBIC_TO, 17.12f, 3.31f, 18.18f, 4.02f, 19.07f, 4.93f,
CUBIC_TO, 19.98f, 5.82f, 20.69f, 6.88f, 21.21f, 8.1f,
CUBIC_TO, 21.74f, 9.32f, 22, 10.62f, 22, 12,
CUBIC_TO, 22, 13.38f, 21.74f, 14.68f, 21.21f, 15.9f,
CUBIC_TO, 20.69f, 17.12f, 19.98f, 18.18f, 19.07f, 19.07f,
CUBIC_TO, 18.18f, 19.98f, 17.12f, 20.69f, 15.9f, 21.21f,
CUBIC_TO, 14.68f, 21.74f, 13.38f, 22, 12, 22,
CLOSE,
MOVE_TO, 12, 20,
CUBIC_TO, 14.23f, 20, 16.13f, 19.23f, 17.68f, 17.68f,
CUBIC_TO, 19.23f, 16.13f, 20, 14.23f, 20, 12,
CUBIC_TO, 20, 11.88f, 20, 11.76f, 19.99f, 11.64f,
CUBIC_TO, 19.98f, 11.51f, 19.98f, 11.41f, 19.98f, 11.32f,
CUBIC_TO, 19.89f, 11.81f, 19.67f, 12.21f, 19.3f, 12.52f,
CUBIC_TO, 18.93f, 12.84f, 18.5f, 13, 18, 13,
LINE_TO, 16, 13,
CUBIC_TO, 15.45f, 13, 14.98f, 12.8f, 14.59f, 12.41f,
CUBIC_TO, 14.2f, 12.02f, 14, 11.55f, 14, 11,
LINE_TO, 14, 10,
LINE_TO, 10, 10,
LINE_TO, 10, 8,
CUBIC_TO, 10, 7.45f, 10.2f, 6.98f, 10.59f, 6.59f,
CUBIC_TO, 10.98f, 6.2f, 11.45f, 6, 12, 6,
LINE_TO, 13, 6,
CUBIC_TO, 13, 5.62f, 13.11f, 5.28f, 13.31f, 4.99f,
CUBIC_TO, 13.52f, 4.7f, 13.77f, 4.46f, 14.07f, 4.27f,
CUBIC_TO, 13.74f, 4.19f, 13.4f, 4.13f, 13.06f, 4.07f,
CUBIC_TO, 12.72f, 4.02f, 12.37f, 4, 12, 4,
CUBIC_TO, 9.77f, 4, 7.88f, 4.77f, 6.32f, 6.32f,
CUBIC_TO, 4.77f, 7.88f, 4, 9.77f, 4, 12,
LINE_TO, 9, 12,
CUBIC_TO, 10.1f, 12, 11.04f, 12.39f, 11.82f, 13.18f,
CUBIC_TO, 12.61f, 13.96f, 13, 14.9f, 13, 16,
LINE_TO, 13, 17,
LINE_TO, 10, 17,
LINE_TO, 10, 19.75f,
CUBIC_TO, 10.33f, 19.83f, 10.66f, 19.89f, 10.99f, 19.94f,
CUBIC_TO, 11.31f, 19.98f, 11.65f, 20, 12, 20,
CLOSE,
MOVE_TO, 12, 20,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kWebidGlobeRepList, kWebidGlobeIcon, {kWebidGlobePath, std::size(kWebidGlobePath)})
VECTOR_ICON_REP_TEMPLATE(kZoomInPath, CANVAS_DIMENSIONS, 16,
FILL_RULE_NONZERO,
MOVE_TO, 12.74f, 13.75f,
LINE_TO, 8.73f, 9.72f,
R_ARC_TO, 4.03f, 4.03f, 0, 0, 1, -1.09f, 0.57f,
R_CUBIC_TO, -0.4f, 0.13f, -0.82f, 0.2f, -1.27f, 0.2f,
R_CUBIC_TO, -1.13f, 0, -2.1f, -0.4f, -2.89f, -1.2f,
R_CUBIC_TO, -0.8f, -0.8f, -1.2f, -1.76f, -1.2f, -2.9f,
R_CUBIC_TO, 0, -1.13f, 0.4f, -2.1f, 1.2f, -2.89f,
R_CUBIC_TO, 0.79f, -0.8f, 1.76f, -1.19f, 2.89f, -1.19f,
R_CUBIC_TO, 1.14f, 0, 2.1f, 0.4f, 2.9f, 1.2f,
R_CUBIC_TO, 0.8f, 0.8f, 1.2f, 1.76f, 1.2f, 2.9f,
R_CUBIC_TO, 0, 0.45f, -0.07f, 0.88f, -0.2f, 1.27f,
R_ARC_TO, 4.1f, 4.1f, 0, 0, 1, -0.55f, 1.08f,
R_LINE_TO, 4.02f, 4.02f,
CLOSE,
MOVE_TO, 6.37f, 9.11f,
R_ARC_TO, 2.63f, 2.63f, 0, 0, 0, 1.92f, -0.79f,
R_ARC_TO, 2.63f, 2.63f, 0, 0, 0, 0.79f, -1.92f,
R_CUBIC_TO, 0, -0.75f, -0.27f, -1.39f, -0.79f, -1.92f,
R_ARC_TO, 2.62f, 2.62f, 0, 0, 0, -1.92f, -0.79f,
R_CUBIC_TO, -0.75f, 0, -1.39f, 0.26f, -1.92f, 0.79f,
R_ARC_TO, 2.61f, 2.61f, 0, 0, 0, -0.79f, 1.92f,
R_CUBIC_TO, 0, 0.75f, 0.26f, 1.4f, 0.79f, 1.92f,
R_CUBIC_TO, 0.52f, 0.52f, 1.16f, 0.79f, 1.92f, 0.79f,
CLOSE,
R_MOVE_TO, -0.6f, -0.93f,
V_LINE_TO, 7,
H_LINE_TO, 4.59f,
V_LINE_TO, 5.8f,
H_LINE_TO, 5.77f,
V_LINE_TO, 4.63f,
R_H_LINE_TO, 1.2f,
R_V_LINE_TO, 1.18f,
R_H_LINE_TO, 1.18f,
V_LINE_TO, 7,
H_LINE_TO, 6.97f,
R_V_LINE_TO, 1.18f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomInRepList, kZoomInIcon, {kZoomInPath, std::size(kZoomInPath)})
VECTOR_ICON_REP_TEMPLATE(kZoomMinusPath, MOVE_TO, 32.15f, 30.43f,
CUBIC_TO, 34.57f, 27.62f, 36.02f, 23.98f, 36.02f, 20.01f,
CUBIC_TO, 36.02f, 11.17f, 28.85f, 4, 20.01f, 4,
CUBIC_TO, 11.17f, 4, 4, 11.17f, 4, 20.01f,
CUBIC_TO, 4, 28.85f, 11.17f, 36.02f, 20.01f, 36.02f,
CUBIC_TO, 23.98f, 36.02f, 27.62f, 34.57f, 30.43f, 32.15f,
LINE_TO, 31.09f, 32.84f,
LINE_TO, 31.09f, 34.79f,
LINE_TO, 40.33f, 44,
LINE_TO, 44, 40.33f,
LINE_TO, 34.79f, 31.09f,
LINE_TO, 32.84f, 31.09f,
LINE_TO, 32.15f, 30.43f,
CLOSE,
MOVE_TO, 20, 32,
CUBIC_TO, 13.36f, 32, 8, 26.64f, 8, 20,
CUBIC_TO, 8, 13.36f, 13.36f, 8, 20, 8,
CUBIC_TO, 26.64f, 8, 32, 13.36f, 32, 20,
CUBIC_TO, 32, 26.64f, 26.64f, 32, 20, 32,
LINE_TO, 20, 32,
CLOSE,
MOVE_TO, 13, 21.4f,
LINE_TO, 13, 18.6f,
LINE_TO, 27, 18.6f,
LINE_TO, 27, 21.4f,
LINE_TO, 13, 21.4f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomMinusRepList, kZoomMinusIcon, {kZoomMinusPath, std::size(kZoomMinusPath)})
VECTOR_ICON_REP_TEMPLATE(kZoomMinusChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
R_MOVE_TO, 15.94f, 17,
R_LINE_TO, -4.98f, -5,
R_CUBIC_TO, -0.41f, 0.32f, -0.87f, 0.57f, -1.37f, 0.74f,
ARC_TO, 4.71f, 4.71f, 0, 0, 1, 8, 13,
R_CUBIC_TO, -1.39f, 0, -2.57f, -0.48f, -3.54f, -1.46f,
CUBIC_TO, 3.48f, 10.57f, 3, 9.39f, 3, 8,
R_CUBIC_TO, 0, -1.39f, 0.48f, -2.57f, 1.46f, -3.54f,
CUBIC_TO, 5.43f, 3.48f, 6.61f, 3, 8, 3,
R_CUBIC_TO, 1.39f, 0, 2.57f, 0.48f, 3.54f, 1.46f,
CUBIC_TO, 12.52f, 5.43f, 13, 6.61f, 13, 8,
R_CUBIC_TO, 0, 0.56f, -0.09f, 1.09f, -0.26f, 1.59f,
R_CUBIC_TO, -0.17f, 0.5f, -0.41f, 0.96f, -0.72f, 1.37f,
R_LINE_TO, 4.98f, 4.98f,
CLOSE,
MOVE_TO, 8, 11.5f,
R_CUBIC_TO, 0.97f, 0, 1.8f, -0.34f, 2.48f, -1.02f,
R_CUBIC_TO, 0.68f, -0.68f, 1.02f, -1.51f, 1.02f, -2.48f,
R_CUBIC_TO, 0, -0.97f, -0.34f, -1.8f, -1.02f, -2.48f,
CUBIC_TO, 9.8f, 4.84f, 8.97f, 4.5f, 8, 4.5f,
R_CUBIC_TO, -0.97f, 0, -1.8f, 0.34f, -2.48f, 1.02f,
CUBIC_TO, 4.84f, 6.2f, 4.5f, 7.03f, 4.5f, 8,
R_CUBIC_TO, 0, 0.97f, 0.34f, 1.8f, 1.02f, 2.48f,
R_CUBIC_TO, 0.68f, 0.68f, 1.51f, 1.02f, 2.48f, 1.02f,
CLOSE,
MOVE_TO, 6, 8.75f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 4,
R_V_LINE_TO, 1.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomMinusChromeRefreshRepList, kZoomMinusChromeRefreshIcon, {kZoomMinusChromeRefreshPath, std::size(kZoomMinusChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kZoomMinusMenuRefreshPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 3.91f, 8.69f,
V_LINE_TO, 7.31f,
R_H_LINE_TO, 8.18f,
R_V_LINE_TO, 1.38f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomMinusMenuRefreshRepList, kZoomMinusMenuRefreshIcon, {kZoomMinusMenuRefreshPath, std::size(kZoomMinusMenuRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kZoomPlusChromeRefreshPath, CANVAS_DIMENSIONS, 20,
FILL_RULE_NONZERO,
R_MOVE_TO, 15.94f, 17,
R_LINE_TO, -4.98f, -5,
R_CUBIC_TO, -0.41f, 0.32f, -0.87f, 0.57f, -1.37f, 0.74f,
ARC_TO, 4.71f, 4.71f, 0, 0, 1, 8, 13,
R_CUBIC_TO, -1.39f, 0, -2.57f, -0.48f, -3.54f, -1.46f,
CUBIC_TO, 3.48f, 10.57f, 3, 9.39f, 3, 8,
R_CUBIC_TO, 0, -1.39f, 0.48f, -2.57f, 1.46f, -3.54f,
CUBIC_TO, 5.43f, 3.48f, 6.61f, 3, 8, 3,
R_CUBIC_TO, 1.39f, 0, 2.57f, 0.48f, 3.54f, 1.46f,
CUBIC_TO, 12.52f, 5.43f, 13, 6.61f, 13, 8,
R_CUBIC_TO, 0, 0.56f, -0.09f, 1.09f, -0.26f, 1.59f,
R_CUBIC_TO, -0.17f, 0.5f, -0.41f, 0.96f, -0.72f, 1.37f,
R_LINE_TO, 4.98f, 4.98f,
CLOSE,
MOVE_TO, 8, 11.5f,
R_CUBIC_TO, 0.97f, 0, 1.8f, -0.34f, 2.48f, -1.02f,
R_CUBIC_TO, 0.68f, -0.68f, 1.02f, -1.51f, 1.02f, -2.48f,
R_CUBIC_TO, 0, -0.97f, -0.34f, -1.8f, -1.02f, -2.48f,
CUBIC_TO, 9.8f, 4.84f, 8.97f, 4.5f, 8, 4.5f,
R_CUBIC_TO, -0.97f, 0, -1.8f, 0.34f, -2.48f, 1.02f,
CUBIC_TO, 4.84f, 6.2f, 4.5f, 7.03f, 4.5f, 8,
R_CUBIC_TO, 0, 0.97f, 0.34f, 1.8f, 1.02f, 2.48f,
R_CUBIC_TO, 0.68f, 0.68f, 1.51f, 1.02f, 2.48f, 1.02f,
CLOSE,
R_MOVE_TO, -0.75f, -1.25f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, -1.5f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, -1.5f,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
R_H_LINE_TO, 1.5f,
R_V_LINE_TO, 1.5f,
R_H_LINE_TO, -1.5f,
R_V_LINE_TO, 1.5f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomPlusChromeRefreshRepList, kZoomPlusChromeRefreshIcon, {kZoomPlusChromeRefreshPath, std::size(kZoomPlusChromeRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kZoomPlusMenuRefreshPath, CANVAS_DIMENSIONS, 16,
MOVE_TO, 7.31f, 12.09f,
R_V_LINE_TO, -3.4f,
H_LINE_TO, 3.91f,
V_LINE_TO, 7.31f,
R_H_LINE_TO, 3.4f,
R_V_LINE_TO, -3.4f,
R_H_LINE_TO, 1.38f,
R_V_LINE_TO, 3.4f,
R_H_LINE_TO, 3.4f,
R_V_LINE_TO, 1.38f,
H_LINE_TO, 8.69f,
R_V_LINE_TO, 3.4f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kZoomPlusMenuRefreshRepList, kZoomPlusMenuRefreshIcon, {kZoomPlusMenuRefreshPath, std::size(kZoomPlusMenuRefreshPath)})
VECTOR_ICON_REP_TEMPLATE(kSecurePaymentConfirmationFingerprintPath, CANVAS_DIMENSIONS, 448,
PATH_COLOR_ARGB, 0xFF, 0xD2, 0xE3, 0xFC,
CIRCLE, 223.99f, 74.37f, 45.68f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
R_MOVE_TO, 239.83f, 53.86f,
R_CUBIC_TO, -0.22f, 0, -0.43f, -0.05f, -0.63f, -0.16f,
R_CUBIC_TO, -5.23f, -2.7f, -9.75f, -3.84f, -15.17f, -3.84f,
R_CUBIC_TO, -5.4f, 0, -10.51f, 1.28f, -15.17f, 3.84f,
R_CUBIC_TO, -0.65f, 0.35f, -1.47f, 0.11f, -1.85f, -0.55f,
R_CUBIC_TO, -0.35f, -0.65f, -0.11f, -1.5f, 0.55f, -1.85f,
R_CUBIC_TO, 5.07f, -2.75f, 10.62f, -4.17f, 16.48f, -4.17f,
R_CUBIC_TO, 5.8f, 0, 10.87f, 1.28f, 16.42f, 4.14f,
R_CUBIC_TO, 0.68f, 0.35f, 0.93f, 1.17f, 0.57f, 1.82f,
R_CUBIC_TO, -0.24f, 0.5f, -0.7f, 0.77f, -1.2f, 0.77f,
CLOSE,
R_MOVE_TO, -38.97f, 14.3f,
R_CUBIC_TO, -0.27f, 0, -0.55f, -0.08f, -0.79f, -0.25f,
R_CUBIC_TO, -0.63f, -0.43f, -0.76f, -1.28f, -0.33f, -1.9f,
R_CUBIC_TO, 2.7f, -3.81f, 6.13f, -6.81f, 10.21f, -8.9f,
R_CUBIC_TO, 8.55f, -4.41f, 19.5f, -4.44f, 28.08f, -0.03f,
R_CUBIC_TO, 4.08f, 2.1f, 7.52f, 5.07f, 10.21f, 8.85f,
R_CUBIC_TO, 0.43f, 0.6f, 0.3f, 1.47f, -0.33f, 1.9f,
R_CUBIC_TO, -0.63f, 0.43f, -1.47f, 0.3f, -1.9f, -0.33f,
R_CUBIC_TO, -2.45f, -3.43f, -5.55f, -6.13f, -9.23f, -8,
R_CUBIC_TO, -7.82f, -4, -17.81f, -4, -25.6f, 0.03f,
R_CUBIC_TO, -3.7f, 1.9f, -6.81f, 4.63f, -9.26f, 8.06f,
R_CUBIC_TO, -0.22f, 0.38f, -0.63f, 0.57f, -1.06f, 0.57f,
CLOSE,
R_MOVE_TO, 17.02f, 32.87f,
R_CUBIC_TO, -0.35f, 0, -0.7f, -0.13f, -0.95f, -0.41f,
R_CUBIC_TO, -2.37f, -2.37f, -3.65f, -3.9f, -5.47f, -7.19f,
R_CUBIC_TO, -1.88f, -3.35f, -2.86f, -7.43f, -2.86f, -11.82f,
R_CUBIC_TO, 0, -8.09f, 6.92f, -14.68f, 15.41f, -14.68f,
R_CUBIC_TO, 8.5f, 0, 15.41f, 6.59f, 15.41f, 14.68f,
R_CUBIC_TO, 0, 0.76f, -0.6f, 1.36f, -1.36f, 1.36f,
R_CUBIC_TO, -0.76f, 0, -1.36f, -0.6f, -1.36f, -1.36f,
R_CUBIC_TO, 0, -6.59f, -5.7f, -11.95f, -12.69f, -11.95f,
R_CUBIC_TO, -7, 0, -12.69f, 5.37f, -12.69f, 11.95f,
R_CUBIC_TO, 0, 3.92f, 0.87f, 7.55f, 2.53f, 10.48f,
R_CUBIC_TO, 1.75f, 3.13f, 2.95f, 4.47f, 5.04f, 6.59f,
R_CUBIC_TO, 0.52f, 0.55f, 0.52f, 1.39f, 0, 1.93f,
R_CUBIC_TO, -0.28f, 0.27f, -0.64f, 0.4f, -1, 0.4f,
CLOSE,
R_MOVE_TO, 19.53f, -5.04f,
R_CUBIC_TO, -3.24f, 0, -6.1f, -0.82f, -8.45f, -2.42f,
R_CUBIC_TO, -4.06f, -2.75f, -6.48f, -7.22f, -6.48f, -11.95f,
R_CUBIC_TO, 0, -0.76f, 0.6f, -1.36f, 1.36f, -1.36f,
R_CUBIC_TO, 0.76f, 0, 1.36f, 0.6f, 1.36f, 1.36f,
R_CUBIC_TO, 0, 3.84f, 1.96f, 7.46f, 5.28f, 9.7f,
R_CUBIC_TO, 1.93f, 1.3f, 4.2f, 1.93f, 6.92f, 1.93f,
R_CUBIC_TO, 0.65f, 0, 1.75f, -0.08f, 2.83f, -0.27f,
R_CUBIC_TO, 0.73f, -0.13f, 1.45f, 0.35f, 1.58f, 1.12f,
R_CUBIC_TO, 0.13f, 0.73f, -0.35f, 1.45f, -1.12f, 1.58f,
R_CUBIC_TO, -1.56f, 0.28f, -2.92f, 0.31f, -3.3f, 0.31f,
CLOSE,
R_MOVE_TO, -5.48f, 5.61f,
R_CUBIC_TO, -0.11f, 0, -0.25f, -0.03f, -0.35f, -0.05f,
R_CUBIC_TO, -4.33f, -1.2f, -7.16f, -2.8f, -10.13f, -5.72f,
R_CUBIC_TO, -3.81f, -3.78f, -5.91f, -8.83f, -5.91f, -14.22f,
R_CUBIC_TO, 0, -4.41f, 3.76f, -8, 8.39f, -8,
R_CUBIC_TO, 4.63f, 0, 8.39f, 3.6f, 8.39f, 8,
R_CUBIC_TO, 0, 2.92f, 2.53f, 5.28f, 5.67f, 5.28f,
R_CUBIC_TO, 3.13f, 0, 5.67f, -2.37f, 5.67f, -5.28f,
R_CUBIC_TO, 0, -10.27f, -8.85f, -18.6f, -19.75f, -18.6f,
R_CUBIC_TO, -7.73f, 0, -14.81f, 4.3f, -18, 10.98f,
R_CUBIC_TO, -1.06f, 2.2f, -1.6f, 4.8f, -1.6f, 7.63f,
R_CUBIC_TO, 0, 2.12f, 0.19f, 5.47f, 1.82f, 9.83f,
R_CUBIC_TO, 0.27f, 0.7f, -0.08f, 1.5f, -0.79f, 1.75f,
R_CUBIC_TO, -0.7f, 0.27f, -1.5f, -0.11f, -1.75f, -0.79f,
R_CUBIC_TO, -1.33f, -3.57f, -1.99f, -7.11f, -1.99f, -10.78f,
R_CUBIC_TO, 0, -3.27f, 0.63f, -6.23f, 1.85f, -8.83f,
R_CUBIC_TO, 3.62f, -7.6f, 11.65f, -12.53f, 20.45f, -12.53f,
R_CUBIC_TO, 12.39f, 0, 22.47f, 9.56f, 22.47f, 21.33f,
R_CUBIC_TO, 0, 4.41f, -3.76f, 8, -8.39f, 8,
R_CUBIC_TO, -4.63f, 0, -8.39f, -3.6f, -8.39f, -8,
R_CUBIC_TO, 0, -2.92f, -2.53f, -5.28f, -5.67f, -5.28f,
R_CUBIC_TO, -3.13f, 0, -5.67f, 2.37f, -5.67f, 5.28f,
R_CUBIC_TO, 0, 4.65f, 1.8f, 9.01f, 5.1f, 12.28f,
R_CUBIC_TO, 2.59f, 2.56f, 5.07f, 3.98f, 8.9f, 5.04f,
R_CUBIC_TO, 0.73f, 0.19f, 1.15f, 0.95f, 0.95f, 1.66f,
R_CUBIC_TO, -0.13f, 0.61f, -0.7f, 1.02f, -1.27f, 1.02f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
R_MOVE_TO, 175.26f, 137.79f,
R_CUBIC_TO, -2.21f, 1.85f, -5.51f, 1.55f, -7.35f, -0.66f,
R_CUBIC_TO, -1.85f, -2.21f, -1.55f, -5.51f, 0.66f, -7.35f,
R_LINE_TO, 8.01f, -6.69f,
R_CUBIC_TO, 2.21f, -1.85f, 5.51f, -1.55f, 7.35f, 0.66f,
R_CUBIC_TO, 1.85f, 2.21f, 1.55f, 5.51f, -0.66f, 7.35f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
R_MOVE_TO, 277.97f, 24,
R_CUBIC_TO, -1.2f, 0.21f, -1.72f, 1.63f, -0.94f, 2.57f,
R_LINE_TO, 9.35f, 11.2f,
R_CUBIC_TO, 0.78f, 0.93f, 2.28f, 0.67f, 2.69f, -0.47f,
R_LINE_TO, 5.02f, -13.69f,
R_CUBIC_TO, 0.42f, -1.15f, -0.55f, -2.32f, -1.75f, -2.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
R_MOVE_TO, 152, 26.5f,
R_CUBIC_TO, -0.26f, -0.54f, -0.04f, -1.19f, 0.49f, -1.45f,
R_LINE_TO, 7.81f, -3.84f,
R_CUBIC_TO, 0.54f, -0.26f, 1.19f, -0.04f, 1.45f, 0.49f,
R_LINE_TO, 3.84f, 7.81f,
R_CUBIC_TO, 0.26f, 0.54f, 0.05f, 1.19f, -0.49f, 1.45f,
R_LINE_TO, -7.81f, 3.84f,
R_CUBIC_TO, -0.54f, 0.27f, -1.19f, 0.05f, -1.45f, -0.49f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 1.49f,
MOVE_TO, 133.45f, 105.41f,
LINE_TO, 142.93f, 102.24f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xEA, 0x43, 0x35,
STROKE, 1.49f,
MOVE_TO, 305.58f, 60.83f,
LINE_TO, 314.55f, 58.24f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
STROKE, 1.49f,
MOVE_TO, 229.57f, 22.8f,
LINE_TO, 232.04f, 9.75f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 274.53f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xD2, 0xE3, 0xFC,
STROKE, 2.24f,
CIRCLE, 274.53f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFF, 0xFF, 0xFF,
CIRCLE, 173.46f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xD2, 0xE3, 0xFC,
STROKE, 2.24f,
CIRCLE, 173.46f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.24f,
R_MOVE_TO, 179.52f, 82.71f,
R_H_LINE_TO, -12.11f,
R_CUBIC_TO, -0.47f, 0, -0.85f, -0.38f, -0.85f, -0.85f,
R_V_LINE_TO, -9.4f,
R_CUBIC_TO, 0, -0.47f, 0.38f, -0.85f, 0.85f, -0.85f,
R_H_LINE_TO, 12.11f,
R_CUBIC_TO, 0.47f, 0, 0.85f, 0.38f, 0.85f, 0.85f,
R_V_LINE_TO, 9.4f,
R_CUBIC_TO, 0, 0.47f, -0.38f, 0.85f, -0.85f, 0.85f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.24f,
R_MOVE_TO, 176.34f, 71.6f,
R_V_LINE_TO, -4.03f,
R_CUBIC_TO, 0, -1.59f, -1.29f, -2.88f, -2.88f, -2.88f,
R_CUBIC_TO, -1.59f, 0, -2.88f, 1.29f, -2.88f, 2.88f,
R_V_LINE_TO, 4.03f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.80f,
MOVE_TO, 265.45f, 76.65f,
LINE_TO, 271.07f, 82.27f,
LINE_TO, 283.62f, 69.73f,
MOVE_TO, 283.62f, 69.73f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSecurePaymentConfirmationFingerprintRepList, kSecurePaymentConfirmationFingerprintIcon, {kSecurePaymentConfirmationFingerprintPath, std::size(kSecurePaymentConfirmationFingerprintPath)})
VECTOR_ICON_REP_TEMPLATE(kSecurePaymentConfirmationFingerprintDarkPath, CANVAS_DIMENSIONS, 448,
PATH_COLOR_ARGB, 0xFF, 0x18, 0x5A, 0xBC,
R_MOVE_TO, 257.48f, 74.37f,
R_CUBIC_TO, 0, -6.73f, 3.9f, -12.55f, 9.56f, -15.32f,
R_CUBIC_TO, -2.2f, -6.2f, -5.78f, -12.02f, -10.75f, -16.99f,
R_CUBIC_TO, -17.84f, -17.84f, -46.77f, -17.84f, -64.61f, 0,
R_CUBIC_TO, -4.96f, 4.96f, -8.54f, 10.78f, -10.75f, 16.99f,
R_CUBIC_TO, 5.67f, 2.77f, 9.56f, 8.59f, 9.56f, 15.32f,
R_CUBIC_TO, 0, 6.73f, -3.9f, 12.55f, -9.56f, 15.32f,
R_CUBIC_TO, 2.2f, 6.2f, 5.78f, 12.02f, 10.75f, 16.99f,
R_CUBIC_TO, 17.84f, 17.84f, 46.77f, 17.84f, 64.61f, 0,
R_CUBIC_TO, 4.96f, -4.96f, 8.54f, -10.78f, 10.75f, -16.99f,
R_CUBIC_TO, -5.66f, -2.77f, -9.56f, -8.6f, -9.56f, -15.32f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x8A, 0xB4, 0xF8,
R_MOVE_TO, 239.83f, 53.86f,
R_CUBIC_TO, -0.22f, 0, -0.43f, -0.05f, -0.63f, -0.16f,
R_CUBIC_TO, -5.23f, -2.7f, -9.75f, -3.84f, -15.17f, -3.84f,
R_CUBIC_TO, -5.4f, 0, -10.51f, 1.28f, -15.17f, 3.84f,
R_CUBIC_TO, -0.65f, 0.35f, -1.47f, 0.11f, -1.85f, -0.55f,
R_CUBIC_TO, -0.35f, -0.65f, -0.11f, -1.5f, 0.55f, -1.85f,
R_CUBIC_TO, 5.07f, -2.75f, 10.62f, -4.17f, 16.48f, -4.17f,
R_CUBIC_TO, 5.8f, 0, 10.87f, 1.28f, 16.42f, 4.14f,
R_CUBIC_TO, 0.68f, 0.35f, 0.93f, 1.17f, 0.57f, 1.82f,
R_CUBIC_TO, -0.24f, 0.5f, -0.7f, 0.77f, -1.2f, 0.77f,
CLOSE,
R_MOVE_TO, -38.97f, 14.3f,
R_CUBIC_TO, -0.27f, 0, -0.55f, -0.08f, -0.79f, -0.25f,
R_CUBIC_TO, -0.63f, -0.43f, -0.76f, -1.28f, -0.33f, -1.9f,
R_CUBIC_TO, 2.7f, -3.81f, 6.13f, -6.81f, 10.21f, -8.9f,
R_CUBIC_TO, 8.55f, -4.41f, 19.5f, -4.44f, 28.08f, -0.03f,
R_CUBIC_TO, 4.08f, 2.1f, 7.52f, 5.07f, 10.21f, 8.85f,
R_CUBIC_TO, 0.43f, 0.6f, 0.3f, 1.47f, -0.33f, 1.9f,
R_CUBIC_TO, -0.63f, 0.43f, -1.47f, 0.3f, -1.9f, -0.33f,
R_CUBIC_TO, -2.45f, -3.43f, -5.55f, -6.13f, -9.23f, -8,
R_CUBIC_TO, -7.82f, -4, -17.81f, -4, -25.6f, 0.03f,
R_CUBIC_TO, -3.7f, 1.9f, -6.81f, 4.63f, -9.26f, 8.06f,
R_CUBIC_TO, -0.22f, 0.38f, -0.63f, 0.57f, -1.06f, 0.57f,
CLOSE,
R_MOVE_TO, 17.02f, 32.87f,
R_CUBIC_TO, -0.35f, 0, -0.7f, -0.13f, -0.95f, -0.41f,
R_CUBIC_TO, -2.37f, -2.37f, -3.65f, -3.9f, -5.47f, -7.19f,
R_CUBIC_TO, -1.88f, -3.35f, -2.86f, -7.43f, -2.86f, -11.82f,
R_CUBIC_TO, 0, -8.09f, 6.92f, -14.68f, 15.41f, -14.68f,
R_CUBIC_TO, 8.5f, 0, 15.41f, 6.59f, 15.41f, 14.68f,
R_CUBIC_TO, 0, 0.76f, -0.6f, 1.36f, -1.36f, 1.36f,
R_CUBIC_TO, -0.76f, 0, -1.36f, -0.6f, -1.36f, -1.36f,
R_CUBIC_TO, 0, -6.59f, -5.7f, -11.95f, -12.69f, -11.95f,
R_CUBIC_TO, -7, 0, -12.69f, 5.37f, -12.69f, 11.95f,
R_CUBIC_TO, 0, 3.92f, 0.87f, 7.55f, 2.53f, 10.48f,
R_CUBIC_TO, 1.75f, 3.13f, 2.95f, 4.47f, 5.04f, 6.59f,
R_CUBIC_TO, 0.52f, 0.55f, 0.52f, 1.39f, 0, 1.93f,
R_CUBIC_TO, -0.28f, 0.27f, -0.64f, 0.4f, -1, 0.4f,
CLOSE,
R_MOVE_TO, 19.53f, -5.04f,
R_CUBIC_TO, -3.24f, 0, -6.1f, -0.82f, -8.45f, -2.42f,
R_CUBIC_TO, -4.06f, -2.75f, -6.48f, -7.22f, -6.48f, -11.95f,
R_CUBIC_TO, 0, -0.76f, 0.6f, -1.36f, 1.36f, -1.36f,
R_CUBIC_TO, 0.76f, 0, 1.36f, 0.6f, 1.36f, 1.36f,
R_CUBIC_TO, 0, 3.84f, 1.96f, 7.46f, 5.28f, 9.7f,
R_CUBIC_TO, 1.93f, 1.3f, 4.2f, 1.93f, 6.92f, 1.93f,
R_CUBIC_TO, 0.65f, 0, 1.75f, -0.08f, 2.83f, -0.27f,
R_CUBIC_TO, 0.73f, -0.13f, 1.45f, 0.35f, 1.58f, 1.12f,
R_CUBIC_TO, 0.13f, 0.73f, -0.35f, 1.45f, -1.12f, 1.58f,
R_CUBIC_TO, -1.56f, 0.28f, -2.92f, 0.31f, -3.3f, 0.31f,
CLOSE,
R_MOVE_TO, -5.48f, 5.61f,
R_CUBIC_TO, -0.11f, 0, -0.25f, -0.03f, -0.35f, -0.05f,
R_CUBIC_TO, -4.33f, -1.2f, -7.16f, -2.8f, -10.13f, -5.72f,
R_CUBIC_TO, -3.81f, -3.78f, -5.91f, -8.83f, -5.91f, -14.22f,
R_CUBIC_TO, 0, -4.41f, 3.76f, -8, 8.39f, -8,
R_CUBIC_TO, 4.63f, 0, 8.39f, 3.6f, 8.39f, 8,
R_CUBIC_TO, 0, 2.92f, 2.53f, 5.28f, 5.67f, 5.28f,
R_CUBIC_TO, 3.13f, 0, 5.67f, -2.37f, 5.67f, -5.28f,
R_CUBIC_TO, 0, -10.27f, -8.85f, -18.6f, -19.75f, -18.6f,
R_CUBIC_TO, -7.73f, 0, -14.81f, 4.3f, -18, 10.98f,
R_CUBIC_TO, -1.06f, 2.2f, -1.6f, 4.8f, -1.6f, 7.63f,
R_CUBIC_TO, 0, 2.12f, 0.19f, 5.47f, 1.82f, 9.83f,
R_CUBIC_TO, 0.27f, 0.7f, -0.08f, 1.5f, -0.79f, 1.75f,
R_CUBIC_TO, -0.7f, 0.27f, -1.5f, -0.11f, -1.75f, -0.79f,
R_CUBIC_TO, -1.33f, -3.57f, -1.99f, -7.11f, -1.99f, -10.78f,
R_CUBIC_TO, 0, -3.27f, 0.63f, -6.23f, 1.85f, -8.83f,
R_CUBIC_TO, 3.62f, -7.6f, 11.65f, -12.53f, 20.45f, -12.53f,
R_CUBIC_TO, 12.39f, 0, 22.47f, 9.56f, 22.47f, 21.33f,
R_CUBIC_TO, 0, 4.41f, -3.76f, 8, -8.39f, 8,
R_CUBIC_TO, -4.63f, 0, -8.39f, -3.6f, -8.39f, -8,
R_CUBIC_TO, 0, -2.92f, -2.53f, -5.28f, -5.67f, -5.28f,
R_CUBIC_TO, -3.13f, 0, -5.67f, 2.37f, -5.67f, 5.28f,
R_CUBIC_TO, 0, 4.65f, 1.8f, 9.01f, 5.1f, 12.28f,
R_CUBIC_TO, 2.59f, 2.56f, 5.07f, 3.98f, 8.9f, 5.04f,
R_CUBIC_TO, 0.73f, 0.19f, 1.15f, 0.95f, 0.95f, 1.66f,
R_CUBIC_TO, -0.13f, 0.61f, -0.7f, 1.02f, -1.27f, 1.02f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
R_MOVE_TO, 175.26f, 137.79f,
R_CUBIC_TO, -2.21f, 1.85f, -5.51f, 1.55f, -7.35f, -0.66f,
R_CUBIC_TO, -1.85f, -2.21f, -1.55f, -5.51f, 0.66f, -7.35f,
R_LINE_TO, 8.01f, -6.69f,
R_CUBIC_TO, 2.21f, -1.85f, 5.51f, -1.55f, 7.35f, 0.66f,
R_CUBIC_TO, 1.85f, 2.21f, 1.55f, 5.51f, -0.66f, 7.35f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
R_MOVE_TO, 277.97f, 24,
R_CUBIC_TO, -1.2f, 0.21f, -1.72f, 1.63f, -0.94f, 2.57f,
R_LINE_TO, 9.35f, 11.2f,
R_CUBIC_TO, 0.78f, 0.93f, 2.28f, 0.67f, 2.69f, -0.47f,
R_LINE_TO, 5.02f, -13.69f,
R_CUBIC_TO, 0.42f, -1.15f, -0.55f, -2.32f, -1.75f, -2.11f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x42, 0x85, 0xF4,
R_MOVE_TO, 152, 26.5f,
R_CUBIC_TO, -0.26f, -0.54f, -0.04f, -1.19f, 0.49f, -1.45f,
R_LINE_TO, 7.81f, -3.84f,
R_CUBIC_TO, 0.54f, -0.26f, 1.19f, -0.04f, 1.45f, 0.49f,
R_LINE_TO, 3.84f, 7.81f,
R_CUBIC_TO, 0.26f, 0.54f, 0.05f, 1.19f, -0.49f, 1.45f,
R_LINE_TO, -7.81f, 3.84f,
R_CUBIC_TO, -0.54f, 0.27f, -1.19f, 0.05f, -1.45f, -0.49f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 1.49f,
MOVE_TO, 133.45f, 105.41f,
LINE_TO, 142.93f, 102.24f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xEA, 0x43, 0x35,
STROKE, 1.49f,
MOVE_TO, 305.58f, 60.83f,
LINE_TO, 314.55f, 58.24f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0xFB, 0xBC, 0x04,
STROKE, 1.49f,
MOVE_TO, 229.57f, 22.8f,
LINE_TO, 232.04f, 9.75f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x18, 0x5A, 0xBC,
STROKE, 2.24f,
CIRCLE, 274.53f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x18, 0x5A, 0xBC,
STROKE, 2.24f,
CIRCLE, 173.46f, 74.37f, 17.04f,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.24f,
R_MOVE_TO, 179.52f, 82.71f,
R_H_LINE_TO, -12.11f,
R_CUBIC_TO, -0.47f, 0, -0.85f, -0.38f, -0.85f, -0.85f,
R_V_LINE_TO, -9.4f,
R_CUBIC_TO, 0, -0.47f, 0.38f, -0.85f, 0.85f, -0.85f,
R_H_LINE_TO, 12.11f,
R_CUBIC_TO, 0.47f, 0, 0.85f, 0.38f, 0.85f, 0.85f,
R_V_LINE_TO, 9.4f,
R_CUBIC_TO, 0, 0.47f, -0.38f, 0.85f, -0.85f, 0.85f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.24f,
R_MOVE_TO, 176.34f, 71.6f,
R_V_LINE_TO, -4.03f,
R_CUBIC_TO, 0, -1.59f, -1.29f, -2.88f, -2.88f, -2.88f,
R_CUBIC_TO, -1.59f, 0, -2.88f, 1.29f, -2.88f, 2.88f,
R_V_LINE_TO, 4.03f,
CLOSE,
NEW_PATH,
PATH_COLOR_ARGB, 0xFF, 0x34, 0xA8, 0x53,
STROKE, 2.80f,
MOVE_TO, 265.45f, 76.65f,
LINE_TO, 271.07f, 82.27f,
LINE_TO, 283.62f, 69.73f,
MOVE_TO, 283.62f, 69.73f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kSecurePaymentConfirmationFingerprintDarkRepList, kSecurePaymentConfirmationFingerprintDarkIcon, {kSecurePaymentConfirmationFingerprintDarkPath, std::size(kSecurePaymentConfirmationFingerprintDarkPath)})
VECTOR_ICON_REP_TEMPLATE(kNearbySharePath, CANVAS_DIMENSIONS, 24,
MOVE_TO, 2.72f, 8.87f,
CUBIC_TO, 2.24f, 9.32f, 2.23f, 10.07f, 2.69f, 10.54f,
LINE_TO, 3.89f, 11.78f,
LINE_TO, 5.54f, 10.08f,
LINE_TO, 4.39f, 8.89f,
CUBIC_TO, 3.94f, 8.42f, 3.19f, 8.41f, 2.72f, 8.87f,
CLOSE,
MOVE_TO, 16.26f, 15.75f,
LINE_TO, 17.26f, 14.73f,
LINE_TO, 18.92f, 13.04f,
LINE_TO, 21.35f, 10.55f,
CUBIC_TO, 21.57f, 10.31f, 21.68f, 10.02f, 21.68f, 9.72f,
CUBIC_TO, 21.68f, 9.41f, 21.56f, 9.11f, 21.32f, 8.87f,
CUBIC_TO, 20.86f, 8.42f, 20.1f, 8.42f, 19.65f, 8.89f,
LINE_TO, 17.26f, 11.35f,
LINE_TO, 15.6f, 13.04f,
LINE_TO, 14.57f, 14.09f,
CUBIC_TO, 13.89f, 14.79f, 12.99f, 15.17f, 12.02f, 15.17f,
CUBIC_TO, 11.15f, 15.17f, 10.35f, 14.86f, 9.71f, 14.3f,
LINE_TO, 8.06f, 15.99f,
CUBIC_TO, 9.15f, 16.99f, 10.54f, 17.54f, 12.02f, 17.54f,
CUBIC_TO, 13.63f, 17.54f, 15.14f, 16.91f, 16.26f, 15.75f,
CLOSE,
MOVE_TO, 12.02f, 8.54f,
CUBIC_TO, 10.41f, 8.54f, 8.9f, 9.17f, 7.78f, 10.33f,
LINE_TO, 4.07f, 14.13f,
LINE_TO, 2.69f, 15.54f,
CUBIC_TO, 2.24f, 16, 2.24f, 16.76f, 2.72f, 17.21f,
CUBIC_TO, 3.18f, 17.66f, 3.94f, 17.66f, 4.39f, 17.19f,
LINE_TO, 6.78f, 14.73f,
LINE_TO, 8.44f, 13.04f,
LINE_TO, 9.47f, 11.99f,
CUBIC_TO, 10.15f, 11.29f, 11.05f, 10.91f, 12.02f, 10.91f,
CUBIC_TO, 12.88f, 10.91f, 13.68f, 11.22f, 14.33f, 11.78f,
LINE_TO, 15.98f, 10.09f,
CUBIC_TO, 14.89f, 9.09f, 13.5f, 8.54f, 12.02f, 8.54f,
CLOSE,
MOVE_TO, 21.68f, 16.37f,
CUBIC_TO, 21.68f, 16.07f, 21.57f, 15.77f, 21.35f, 15.54f,
LINE_TO, 20.16f, 14.32f,
LINE_TO, 18.51f, 16.03f,
LINE_TO, 19.65f, 17.2f,
CUBIC_TO, 20.1f, 17.67f, 20.85f, 17.69f, 21.32f, 17.22f,
CUBIC_TO, 21.56f, 16.99f, 21.68f, 16.68f, 21.68f, 16.37f,
CLOSE)
VECTOR_ICON_TEMPLATE_CC(kNearbyShareRepList, kNearbyShareIcon, {kNearbySharePath, std::size(kNearbySharePath)})
